** USER SETTINGS **


cd "YOUR DIRECTORY\SWAY.Replication.June2013\Data\Raw data and cleaning\"
    
clear
set mem 30m
set more off

use SWAYI\HHsurvey_sectionII_27May2008.dta, clear

keep if hhiid==0
keep if age>=14 & age<=30
drop if camp==.

gen no_survive=cond(died==1 | no_return==1,1,0)

keep id2 camp parish village household block firstname1 surname1 firstname2 surname2 hhiia hhiib age abd abd_per abd_times no_return no_survive
order id2 camp parish village household block firstname1 surname1 firstname2 surname2 hhiia hhiib age abd abd_per abd_times no_return no_survive

sort id2 hhiia

set seed 1234
gen rand=uniform()

sort no_survive abd camp rand
bysort no_survive abd camp: egen rank=rank(rand)

gen sample=0 if no_survive==1
gen sample_prob=1 if no_surv==1

replace sample=1 if abd==1 & no_surv==0
replace sample_prob=1 if abd==1 & no_surv==0

bysort camp: egen n_abd=count(id2) if abd==1 & no_surv==0
bysort camp: egen n_nonabd=count(id2) if abd==0 & no_surv==0
table camp, c(count n_abd count n_nonabd)

replace sample=1 if abd==0 & no_surv==0 & camp==1 & rank<=64
replace sample=1 if abd==0 & no_surv==0 & camp==2 & rank<=58
replace sample=1 if abd==0 & no_surv==0 & camp==3 & rank<=52
replace sample=1 if abd==0 & no_surv==0 & camp==4 & rank<=75
replace sample=1 if abd==0 & no_surv==0 & camp==5 & rank<=62
replace sample=1 if abd==0 & no_surv==0 & camp==6 & rank<=73
replace sample=1 if abd==0 & no_surv==0 & camp==7 & rank<=63
replace sample=1 if abd==0 & no_surv==0 & camp==8 & rank<=79

replace sample_prob=64/n_nonabd if abd==0 & no_surv==0 & camp==1 & rank<=64
replace sample_prob=58/n_nonabd if abd==0 & no_surv==0 & camp==2 & rank<=58
replace sample_prob=52/n_nonabd if abd==0 & no_surv==0 & camp==3 & rank<=52
replace sample_prob=75/n_nonabd if abd==0 & no_surv==0 & camp==4 & rank<=75
replace sample_prob=62/n_nonabd if abd==0 & no_surv==0 & camp==5 & rank<=62
replace sample_prob=73/n_nonabd if abd==0 & no_surv==0 & camp==6 & rank<=73
replace sample_prob=63/n_nonabd if abd==0 & no_surv==0 & camp==7 & rank<=63
replace sample_prob=79/n_nonabd if abd==0 & no_surv==0 & camp==8 & rank<=79

replace sample_prob=0 if sample==.
replace sample=0 if sample==.

ren id2 hh_id
ren hhiia roster_row
ren hhiib name
ren abd_per abd_length
ren firstname1 hh_head1_name
ren surname1 hh_head1_surname
ren firstname2 hh_head2_name
ren surname2 hh_head2_surname

gsort -sample no_surv rand
egen id=seq()
replace id=id+1000
order id hh_id roster_row name

drop n_abd n_nonabd rank rand

save SWAYII\tempdata\temp.dta, replace

use SWAYI\HHsurvey_sectionII_27May2008.dta, clear

keep if hhiid==0
keep if age>30 & age<=35
drop if camp==.

gen no_survive=cond(died==1 | no_return==1,1,0)

keep id2 camp parish village household block firstname1 surname1 firstname2 surname2 hhiia hhiib age abd abd_per abd_times no_return no_survive
order id2 camp parish village household block firstname1 surname1 firstname2 surname2 hhiia hhiib age abd abd_per abd_times no_return no_survive

sort id2 hhiia

set seed 1234
gen rand=uniform()

sort no_survive abd camp rand
bysort no_survive abd camp: egen rank=rank(rand)

gen sample=0 if no_survive==1
gen sample_prob=1 if no_surv==1

replace sample=1 if abd==1 & no_surv==0
replace sample_prob=1 if abd==1 & no_surv==0

bysort camp: egen n_abd=count(id2) if abd==1 & no_surv==0
bysort camp: egen n_nonabd=count(id2) if abd==0 & no_surv==0
table camp, c(count n_abd count n_nonabd)

replace sample=1 if abd==0 & no_surv==0 & rank<=4

replace sample_prob=4/n_nonabd if abd==0 & no_surv==0 & rank<=4

replace sample_prob=0 if sample==.
replace sample=0 if sample==.

ren id2 hh_id
ren hhiia roster_row
ren hhiib name
ren abd_per abd_length
ren firstname1 hh_head1_name
ren surname1 hh_head1_surname
ren firstname2 hh_head2_name
ren surname2 hh_head2_surname

gsort -sample no_surv rand
egen id=seq()
replace id=id+3000
order id hh_id roster_row name

drop n_abd n_nonabd rank rand

append using temp.dta

gen i_weight=1/sample_prob

gen stratum=1 if abd==1
replace stratum=2 if abd==0
replace stratum=3 if abd==0 & age>30 & age<35
replace stratum=4 if no_survive==1
label define strata 1 "Abducted" 2 "Non-Abd, 14 to 30" 3 "Non-Abd, 30 to 35" 4 "Did not survive"
label values stratum strata

label var id "Respondent ID"
label var hh_id "Household ID"
label var roster_row "HH roster row number"
label var name "Respondent name"
label var camp "Camp"
label var parish "Parish"
label var village "Village"
label var household "Household"
label var block "Block"
label var hh_head1_name "HH survey respondent 1, first name"
label var hh_head1_surname "HH survey respondent 1, second name"
label var hh_head2_name "HH survey respondent 2, first name"
label var hh_head2_surname "HH survey respondent 2, second name"
label var age "Estimated age"
label var abd "Abduction indicator"
label var abd_length "Estimated abduction length"
label var abd_times "Estimated times abducted"
label var no_return "Did not return from abduction"
label var no_survive "Died or did not return from abduction"
label var sample "In the sample"
label var sample_prob "Sampling probability"
label var i_weight "Individual sampling weight"
label var stratum "Sampling stratum"

keep if sample==1 | no_survive==1

save SWAYII\rawdata\sway_ii_sample.dta, replace
