* DATA CREATION AND CLEANING FILE FOR SWAY-I DATA

/* This file contains code to create the SWAYI dataset. It includes:
		 1) The creation code (SWAY_I_create_27Jan2009.do)
		 2) The data cleaning code for the individual survey (SWAY_I_cleaning_Isurvey.do)
		 3) The data cleaning code for the household survey (SWAY_I_cleaning_HHsurvey.do)
		*/ 



** USER SETTINGS **


cd "YOUR DIRECTORY\SWAY.Replication.June2013\Data\Raw data and cleaning\"

	clear
	set more off
	set mem 100m
	set matsize 800

** BUILD DATASET

// Bring in data
	
	insheet using "SWAYI/rawdata/I to VI.txt", clear names
	order surveycontrol_yqnno district camp parish village household person
	sort surveycontrol_yqnno
	save "SWAYI/tempdata/temp1.dta", replace 

	insheet using "SWAYI/rawdata/VII to VIII.txt", clear names
	sort surveycontrol_yqnno
	save "SWAYI/tempdata/temp2.dta", replace 

	insheet using "SWAYI/rawdata/IXi.txt", clear names
	sort surveycontrol_yqnno
	save "SWAYI/tempdata/temp3.dta", replace 

	insheet using "SWAYI/rawdata/IXii.txt", clear names
	sort surveycontrol_yqnno
	save "SWAYI/tempdata/temp4.dta", replace 

	insheet using "SWAYI/rawdata/X to XII.txt", clear names
	sort surveycontrol_yqnno
	save "SWAYI/tempdata/temp5.dta", replace

	insheet using "SWAYI/rawdata/IV Work Activities.txt", clear names
	ren qn id
	*!Change these below?
		ren iii6 iv6
		ren iii6other iv6other
		ren iii7 iv7
		ren iii8 iv8
		ren iii9 iv9
		ren iii10 iv10
	*drop blanks
		drop if iv6=="" & ((iv7==. | iv7==0) & (iv8==. | iv8==0)) 
	*classify "Other"
		replace iv6="S" if iv6==""

		*!Check and recode these

// Number jobs
	bysort id: gen a = cond(_N==1,1,_n)
	gen str1 b="_"
	egen job=concat(b a)
	drop a b
	reshape wide iv6 iv6other iv7 iv8 iv9 iv10, i(id) j(job) string
	sort id
	save "SWAYI/tempdata/temp6.dta", replace

	use "SWAYI/tempdata/temp1.dta"
	foreach x in 2 3 4 5 {
		merge surveycontrol_yqnno using SWAYI/tempdata/temp`x'.dta
		tab _merge
		drop _merge
		sort surveycontrol_yqnno
		}

// Generate unique ID
	drop hhinfo_yqnno education_yqnno health_yqnno economicactivities_yqnno food_drink_yqnno communityinvolv_yqnno psychosocialadj_yqnno abduction_return1_30_yqnno abduction_return31_61_yqnno abduction_return62_91_yqnno war_trauma_yqnno tracking_information_yqnno conclusion_yqnno
	ren surveycontrol_yqnno id
	la var id "Questionnaire ID"

	sort id
	merge id using "SWAYI/tempdata/temp6"
	tab _merge
	list id iv1 iv2 if _merge==1
	drop _merge

save "SWAYI/tempdata/sway_import.dta", replace

** DATA CLEANING 
****************

	********************************************************************
	** CLEANING INDIVIDUAL SURVEY (code from SWAY_I_cleaning_Isurvey.do)
	********************************************************************
	
	
	use "SWAYI/tempdata/sway_import.dta", clear

	***************
	** SECTION I **
	***************	
	
	drop session
	
	*i1: Generate Respondent ID
	
		*!Check for missing or unusual values	
			codebook district camp parish village household person 
			tab1 district camp parish village household person
	
				list district camp if district>2
				replace district=2 if district>2 & camp==6
	
				replace parish="1" if parish=="A"	
				gen par=real(parish)
				drop parish
				ren par parish
				order id district camp parish
	
			list district camp parish village household person if person>5
				replace person=2 if id==561
				replace person=1 if person==.
	
			*erroneous ids
				replace village=42 if id==7
				replace village=42 if id==8
				replace household=4 if id==33
				replace person=1 if id==33
				replace village=53 if id==56
				replace village=5 if id==73
				replace village=15 if id==83
				replace household=3 if id==95
				replace household=2 if id==115
				replace household=3 if id==120
				replace household=5 if id==143
				replace village=12 if id==152
				replace parish=1 if id==153
				replace village=22 if id==168
				replace village=41 if id==188
				replace parish=5 if id==203
				replace household=5 if id==215
				replace parish=6 if id==219
				replace parish=5 if id==278
				replace village=32 if id==278
				replace household=6 if id==278
				replace person=1 if id==278
				replace village=16 if id==234
				replace parish=3 if id==259
				replace household=1 if id==262
				replace household=3 if id==311
				replace parish=8 if id==320
				replace village=57 if id==320
				replace village=39 if id==332
				replace parish=26 if id==350
				replace household=1 if id==370
				replace household=10 if id==371
				replace village=50 if id==381
				replace camp=6 if id==383
				replace village=6 if id==406
				replace village=65 if id==408
				replace village=50 if id==412
				replace village=50 if id==412
				replace village=50 if id==413
				replace village=50 if id==423
				replace household=46 if id==426
				replace village=65 if id==430
				replace village=69 if id==443
				replace camp=8 if id==444
				replace parish=3 if id==444
				replace village=2 if id==444
				replace household=2 if id==453
				replace household=1 if id==457
				replace village=14 if id==459
				replace household=1 if id==459
				replace camp=3 if id==462
				replace household=1 if id==462
				replace village=14 if id==464
				replace village=18 if id==471
				replace person=2 if id==479
				replace household=3 if id==481
				replace village=32 if id==488
				replace parish=2 if id==507
				replace household=2 if id==509
				replace person=8 if id==509
				replace village=19 if id==514
				replace household=8 if id==518
				replace household=8 if id==533
				replace village=41 if id==552
				replace household=3 if id==552
				replace parish=5 if id==557
				replace village=3 if id==557
				replace village=29 if id==562
				replace household=3 if id==585
				replace village=47 if id==586
				replace household=3 if id==586
				replace household=1 if id==591
				replace household=1 if id==592
				replace village=53 if id==615
				replace person=1 if id==615
				replace block="6" if id==617
				replace village=53 if id==649
				replace village=22 if id==651
				replace village=59 if id==654
				replace parish=1 if id==654
				replace village=55 if id==658
				replace village=29 if id==662
				replace parish=14 if id==666
				replace village=47 if id==670
				replace parish=6 if id==690
				replace parish=5 if id==693
				replace parish=8 if id==694
				replace parish=23 if id==695
				replace block="21" if id==696
				replace household=4 if id==696
				replace parish=15 if id==700
				replace household=4 if id==712
				replace parish=8 if id==727
				replace village=94 if id==727
				replace household=2 if id==730
				replace household=4 if id==744
	
			label var parish "Parish"
			label var block "Block"
	
			label define districts 1 Kitgum 2 Pader
			label values district districts
			label define camps 1 Acholibur 2 Akwang 3 K.Matidi 4 Orom 5 Pader 6 Pajule 7 Palabek 8 Atanga
			label values camp camps
	
		egen i1_par=concat(camp parish), punct(".")
		label var i1_par "I1 Parish ID Number"
	
		egen i1_hh=concat(district camp parish village household), punct(".")
		label var i1_hh "I1 HH ID Number"
	
		egen i1=concat(district camp parish village household person), punct(".")
		label var i1 "I1 Respondent ID Number"
	
		duplicates tag i1, g(dup_id)
		gen missing_id=1 if district==. | camp==. | parish==. | village==. | household==. | person==.
		sort i1
		list id i1 district camp parish village household person if missing_id==1
		list id i1 district camp parish village household person if dup_id>=1
	
	
	* LOCATION DUMMIES
	
		gen C_ach=cond(camp==1,1,0)
		gen C_akw=cond(camp==2,1,0)
		gen C_kma=cond(camp==3,1,0)
		gen C_oro=cond(camp==4,1,0)
		gen C_pad=cond(camp==5,1,0)
		gen C_paj=cond(camp==6,1,0)
		gen C_pal=cond(camp==7,1,0)
		gen C_ata=cond(camp==8,1,0)
	
		label var C_ach "Acholibur Dummy"
		label var C_akw "Akwang Dummy"
		label var C_kma "K. Matidi Dummy"
		label var C_oro "Orom Dummy"
		label var C_pad "Pader Dummy"
		label var C_paj "Pajule Dummy"
		label var C_pal "Palabek Dummy"
		label var C_ata "Atanga Dummy"
	
		
	*i3 & i4: Permission
		codebook i3 i4
		label var i3 "I3 Permission to Interview"	
		label var i4 "I4 Refusal reason"	
	
	
	*i5: Date & Time
		
		*!Check for unusual values
			tab i5aday
			tab i5amonth
			tab i5ayear
		
		codebook i5ayear i5amonth
	
		list id i5aday i5amonth i5ayear if i5ayear!=5 & i5ayear!=6
		replace i5ayear=5 if i5ayear!=6 & id<500
	
		list id i5aday i5amonth i5ayear if i5amonth>3 & i5amonth<8
		replace i5amonth=10 if i5amonth==. & i5ayear==5
		replace i5amonth=1 if id==564
		replace i5amonth=2 if id==713
	
		replace i5ayear=i5ayear+2000
	
		*Fix times
			egen time=ends(i5b), tail punct(" ") trim
			egen hr=ends(time), head punct(":") trim
			egen mins=ends(time), tail punct(":") trim
			egen min=ends(mins), head punct(":") trim
			gen i5bhr=real(hr)
			gen i5bmin=real(min)
			drop i5b time mins hr min
			tab i5bhr
			tab i5bmin
	
			replace i5bhr=i5bhr+12 if i5bhr>=1 & i5bhr<=6
	
		label var i5aday "I5a Interview day"
		label var i5amonth "I5a Interview month"
		label var i5ayear "I5a Interview year"
		label var i5bhr "I5b Interview start hour"
		label var i5bmin "I5b Interview start minute"
	
			
	
	* i8: Suspicious Respondents
		tab i8
		list id if i8==2
		label var i8 "I8 Suspicious Respondent?"
	
	
	****************
	** SECTION II **
	****************	
		
	*ii1: Age
		codebook ii1
		gen CHECK_AGE=cond(ii1<14 | ii1>30,1,0)
		list id i1 ii1 if ii1<14
		list id i1 ii1 if ii1>30
		*!Correct missing or erroneous ages
			replace ii1=18 if id==5
			replace ii1=30 if id==63
			replace ii1=22 if id==71
			replace ii1=14 if id==412
	
		gen age=ii1
		label var ii1 "II1 Age"
		label var age "Respondent Age"
	
	* Age dummies
	
		gen adult=cond(age>=18,1,0)
		gen child=cond(age<18,1,0)
	
		gen age_14_15=cond(age>=14 & age<=15,1,0)
		gen age_16_17=cond(age>=16 & age<=17,1,0)
		gen age_18_19=cond(age>=18 & age<=19,1,0)
		gen age_20_21=cond(age>=20 & age<=21,1,0)
		gen age_22_23=cond(age>=22 & age<=23,1,0)
		gen age_24_25=cond(age>=24 & age<=25,1,0)
		gen age_26_27=cond(age>=26 & age<=27,1,0)
		gen age_28_29=cond(age>=28 & age<=29,1,0)
	
	*ii2-ii9: Parents alive?
		tab ii2
		tab ii6
	
			label var ii2 "II2 Father Alive?"
			label var ii6 "II6 Mother Alive?"
	
			list id ii2 ii3 ii4 ii5 if ii2==.
			list id ii6 ii7 ii8 ii9 if ii6==.
			replace ii2=1 if ii2==. & ii3==""
			replace ii6=1 if ii6==. & ii7==.
			replace ii6=2 if ii6==. & ii7!=.
	
		tab ii3
		tab ii7
	
			replace ii3="" if ii3=="DK" | ii3=="DR" | ii3=="9999" | ii3=="99"
			gen ii3_=real(ii3)
			drop ii3
			ren ii3_ ii3
	
			replace ii7=. if ii7==99 | ii7==9999
	
			label var ii3 "II3 Father yr of death"
			label var ii7 "II7 Mother yr of death"
			list id ii2 ii3 ii4 ii5  if ii3==. & ii2!=1
			list id ii6 ii7 ii8 ii9 if ii7==. & ii6!=1
		
	
		tab ii4
		tab ii8
	
			label var ii4 "II4 Father cause of death"
			label var ii8 "II8 Mother cause of death"
			label var ii4other "II4 Father cause of death - Other"
			label var ii8other "II8 Mother cause of death - Other"
	
		!* Check for odd values
		list id i1 ii2 ii3 ii4 if ii2==1 & ii3!=.
			replace ii2=2 if ii2==1 & (ii3!=. | ii4!=.)
		list id ii6 ii7 ii8 ii9  if ii6==1 & (ii7!=. | ii8!=.)
			replace ii8=. if ii6==1 & ii7==.
	
		gen no_fthr=0
		replace no_fthr=1 if ii2!=1
	
		gen no_fthr96=0
		gen no_fthr99=0
		gen no_fthr02=0
		replace no_fthr96=1 if ii2!=1 & (ii3<=1996 | ii3==.)
		replace no_fthr99=1 if ii2!=1 & (ii3<=1999 | ii3==.)
		replace no_fthr02=1 if ii2!=1 & (ii3<=2002 | ii3==.)
	
		gen no_mthr=0
		replace no_mthr=1 if ii6!=1
	
		gen no_mthr96=0
		gen no_mthr99=0
		gen no_mthr02=0
		replace no_mthr96=1 if ii6!=1 & (ii7<=1996 | ii7==.)
		replace no_mthr99=1 if ii6!=1 & (ii7<=1999 | ii7==.)
		replace no_mthr02=1 if ii6!=1 & (ii7<=2002 | ii7==.)
	
		gen orphan=0
		replace orphan=1 if no_fthr==1 & no_mthr==1
		gen orphan96=0
		replace orphan96=1 if no_mthr96==1 & no_fthr96==1
	
	
		label var no_fthr "=1 if father died"
		label var no_mthr "=1 if father died"
		label var orphan "=1 if father and mother died"
		label var no_fthr96 "=1 if father died before 1997"
		label var no_mthr96 "=1 if mother died before 1997"
		label var orphan96 "=1 if parents died before 1997"
		label var no_fthr99 "=1 if father died before 2000"
		label var no_mthr99 "=1 if mother died before 2000"
		label var no_fthr02 "=1 if father died before 2003"
		label var no_mthr02 "=1 if mother died before 2003"
		
	
	*ii5,ii9: Parent's education
	
		tab ii5
		tab ii9
	 
		replace ii5=15 if ii5==5
		replace ii5=13 if ii5==4
		replace ii5=10 if ii5==3
		replace ii5=7 if ii5==2
		replace ii5=4 if ii5==1
		replace ii5=. if ii5==99 
	
		replace ii9=15 if ii9==5
		replace ii9=13 if ii9==4
		replace ii9=10 if ii9==3
		replace ii9=7 if ii9==2
		replace ii9=4 if ii9==1
		replace ii9=. if ii9==99 
	
		*Predict missing values:
			reg ii5 ii1 ii9 C_*
			predict e, xb
			reg ii5 ii1 C_*
			predict e2, xb
			gen fthr_ed=ii5
			replace fthr_ed=e if fthr_ed==.
			replace fthr_ed=e2 if fthr_ed==.
			replace fthr_ed=0 if fthr_ed<3
			replace fthr_ed=4 if fthr_ed>=3 & fthr_ed<6
			replace fthr_ed=7 if fthr_ed>=6 & fthr_ed<=8
			replace fthr_ed=10 if fthr_ed>8 & fthr_ed<=11
			replace fthr_ed=12 if fthr_ed>11 & fthr_ed<=13
			replace fthr_ed=15 if fthr_ed>13 & fthr_ed!=.
			codebook fthr_ed
			drop e e2
	
			reg ii9 ii1 fthr_ed C_*
			predict e, xb
			gen mthr_ed=ii9
			replace mthr_ed=e if mthr_ed==.
			replace mthr_ed=0 if mthr_ed<3
			replace mthr_ed=4 if mthr_ed>=3 & mthr_ed<6
			replace mthr_ed=7 if mthr_ed>=6 & mthr_ed<=8
			replace mthr_ed=10 if mthr_ed>8 & mthr_ed<=11
			replace mthr_ed=12 if mthr_ed>11 & mthr_ed<=13
			replace mthr_ed=15 if mthr_ed>13 & mthr_ed!=.
			codebook mthr_ed
			drop e
	
	
		*Indicator for parent's education	
			gen fthr_ed0=cond(fthr_ed==0,1,0)
			gen fthr_ed4=cond(fthr_ed==4,1,0)
			gen fthr_ed7=cond(fthr_ed==7,1,0)
			gen fthr_edhi=cond(fthr_ed>7,1,0)
	
			gen mthr_ed0=cond(mthr_ed==0,1,0)
			gen mthr_ed4=cond(mthr_ed==4,1,0)
			gen mthr_ed7=cond(mthr_ed==7,1,0)
			gen mthr_edhi=cond(mthr_ed>7,1,0)
	
			label var ii5 "II5 Father education"
			label var ii9 "II9 Mother education"
	
			label var fthr_ed "Father's education"
			label var mthr_ed "Mother's education"
	
			label var fthr_ed0 "=1 if father has no schooling"
			label var fthr_ed4 "=1 if father has some primary school"
			label var fthr_ed7 "=1 if father completed P7"
			label var fthr_edhi "=1 if father attended secondary school"
	
			label var mthr_ed0 "=1 if mother has no schooling"
			label var mthr_ed4 "=1 if mother has some primary school"
			label var mthr_ed7 "=1 if mother completed P7"
			label var mthr_edhi "=1 if mother attended secondary school"
	
	*ii10: Adults you live with
		
		tab ii10a	
		tab ii10b	
		tab ii10c	
		tab ii10d	
		tab ii10other	
	
		label var ii10a "II10 Adults living with (a)"
		label var ii10b "II10 Adults living with (b)"
		label var ii10c "II10 Adults living with (c)"
		label var ii10d "II10 Adults living with (d)"
		label var ii10other "II10 Adults living with (other)"
	
	
	*ii11-ii13: Family harmony
	
		tab ii11
		tab ii12
		tab ii13
	
		label var ii11 "II11 Comfortable with family?"
		label var ii12 "II12 Quarrels with family?"
		label var ii13 "II13 Caring family?"
	
	
	*ii14-ii17
		
		tab ii14
		tab ii14other
		tab ii15
		tab ii16
		tab ii17
	
			list id ii14 ii14other ii15 ii16 ii17 if ii14==. | ii15==. | ii16==.
				replace ii14=1 if ii14other=="1"
				replace ii14=1 if ii17!=.
	
			list id ii14 ii14other if ii14==6 | ii14other!=""
				replace ii14=2 if id==46
				replace ii14other="" if ii14!=6
	
		gen ii17_dens=ii16/ii17
		tab ii17_dens
	
		list id ii14 ii15 ii16 ii17 ii17_dens  if ii17_dens<.5 | ii17_dens>5 & ii17_dens!=.
	
		label var ii14 "II14 Dwelling type"
		label var ii14other "II14 Dwelling type (other)"
		label var ii15 "II15 Own dwelling?"
		label var ii16 "II16 # HH members"
		label var ii17 "II17 # huts"
		label var ii17_dens "II17 # people per hut (II16/II17)"
	
	
	*ii18: Assets
	
		codebook ii18*, 
		foreach x in a b c d e f g h i j k l {
			list id ii18* if ii18`x'==.
			replace ii18`x'=0 if ii18`x'==. | ii18`x'==88 | ii18`x'==99 | ii18`x'==77		
			}
	
		label var ii18a "ii18a Assets (Jerry cans)"
		label var ii18b "ii18b Assets (Basins)"
		label var ii18c "ii18c Assets (Bicycles)"
		label var ii18d "ii18d Assets (Mattresses)"
		label var ii18e "ii18e Assets (Radios)"
		label var ii18f "ii18f Assets (Plates)"
		label var ii18g "ii18g Assets (Cattle)"
		label var ii18h "ii18h Assets (Pigs/Goats)"
		label var ii18i "ii18i Assets (Birds)"
		label var ii18j "ii18j Assets (Chairs)"
		label var ii18k "ii18k Assets (Mobiles)"
		label var ii18l "ii18l Assets (Ploughs)"
	
		pca ii18a ii18b ii18c ii18d ii18e ii18f ii18g ii18h ii18i ii18j ii18k ii18l, means
		predict score assets
		codebook assets
		replace assets=10 if assets>20
		egen aindex=std(assets)
		codebook aindex assets
	
		label var aindex "Index of asset ownership"
	
	
	*ii19-ii21: Condoms
	
		codebook ii19 ii20 ii21
		
		label var ii19 "ii19 Condoms - Know where to find?"
		label var ii20 "ii20 Condoms - Protect?"
		label var ii21 "ii21 Condoms - Own?"
	
	*ii22-ii30 Marriage and condoms
	
		codebook ii22-ii30other
		
		*!Check for erroneously missing marriage variables and correct
			list id ii23 ii24 ii27  if ii23==.
			replace ii23=1 if ii23==. & ii27!=.
			replace ii23=2 if ii23==. & ii24!=.
	
		*! Check for other inconsistencies or mistakes
			list id ii23 ii24 ii25 ii27 ii29  if ii23==1 & ii24!=.
				replace ii24=. if id==42 | id==232
			list id ii23 ii24 ii25 ii27 ii29  if ii23==1 & (ii27==. | ii29==.)
			list id ii23 ii24 ii25 ii27 ii29 if ii23==2 & ii27!=.
				replace ii23=1 if ii24==. & ii27!=.
				replace ii27=. if ii23==2 & ii27!=.
				replace ii29=. if ii23==2 & ii29!=.
			list id ii23 ii24 ii25 ii27 ii29  if ii23==2 & ii24==.
	
			list id ii24 ii25 ii26  if ii24==2 & ii25!=.
			list id ii24 ii25 ii26 if (ii25==1 | ii25==3) & ii26!=.
			list id ii24 ii25 ii26 if (ii26!=. & ii25==.) | (ii25==2 & ii26==.)
	
			list id ii27 ii28 ii29 ii30 if ii27==2 & ii28==.
			list id ii27 ii28 ii29 ii30 if ii29==2 & ii30==.
			list id ii27 ii28 ii29 ii30 if (ii27==1 | ii27==3) & ii28!=.
				replace ii28=. if ii27==1 | ii27==3
			list id ii27 ii28 ii29 ii30 if (ii29==1 | ii27==9) & ii30!=.
				replace ii30=. if ii29==1 | ii29==3
	
		label var ii22 "II22 # children?"
		label var ii23 "II23 Ever married?"
		label var ii24 "II24 Sex partners?"
		label var ii25 "II25 Condom with sex partners?"
		label var ii26 "II26 Why no condom?"
		label var ii26other "II26 Why no condom? (other)"
		label var ii27 "II27 # children?"
		label var ii28 "II28 Condom with wife?"
		label var ii28other "II28 Why no condom? (other)"
		label var ii29 "II29 Condom with others?"
		label var ii30 "II30 Why no condom?"
		label var ii30other "II30 Why no condom? (other)"
	
	
	
	*****************
	** SECTION III **
	*****************
	
	*iii1-iii3: Literacy
	
		codebook iii1-iii3
	
		list id iii1 iii2 iii3  if iii1==. | iii3==.
		list id iii1 iii2 iii3 i5aday i5amonth if iii1==3 & iii2==.
		list id iii1 iii2 iii3 i5aday i5amonth if iii1<3 & iii2!=.
			replace iii2=. if iii1!=3	
	
		*Generate literacy measure
			gen literate=cond(iii1==1 & iii3<=2,1,0)
			gen illiterate=cond(iii1==3,1,0)
			
		label var iii1 "III1 Read well?"
		label var iii2 "III1 Read well?"
		label var iii3 "III1 Read well?"
		label var literate "Equals 1 if Reads without difficulty and writes with some or no difficulty"
		label var illiterate "Equals 1 if cannot read or write"
	
	
	*iii4-iii10: Schooling
	
		codebook iii4
	
		list id iii4 iii6 iii7 iii8 iii9  if iii4==.
			replace iii4=1 if iii4==. & iii6!=.
			replace iii4=2 if iii4==. & iii6==.
			
		gen inschool=0
		replace inschool=1 if iii4==1
		
		codebook iii5*
	
		codebook iii6-iii9
		list id iii4 iii6 iii7 iii8 iii9  if iii4==1 & iii6==.
	
	
		* Educational attainment
		codebook iii10
			tab iii10
			list id iii4 iii10  if iii10==.
				gen NO_SCHOOL_DATA=1 if iii10==.
	
			gen p7=0
			replace p7=1 if iii10>=7 & iii10!=.
			gen p4=0
			replace p4=1 if iii10>=4
			gen p1=0
			replace p1=1 if iii10>=1
	
	*iii11-iii15: Higher education
	
		codebook iii11 iii12a iii12b
		list id iii12a  if iii12a!="1" & iii12a!="2" & iii12a!="3" & iii12a!=""
			replace iii12a="2" if iii12a!="1" & iii12a!="2" & iii12a!="3" & iii12a!=""
			gen iii12a_=real(iii12a)
			drop iii12a
			ren iii12a_ iii12a
	
		list id iii12b  if iii12b!="1" & iii12b!="2" & iii12b!="3" & iii12b!=""
			replace iii12b="2" if iii12b!="1" & iii12b!="2" & iii12b!="3" & iii12b!=""
			gen iii12b_=real(iii12b)
			drop iii12b
			ren iii12b_ iii12b
	
			list id iii4 iii10 iii11 iii12a iii13a  if iii11==.
				replace iii11=1 if iii11==. & iii12a!=.
				replace iii11=2 if iii11==. & iii12a==.
	
			list id iii11 iii12a iii13a  if iii11==1 & iii12a==.
				*! Fix?
			list id iii11 iii12a iii13a iii14ia  if iii11==2 & iii12a!=.
				replace iii12a=. if iii11==2 & iii13a==""
		
		tab iii13a 
			list id iii11 iii12a iii13a if iii12a!=. & iii13a==""
		tab iii13b 
			list id iii11 iii12b iii13b  if iii12b!=. & iii13b==""
	
		codebook iii14ia iii14ib iii14iia iii14iib
			replace iii14ia=. if iii14ia==0 & iii12a==. 
			replace iii14ib=. if iii14ib==0 & iii12b==. 
	
			list id iii11 iii12a iii13a iii14ia iii14iia  if iii12a==. & iii14ia!=. & iii14ia!=0
			list id iii11 iii12a iii13a iii14ia iii14iia if iii14ia==0
			list id iii11 iii12b iii13b iii14ib iii14iib if iii14ib!=. & iii14iib==""
			list id iii11 iii12b iii13b iii14ib iii14iib if iii14ib!=.
			list id iii11 iii12b iii13b iii14ib iii14iib if iii12b!=. & iii14iib==""
		
			gen iii14a=.
			replace iii14a=iii14ia if iii14iia=="D"
			replace iii14a=iii14ia*7 if iii14iia=="W"
			replace iii14a=iii14ia*30 if iii14iia=="M"
			replace iii14a=iii14ia*365 if iii14iia=="Y"
			replace iii14a=iii14a/365
			tab iii14a
			
			gen iii14b=.
			replace iii14b=iii14ib if iii14iib=="D"
			replace iii14b=iii14ib*7 if iii14iib=="W"
			replace iii14b=iii14ib*30 if iii14iib=="M"
			replace iii14b=iii14ib*365 if iii14iib=="Y"
			replace iii14b=iii14b/365
			tab iii14b
	
			drop iii14ia iii14iia iii14ib iii14iib
			
		codebook iii15a iii15b
		
		label var iii11 "III11 Higher education"
		label var iii12a "III12a Higher Ed: Inst A"
		label var iii12b "III12b Higher Ed: Inst B"
		label var iii13a "III13a Higher Ed: Discipline A"
		label var iii13b "III13b Higher Ed: Discipline B"
		label var iii14a "III14a Higher Ed: Length (Years) A"
		label var iii14b "III14b Higher Ed: Length (Years) B"
		label var iii15a "III15a Higher Ed: Paid for by A"
		label var iii15b "III15b Higher Ed: Paid for by B"
	
	
		gen university=cond(iii12a==1 | iii12b==1,1,0)
		gen voc_time=iii14a+iii14b
		gen voc=cond(iii11==1 & university!=1 & voc_time>.16,1,0)
		gen voc2=cond(iii11==1 & university!=1 & voc_time>6,1,0)
		drop voc_time
	
		gen ngo_paid=cond(iii15a==4 | iii15b==4,1,0)
	
		* Attainment
		codebook iii10
		list iii10 iii11 if iii10==. 
		codebook iii10 if iii11==1
		codebook iii10 if iii11==2
		gen educ=iii10
		replace educ=educ+iii14a if iii14a!=.	
		replace educ=educ+iii14b if iii14b!=.	
		replace educ=round(educ)
		replace educ=8 if iii10==. & iii11==1
		replace educ=6 if iii10==. & iii11==2
		replace educ=16 if educ>16 & educ!=.
	
		gen leduc=ln(educ)
	
		gen secskool=cond(educ>7,1,0)
	
		reg educ age_* C_*, robust
		predict educhat, resid
	
		gen educ_cat=.
		replace educ_cat=1 if educ>=0 & educ<=3 
		replace educ_cat=2 if educ>=4 & educ<=7 
		replace educ_cat=3 if educ>=8 & educ<=11 
		replace educ_cat=4 if educ>=12 & educ<=16
		label define education 1 "0-3" 2 "4-7" 3 "8-11" 4 "12-16"
		label values educ_cat education
	
	
		label var educ "Years of education attained"
		label var educhat "Age and Location-Adj education attained"
		label var leduc "Log of Years of education attained"
	
		label var iii4 "III4 Currently in school?"
		label var inschool "=1 if currently in school"
	
		label var iii5a "III5a Reason not in school: abdcuted"
		label var iii5b "III5b Reason not in school: closed"
		label var iii5c "III5c Reason not in school: displaced"
		label var iii5d "III5d Reason not in school: fees"
		label var iii5e "III5e Reason not in school: too many kids"
		label var iii5f "III5f Reason not in school: difficulty"
		label var iii5g "III5g Reason not in school: work"
		label var iii5h "III5h Reason not in school: marriage"
		label var iii5i "III5i Reason not in school: repeat"
		label var iii5j "III5j Reason not in school: care for family"
		label var iii5k "III5k Reason not in school: other"
		label var iii5kother "III5k Reason not in school: other (explanation)"
	
		label var iii6 "III6 If in school: Teachers like you?"
		label var iii7 "III7 If in school: Quarrel with students?"
		label var iii8 "III8 If in school: Part of your school?"
		label var iii9 "III9 If in school: Like brothers and sisters?"
		
		label var iii10 "III10 Highest primary/secondary schooling"
	
		label var p1 "=1 if completed P1"
		label var p4 "=1 if completed P4"
		label var p7 "=1 if completed P7"
	
	* Drop-outs
	
		list id iii5kother if iii5k==1
		replace iii5a="1" if iii5a=="A"
		ren iii5a iii5a_
		gen iii5a=real(iii5a_)
		drop iii5a_
		replace iii5d=1 if iii5e==1 | id==6 | id==7 | id==24 | id==68 | id==160 | id==204 | id==471 | id==618 
		replace iii5a=1 if id==144 | id==372  
		replace iii5c=1 if id==728  
		replace iii5f=1 if id==202 | id==730
		replace iii5j=1 if id==355  
		replace iii5g=1 if id==544 | id==584 | id==647 
		gen iii5oth1=1 if id==33 | id==46 | id==195 | id==216 | id==240 | id==274 | id==342 | id==351 | id==389 | id==423 | id==543 | id==636 | id==663 | id==690 | id==699 | id==744 | id==746
		gen iii5oth2=1 if id==3 | id==14 | id==16 | id==50 | id==127 | id==138 | id==314 | id==342 | id==146 | id==150 | id==182 | id==213 |  id==214 |  id==223 | id==367 | id==372 | id==374 | id==428 | id==478 | id==458 | id==487 | id==459 | id==497 | id==460 | id==516 | id==648 | id==653 | id==658 | id==678 | id==697 | id==705 | id==751
		replace iii5d=1 if id==3 | id==14 | id==16 | id==50 | id==127 | id==138 | id==314 | id==342 | id==146 | id==150 | id==182 | id==213 |  id==214 |  id==223 | id==367 | id==372 | id==374 | id==428 | id==478 | id==458 | id==487 | id==459 | id==497 | id==460 | id==516 | id==648 | id==653 | id==658 | id==678 | id==697 | id==705
		replace iii5k=. if id==744 | id==746 | id==751 | id==314 | id==6 | id==7 | id==24 | id==68 | id==160 | id==204 | id==471 | id==618 | id==144 | id==372 | id==202 | id==355 | id==544 | id==584 | id==647 | id==33 | id==46 | id==195 | id==216 | id==240 | id==274 | id==342 | id==351 | id==389 | id==423 | id==543 | id==636 | id==663 | id==690 | id==699 | id==3 | id==14 | id==16 | id==50 | id==127 | id==138 | id==342 | id==146 | id==150 | id==182 | id==213 |  id==214 |  id==223 | id==367 | id==372 | id==374 | id==428 | id==478 | id==458 | id==487 | id==459 | id==497 | id==460 | id==516 | id==648 | id==653 | id==658 | id==678  | id==697 | id==705
		replace iii5kother="" if id==744 | id==746 | id==751 | id==314 | id==6 | id==7 | id==24 | id==68 | id==160 | id==204 | id==471 | id==618 | id==144 | id==372 | id==202 | id==355 | id==544 | id==584 | id==647 | id==33 | id==46 | id==195 | id==216 | id==240 | id==274 | id==342 | id==351 | id==389 | id==423 | id==543 | id==636 | id==663 | id==690 | id==699 | id==3 | id==14 | id==16 | id==50 | id==127 | id==138 | id==342 | id==146 | id==150 | id==182 | id==213 |  id==214 |  id==223 | id==367 | id==372 | id==374 | id==428 | id==478 | id==458 | id==487 | id==459 | id==497 | id==460 | id==516 | id==648 | id==653 | id==658 | id==678  | id==697 | id==705
		replace iii5k=1 if iii5kother!="" | iii5f==1 | iii5g==1 | iii5i==1
		
		label var iii5a "Abduction"
		label var iii5b "School closed"
		label var iii5c "Displaced"
		label var iii5d "No fees"
		label var iii5h "Got married"
		label var iii5j "Caring for family"
		label var iii5k "Other"
		label var iii5oth1 "Insecurity"
		label var iii5oth2 "Parents died"
		
	
	*****************
	** SECTION IV **
	*****************
	
	*iv1-iv2: Earnings
	
		codebook iv1 iv2
		list id iv1 iv2 iv6_1 iv7_1 iv8_1 iv6_2 iv7_2 iv8_2  if iv1==. | iv2==.
			!* Check these
			replace iv1=0 if iv1==.
			replace iv2=0 if iv2==.
			replace iv1=iv1
			replace iv2=iv2
	
		list id iv1 iv2  if iv2>(iv1+.01)
			*! check into these
				replace iv1=iv1+iv2 if iv2>(iv1+.01)
	
		label var iv1 "IV1 Earnings: Last 4 weeks (USH)"
		label var iv2 "IV1 Earnings: Last 7 days (USH)"
	
	
	*iv3-iv5: Land access
	*!TO DO
	
		codebook  iv3-iv5
		list id iv3 iv4a iv4b iv5 if iv3==.
			replace iv4a=. if iv4a==0
			replace iv3=1 if iv4a!=. & iv3==.
			replace iv3=2 if iv4a==. & iv3==.
		
		label var iv3 "IV3 Access to land?"
		
		label var iv5 "IV5 If land: Own, pay rent, or borrow?"
		
	
	*iv6-iv10
	
		foreach x of numlist 1/6 {
			sort iv6_`x'
			egen iv6num_`x'=group(iv6_`x')
			}
		
		list id iv1 iv2  if iv6num_1==.
		list id iv1 iv2 if iv6num_1==. & iv1!=0 & iv1!=.
	
		foreach x of numlist 1/6 {
			codebook iv7_`x' iv8_`x'
			list id iv6_`x' iv7_`x' iv8_`x' iv9_`x' iv10_`x'  if iv7_`x'==. & iv6num_`x'!=.
			list id iv6_`x' iv7_`x' iv8_`x' iv9_`x' iv10_`x' if iv8_`x'==. & iv6num_`x'!=.
				replace iv8_`x'=0 if  iv8_`x'==. & iv7_`x'!=.
			list id iv6_`x' iv7_`x' iv8_`x' iv9_`x' iv10_`x' if iv8_`x'>7 & iv8_`x'!=.
				replace iv8_`x'=7 if iv8_`x'>7 & iv8_`x'!=. 
			list id iv6_`x' iv7_`x' iv8_`x' iv9_`x' iv10_`x' if iv8_`x'>iv7_`x' & iv8_`x'!=.
				*Check how best to correct this
				replace iv7_`x'=iv7_`x'+iv8_`x' if iv8_`x'>iv7_`x' & iv8_`x'!=. 
				replace iv7_`x'=iv8_`x' if iv8_`x'!=. & iv7_`x'==.
			}
	
		foreach x of numlist 1/6 {
			replace iv9_`x'=. if iv8_`x'==0
			replace iv10_`x'=. if iv8_`x'==0
			codebook iv9_`x' iv10_`x'
			list id iv6_`x' iv7_`x' iv8_`x' iv9_`x' iv10_`x'  if (iv9_`x'==. | iv9_`x'==0) & iv8_`x'!=. & iv8_`x'!=0
				replace iv9_`x'=6 if (iv9_`x'==. | iv9_`x'==0) & iv8_`x'!=. & iv8_`x'!=0
			list id iv6_`x' iv7_`x' iv8_`x' iv9_`x' iv10_`x' if iv9_`x'!=. & iv8_`x'==.
			list id iv6_`x' iv7_`x' iv8_`x' iv9_`x' iv10_`x' if iv10_`x'==. & iv8_`x'!=. & iv8_`x'!=0
			}
	
		foreach x of numlist 1/6 {
			label var iv6_`x' "IV6 Job `x': Activity"
			label var iv6num_`x' "IV6 Job `x': Activity (number)"
			label var iv6other_`x' "IV6 Job `x': Activity (other)"
			label var iv7_`x' "IV7 Job `x': Days worked in past 4 weeks"
			label var iv8_`x' "IV8 Job `x': Days worked in past 7 days"
			label var iv9_`x' "IV9 Job `x': Hours per day in last 7 days"
			label var iv10_`x' "IV10 Job `x': Paid?"
			}
	
	
	*iv11-iv15: Military service
		
		codebook iv11-iv15
	
		list id iv11-iv15  if iv11==.
			replace iv11=3 if iv11==.
		list id iv11-iv15  if iv11==3 & (iv12!="" | iv13!=.)
			replace iv12="" if iv11==3
			replace iv13=. if iv11==3
	
		label var iv11 "IV11 Armed forces?"
		label var iv12 "IV12 Highest rank"
		label var iv13 "IV13 Join voluntarily?"
		label var iv14 "IV14 Why join?"
		label var iv15 "IV15 How involuntarily?"
	
	
	***************
	** SECTION V **
	***************
	
	*v1-v2: General health
	
		codebook v1 v2
		gen bad_hlth=0
		replace bad_hlth=1 if v1==3
	
		label var v1 "V1 Self-described health"
		label var bad_hlth "=1 if self-described health is 'not good'"
		label var v2 "V2 Days work missed"
	
	*v3-v12
		
		codebook v3a-v10b
	
		list id v3a v3b v4a v4b v5a v5b v6a v6b v7a v7b v8a v8b v9a v9b v10a v10b  if (v3a==1 & v3b!=.) | (v4a==1 & v4b!=.) | (v5a==1 & v5b!=.) | (v6a==1 & v6b!=.) | (v7a==1 & v7b!=.) | (v8a==1 & v8b!=.) | (v9a==1 & v9b!=.) | (v10a==1 & v10b!=.)
		list id v3a v3b v4a v4b v5a v5b v6a v6b v7a v7b v8a v8b v9a v9b v10a v10b if ((v3a==1 & v3b!=.) | (v4a==1 & v4b!=.) | (v5a==1 & v5b!=.) | (v6a==1 & v6b!=.) | (v7a==1 & v7b!=.) | (v8a==1 & v8b!=.) | (v9a==1 & v9b!=.) | (v10a==1 & v10b!=.)) 
	
		list id v3a v3b v4a v4b v5a v5b v6a v6b v7a v7b v8a v8b v9a v9b v10a v10b if (v3a==2 & v3b==.) | (v4a==2 & v4b==.) | (v5a==2 & v5b==.) | (v6a==2 & v6b==.) | (v7a==2 & v7b==.) | (v8a==2 & v8b==.) | (v9a==2 & v9b==.) | (v10a==2 & v10b==.)
		list id v3a v3b v4a v4b v5a v5b v6a v6b v7a v7b v8a v8b v9a v9b v10a v10b if ((v3a==2 & v3b==.) | (v4a==2 & v4b==.) | (v5a==2 & v5b==.) | (v6a==2 & v6b==.) | (v7a==2 & v7b==.) | (v8a==2 & v8b==.) | (v9a==2 & v9b==.) | (v10a==2 & v10b==.)) 
	
		codebook v11 v12
		tab v11
		list id v11 v12  if v11=="" & v12!=.
	
		label var v3a "V3a Daily task: Walk easily?"
		label var v4a "V4a Daily task: Carry 20L?"
		label var v5a "V5a Daily task: Digging?"
		label var v6a "V6a Daily task: Run?"
		label var v7a "V7a Daily task: carry 10L?"
		label var v8a "V8a Daily task: Work half day?"
		label var v9a "V9a Daily task: Stand up?"
		label var v10a "V10a Daily task: Bow/squat/kneel?"
	
		label var v3b "V3b Daily task (degree): Walk easily?"
		label var v4b "V4b Daily task (degree): Carry 20L?"
		label var v5b "V5b Daily task (degree): Digging?"
		label var v6b "V6b Daily task (degree): Run?"
		label var v7b "V7b Daily task (degree): carry 10L?"
		label var v8b "V8b Daily task (degree): Work half day?"
		label var v9b "V9b Daily task (degree): Stand up?"
		label var v10b "V10b Daily task (degree): Bow/squat/kneel?"
	
		label var v11 "V11 Injury description"
		label var v12 "V12 Injury from LRA?"
	
	*v13-v15
	
		codebook v13-v15
		tab v14
		list id v13 v14 v15 if v13==1 & (v14=="" | v15==.)
		list id v13 v14 v15 if v13!=1 & (v14!="" | v15!=.)
	
		gen head_inj=0
		replace head_inj=1 if v13==1 & (v15==1 | v15==.)
	
		label var v13 "V13 Head injury?"
		label var head_inj "=1 if unconscious from head injury?"
		label var v14 "V14 Describe head injury"
		label var v15 "V15 Go unconscious?"
	
	
	
	****************
	** SECTION VI **
	****************	
	
	*vi1-vi2: Meals
	
		codebook vi1 vi2
		list id vi1 vi2  if vi1>4 | vi2>7
	
	*vi3-vi4: Smoking
	
		codebook vi3 vi4 vi4unit
		list id vi3 vi4 vi4unit if vi3==.
			replace vi3=2 if vi3==.
		list id vi3 vi4 vi4unit if vi4!=. & vi3==2
			replace vi4=. if vi3==2 & (vi4==0 | vi4==88)
			*! Fix the others?
		list id vi3 vi4  if vi4==. & vi3==1
		list id vi3 vi4 vi4unit  if vi4unit=="" & vi4!=.
			replace vi4unit="D" if vi4unit=="" & vi4!=.
		list id vi3 vi4 vi4unit if vi4unit!=""
			replace vi4=4 if id==63
			replace vi4unit="D" if id==63
	
		replace vi4=vi4/7 if vi4unit=="W"
		drop vi4unit
	
		label var vi3 "VI3 Smoke?"
		label var vi4 "VI4 Cigarettes per day"
	
	
	*vi5-vi10: Alcohol
	
		codebook vi5-vi9
	
		tab vi5
			replace vi5=. if vi5==88 
			list id vi5 vi5unit  if vi5unit!=""		
				replace vi5unit="" if vi5unit=="88"
				replace vi5unit="Y" if vi5unit=="1year ago"
				replace vi5=7 if vi5unit=="7/D"
				replace vi5unit="D" if vi5unit=="7/D"
			list vi5 vi6 vi7 vi8 vi9  if vi5==0
				replace vi5=. if vi5==0
				*! check these
	
			replace vi5=vi5*7 if vi5unit=="W"
			replace vi5=vi5*30 if vi5unit=="M"
			replace vi5=vi5*365 if vi5unit=="Y"
			drop vi5unit
			tab vi5
	
		codebook vi6 vi7 vi8 vi9
			list id vi5 vi6  if vi5>30 & vi6!=.
				replace vi6=. if vi5>30
			list id vi5 vi6 vi7 vi8  if vi5<=30 & vi6==.
	
			gen drinks=0
			replace drinks=1 if vi5<=7
	
			gen drinks_hvy=0
			replace drinks_hvy=1 if vi5<=7 & vi7==2 & vi6>1
	
			label var vi5 "VI5 Days since last took alcohol"
			label var vi6 "VI6 Drinks per week"
			label var vi7 "VI7 Get drunk?"
			label var vi8 "VI8 Take in morning?"
			label var vi9 "VI9 Alcohol type"
			label var drinks "=1 if took alcohol in past week"
			label var drinks_hvy "=1 if heavy drinker"
	
		codebook vi10*
			foreach x in a b c d e f g h {
				replace vi10`x'=1 if vi10`x'==2
				}
	
			label var vi10a "VI10 Reason for drinking: Forget"
			label var vi10b "VI10 Reason for drinking: Fun"
			label var vi10c "VI10 Reason for drinking: Hunger"
			label var vi10d "VI10 Reason for drinking: Nutrition"
			label var vi10e "VI10 Reason for drinking: Friends"
			label var vi10f "VI10 Reason for drinking: Boredom"
			label var vi10g "VI10 Reason for drinking: Stress"
			label var vi10h "VI10 Reason for drinking: Other"
	
	*vi11-vi15: Drugs
		
			replace vi11=2 if vi11==.

	
	*****************
	** SECTION VII **
	*****************	
	
	*vi1: Groups
	
		codebook vii1a-vii1jdescribe
			foreach x in a b c d e f g h i j {
				replace vii1`x'=0 if vii1`x'==.
				replace vii1`x'=1 if vii1`x'==2
				}
			tab vii1jdescribe
	
		replace vii1a=1 if id==225 | id==510
		replace vii1c=1 if id==8 | id==9 | id==310 | id==364 | id==436 | id==616 | id==139 | id==612
		replace vii1e=1 if id==36 | id==51 | id==91 | id==95 | id==123 | id==470 | id==588 | id==650 | id==707
		replace vii1f=1 if id==26 | id==188 | id==197 | id==219 | id==257 | id==310 | id==365 | id==583 | id==688 | id==156 | id==243 | id==57 | id==167 | id==287 | id==528 | id==713 | id==722
		replace vii1g=1 if id==753 | id==240 | id==73 | id==262 | id==373 | id==376 | id==486 | id==525 | id==542 | id==552 | id==553 | id==568 | id==576 | id==634 | id==672 | id==674 | id==698 | id==706 | id==711
		replace vii1h=1 if id==341 | id==441 | id==684 | id==573 | id==731
		replace vii1j=1 if vii1jd!=""
		replace vii1j=0 if id==753 | id==74 | id==215 | id==584 | id==697 | id==225 | id==510 | id==8 | id==9 | id==310 | id==364 | id==436 | id==616 | id==139 | id==612 | id==36 | id==51 | id==91 | id==95 | id==123 | id==470 | id==588 | id==650 | id==707 | id==26 | id==188 | id==197 | id==219 | id==257 | id==310 | id==365 | id==583 | id==688 | id==156 | id==243 | id==57 | id==167 | id==287 | id==528 | id==713 | id==240 | id==73 | id==262 | id==373 | id==376 | id==486 | id==525 | id==542 | id==552 | id==553 | id==568 | id==576 | id==634 | id==672 | id==674 | id==698 | id==706 | id==711 | id==341 | id==441 | id==684 | id==573 | id==731 | id==722
	
		label var vii1a "VII1A: Group member: Drama/music/dance"
		label var vii1b "VII1B: Group member: Peace"
		label var vii1c "VII1C: Group member: Farmer/Professional"
		label var vii1d "VII1D: Group member: Water"
		label var vii1e "VII1E: Group member: Religious"
		label var vii1f "VII1F: Group member: School"
		label var vii1g "VII1G: Group member: Sports"
		label var vii1h "VII1H: Group member: NGO"
		label var vii1i "VII1I: Group member: Mobilizer"
		label var vii1j "VII1J: Group member: Other"
		label var vii1jdescribe "VII1J: Group member: Other (Description)"
	
	
	*vii2-vii6: Closest groups
	
		codebook vii2
		label var vii2 "VII2 Closest group"
	
		codebook vii3-vii6
		
	*vii7: Religion
		
		tab vii7
		tab vii7other
		
		gen catholic=0
		replace catholic=1 if vii7==1
		gen saved=0
		replace saved=1 if vii7==2 | vii7==5
	
		codebook vii8
			gen church=0
			replace church=1 if vii8==1
	
		label var vii7 "VI7 Religion"
		label var vii8 "VI8 Attend church?"
		label var catholic "=1 if Catholic"
		label var saved "=1 if Saved"
		label var church "=1 if attends church"
	
	
	*vii9-17: Attitudes towards violence
	
		ren vii13a viii13a
		ren vii14a viii14a
		ren vii15a viii15a
		codebook vii9-vii16
	
		list id vii9 vii10  if (vii9==2 | vii9==.) & vii10!=.
			replace vii9=1 if vii9==. & vii10!=.
			replace vii10=. if vii9==2
		list id vii11 vii12 if (vii11==2 | vii11==.) & vii12!=.
			replace vii11=1 if vii11==. & vii12!=.
			replace vii12=. if vii11==2
		list id vii13 vii14 if (vii13==2 | vii13==.) & vii14!=.
			replace vii13=1 if vii13==. & vii14!=.
			replace vii14=. if vii13==2
		list id vii15 vii16 if (vii15==2 | vii15==.) & vii16!=.
			replace vii15=1 if vii15==. & vii16!=.
			replace vii16=. if vii15==2
	
		codebook vii17
		gen fight=0
		replace fight=1 if vii17>0 & vii17!=.
	
		label var vii9 "VII9 Ok for neighbor to beat insulter?"
		label var vii10 "VII10 Ok for you to beat insulter?"
		label var vii11 "VII11 Ok for neighbor to beat thief?"
		label var vii12 "VII12 Ok for you to beat thief?"
		label var vii13 "VII13 Ok for neighbor to beat wife for burning?"
		label var vii14 "VII14 Ok for you to beat wife for burning?"
		label var vii15 "VII15 Ok for neighbor to beat wife for talking back?"
		label var vii16 "VII16 Ok for you to beat wife for talking back?"
	
		label var vii17 "VII17 Number of fights in last 6 months"
		label var fight "=1 if been in fight in last six months"
	
	
	*vii18-vii19: Voting
		
		codebook vii18 vii19
		list id ii1 vii18  if vii18==3 & ii1>17
		list id ii1 vii19 if vii19==3 & ii1>14
		
		gen vote05=0
		replace vote05=1 if vii18==1
		replace vote05=. if vii18==3 | ii1<17
	
		gen vote01=0
		replace vote01=1 if vii19==1
		replace vote01=. if vii19==3 | ii1<13
	
		label var vii18 "VII18 Voted in 2005 election?"
		label var vii19 "VII19 Voted in 2001 election?"
		label var vote05 "=1 if voted in 2005"
		label var vote01 "=1 if voted in 2001"
	
	*vii20-vii22: Attitudes to justice
	
		codebook vii20a-vii20mother
		codebook vii21a-vii21l
		codebook vii22
	
		gen punish=0
		replace punish=1 if vii22==1
		replace punish=. if vii22==.
	
		label var vii20a "VII20 Kony: Forgive"
		label var vii20b "VII20 Kony: Reconcile"
		label var vii20c "VII20 Kony: Reintegrate"
		label var vii20d "VII20 Kony: Confont"
		label var vii20e "VII20 Kony: Confess"
		label var vii20f "VII20 Kony: Compensate"
		label var vii20g "VII20 Kony: Kill"
		label var vii20h "VII20 Kony: Punish"
		label var vii20i "VII20 Kony: Trial/Kill"
		label var vii20j "VII20 Kony: Trial/Prison"
		label var vii20k "VII20 Kony: DK"
		label var vii20l "VII20 Kony: Other"
		label var vii20lother "VII20 Kony: Other (Describe)"
		label var vii20m "VII20 Kony: Other"
		label var vii20mother "VII20 Kony: Other (Describe)"
	
		label var vii21a "VII21 LRA Commanders: Forgive"
		label var vii21b "VII21 LRA Commanders: Reconcile"
		label var vii21c "VII21 LRA Commanders: Reintegrate"
		label var vii21d "VII21 LRA Commanders: Confont"
		label var vii21e "VII21 LRA Commanders: Confess"
		label var vii21f "VII21 LRA Commanders: Compensate"
		label var vii21g "VII21 LRA Commanders: Kill"
		label var vii21h "VII21 LRA Commanders: Punish"
		label var vii21i "VII21 LRA Commanders: Trial/Kill"
		label var vii21j "VII21 LRA Commanders: Trial/Prison"
		label var vii21k "VII21 LRA Commanders: DK"
		label var vii21l "VII21 LRA Commanders: Other"
	
		label var vii22 "VI22 Trial/Punish or Reintegrate?"
		label var punish "=1 if supports trials and punishments for Commanders"
	
	
	******************
	** SECTION VIII **
	******************
	
		foreach n of numlist 2/44 {
			codebook viii`n'*
			list id viii`n'a viii`n'b  if viii`n'a==2 & viii`n'b!=. 
				quietly replace viii`n'b=. if viii`n'a==2
			list id viii`n'a viii`n'b if viii`n'a==1 & viii`n'b==.
				*!Impute for missing data?
			label var viii`n'a "VIII`n' Psychosocial Q`n': Ever?"
			label var viii`n'b "VIII`n' Psychosocial Q`n': Frequency"
			}
	
		codebook viii45*
		label var viii451 "VII45 To feel better: Go somewhere"
		label var viii452 "VII45 To feel better: Relax"
		label var viii453 "VII45 To feel better: Humor"
		label var viii454 "VII45 To feel better: Work"
		label var viii455 "VII45 To feel better: Wait"
		label var viii456 "VII45 To feel better: Friends"
		label var viii457 "VII45 To feel better: Solve"
		label var viii458 "VII45 To feel better: Talk"
		label var viii459 "VII45 To feel better: Avoid"
		label var viii4510 "VII45 To feel better: Pray"
		label var viii4511 "VII45 To feel better: Quiet"
		label var viii4512 "VII45 To feel better: Other"
		label var viii4512other "VII45 To feel better: Other (describe)"
	
	
	****************
	** SECTION IX **
	****************	
	
	*ix1: abducted?
	
		codebook ix1 ix2ayear ix2amonth
		tab ix1, m
		
		list id ix1 ix2amonth ix2ayear ix3a ix2bmonth ix2byear ix3b  if ix1==. & (ix2ayear!=. | ix2amonth!="")
		list id ix1 ix2amonth ix2ayear ix3a ix2bmonth ix2byear ix3b if ix1==0 & (ix2ayear!=. | ix2amonth!="")
			*!double check these
			replace ix1=1 if ix1==. & ix2ayear!=.
			replace ix1=1 if ix1==0 & ix2ayear!=.
		list id ix1 ix2ayear ix2byear ix2cyear  if (ix1==. | ix1==1) & (ix2byear!=.)
			replace ix1=2 if (ix1==. | ix1==1) & ix2byear!=.
		list id ix1 ix2ayear ix2byear ix2cyear if (ix1==. | ix1==1 | ix1==2) & (ix2cyear!=.)
			replace ix1=3 if (ix1==. | ix1==1 | ix1==2) & ix2cyear!=.
	
		list id ix1 ix2amonth ix2ayear ix3a ix4a ix5a  if (ix1==0 | ix1==.) & (ix2amonth!="" | ix2ayear!=. | ix3a!=. | ix5a!=. | ix6a!=. )
	
		replace ix1=0 if ix1==.
		tab ix1, m
	
		list id ix1 ix2amonth ix2ayear ix3a ix4a ix5a if ix1>0 & (ix2amonth=="" | ix2ayear==. | ix3a==. | ix5a==. | ix6a==.)
	
		replace ix1=1 if id==735
	
		replace ix1=0 if id==315
		replace ix2bmonth="" if id==315
		replace ix2byear=. if id==315
	
		gen abd=0
		replace abd=1 if ix1>0 & ix1!=.
	
		gen abd2=0
		replace abd2=1 if ix1>1 & ix1!=.
	
		gen abd3=0
		replace abd3=1 if ix1>2 & ix1!=.
	
		label var ix1 "IX1 Times abducted"
		label var abd "=1 if ever abducted"
	
	
	*ix2a, ix14a, ix15a: Abduction date, age, length (abduction A)
		
		tab ix2amonth 
		tab ix2ayear
	
		list id ix2amonth if ix2amonth=="4D"
		replace ix2amonth="" if ix2amonth=="4D"
		ren ix2amonth x
		gen ix2amonth=real(x)
		drop x
	
		*Look for failure to fill abduction date data
		* M or Y Abducted missing
		list id ix1 ix2amonth ix2ayear ix3a ix14amonth ix14ayear ix15a ix15aunit  if ix1>=1 & (ix2amonth==. | ix2ayear==.)
		* Too little data to calculate abduction length
		list id ix1 ix2amonth ix2ayear ix3a ix14amonth ix14ayear ix15a ix15aunit if ix1>=1 & ((ix2ayear==. & ix14ayear==.) | ((ix15a==. | ix15aunit=="") & (ix2ayear==. | ix14ayear==.)))
		 	
		* DK month or year
		list id ix1 ix2amonth ix2ayear  if ix2amonth==99
		list id ix1 ix2amonth ix2ayear if ix2amonth==. & ix2ayear!=.
		list id ix1 ix2amonth ix2ayear if ix2ayear==. & ix2amonth!=.
			replace ix2amonth=. if ix1==0
		* Missing abduction end/length
		list id ix1 ix2amonth ix2ayear ix3a ix14amonth ix14ayear ix15a ix15aunit if ix1>=1 & (ix14amonth==. | ix14ayear==. | ix15a==. | ix15aunit=="")
	
		* Replace DKs & make 4-digit years
		replace ix2amonth=. if ix2amonth==99
		replace ix2ayear=. if ix2amonth==99 & ix2ayear==99
		replace ix2ayear=ix2ayear+1900 if ix2ayear>6 & ix2ayear<100
		replace ix2ayear=ix2ayear+2000 if ix2ayear<=6
		replace ix14amonth=. if ix14amonth==99
		replace ix14ayear=. if ix14amonth==99 & ix14ayear==99
		replace ix14ayear=ix14ayear+1900 if ix14ayear>6 & ix14ayear<100
		replace ix14ayear=ix14ayear+2000 if ix14ayear<=6
	
		*Weird years
		list id ix1 age ix2amonth ix2ayear ix3a ix14amonth ix14ayear ix15a ix15aunit  if ix1>=1 & (ix2ayear<1989 | ix14ayear<1989)
	
		tab ix2ayear
		tab ix14ayear
		tab ix2amonth
		tab ix14amonth
	
		*Fix and standardize length of abduction data
		tab ix15a
		replace ix15a=. if ix1==0
	
		tab ix15aunit
		replace ix15a=ix15a/30 if ix15aunit=="D"
		replace ix15a=ix15a/4 if ix15aunit=="W"
		replace ix15a=ix15a*12 if ix15aunit=="Y"
		drop ix15aunit
	
		*Generate abduction length
		replace ix2ayear=2002 if id==735
		gen abd_lgth_a=((ix14ayear-ix2ayear)*12)+(ix14amonth-ix2amonth) if ix1>=1
			list id age ix2amonth ix2ayear ix3a ix14amonth ix14ayear ix15a abd_lgth_a if ix1>0 & abd_lgth_a==.
			replace abd_lgth_a=ix15a if abd_lgth_a==. & ix1>0	
			replace abd_lgth_a=36 if id==66
			replace abd_lgth_a=9 if id==442
			replace abd_lgth_a=6 if id==242
	
		replace abd_lgth_a=2.5 if id==621
	
	
		* Look for weird values
		list id ix1 age ix2amonth ix2ayear ix3a ix14amonth ix14ayear ix15a abd_lgth_a if abd_lgth_a<0
		list id ix1 age ix2amonth ix2ayear ix3a ix14amonth ix14ayear ix15a abd_lgth_a if abd_lgth_a<=3
			replace abd_lgth_a=ix15a if abd_lgth_a<=3 & ix15a!=.
		gen x=abd_lgth_a-ix15a
		tab x
		list id age ix2amonth ix2ayear ix3a ix14amonth ix14ayear ix15a abd_lgth_a x if x!=0 & x!=.
		drop x
	
		*Generate abduction Age
		gen abd_age_a=age-((i5ayear-ix2ayear)+((i5amonth-ix2amonth)/12)) if ix1>=1
		replace abd_age_a=age-(i5ayear-ix2ayear) if ix1>=1 & abd_age_a==.
	
		replace abd_age_a=14 if id==270
	
	
	*ix2b, ix14b, ix15b: Abduction date, age, length (abduction B)
		
		tab ix2bmonth 
		tab ix2byear
	
		list id ix2bmonth if ix2bmonth=="1W"
		replace ix2bmonth="" if ix2bmonth=="1W"
		ren ix2bmonth x
		gen ix2bmonth=real(x)
		drop x
	
		list id ix14byear if ix14byear=="M"
		replace ix14byear="" if ix14byear=="M"
		ren ix14byear x
		gen ix14byear=real(x)
		drop x
	
		*Look for failure to fill abduction date data
		* M or Y Abducted missing
		list id ix1 ix2bmonth ix2byear ix3b ix14bmonth ix14byear ix15b ix15bunit  if ix1>=2 & (ix2bmonth==. | ix2byear==.)
			replace ix1=1 if ix1==2 & ix2byear==. & ix2bmonth==.
		* Too little data to calculate abduction length
		list id ix1 ix2bmonth ix2byear ix3b ix14bmonth ix14byear ix15b ix15bunit  if ix1>=2 & ((ix2byear==. & ix14byear==.) | ((ix15b==. | ix15bunit=="") & (ix2byear==. | ix14byear==.)))
		
		* DK month or year
		list id ix1 ix2bmonth ix2byear  if ix2bmonth==99
		list id ix1 ix2bmonth ix2byear  if ix2bmonth==. & ix2byear!=.
		list id ix1 ix2bmonth ix2byear  if ix2byear==. & ix2bmonth!=.
			replace ix2bmonth=. if ix1==0
		* Missing abduction end/length
		list id ix1 ix2bmonth ix2byear ix3b ix14amonth ix14byear ix15b ix15bunit  if ix1>=2 & (ix14bmonth==. | ix14byear==. | ix15b==. | ix15bunit=="")
	
		* Replace DKs & make 4-digit years
		replace ix2bmonth=. if ix2bmonth==99
		replace ix2byear=. if ix2bmonth==99 & ix2byear==99
		replace ix2byear=ix2byear+1900 if ix2byear>6 & ix2byear<100
		replace ix2byear=ix2byear+2000 if ix2byear<=6
		replace ix14bmonth=. if ix14bmonth==99
		replace ix14byear=. if ix14bmonth==99 & ix14byear==99
		replace ix14byear=ix14byear+1900 if ix14byear>6 & ix14byear<100
		replace ix14byear=ix14byear+2000 if ix14byear<=6
	
		*Weird years
		list id ix1 age ix2bmonth ix2byear ix3a ix14bmonth ix14byear ix15b ix15bunit  if ix1>=2 & (ix2byear<1989 | ix14byear<1989)
	
		tab ix2byear
		tab ix14byear
		tab ix2bmonth
		tab ix14bmonth
	
		*Fix and standardize length of abduction data
		tab ix15b
		replace ix15b=. if ix1<2
	
		tab ix15bunit
		replace ix15b=ix15b/30 if ix15bunit=="D"
		replace ix15b=ix15b/4 if ix15bunit=="W"
		replace ix15b=ix15b*12 if ix15bunit=="Y"
		drop ix15bunit
	
		*Generate abduction length
		gen abd_lgth_b=((ix14byear-ix2byear)*12)+(ix14bmonth-ix2bmonth) if ix1>=2
			list id age ix2bmonth ix2byear ix3a ix14bmonth ix14byear ix15b abd_lgth_b if ix1>=2 & abd_lgth_b==.
			replace abd_lgth_b=ix15b if abd_lgth_b==. & ix1>=2	
	
		* Look for weird values
		list id ix1 age ix2bmonth ix2byear ix3a ix14bmonth ix14byear ix15b abd_lgth_b if abd_lgth_b<0
		list id ix1 age ix2bmonth ix2byear ix3a ix14bmonth ix14byear ix15b abd_lgth_b if abd_lgth_b<=3
			replace abd_lgth_b=ix15b if abd_lgth_b<=3 & ix15b!=.
		gen x=abd_lgth_b-ix15b
		tab x
		list id age ix2bmonth ix2byear ix3a ix14bmonth ix14byear ix15b abd_lgth_b x if x!=0 & x!=.
		drop x
	
		*Generate abduction Age
		gen abd_age_b=age-((i5ayear-ix2byear)+((i5amonth-ix2bmonth)/12)) if ix1>=2
		replace abd_age_b=age-(i5ayear-ix2byear) if ix1>=2 & abd_age_b==.
	
	
	*ix2c, ix14c, ix15c: Abduction date, age, length (abduction C)
		
		tab ix2cmonth 
		tab ix2cyear
	
		list id ix14cyear if ix14cyear =="D"
		replace ix14cyear ="" if ix14cyear =="D"
		ren ix14cyear x
		gen ix14cyear =real(x)
		drop x
	
		*Look for failure to fill abduction date data
		* M or Y Abducted missing
		list id ix1 ix2cmonth ix2cyear ix3c ix14cmonth ix14cyear ix15c ix15cunit  if ix1>=3 & (ix2cmonth==. | ix2cyear==.)
		* Too little data to calculate abduction length
		list id ix1 ix2cmonth ix2cyear ix3c ix14cmonth ix14cyear ix15c ix15cunit  if ix1>=3 & ((ix2cyear==. & ix14cyear==.) | ((ix15c==. | ix15cunit=="") & (ix2cyear==. | ix14cyear==.)))
		 	
		* DK month or year
		list id ix1 ix2cmonth ix2cyear  if ix2cmonth==99
		list id ix1 ix2cmonth ix2cyear  if ix2cmonth==. & ix2cyear!=.
		list id ix1 ix2cmonth ix2cyear  if ix2cyear==. & ix2cmonth!=.
		* Missing abduction end/length
		list id ix1 ix2cmonth ix2cyear ix3b ix14amonth ix14cyear ix15c ix15cunit  if ix1>=3 & (ix14cmonth==. | ix14cyear==. | ix15c==. | ix15cunit=="")
	
		* Replace DKs & make 4-digit years
		replace ix2cmonth=. if ix2cmonth==99
		replace ix2cyear=. if ix2cmonth==99 & ix2cyear==99
		replace ix2cyear=ix2cyear+1900 if ix2cyear>6 & ix2cyear<100
		replace ix2cyear=ix2cyear+2000 if ix2cyear<=6
		replace ix14cmonth=. if ix14cmonth==99
		replace ix14cyear=. if ix14cmonth==99 & ix14cyear==99
		replace ix14cyear=ix14cyear+1900 if ix14cyear>6 & ix14cyear<100
		replace ix14cyear=ix14cyear+2000 if ix14cyear<=6
	
		*Weird years
		list id ix1 age ix2cmonth ix2cyear ix3a ix14cmonth ix14cyear ix15c ix15cunit  if ix1>=3 & (ix2cyear<1989 | ix14cyear<1989)
	
		tab ix2cyear
		tab ix14cyear
		tab ix2cmonth
		tab ix14cmonth
	
		*Fix and standardize length of abduction data
		tab ix15c
		replace ix15c=. if ix1<3
	
		tab ix15cunit
		replace ix15c=ix15c/30 if ix15cunit=="D"
		replace ix15c=ix15c/4 if ix15cunit=="W"
		replace ix15c=ix15c*12 if ix15cunit=="Y"
		drop ix15cunit
	
		*Generate abduction length
		gen abd_lgth_c=((ix14cyear-ix2cyear)*12)+(ix14cmonth-ix2cmonth) if ix1>=3
			list id age ix2cmonth ix2cyear ix3a ix14cmonth ix14cyear ix15c abd_lgth_c if ix1>=3 & abd_lgth_c==.
			replace abd_lgth_c=ix15c if abd_lgth_c==. & ix1>=3	
	
		* Look for weird values
		list id ix1 age ix2cmonth ix2cyear ix3a ix14cmonth ix14cyear ix15c abd_lgth_c if abd_lgth_c<0
		list id ix1 age ix2cmonth ix2cyear ix3a ix14cmonth ix14cyear ix15c abd_lgth_c if abd_lgth_c<=3
			replace abd_lgth_c=ix15c if abd_lgth_c<=3 & ix15c!=.
		gen x=abd_lgth_c-ix15c
		tab x
		list id age ix2cmonth ix2cyear ix3a ix14cmonth ix14cyear ix15c abd_lgth_c x if x!=0 & x!=.
		drop x
	
		*Generate abduction Age
		gen abd_age_c=age-((i5ayear-ix2cyear)+((i5amonth-ix2cmonth)/12)) if ix1>=3
		replace abd_age_c=age-(i5ayear-ix2cyear) if ix1>=3 & abd_age_c==.
	
		list id ix1 age abd_age_a abd_lgth_a abd_age_b abd_lgth_b abd_age_c abd_lgth_c  if ix1>0 & abd_age_a<=8
		gen tooyoung=1 if abd_age_a<=8 & abd_lgth_a<=.25
		label var tooyoung "=1 if age was less than 14"
		list id ix1 abd age abd_age_a abd_lgth_a abd_age_b abd_lgth_b abd_age_c abd_lgth_c if ix1>0 & tooyoung==1
	
	
		*Other measures
	
		egen abd_lgth=rsum(abd_lgth_a abd_lgth_b abd_lgth_c)
			replace abd_lgth=. if abd_lgth_a==. & abd_lgth_b==. & abd_lgth_c==. & abd==1
	
	
		label define period 0 "Never" 1 "<1W" 2 "1W-1M" 3 "1-3M" 4 "3-12M" 5 "1Y+"
	
		gen abd_per=.
		replace abd_per=0 if abd==0
		replace abd_per=1 if abd==1 & abd_lgth<.25
		replace abd_per=2 if abd_lgth>=.25 & abd_lgth<1
		replace abd_per=3 if abd_lgth>=1 & abd_lgth<3
		replace abd_per=4 if abd_lgth>=3 & abd_lgth<12
		replace abd_per=5 if abd_lgth>=12 & abd_lgth!=.
		label values abd_per period
		
		gen abd_per_a=.
		replace abd_per_a=0 if abd==0
		replace abd_per_a=1 if abd==1 & abd_lgth_a<.25
		replace abd_per_a=2 if abd_lgth_a>=.25 & abd_lgth_a<1
		replace abd_per_a=3 if abd_lgth_a>=1 & abd_lgth_a<3
		replace abd_per_a=4 if abd_lgth_a>=3 & abd_lgth_a<12
		replace abd_per_a=5 if abd_lgth_a>=12 & abd_lgth_a!=.
		label values abd_per_a period
	
	
		label var ix2amonth "IX2a 1st Abduction: Month abducted"
		label var ix2ayear "IX2a 1st Abduction: Year abducted"
		label var ix14bmonth "IX14a 1st Abduction: Month abducted"
		label var ix14ayear "IX14a 1st Abduction: Year abducted"
		label var ix15a "IX15a Length of 1st abduction (months)"
		label var abd_age_a "Age abducted (1st abduction)"
		label var abd_lgth_a "Months abducted (1st abduction)"
	
		label var ix2bmonth "IX2b 2nd Abduction: Month abducted"
		label var ix2byear "IX2b 2nd Abduction: Year abducted"
		label var ix14bmonth "IX14b 2nd Abduction: Month abducted"
		label var ix14byear "IX14b 2nd Abduction: Year abducted"
		label var ix15b "IX15b Length of 2nd abduction (months)"
		label var abd_age_b "Age abducted (2nd abduction)"
		label var abd_lgth_b "Months abducted (2nd abduction)"
	
		label var ix2cmonth "IX2c 3rd Abduction: Month abducted"
		label var ix2cyear "IX2c 3rd Abduction: Year abducted"
		label var ix14cmonth "IX14c 3rd Abduction: Month abducted"
		label var ix14cyear "IX14c 3rd Abduction: Year abducted"
		label var ix15c "IX15c Length of 3rd abduction (months)"
		label var abd_age_c "Age abducted (3rd abduction)"
		label var abd_lgth_c "Months abducted (3rd abduction)"
	
		label var abd_lgth "Months abducted (total)"
		label var abd_per "Period of abduction--categories"
		label var abd_per_a "Period of 1st abduction--categories"
	
	
	*ix3-ix9:Abduction events
	
		codebook ix3a-ix3c
		sort ix1
		list id ix1 ix3a ix3b ix3c  if (ix1==1 & ix3a==.) | (ix1==2 & ix3b==.) | (ix1==3 & ix3c==.)
	
		codebook ix4a-ix4c
	
		codebook ix5a-ix5c
	
		codebook ix6a-ix6c
	
		codebook ix7a-ix7c
		tab ix7a
		tab ix7b
		tab ix7c
	
		codebook ix8a-ix8c
	
		codebook ix9a1-ix9cother
	
		foreach x in a b c {
			replace ix3`x'=2 if ix3`x'==. & abd==1
			replace ix4`x'=. if ix3`x'==2 | abd==0
			replace ix8`x'=. if (ix7`x'==0 | ix7`x'==.) & ix8`x'==0
			}
	
		foreach x in a b c {
			label var ix3`x' "IX3`x' Abducted Willingly?"
			label var ix4`x' "IX3`x' If willingly: Why?"
			label var ix5`x' "IX3`x' Abduction place"
			label var ix6`x' "IX3`x' Abduction location"
			label var ix7`x' "IX3`x' How many other abducted?"
			label var ix8`x' "IX3`x' How many released?"
			label var ix9`x'1 "IX3`x' Who/why released? (Reason 1)"
			label var ix9`x'2 "IX3`x' Who/why released? (Reason 2)"
			label var ix9`x'3 "IX3`x' Who/why released? (Reason 3)"
			label var ix9`x'other "IX3`x' Who/why released? (Other-description)"
			}
	
	*ix10-ix13: Initiation
	
		codebook ix10a-ix10c
		list id ix1 ix10a abd_lgth_a  if ix10a==. & abd==1
	
		codebook ix11a-ix11cunit
	
		tab ix11aunit
		replace ix11a=1/30 if ix11aunit=="1/D"
		replace ix11a=1 if ix11aunit=="1/M"
		replace ix11a=7/30 if ix11aunit=="1/W"
		replace ix11a=7/30 if ix11aunit=="1/w"
		replace ix11a=1 if ix11aunit=="1M"
		replace ix11a=4 if ix11aunit=="4M"
		
		tab ix11bunit
		replace ix11b=1/30 if ix11bunit=="1/D"
	
		tab ix11cunit
		replace ix11c=1/30 if ix11cunit=="1/D"
	
		foreach x in a b c {
			replace ix11`x'=. if ix10`x'==2 | (ix10`x'==. & ix11`x'unit=="")
			list id ix10`x' ix11`x' ix11`x'unit  if ix10`x'==1 & (ix11`x'==. | ix11`x'==0)
			list id ix10`x' ix11`x' ix11`x'unit  if ix11`x'!=. & ix11`x'unit==""
			replace ix11`x'=. if ix11`x'==0 & ix11`x'unit==""
			replace ix11`x'=ix11`x'/30 if ix11`x'unit=="D"
			replace ix11`x'=ix11`x'/4 if ix11`x'unit=="W"
			replace ix11`x'=ix11`x'*12 if ix11`x'unit=="Y"
			}
	
		drop ix11aunit ix11bunit ix11cunit 
	
		codebook ix12*
		tab ix12a7other
	
		codebook ix13*
	
		foreach x in a b c {
			label var ix10`x' "IX10`x' Initiation ceremony?"
			label var ix11`x' "IX11`x' Months with LRA before initiation"
			label var ix12`x'1 "IX12`x' Initiation: Prayers"
			label var ix12`x'2 "IX12`x' Initiation: Oil"
			label var ix12`x'3 "IX12`x' Initiation: Give Beating"
			label var ix12`x'4 "IX12`x' Initiation: Kill"
			label var ix12`x'5 "IX12`x' Initiation: Drink blood"
			label var ix12`x'6 "IX12`x' Initiation: Receive beating"
			label var ix12`x'7 "IX12`x' Initiation: Other"
			label var ix12`x'7other "IX12`x' Initiation: Other (describe)"
			label var ix13`x'0 "IX13`x' Initiation: None"
			label var ix13`x'1 "IX13`x' Initiation: Prayers"
			label var ix13`x'2 "IX13`x' Initiation: Oil"
			label var ix13`x'3 "IX13`x' Initiation: Give Beating"
			label var ix13`x'4 "IX13`x' Initiation: Kill"
			label var ix13`x'5 "IX13`x' Initiation: Drink blood"
			label var ix13`x'6 "IX13`x' Initiation: Receive beating"
			label var ix13`x'7 "IX13`x' Initiation: Other"
			label var ix13`x'7other "IX13`x' Initiation: Other (describe)"
			}
	
	*ix16-ix21: Escaped?
	
		codebook ix16*
	
		foreach x in a b c {
			list id ix1 ix16`x' ix17`x' ix18`x' ix19`x' ix20`x' ix21`x'  if ix16`x'==. & abd2==1
				replace ix16`x'=3 if ix16`x'==. & ix19`x'!=.
			}
	
		codebook ix17*
		tab ix17aother 
		tab ix17bother 
		tab ix17cother 
	
		codebook ix18*
	
		foreach x in a b c {
			list id ix1 ix16`x' ix17`x' ix18`x' ix19`x' ix20`x' ix21`x'  if ix17`x'==. & ix16`x'==1
			list id ix1 ix16`x' ix17`x' ix18`x' ix19`x' ix20`x' ix21`x'  if ix18`x'==. & ix16`x'==2
			}
	
		codebook ix19*
		codebook ix20*
		codebook ix21*
	
		foreach x in a b c {
			list id ix1 ix16`x' ix17`x' ix18`x' ix19`x' ix20`x' ix21`x'  if ix19`x'==. & ix16`x'==3
			list id ix1 ix16`x' ix17`x' ix18`x' ix19`x' ix20`x' ix21`x'  if ix20`x'==. & ix16`x'==3
			list id ix1 ix16`x' ix17`x' ix18`x' ix19`x' ix20`x' ix21`x'  if ix21`x'==. & ix16`x'==3
			}
	
		foreach x in a b c {
			label var ix16`x' "IX16`x' Released, Rescued, or Escaped?"
			label var ix17`x' "IX17`x' Why Released?"
			label var ix17`x'other "IX17`x' Why Released? (other)"
			label var ix18`x' "IX18`x' Rescue circumstances"
			label var ix19`x' "IX19`x' How escaped?"
			label var ix20`x' "IX20`x' Know location when escaped?"
			label var ix21`x' "IX21`x' Why escape then?"
			}
		
	
	*ix22-ix25: UPDF
	
		codebook ix22*
		
		list id ix1 ix22a ix23a ix24a ix24aunit ix26a  if ix1>=1 & ix22a==.
			replace ix22a=1 if ix1>=1 & ix22a==. & ix23a!=.
			replace ix22a=2 if ix1>=1 & ix22a==. & ix24a==0
			replace ix22a=2 if ix1>=1 & ix22a==. & ix26a==0
	
		list id ix1 ix22b ix23b ix24b ix24bunit ix26b  if ix1>=2 & ix22b==.
			replace ix22b=1 if ix1>=2 & ix22b==. & ix23b!=.
			replace ix22b=2 if ix1>=2 & ix22b==. & ix24b==0
			replace ix22b=2 if ix1>=2 & ix22b==. & ix26b==0
	
		list id ix1 ix22c ix23c ix24c ix24cunit ix26c  if ix1>=3 & ix22c==.
			replace ix22c=1 if ix1>=3 & ix22c==. & ix23c!=.
			replace ix22c=2 if ix1>=3 & ix22c==. & ix24c==0
			replace ix22c=2 if ix1>=3 & ix22c==. & ix26c==0
	
		codebook ix23*
		foreach x in a b c {
			list id ix1 ix22`x' ix23`x' ix24`x' ix24`x'unit ix26`x'  if ix22`x'==2 & ix23`x'!=.
				replace ix23`x'=. if ix22`x'==2
			list id ix1 ix22`x' ix23`x' ix24`x' ix24`x'unit ix26`x'  if ix22`x'==1 & ix23`x'==.
				replace ix23`x'=2 if ix23`x'==. & ix22`x'==1
			}
	
		codebook ix24*
	
		*! Return to this one after corrected
		foreach x in a b c {
			replace ix24`x'=. if ix22`x'==2
			}
	
		codebook ix25*
	
		gen updf=.
		replace updf=0 if ix22a==2 | ix22b==2 | ix22c==2
		replace updf=1 if ix22a==1 | ix22b==1 | ix22c==1
		label var updf "=1 if reported ever passing through UPDF"
	
		foreach x in a b c {
			label var ix22`x' "IX22`x' Pass through UPDF?"
			label var ix23`x' "IX23`x' Harmed by UPDF?"
			label var ix24`x' "IX24`x' # Days with UPDF?"
			label var ix25`x' "IX25`x' Why with UPDF >3 days?"
			label var ix25`x'other "IX25`x' Why with UPDF >3 days? (other)"
			}
	
	*ix26-ix29: Return
	
		codebook ix26*
	
		list id ix26a ix27a* ix28a ix29a abd_lgth_a  if ix26a==. & abd==1
			replace ix26a=7 if ix26a==. & (ix27a1==1 | ix27a2==1 | ix27a3==1 | ix27a4==1 | ix27a5==1 | ix27a6==1 | ix27a99==1)
			replace ix26a=0 if ix26a==. & abd==1 & ix29a!=.
	
		list ix1 ix26b ix27b* ix28b ix29b abd_lgth_b  if ix26b==. & ix1>=2
			replace ix26b=7 if ix26b==. & (ix27b1==1 | ix27b2==1 | ix27b3==1 | ix27b4==1 | ix27b5==1 | ix27b6==1 | ix27b99==1)
			replace ix26b=0 if ix26b==. & ix1>=2 & ix29b!=.
		
		list ix1 ix26c ix27c* ix28c ix29c abd_lgth_c  if ix26c==. & ix1>=2
			replace ix26c=7 if ix26c==. & (ix27c1==1 | ix27c2==1 | ix27c3==1 | ix27c4==1 | ix27c5==1 | ix27c6==1 | ix27c99==1)
			replace ix26c=0 if ix26c==. & ix1>=2 & ix29c!=.
	
		codebook ix28*
		list id ix1 ix26a ix28a ix29a abd_lgth_a  if ix28a==. & ix1>=1
			replace ix28a=2 if ix28a==. & ix29a!=. & ix1>=1
		list id ix1 ix26b ix28b ix29b abd_lgth_b  if ix28b==. & ix1>=2
			replace ix28b=2 if ix28b==. & ix29b!=. & ix1>=2
		list id ix1 ix26c ix28c ix29c abd_lgth_c  if ix28c==. & ix1>=3
			replace ix28c=2 if ix28c==. & ix29c!=. & ix1>=3
	
		codebook ix29*
		list id ix1 ix26a ix28a ix29a abd_lgth_a  if ix29a==. & ix1>=1
		list id ix1 ix26b ix28b ix29b abd_lgth_b  if ix29b==. & ix1>=2
		list id ix1 ix26c ix28c ix29c abd_lgth_c  if ix29c==. & ix1>=3
			*! can we fix these?
	
		foreach x in a b c {
			label var ix26`x' "IX26`x' Reception Center"
			label var ix26`x'other "IX26`x' Reception Center (other)"
			label var ix27`x'1 "IX27`x' Services: Medical"
			label var ix27`x'2 "IX27`x' Services: Counseling"
			label var ix27`x'3 "IX27`x' Services: Skills"
			label var ix27`x'4 "IX27`x' Services: Cash"
			label var ix27`x'5 "IX27`x' Services: Goods"
			label var ix27`x'6 "IX27`x' Services: Other"
			label var ix27`x'99 "IX27`x' Services: DK"
			label var ix28`x' "IX28`x' Amnesty certificate?"
			label var ix29`x' "IX29`x' Return Home?"
			}
	
		gen reception=.
		replace reception=0 if ix26a==0 | ix26b==0 | ix26c==0
		replace reception=1 if (ix26a>0 & ix26a<=7) | (ix26b>0 & ix26b<=7) | (ix26c>0 & ix26c<=7)
		label var reception "=1 if reported ever passing through a reception center"
	
	*ix30: pass through LRA again?
	
		codebook ix30
		list id ix1 ix30 ix2byear ix15b  if ix1>1 & ix30!=1
		drop ix30
	
	*ix31: Abducted less than 2 weeks?
	
		codebook ix31
		list id ix31 ix32 abd_lgth  if ix31==. & abd==1
			replace ix31=1 if ix31==. & ix32!=. & abd==1
		list id ix31 abd_lgth  if abd_lgth==.
		list id ix31 abd_lgth  if abd_lgth<.4 & ix31==1
		list id ix31 ix32 abd_lgth  if abd_lgth>.4 & abd_lgth!=. & ix31==2
			replace ix31=1 if abd_lgth>.4 & abd_lgth!=. & ix31==2
	
		label var ix31 "IX31 Abducted less than 2 weeks?"
	
		foreach x of varlist ix32-ix91 {
			cap replace `x'=. if ix31==2 | ix31==.
			}
	
	
	*ix32-ix44: Used a gun?
	
		codebook ix32
	
		list id ix1 ix31 ix32 ix35 ix36  if ix32!=. & (ix31==2 | ix31==.)
			replace ix32=. if ix31==2
		list id ix1 ix31 ix32 ix35 ix36  if ix32==. & ix31==1
			replace ix32=1 if ix32==. & ix35!=.
			replace ix32=2 if ix32==. & ix35==. & ix31==1
	
		codebook ix33*
	
		codebook ix34*
			replace ix34=1 if ix34unit=="1/M" | ix34unit=="1M"
			replace ix34unit="M" if ix34unit=="1/M" | ix34unit=="1M"
			replace ix34=2 if ix34unit=="2/W" | ix34unit=="2/w"
			replace ix34unit="W" if ix34unit=="2/W" | ix34unit=="2/w"
			replace ix34=1 if ix34unit=="1/Y" | ix34unit=="1Y"
			replace ix34unit="Y" if ix34unit=="1/Y" | ix34unit=="1Y"
	
			replace ix34=ix34/30 if ix34unit=="D"
			replace ix34=ix34/4 if ix34unit=="W"
			replace ix34=ix34*12 if ix34unit=="Y"
	
			replace ix34=. if ix32==2 | ix32==.	
	
			list id ix32 ix34 ix34unit  if ix32==1 & ix34==.		
			list id ix32 ix34 ix34unit  if ix34!=. & ix34unit==""	
	
			drop ix34unit
			tab ix34
	
		codebook ix35
	
		codebook ix36
	
		codebook ix37*
	
		codebook ix38
	
		list id ix32 ix34 ix35 ix36 ix37* ix38  if ix32==1 & (ix34==. | ix35==. | ix36==. | ix38==.) 
	
		codebook ix39
	
		list id ix32 ix38 ix39 ix40*  if ix39!=. & (ix38==. | ix38==2)
			replace ix38=1 if ix39!=. & ix38==2
		
		codebook ix40*
	
		codebook ix41*
	
		codebook ix42
	
		codebook ix43
		
		sort ix42
		list id ix32 ix34 ix35 ix36 ix37* ix38 ix42 ix43 abd_lgth  if ix32==1 & (ix42==. | ix43==.) 
			replace ix42=2 if ix43==0 & ix42==.
			replace ix43=0 if ix42==2 & ix43==.
	
		codebook ix44*
	
	
			label var ix32 "IX32 Given a gun?"
			label var ix331 "IX33 Which abduction given?"
			label var ix332 "IX33 Which abduction given?"
			label var ix333 "IX33 Which abduction given?"
			label var ix34 "IX34 Months before given a gun?"
			label var ix35 "IX35 Sleep with gun?"
			label var ix36 "IX36 Dependable?"
			label var ix37min "IX37 # battles? (Min)"
			label var ix37max "IX37 # battles? (Max)"
			label var ix38 "IX38 Rewarded?"
			label var ix39 "IX39 Frequency rewarded?"
	
			label var ix40a "IX40 Actions rewarded"
			label var ix40b "IX40 Actions rewarded"
			label var ix40c "IX40 Actions rewarded"
			label var ix40d "IX40 Actions rewarded"
			label var ix40e "IX40 Actions rewarded"
	
			label var ix42 "IX42 Lead others?"
			label var ix43 "IX43 Rank"
	
	
	aorder 
	order i* v* x*
	
	*ix45-ix55: Attitudes in group
	
		codebook ix45-ix53
	
		*!Check these
		replace ix53="" if ix1==0 | ix31==2
		list id ix1 ix31 ix53  if ix53!="1" & ix53!="2" & ix53!=""
		replace ix53="2" if ix53=="-1" | ix53=="0"
		gen ix53_=real(ix53)
		drop ix53
		ren ix53_ ix53
	
		list id ix45 ix46 ix47 ix48 ix49 ix50 ix51 ix52 ix53  if ix31==1 & (ix45==. | (ix46==. & ix45==1) | (ix46!=. & ix45==2) | ix47==. | ix48==. | (ix49==. & ix48==1) | (ix49!=. & ix48==2) |  (ix50==. & ix48==1) | (ix50!=. & ix48==2) | ix51==. | (ix52==. & ix51==1) | (ix52!=. & ix51==2) | ix53==.) 
	
			replace ix46=. if ix45==2
			replace ix49=. if ix48==2
			replace ix50=. if ix48==2
			replace ix52=. if ix51==2
		
			replace ix45=1 if ix45==. & ix46!=.
			replace ix48=1 if ix48==. & (ix49!=. | ix50!=.)
			replace ix51=1 if ix51==. & ix52!=.
	
		codebook ix54*
	
		codebook ix55*
	
		label var ix45 "IX45 Safer inside LRA?"
		label var ix46 "IX46 Safer inside LRA? (Frequency)"
		label var ix47 "IX47 Protection from bullets?"
		label var ix48 "IX48 Allegiance to Kony?"
		label var ix49 "IX49 Allegiance frequency"
		label var ix50 "IX50 Allegiance still?"
		label var ix51 "IX51 Wanted to stay?"
		label var ix52 "IX52 Wanted to stay? (frequency)"
		label var ix53 "IX53 Want to become commander?"
	
		label var ix541 "IX54 Escape propaganda: Caught"
		label var ix542 "IX54 Escape propaganda: Smell oil"
		label var ix543 "IX54 Escape propaganda: Family killed"
		label var ix544 "IX54 Escape propaganda: UPDF kill/imprison"
		label var ix545 "IX54 Escape propaganda: Troops kill you"
		label var ix546 "IX54 Escape propaganda: Fight to overthrow"
		label var ix547 "IX54 Escape propaganda: Other 1"
		label var ix548 "IX54 Escape propaganda: Other 2"
	
		label var ix551 "IX55 Escape propaganda repeated: Caught"
		label var ix552 "IX55 Escape propaganda repeated: Smell oil"
		label var ix553 "IX55 Escape propaganda repeated: Family killed"
		label var ix554 "IX55 Escape propaganda repeated: UPDF kill/imprison"
		label var ix555 "IX55 Escape propaganda repeated: Troops kill you"
		label var ix556 "IX55 Escape propaganda repeated: Fight to overthrow"
		label var ix557 "IX55 Escape propaganda repeated: Other 1"
		label var ix558 "IX55 Escape propaganda repeated: Other 2"
	
	
	*ix56-ix61: Unit composition
	
		codebook ix56-ix61
		list id ix56-ix61  if ix31==1 & (ix56==. |  ix57==. |  (ix58==. & ix57!=.) |  ix59==. |  ix60=="" |  ix61==.)
	
		tab ix56
			replace ix56=. if ix56==888
			replace ix56=. if ix56>=3000
	
		tab ix57
			replace ix57=. if ix57==888
			replace ix57=. if ix57==99
	
		tab ix58
			replace ix58=. if ix58==888
			replace ix58=. if ix58==99
			replace ix58=0 if ix57==0
			list ix56 ix57 ix58  if ix58==. & ix57!=.
	
		tab ix59
			replace ix59=. if ix59==888
			replace ix59=. if ix59==99
	
		tab ix60
			replace ix60="" if ix60=="99"
	
		tab ix61
			replace ix61=. if ix61==99
	
		label var ix56 "IX56 # Male fighters in unit"
		label var ix57 "IX57 # Females in unit"
		label var ix58 "IX58 # Female fighters in unit"
		label var ix59 "IX59 # Commanders in unit"
		label var ix60 "IX60 Commander name"
		label var ix61 "IX61 Feel important part of unit?"
	
	
	*ix62: Do to civilians
	
		codebook ix62*
		label var ix620 "IX62 Civilian orders: None"
		label var ix621 "IX62 Civilian orders: Whatever you want"
		label var ix622 "IX62 Civilian orders: Leave be"
		label var ix623 "IX62 Civilian orders: Abduct"
		label var ix624 "IX62 Civilian orders: Beat"
		label var ix625 "IX62 Civilian orders: Steal"
		label var ix626 "IX62 Civilian orders: Mutilate"
		label var ix627 "IX62 Civilian orders: Tie up"
		label var ix628 "IX62 Civilian orders: Kill"
		label var ix629 "IX62 Civilian orders: Other"
		label var ix629other "IX62 Civilian orders: Other (describe)"
	
	
	*ix63-ix68: Unit violence
		
		codebook ix63a-ix68b
	
		foreach x of numlist 63/68 {
			list id ix`x'a ix`x'b  if ix31==1 & ix`x'a==.
			replace ix`x'a=1 if ix`x'a==. & ix`x'b!=.
			list id ix`x'a ix`x'b  if ix`x'b==. & ix`x'a==1
			list id ix`x'a ix`x'b  if ix`x'b!=. & ix`x'a==2
			}
	
		label var ix63a "IX63a New abductees beaten?"
		label var ix63b "IX63b New abductees beaten? (Frequency)"
		label var ix64a "IX64a New abductees beaten/killed by other abductees?"
		label var ix64b "IX64b New abductees beaten/killed by other abductees? (Frequency)"
		label var ix65a "IX65a Mutilate civilians?"
		label var ix65b "IX65b Mutilate civilians? (Frequency)"
		label var ix66a "IX66a Desecrate bodies?"
		label var ix66b "IX66b Desecrate bodies? (Frequency)"
		label var ix67a "IX67a Carry body parts?"
		label var ix67b "IX67b Carry body parts? (Frequency)"
		label var ix68a "IX68a Punished for no reason?"
		label var ix68b "IX68b Punished for no reason? (Frequency)"
	
	
	*ix69-ix74: Discipline
	
		codebook ix69a-ix74b
		list id ix69a ix70a ix71a ix72a ix73a ix74a  if ix31==1 & (ix69a==. | ix70a==. | ix71a==. | ix72a==. | ix73a==. | ix74a==.)
	
		foreach x of numlist 69/74 {
			replace ix`x'a=. if ix`x'a==99
			replace ix`x'a=1 if ix`x'a==. & ix`x'b!=.
			}
	
		label var ix69a "IX69a Theft discipined?"
		label var ix70a "IX70a Beating fighters discipined?"
		label var ix71a "IX71a Beating abductees discipined?"
		label var ix72a "IX72a Killing fighter discipined?"
		label var ix73a "IX73a Killing abductee discipined?"
		label var ix74a "IX74a Killing civilans discipined?"
		label var ix69b "IX69b How theft discipined?"
		label var ix70b "IX70b How beating fighters discipined?"
		label var ix71b "IX71b How beating abductees discipined?"
		label var ix72b "IX72b How killing fighter discipined?"
		label var ix73b "IX73b How killing abductee discipined?"
		label var ix74b "IX74b How killing civilans discipined?"
	
	
	*ix75-ix76: Returned to school?
		
		codebook ix75
		list id ix75 ix76*  if ix75==. & ix31==1
			replace ix75=2 if ix31==1 & ix75==. & (ix76a==1 | ix76b==1 | ix76c==1 | ix76d==1 | ix76e==1 | ix76f==1 | ix76g==1 | ix76h==1 | ix76h==1 | ix76i==1 | ix76j==1) 
		
		codebook ix76*
	
		label var ix75 "IX75 returned to school?"
	
		label var ix76a "IX76 Did not return: Too old"
		label var ix76b "IX76 Did not return: Finished"
		label var ix76c "IX76 Did not return: No schools"
		label var ix76d "IX76 Did not return: No money"
		label var ix76e "IX76 Did not return: Not worthwhile"
		label var ix76f "IX76 Did not return: Don't like"
		label var ix76g "IX76 Did not return: Need job"
		label var ix76h "IX76 Did not return: Had job"
		label var ix76i "IX76 Did not return: Did not want to repeat"
		label var ix76j "IX76 Did not return: Difficulty concentrating"
		label var ix76k "IX76 Did not return: Other"
		label var ix76kother "IX76 Did not return: Other (describe)"
		
	*ix77-ix83
	
		codebook ix77*
		list id ix77*  if ix31==1 & ix771==0 & ix772==0 & ix773==0 & ix774==0 & ix775==0 & ix776==0
	
		codebook ix78*
		list id ix780-ix787  if ix31==1 & ix780==0 & ix781==0 & ix782==0 & ix783==0 & ix787==0 & ix785==0 & ix786==0
		replace ix780=1 if ix31==1 & ix780==0 & ix781==0 & ix782==0 & ix783==0 & ix787==0 & ix785==0 & ix786==0
	
		codebook ix79-ix80
		list id ix79-ix83  if ix31==1 & ix79==.
			replace ix79=. if ix79==99
		list id ix79-ix83  if ix31==1 & ix79==3 & ix80==""
		list id ix79-ix83  if ix31==1 & ix80!=""
			*! Fix these?
	
		codebook ix90-ix91
		list id ix90 ix91  if ix31==1 & ix90==.
			replace ix90=. if ix90==99
		list id ix90 ix91  if ix31==1 & ix90==3 & ix91==""
		list id ix90 ix91  if ix31==1 & ix90!=3 & ix91!=""
			*! Fix these?
	
		codebook ix81-ix83
	
		label var ix771 "IX77 Relatives returned to: Mother"
		label var ix772 "IX77 Relatives returned to: Father"
		label var ix773 "IX77 Relatives returned to: Aunt/Uncle"
		label var ix774 "IX77 Relatives returned to: Grandparent"
		label var ix775 "IX77 Relatives returned to: Sibling"
		label var ix776 "IX77 Relatives returned to: Other"
		label var ix776other "IX77 Relatives returned to: Other"
	
		label var ix780 "XI78 Services after reception center: None"
		label var ix781 "XI78 Services after reception center: Medical"
		label var ix782 "XI78 Services after reception center: Counseling"
		label var ix783 "XI78 Services after reception center: IGA"
		label var ix785 "XI78 Services after reception center: School"
		label var ix786 "XI78 Services after reception center: Amnesty package"
		label var ix787 "XI78 Services after reception center: Other"
		label var ix787other "XI78 Services after reception center: Other"
	
		label var ix79 "IX79 Accepted by family?"
		label var ix80 "IX80 Why not?"
		label var ix81 "IX81 Family happy to see you?"
		label var ix82 "IX82 Step on egg?"
		label var ix83 "IX83 Cleansing ceremony?"
	
		label var ix90 "IX90 Accepted by community?"
		label var ix91 "IX91 Why not?"
	
	
	*ix84-ix89
	
		codebook ix84a-ix89b
		list ix84a-ix89b  if ix31==1 & (ix84a==. | ix85a==. | ix86a==. | ix87a==. | ix88a==. | ix89a==.)
		list ix84a-ix89b  if ix31==1 & ((ix84a==1 & ix84b==.) | (ix85a==1 & ix85b==.) | (ix86a==1 & ix86b==.) | (ix87a==1 & ix87b==.) | (ix88a==1 & ix88b==.) | (ix89a==1 & ix89b==.))
		list ix84a-ix89b  if ix31==1 & ((ix84a==2 & ix84b!=.) | (ix85a==2 & ix85b!=.) | (ix86a==2 & ix86b!=.) | (ix87a==2 & ix87b!=.) | (ix88a==2 & ix88b!=.) | (ix89a==2 & ix89b!=.))
			
			foreach x of numlist 84/89 {
				replace ix`x'a=1 if ix31==1 & ix`x'a==. & ix`x'b!=.
				replace ix`x'a=1 if ix31==1 & ix`x'a==2 & ix`x'b==1
				replace ix`x'a=. if ix31==1 & ix`x'a==2 & ix`x'b==2
				}
	
		label var ix84a "IX84a Family ever insult you?"	
		label var ix84b "IX84b Family still insult you?"
		label var ix85a "IX85a Family ever blame you?"	
		label var ix85b "IX85b Family still blame you?"
		label var ix86a "IX86a Family ever hurt you?"	
		label var ix86b "IX86b Family still hurt you?"
		label var ix87a "IX87a Community ever insult you?"	
		label var ix87b "IX87b Community still insult you?"
		label var ix88a "IX88a Community ever blame you?"	
		label var ix88b "IX88b Community still blame you?"
		label var ix89a "IX89a Community ever afraid of you?"	
		label var ix89b "IX89b Community still afraid of you?"
	
	
	
	***************
	** SECTION X **
	***************
	
	*x2-x34: Traumas
	
		codebook x2-x32
	
		gen MISSING_TRAUMA=0
				
				foreach k of numlist 2,3,4,5,7,9,11,12,13,14,16,17,19 {
			encode x`k', generate(x`k'_a)
			}
			
					foreach k of numlist 2,3,4,5,7,9,11,12,13,14,16,17,19 {
			drop x`k'
			rename x`k'_a x`k'
			}
			
			
		foreach k of numlist 2/32 {
			list id x`k'  if x`k'==.
			capture replace MISSING_TRAUMA = MISSING_TRAUMA+1 if x`k'==.
			}
			
		tab x33
			*Check out these
		list id x32 x33  if x32==1 & x33==""
		list id x32 x33  if x32==2 & x33!=""
			*Check out these
		list id x32 x33  if x32==. & x33!=""
			*Check out these
	
		codebook x34*
	
		label var x2 "X2 War trauma: Property destroyed"
		label var x3 "X3 War trauma: Gunfire"
		label var x4 "X4 War trauma: Hom shot"
		label var x5 "X5 War trauma: Witness attack"
		label var x6 "X6 War trauma: Beating"
		label var x7 "X7 War trauma: Armed attack"
		label var x8 "X8 War trauma: Imprisioned or tied"
		label var x9 "X9 War trauma: Battle injury"
		label var x10 "X10 War trauma: Carry heavy loads"
		label var x11 "X11 War trauma: Betrayed"
		label var x12 "X12 War trauma: Witness beatings"
		label var x13 "X13 War trauma: Witness killing"
		label var x14 "X14 War trauma: Witness massacre"
		label var x15 "X15 War trauma: Witness setting fire to houses with people"
		label var x16 "X16 War trauma: Witnessed rape"
		label var x17 "X17 War trauma: Parent died violently"
		label var x18 "X18 War trauma: Parent disappeared/abducted"
		label var x19 "X19 War trauma: Friend/family died violently"
		label var x20 "X20 War trauma: Friend/family disappeared/abducted"
		label var x21 "X21 War trauma: Friend/family landmine injury"
		label var x22 "X22 War trauma: Forced to steal"
		label var x23 "X23 War trauma: Forced to kill soldier"
		label var x24 "X24 War trauma: Forced to beat stranger"
		label var x25 "X25 War trauma: Forced to kill stranger"
		label var x26 "X26 War trauma: Forced to kill strange civilian"
		label var x27 "X27 War trauma: Forced to kill family/friend"
		label var x28 "X28 War trauma: Forced to have sex"
		label var x29 "X29 War trauma: Forced to abuse bodies"
		label var x30 "X30 War trauma: Forced to betray family/friend"
		label var x31 "X31 War trauma: Forced to betray stranger"
		label var x32 "X32 War trauma: Other"
		label var x33 "X33 War trauma: Other (describe)"
		label var x34a "X34 War trauma: Worst experience 1"
		label var x34b "X34 War trauma: Worst experience 2"
		label var x34c "X34 War trauma: Worst experience 3"
		
	
	*x35-x48
	
		codebook x35-x37
	
		codebook x38-x41
		list id x37-x41 if x38==.
		list id x38-x41 if (x38==1 | x38==3) & (x39==. | x41=="")
		list id x38-x41 if x38==2 & (x39!=. | x41!="")
			replace x39=. if x38==2
			replace x40=. if x38==2
			replace x41="" if x38==2
	
		codebook x42-x48
	
		tab x49
		tab x49who
		drop x49
		ren x49who x49
		replace x49="No one" if x49=="0"
	
		label var x35 "X35 Not strong enough?"
		label var x36 "X36 Parents to blame?"
		label var x37 "X37 Self blame?"
		label var x38 "X38 Haunted?"
		label var x39 "X39 Know spirits?"
		label var x40 "X40 Who are spirits?"
		label var x41 "X41 What to stop haunting?"
		label var x42 "X42 Caused trouble to family?"
		label var x43 "X43 Guilt?"
		label var x44 "X44 Caused trouble for community?"
		label var x45 "X45 Seen as bad person by community?"
		label var x46 "X46 Kony to blame?"
		label var x47 "X47 God's plan for you?"
		label var x48 "X48 Cursed?"
		label var x49 "X49 Who else to blame?"
	
	
	*x50-66
	
		codebook x50a-x66b
	
		foreach n of numlist 50/66 {
			list id x`n'a x`n'b  if x`n'a==.
				replace x`n'a=1 if x`n'a==. & x`n'b!=.
			list id x`n'a x`n'b  if x`n'b==. & x`n'a==1
			list id x`n'a x`n'b  if x`n'b!=. & x`n'a==2
				replace x`n'b=. if x`n'a==2
			list id x`n'a x`n'b  if x`n'b==3
				replace x`n'b=2 if x`n'b==3
			}
	
		label var x50a "X50a Support: Looked after your things?"
		label var x50b "X50b Support: Looked after your things? (frequency)"
		label var x51a "X51a Support: Sat with you?"
		label var x51b "X51b Support: Sat with you? (frequency)"
		label var x52a "X52a Support: Shared with you?"
		label var x52b "X52b Support: Shared with you? (frequency)"
		label var x53a "X53a Support: Work with you"
		label var x53b "X53b Support: Work with you (frequency)"
		label var x54a "X54a Support: Appreciated you"
		label var x54b "X54b Support: Appreciated you (frequency)"
		label var x55a "X55a Support: Gave info"
		label var x55b "X55b Support: Gave info (frequency)"
		label var x56a "X56a Support: Take heart"
		label var x56b "X56b Support: Take heart (frequency)"
		label var x57a "X57a Support: Help find direction"
		label var x57b "X57b Support: Help find direction (frequency)"
		label var x58a "X58a Support: Gave advice"
		label var x58b "X58b Support: Gave advice (frequency)"
		label var x59a "X59a Support: Lent you things for free"
		label var x59b "X59b Support: Lent you things for free(frequency)"
		label var x60a "X60a Support: Listened"
		label var x60b "X60b Support: Listened (frequency)"
		label var x61a "X61a Support: Gave you something"
		label var x61b "X61b Support: Gave you (frequency)"
		label var x62a "X62a Support: Taught you"
		label var x62b "X62b Support: Taught you (frequency)"
		label var x63a "X63a Support: Joked"
		label var x63b "X63b Support: Joked (frequency)"
		label var x64a "X64a Support: Prayed"
		label var x64b "X64b Support: Prayed (frequency)"
		label var x65a "X65a Support: Took to healer"
		label var x65b "X65b Support: Took to healer (frequency)"
		label var x66a "X66a Support: Helped you"
		label var x66b "X66b Support: Helped you (frequency)"
	
	
	****************
	** SECTION XI **
	****************
	
	* Raw data for this section removed for anonymity. 
	
	
	
	*****************
	** SECTION XII **
	*****************
	
	*xii6: Time interview
	
		codebook xii6
		tab xii6
	
		*Fix times
			egen time=ends(xii6), tail punct(" ") trim
			egen hr=ends(time), head punct(":") trim
			egen mins=ends(time), tail punct(":") trim
			egen min=ends(mins), head punct(":") trim
			gen xii6hr=real(hr)
			gen xii6min=real(min)
			drop xii6 time mins hr min
			tab xii6hr
			tab xii6min
			replace xii6hr=xii6hr+12 if xii6hr>=1 & xii6hr<=6
	
		*Length of interview
			gen xii6_length=((xii6hr-i5bhr)*60)+(xii6min-i5bmin)
			
			sort xii6_length
			list xii6hr i5bhr xii6min i5bmin xii6_length  if xii6_length<=40 | xii6_length>120
	
			
			list xii6_length if xii6_length<=50 | xii6_length>120
	
			replace xii6_length=. if xii6_length<20 | xii6_length>180
	
		label var xii6hr "XII6 Interview end hour"
		label var xii6min "XII6 Interview end min"
		label var xii6_length "Interview length (mins)"
	
			
	*xii7-xii11: Survey quality measures
	
		codebook xii7-xii11
	
		list id xii7  if xii7>1
		list id xii10  if xii7>1
		list id xii11  if xii7>1
	
		label var xii7 "XII7 Quality: Respondent comprehension"
		label var xii8 "XII8 Quality: People present?"
		label var xii10 "XII10 Quality: Respondent thought process"
		label var xii11 "XII11 Quality: Respondent emotional state"
		
	
	*xii12-xii14: Referrals
	
		codebook xii12-xii14
	
		label var xii12 "XII12 Referral: Details"
		label var xii13 "XII13 Referral: Priority"
		label var xii14 "XII14 Referral: Permission"
	
	

	
	
	save "SWAYI/tempdata/SWAY cleaning - I survey.dta", replace
	


	**************************************************************************
	** CLEANING HOUSEHOLD LEVEL SURVEY (code from SWAY_I_cleaning_HHsurvey.do)
	**************************************************************************
	
	clear
	
	use "SWAYI/tempdata/SWAY cleaning - I survey.dta", clear
	
	
	************
	** IMPORT **
	************
	
		insheet using "SWAYI/rawdata/HHii_suppl.csv", clear names comma
		tostring f, replace
		tostring hii, replace
		save "SWAYI/tempdata/temp1.dta", replace
	
		insheet using "SWAYI/rawdata/HHi_suppl.csv", clear names
		tostring initials, replace
		tostring parish, replace
		tostring block, replace
		tostring district2, replace
		tostring parish2, replace
		tostring interviewtime, replace
		tostring qn4a, replace
		tostring qn7month, replace
		tostring qn7year, replace
		tostring qn12other, replace
		tostring qn16, replace
	
		
		save "SWAYI/tempdata/temp2.dta", replace
	
		insheet using "SWAYI/rawdata/HHii.txt", clear names
			tostring hinumber, replace
			tostring ii, replace
		append using "SWAYI/tempdata/temp1.dta"
		ren qnno id2
		label var id2 "HH Questionnaire ID"
		sort id2
		save "SWAYI/tempdata/hhiitemp.dta", replace
	
		insheet using "SWAYI/rawdata/HHi.txt", clear names
		
		append using "SWAYI/tempdata/temp2.dta"
		ren qnno id2
		label var id2 "HH Questionnaire ID"
		sort id2
	
	
	***************
	** SECTION I **
	***************
	
	drop phase initials
	*hhi1: Generate Respondent ID
	
		*!Check for missing or unusual values	
			codebook district camp parish village household	block
			tab parish
			tab block
	
			replace block="1" if block=="A"
			replace block="2" if block=="B"
			replace block="3" if block=="C"
			replace block="4" if block=="D"
			replace block="5" if block=="E"
			replace block="6" if block=="F"
			gen block_=real(block)
			drop block
			ren block_ block
	
			replace parish="1" if parish=="A"
			replace parish="4" if parish=="D"
			replace parish="6" if parish=="F"
			gen parish_=real(parish)
			drop parish
			ren parish_ parish
	
		codebook district2 camp2 parish2 village2 household2
		tab district2 
		tab parish2
	
			replace district2="1" if district2=="I"
			gen district_=real(district2)
			drop district2
			ren district_ district2
	
			replace parish2="1" if parish2=="A" | parish2=="I"
			replace parish2="4" if parish2=="D"
			replace parish2="6" if parish2=="F"
			gen parish_=real(parish2)
			drop parish2
			ren parish_ parish2
	
		replace district2=2 if district2==5
		list id district district2 if district!=district2 & district2!=.
		replace district=district2 if district!=district2 & district2!=.
		
		list id camp camp2 if camp!=camp2 & camp2!=.
		replace camp=camp2 if camp!=camp2 & camp2!=.
		
		list id parish parish2 if parish !=parish2 & parish2 !=.
		replace parish=parish2 if parish !=parish2 & parish2 !=.
		
		list id village village2 if village!=village2 & village2 !=.
		replace village=village2 if village!=village2 & village2 !=.	
		
		list id household household2 if household !=household2 & household2 !=.
		replace household =household2 if household !=household2 & household2 !=.
		
		drop district2 camp2 parish2 village2 household2
		
		*replace erroneous ids
			
			replace village=41 if id==28
			replace household=3 if id==50
			replace household=8 if id==55
			replace village=1 if id==72
			replace household=3 if id==98
			replace village=53 if id==103
			replace village=46 if id==106
			replace household=2 if id==106
			replace village=23 if id==132
			replace village=22 if id==161
			replace parish=2 if id==173
			replace parish=3 if id==189
			replace village=51 if id==195
			replace household=9 if id==201
			replace camp=6 if id==214
			replace village=27 if id==230
			replace household=10 if id==262
			replace district=2 if id==274
			replace parish=2 if id==274
			replace village=32 if id==342
			replace household=3 if id==342
			replace household=2 if id==351
			replace household=8 if id==477
			replace household=4 if id==478
			replace village=29 if id==478
			replace village=4 if id==480
			replace parish=5 if id==526
			replace village=2 if id==623
			replace household=3 if id==706
			replace parish=4 if id==706
			replace household=8 if id==716
			replace household=15 if id==746
			replace household=3 if id==767
			replace household=4 if id==790
			replace household=3 if id==853
			replace village=99 if id==857
			replace household=7 if id==866
			replace household=1 if id==865
			replace household=4 if id==868
			replace household=2 if id==869
			replace household=1 if id==870
			replace household=3 if id==871
			replace household=5 if id==872
			replace household=1 if id==873
			replace household=3 if id==881
			replace household=2 if id==892
			replace village=37 if id==905
			replace village=14 if id==906
	
		* Same ID, Different block #
			replace household=5 if id==17
			replace household=1 if id==795
			replace household=2 if id==796
			replace household=1 if id==221
			replace household=3 if id==259
			replace household=3 if id==478
			*! check these last two!
	
		* duplicate entry of photocopied roster
		drop if id==744 | id==837 | id==835 | id==392 | id==882 | id==876 
	
		* duplicate roster created
		drop if id==73 | id==873 | id==899 | id==360 | id==82 | id==369 | id==271 | id==651
	
		replace householdsize=8 if id==348
	 
	
		egen hhi1=concat(district camp parish village household), punct(".")
		label var hhi1 "HHI1 Household ID Number"
	
		duplicates tag hhi1, g(dup)
		sort district camp parish village household block
		list id2 district camp parish village household dup if dup!=0
		list id2 district camp parish village household if district==. | camp==. | parish==. | village==. | household==.
	
		gen C_ach=cond(camp==1,1,0)
		gen C_akw=cond(camp==2,1,0)
		gen C_kma=cond(camp==3,1,0)
		gen C_oro=cond(camp==4,1,0)
		gen C_pad=cond(camp==5,1,0)
		gen C_paj=cond(camp==6,1,0)
		gen C_pal=cond(camp==7,1,0)
		gen C_ata=cond(camp==8,1,0)
	
		label values camp camps
	
		label var C_ach "Acholibur Dummy"
		label var C_akw "Akwang Dummy"
		label var C_kma "K. Matidi Dummy"
		label var C_oro "Orom Dummy"
		label var C_pad "Pader Dummy"
		label var C_paj "Pajule Dummy"
		label var C_pal "Palabek Dummy"
		label var C_ata "Atanga Dummy"
	
		label var district "HHI1 District ID"
		label var camp "HHI1 Camp ID"
		label var parish "HHI1 Parish ID"
		label var village "HHI1 Village ID"
		label var household "HHI1 Household ID"
	
	list id2 if camp==4 & parish==5 & village==51 & household==2
	list id2 if camp==6 & parish==1 & village==55 & household==3
	
	
	
	*HH size & selection probability
	
		tab householdsize, m
	
		list id2 district camp parish village household if householdsize==.
	
		ren householdsize hhsize
		replace hhsize=0 if hhsize==88
		
		*! fix this!
			replace hhsize=0 if id==23
			replace hhsize=5 if id==82
			replace hhsize=9 if id==212
			replace hhsize=3 if id==299
			replace hhsize=5 if id==335
			replace hhsize=8 if id==348
			replace hhsize=7 if id==360
			replace hhsize=8 if id==376
			replace hhsize=5 if id==377
			replace hhsize=2 if id==392
			replace hhsize=7 if id==403
			replace hhsize=4 if id==522
			replace hhsize=0 if id==535
			replace hhsize=0 if id==554
			replace hhsize=0 if id==571
			replace hhsize=0 if id==586
			replace hhsize=0 if id==602
			replace hhsize=0 if id==605
			replace hhsize=5 if id==786
			replace hhsize=4 if id==792
			replace hhsize=0 if id==811
			replace hhsize=3 if id==835
			replace hhsize=5 if id==837
			replace hhsize=2 if id==956
			replace hhsize=2 if id==966
			replace hhsize=5 if id==969
			replace hhsize=5 if id==978
			replace hhsize=7 if id==995
			replace hhsize=6 if id==999
			replace hhsize=5 if id==1003
			replace hhsize=2 if id==1017
			replace hhsize=0 if id==1020
			replace hhsize=6 if id==1022
	
		replace hhsize=5 if hhsize==.
	
		save "SWAYI/tempdata/temp", replace
	
		insheet using "SWAYI/rawdata/hh_weights.csv", comma clear names
		reshape long camp, i(hhsize) j(x) string
		ren camp hh_select
		gen camp=real(x)
		drop x
		sort camp hhsize
		save "SWAYI/tempdata/hh_weights", replace
	
		use "SWAYI/tempdata/temp", clear
		sort camp hhsize
		merge camp hhsize using "SWAYI/tempdata/hh_weights"
		drop if _merge==2
		drop _merge
	
		tab hh_select
		
		gen hh_weight=1/hh_select
	
		replace hh_weight=50 if hh_weight>50 & hh_weight!=.
		replace hh_weight=20 if hh_weight==.
	
	*hhi2: Respondents
	
		*drop surname1-firstname3
	
	*hhi3,hhi16: Survey Date/Time
	
	
		codebook interviewday interviewmm interviewyy interviewtime qn16
		replace interviewmm=. if interviewmm>4 & interviewmm<8
		replace interviewyy=interviewyy+2000
		replace interviewyy=2005 if interviewyy!=2005 & interviewmm>=8
		replace interviewyy=2006 if interviewyy!=2006 & interviewmm<4
	
		ren interviewday hhi3aday
		ren interviewmm hhi3amo
		ren interviewyy hhi3ayr
	
		*Fix times
			egen time=ends(interviewtime), tail punct(" ") trim
			egen hr=ends(time), head punct(":") trim
			egen mins=ends(time), tail punct(":") trim
			egen min=ends(mins), head punct(":") trim
			gen hhi3bhr=real(hr)
			gen hhi3bmin=real(min)
			drop interviewtime time mins hr min
			tab hhi3bhr
			tab hhi3bmin
	
			replace hhi3bhr=hhi3bhr+12 if hhi3bhr>=1 & hhi3bhr<=5
			replace hhi3bhr=24 if hhi3bhr==0
	
			egen time=ends(qn16), tail punct(" ") trim
			egen hr=ends(time), head punct(":") trim
			egen mins=ends(time), tail punct(":") trim
			egen min=ends(mins), head punct(":") trim
			gen hhi16hr=real(hr)
			gen hhi16min=real(min)
			drop qn16 time mins hr min
			tab hhi16hr
			tab hhi16min
	
			replace hhi16hr=hhi16hr+12 if hhi16hr>=1 & hhi16hr<=5
			replace hhi16hr=24 if hhi16hr==0 
	
		gen int_lgth_hhi=((hhi16hr-hhi3bhr)*60)+(hhi16min-hhi3bmin)
			tab int_lgth_hhi
	
			replace hhi3bhr=12 if hhi3bhr==24 & int_lgth_hhi<0
			replace hhi3bhr=hhi3bhr-1 if int_lgth_hhi<0 & int_lgth_hhi>=-60
			replace int_lgth_hhi=((hhi16hr-hhi3bhr)*60)+(hhi16min-hhi3bmin)
	
			replace int_lgth_hhi=. if int_lgth<0 | int_lgth_hhi>120
			
		label var hhi3aday "HHI3a Interview Day"
		label var hhi3amo "HHI3a Interview Month"
		label var hhi3ayr "HHI3a Interview Year"
		label var hhi3bhr "HHI3b Interview Start: Hour"
		label var hhi3bmin "HHI3b Interview Start: Minute"
		label var hhi16hr "HHI3b Interview End: Hour"
		label var hhi16min "HHI3b Interview End: Minute"
		label var int_lgth_hhi "HHI Interview Length (Minutes)"
	
	
	
	
	*hhi5,hhi6: Agree to interview?
	
		codebook qn5 qn6
	
		ren qn5 hhi5
		ren qn6 hhi6
	
		label var hhi5 "HHI5 Agree to Interview?"
		label var hhi6 "HHI6 Why not agree?"
		
	
	*hhi7: Length in camp
	
		codebook qn7month qn7year
		tab qn7month
			replace qn7month="99" if qn7month=="DD"
			gen hhi7mo=real(qn7month)
			drop qn7month
			replace hhi7mo=. if hhi7mo>12 | hhi7mo<1
		tab qn7year
			gen hhi7yr=real(qn7year)
			drop qn7year
			replace hhi7yr=. if hhi7yr>2006
			replace hhi7yr=hhi7yr+2000 if hhi7yr<100
			tab hhi7yr camp 
	
	
		*!Fix seemingly erroneous dates
			bysort camp: egen medmo=median(hhi7mo)
			replace hhi7mo=round(medmo) if hhi7mo==.
			bysort camp: egen minmode=mode(hhi7yr), min
			replace hhi7yr=minmode if hhi7yr==. | hhi7yr<1996 | (hhi7yr<2001 & camp!=7)
			drop medmo minmode
			tab hhi7yr camp 
	
		*!Find camp opening dates
	
		gen camp_yrs=(hhi3ayr-hhi7yr)+((hhi3amo-hhi7mo)/12)
	
		label var hhi7mo "HHI7 Month Moved to Camp"
		label var hhi7yr "HHI7 Year Moved to Camp"
		label var camp_yrs "Years household in camp"
	
	
	*hhi8-hhi12: assets & occupation
	
		codebook qn8 qn8unit
		tab qn8
			replace qn8=0 if qn8==.
		tab qn8 if qn8unit==""
		tab qn8 if qn8unit=="Katala"
			replace qn8=qn8*2.47 if qn8unit=="Hectare"
			replace qn8=((qn8*2)^2)/4046 if qn8unit=="Katala" & qn8<100
			replace qn8=qn8*2/4046 if qn8unit=="Katala" & qn8>100
		tab qn8
		tab qn8 if qn8unit=="Katala"
		list qn8 qn8unit if qn8>100
		drop qn8unit
	
		*!cut off if acres > 100?
			replace qn8=100 if qn8>100
	
		ren qn8 hhi8
		ren qn9 hhi9
		ren qn10 hhi10
		ren qn11 hhi11
	
		gen hhi8_log=ln(hhi8+.01)
		gen hhi8_small=cond(hhi8<2,1,0)
	
		codebook hhi9-hhi11
			regress hhi9 hhi8_log hhi10 hhi11 C_*
				predict xb
				replace hhi9=xb if hhi9==.
				drop xb
	
		codebook qn12 qn12other
			list id qn12 qn12other if qn12==. | qn12==5 | qn12other!=""
			list id qn12 qn12other if qn12==. | qn12==5 
			replace qn12=2 if id2==19 | id2==143 | id2==315 | id2==318 | id2==340 | id2==531 | id2==614 | id2==668 | id2==725 | id2==800 | id2==874
			replace qn12=3 if id2==23 | id2==120 | id2==137 | id2==202 | id2==245 | id2==481 | id2==576 | id2==680 | id2==681 | id2==497 | id2==505 | id2==512 | id2==706 | id2==759 | id2==875
			replace qn12=4 if id2==9 | id2==18 | id2==563 | id2==606
			replace qn12=. if id2==62 | id2==195 | id2==722
	
	
		codebook qn12
		gen fthr_frm=cond(qn12==1 | qn12==. ,1,0)
		gen fthr_pro=cond(qn12==2 | qn12==3 | qn12==5,1,0)
		gen fthr_sol=cond(qn12==4,1,0)
	
		ren qn12 hhi12
		ren qn12other hhi12other
	
		pca hhi9 hhi10 hhi11, means
		predict score1 hh_assets
		codebook hh_assets
		replace hh_assets=0 if hh_assets==.
		replace hh_assets=9 if hh_assets>9
		egen hh_assets96=std(hh_assets)
		drop hh_assets
	
		factor hhi8_log hhi9 hhi10 hhi11, pcf
		rotate
		pca hhi8_log hhi9 hhi10 hhi11
		predict score2 hh_wealth
		codebook hh_wealth
		replace hh_wealth=0 if hh_wealth==.
		replace hh_wealth=9 if hh_wealth>9
		egen hh_wealth96=std(hh_wealth)
		drop hh_wealth
	
		tab hh_assets96
		gen asset_50th=cond(hh_assets96<-.38,1,0)
		gen asset_75th=cond(hh_assets96>.08,1,0)
		gen asset_90th=cond(hh_assets96>.79,1,0)
	
		tab hh_wealth96
		gen w10=cond(hh_wealth96<-.62,1,0)
		gen w25=cond(hh_wealth96<-.5,1,0)
		gen w75=cond(hh_wealth96>.12,1,0)
		gen w90=cond(hh_wealth96>.85,1,0)
	
		label var hhi8 "HHI8 1996 Land (Acres)"
		label var hhi8_log "Ln 1996 Land (Acres)"
		label var hhi9 "HHI9 1996 Cattle"
		label var hhi10 "HHI10 1996 Goats/Sheep/Pigs"
		label var hhi11 "HHI11 1996 Ploughs"
		label var hhi12 "HHI12 1996 HHI Head's Occupation"
		label var hhi12other "HHI12 1996 HHI Head's Occupation (Other)"
		label var fthr_pro "=1 if Father a professional"
		label var fthr_sol "=1 if Father a soldier"
		label var fthr_frm "=1 if Father a farmer"
		label var hh_assets96 "1996 Assets Index"
		label var hh_wealth96 "1996 Wealth Index"
		label var asset_50th "=1 if Poorest 50% in Assets in 1996"
		label var asset_75th "=1 if Richest 25% in Assets in 1996"
		label var asset_90th "=1 if Richest 10% in Assets in 1996"
		label var w10 "=1 if Poorest 10% in Assets/Land in 1996"
		label var w25 "=1 if Poorest 25% in Assets/Land in 1996"
		label var w75 "=1 if Richest 25% in Assets/Land in 1996"
		label var w90 "=1 if Richest 10% in Assets/Land in 1996"
	
	drop qn13 qn14 qn15 
	
	aorder
	order id2 hh*
	sort id2
	save "SWAYI/HHsurvey_sectionI_27May2008.dta", replace
	
	****************
	** SECTION II **
	****************
	use "SWAYI/HHsurvey_sectionI_27May2008.dta", clear
	
	keep  id2 camp parish village household block hhi8 hhi8_log hhi9 hhi10 hhi11 hh_assets96 hh_wealth96 fthr* asset_* w10-w90 hh_select hh_weight camp_yrs C* 
	sort id2
	
		* duplicate entry of photocopied roster
		drop if id==744 | id==837 | id==835 | id==392 | id==882 | id==876 
		* duplicate roster created
		drop if id==73 | id==873 | id==899 | id==360 | id==82 | id==369 | id==271 | id==651
	
	
	merge id2 using "SWAYI/tempdata/hhiitemp.dta"
	drop _merge
		
		foreach x of varlist a-l {
			ren `x' hhii`x'
			}
	
	*hhiia,b, wfp
		codebook hhiia hhiiwfp

		*drop hhiib
	
		list id2 if hhiia==.
		replace hhiia=17 if hhiia==. & id2==76
		replace hhiia=13 if hhiia==. & id2==668
	
		gen hhiib_05=cond(hhiiwfp=="",0,1)
		drop hhiiwfp
	
		bysort id2: egen hh_size05=sum(hhiib_05)
		
		gen a=0
		replace a=1 if hhiib_05==1 & hhiif!="A"
		bysort id2: egen hh_adults05=sum(a)
		drop a
	
		gen a=0
		replace a=1 if hhiif!="A"
		bysort id2: egen hh_size96=sum(a)
		drop a
	
		sort id2 hhiia
	
	*hhiic,d,e
		codebook hhiic-hhiig
	
		tab hhiic if hhiia==1
		list id2 hhiic  if hhiia==1 & hhiic!=.
			replace hhiic=0 if hhiia==1
		list id2 hhiia-hhiig  if hhiic==.
	
		tab hhiid
		replace hhiid="1" if hhiid=="M"
		replace hhiid="0" if hhiid=="F"
		gen hhiid_=real(hhiid)
		drop hhiid
		ren hhiid_ hhiid
		codebook hhiid
	
		replace hhiid=1 if (id2==378 & hhiia==5) | (id2==979 & hhiia==11) | (id2==202 & hhiia==2) | (id2==727 & hhiia==6) | (id2==971 & hhiia==4) | (id2==973 & hhiia==3)
	
		aorder
		order id2 hhii*
	
		list id2 hhiia-hhiig  if hhiid==.
	
		gen male=hhiid 
		gen died=cond(hhiie==.,0,1)
	
	* hhiif: Age
	
		tab hhiif, m
		list id2 hhiic-hhiig if hhiif==""
		list id2 hhiia-hhiig if hhiif=="G"
		replace hhiif="F" if hhiif=="G"
	
		tab1 hhiiage hhiiageunit, m
			replace hhiiage=hhiiage/365 if hhiiageunit=="DAYS"
			replace hhiiage=hhiiage/52 if hhiiageunit=="WEEKS"
			replace hhiiage=hhiiage/12 if hhiiageunit=="MONTHS"
			drop hhiiageunit
		
		gen age=round(hhiiage)
		replace age=1 if age==0
	
		list hhiif if hhiiage==.
			codebook hhiiage if hhiiage<13
				replace age=6 if hhiiage==. & hhiif=="A"
			codebook hhiiage if hhiiage>=13 & hhiiage<=18
				replace age=15 if hhiiage==. & hhiif=="B"
			codebook hhiiage if hhiiage>=19 & hhiiage<=24
				replace age=21 if hhiiage==. & hhiif=="C"
			codebook hhiiage if hhiiage>=25 & hhiiage<=30
				replace age=27 if hhiiage==. & hhiif=="D"
			codebook hhiiage if hhiiage>=31 & hhiiage<=50
				replace age=38 if hhiiage==. & hhiif=="E"
			codebook hhiiage if hhiiage>=51 & hhiiage!=.
				replace age=62 if hhiiage==. & hhiif=="F"
	
			replace age=30 if id2==836 & hhiia==1
		
		tab hhiiage, m
	
		gen youth=cond(age>=13 & age<=30,1,0)
	
		tab age, g(A)
	
	*hhiig,h,i
	
		list id2 hhiie hhiihii hhiihinumber hhiihiunit hhiiii hhiiiii hhiiiiunit if hhiig==.
		encode hhiihinumber, generate(hhiihinumber2)
		drop hhiihinumber
		rename hhiihinumber2 hhiihinumber
		encode hhiiii, generate(hhiiii2)
		drop hhiiii
		rename hhiiii2 hhiiii
		replace hhiig=1 if hhiig==. & hhiihii!="" & hhiihii!="9999"
		replace hhiig=2 if hhiig==. & hhiiiii!=. & hhiiiii!=9999
		replace hhiig=0 if hhiig==.
		list id2 hhiie hhiihii hhiihinumber hhiihiunit hhiiii hhiiiii hhiiiiunit if hhiig==99
		replace hhiig=0 if hhiig==99
		
	
		gen abd=cond(hhiig>=1 & hhiig!=.,1,0) 
	
	 	codebook hhiih* hhiii*
	
		tab hhiihii 
		tab hhiiiii
			replace hhiihii="" if hhiihii=="XXXX"
			gen hhiih_yr=real(hhiihii)
			ren hhiiiii hhiii_yr
			tab hhiih_yr
			tab hhiii_yr
	
			drop hhiihii 
	
		tab hhiihiunit
		tab hhiiiiunit
	
		gen no_return=0 if abd!=.
		replace no_return=1 if hhiihiunit=="X" | hhiihiunit=="XX" | hhiiiiunit=="X" | hhiiiiunit=="XX"
	
		list id2 hhiig hhiihinumber hhiihiunit hhiih_yr hhiiii hhiiiiunit hhiii_yr if abd==0 & no_return==1
			replace hhiig=1 if no_return==1 & hhiih_yr!=.
			replace no_return=0 if no_return==1 & hhiig==0
			replace hhiihiunit="" if no_return==1 | hhiihiunit=="X"
			replace hhiihinumber=. if no_return==1
	
		tab hhiihinumber
		tab hhiihiunit
	
		list id2 abd hhiihinumber hhiihiunit hhiih_yr hhiiii hhiiiiunit hhiii_yr if hhiihinumber>1900 & hhiihinumber!=.
			replace hhiihinumber=. if hhiihinumber==1990
		list id2 abd hhiihinumber hhiihiunit hhiih_yr hhiiii hhiiiiunit hhiii_yr if hhiihinumber==0
			gen a=1 if hhiihinumber==0
			replace hhiihinumber=real(hhiihiunit) if hhiihinumber==0
			replace hhiihiunit="" if a==1	
			drop a	
		list id2 abd hhiihinumber hhiihiunit hhiih_yr hhiiii hhiiiiunit hhiii_yr if hhiihinumber==99
			replace hhiihinumber=. if hhiihinumber==99 & hhiig==0
	
		tab hhiihiunit
		list id2 abd hhiiage hhiihinumber hhiihiunit hhiih_yr hhiiii hhiiiiunit hhiii_yr if hhiihinumber!=. & hhiihiunit!="D" & hhiihiunit!="W" & hhiihiunit!="M" & hhiihiunit!="Y"
			tab hhiihiunit if hhiiage>=13 & hhiiage<=25	
			replace hhiihiunit="D" if (hhiiage>25 | hhiiage<7) & hhiihinumber!=. & hhiihiunit!="D" & hhiihiunit!="W" & hhiihiunit!="M" & hhiihiunit!="Y"
			replace hhiihinumber=. if abd==0
			tab hhiihiunit if hhiiage>=13 & hhiiage<=25 & hhiihinumber==1
			tab hhiihiunit if hhiiage>=13 & hhiiage<=25 & hhiihinumber>=1 & hhiihinumber!=.
			replace hhiihiunit="M" if hhiihinumber!=. & hhiihiunit!="D" & hhiihiunit!="W" & hhiihiunit!="M" & hhiihiunit!="Y"
	
		gen hhiih_lgth=hhiihinumber if hhiihiunit=="M"
		replace hhiih_lgth=hhiihinumber/30 if hhiihiunit=="D"
		replace hhiih_lgth=hhiihinumber/4 if hhiihiunit=="W"
		replace hhiih_lgth=hhiihinumber*12 if hhiihiunit=="Y"
	
		list id2 abd hhiihinumber hhiihiunit hhiih_yr hhiiii hhiiiiunit hhiii_yr if hhiih_lgth==. & no_return!=1 & hhiihiunit!=""
			replace hhiihiunit="" if abd==0 & hhiihinumber==.
			replace hhiih_yr=. if abd==0 & hhiihinumber==.
			replace hhiih_lgth=1 if hhiih_lgth==. & hhiihiunit=="D"
			*! fix these unknowns?
	
		list id2 hhiig hhiihinumber hhiihiunit hhiih_yr hhiiii hhiiiiunit hhiii_yr if (hhiig<=1 | hhiig==.) & (hhiiii!=. | hhiiiiunit!="" | hhiii_yr!=.)
			replace hhiiii=. if abd==0 & hhiiii==0 & hhiihinumber==.
			replace hhiiii=. if abd==1 & hhiiii==0 & hhiii_yr==.
			replace hhiiii=. if abd==0 & hhiiii==9 & hhiihinumber==.
			replace hhiiiiunit="" if abd==0 & hhiiiiunit=="9" & hhiihinumber==.
			replace hhiii_yr=. if abd==0 & hhiii_yr==9999 & hhiihinumber==.
			replace hhiiiiunit="" if abd==1 & hhiiiiunit=="11" & hhiiii==.
			replace hhiig=2 if (hhiig==1 | hhiig==.) & (hhiiii!=. & hhiiiiunit!="" & hhiii_yr!=.)
			replace no_return=1 if hhiiiiunit=="X"
	
		tab hhiiii
		tab hhiiiiunit
	
		gen hhiii_lgth=hhiiii if hhiiiiunit=="M"
		replace hhiii_lgth=hhiiii/30 if hhiiiiunit=="D"
		replace hhiii_lgth=hhiiii/4 if hhiiiiunit=="W"
		replace hhiii_lgth=hhiiii*12 if hhiiiiunit=="Y"
	
		list id2 abd hhiiage hhiihinumber hhiihiunit hhiih_yr hhiiii hhiiiiunit hhiii_yr if hhiii_lgth==. & no_return!=1 & hhiiiiunit!=""
		replace hhiii_lgth=.25 if hhiii_lgth==. & no_return!=1 & hhiiiiunit!=""
		
		list id2 hhiif hhiig hhiih_lgth hhiihiunit hhiih_yr hhiii_lgth hhiiiiunit hhiii_yr if (hhiig==0 | hhiig==.) & (hhiih_lgth!=. | hhiih_yr!=.)
		list id2 hhiif hhiig hhiih_lgth hhiihiunit hhiih_yr hhiii_lgth hhiiiiunit hhiii_yr if (hhiig<=1 | hhiig==.) & (hhiii_lgth!=. | hhiii_yr!=.)
			replace hhiig=2 if hhiig<=1 & hhiih_yr!=. & hhiii_yr!=.
	
		list id2 hhiif hhiig hhiih_lgth hhiihiunit hhiih_yr hhiii_lgth hhiiiiunit hhiii_yr if hhiih_yr==9999
		list id2 hhiif hhiig hhiih_lgth hhiihiunit hhiih_yr hhiii_lgth hhiiiiunit hhiii_yr if hhiii_yr==9999
			replace hhiih_yr=. if hhiih_yr==9999
			replace hhiii_yr=. if hhiii_yr==9999
	
		replace abd=cond(hhiig>=1 & hhiig!=.,1,0) 
	
		drop hhiihinumber hhiihiunit hhiiiiunit
	
		list id2 hhiig hhiiage hhiih_l hhiih_yr hhiii_l hhiii_yr if hhiih_yr>hhiii_yr & hhiih_yr!=.
			gen a=hhiii_l
			gen b=hhiii_yr
			gen c=1 if hhiih_yr>hhiii_yr & hhiih_yr!=.
			replace hhiii_l=hhiih_l if c==1
			replace hhiii_yr=hhiih_yr if c==1
			replace hhiih_l=a if c==1
			replace hhiih_yr=b if c==1
			drop a b c
	
	label var hhiih_yr "Year of Abduction"
	
	*hhiij,k,l
	
		codebook hhiij-hhiil
		list id2 hhiie hhiif hhiij-hhiil  if (hhiij==. | hhiik==. | hhiil==.) & youth==1 & died==0
		list id2 hhiie hhiif hhiij-hhiil  if (hhiij!=. & hhiik!=. & hhiil==.) & youth==1 & died==0
		list id2 hhiie hhiif hhiij-hhiil  if (hhiij==. & hhiik!=. & hhiil!=.) & youth==1 & died==0
		list id2 hhiie hhiif hhiij-hhiil  if hhiik==. & (hhiij!=. | hhiil!=.) & youth==1 & died==0
		list id2 hhiie hhiif hhiij-hhiil  if (hhiij!=. | hhiik!=. | hhiil!=.) & youth==0
	
			replace hhiij=. if youth==0
			replace hhiik=. if youth==0
			replace hhiil=. if youth==0
	
			replace hhiij=. if hhiij==99
			replace hhiik=. if hhiik==99
			replace hhiil=. if hhiil==99
	
		gen nothing=cond(hhiil==1,1,0)
		replace nothing=. if hhiil==.
		gen inschool=cond(hhiil==2,1,0)
		replace inschool=. if hhiil==.
		gen army=cond(hhiil==3 | hhiil==4,1,0)
		replace army=. if hhiil==.
		gen job=cond(hhiil>=5 & hhiil<10,1,0)
		replace job=. if hhiil==.
		gen farm=cond(hhiil==5,1,0)
		replace farm=. if hhiil==.
		gen work=cond(job==1 | army==1,1,0)
		replace work=. if hhiil==.
	
		gen educ=hhiij
	
		gen incamp=cond(hhiik==1 | hhiik==2,1,0)
		replace incamp=. if hhiik==.
	
		*People who didn't return
		list hhiie hhiif hhiig if no_return==1
		
		*Sibling education
		bysort id2: egen hh_avg_educ=mean(hhiij)
		
		*War death in family
		gen war_death=cond(hhiie==3 | hhiie==4 | hhiie==5,1,0)
		bysort id2: egen hh_war_death=sum(war_death)
	
		*HH abductions
		bysort id2: egen hh_abd_total=sum(abd)
		gen abd_youth=cond(abd==1 & youth==1,1,0)
		bysort id2: egen hh_abd_youth=sum(abd_youth)
		
	cap drop _merge
	
	replace hhiik=5 if no_return==1
	replace hhiik=11 if no_return==0 & died==1
	replace hhiik=. if age<13 | age>30
	drop incamp
	gen incamp=cond(hhiik==1 | hhiik==2,1,0)
	replace incamp=. if youth!=1
	
	gen abd_age1=age-2005+hhiih_yr
	replace abd_age1=1 if abd_age1<1
	
	gen abd_age2=age-2005+hhiii_yr
	replace abd_age2=1 if abd_age2<1
	list abd_age1 hhiih_l hhiih_yr abd_age2 hhiii_l hhiii_yr if abd_age1>abd_age2 & hhiih_yr!=.
	
	gen female=cond(male==0,1,0)
	gen nonabd=cond(abd==0,1,0)
	
	label define gender 0 "Female" 1 "Male"
	label values male gender
	
	gen abd_age_cat=0
	replace abd_age_cat=1 if abd_age1<=4
	replace abd_age_cat=2 if abd_age1>=5 & abd_age1<=9
	replace abd_age_cat=3 if abd_age1>=10 & abd_age1<=17
	replace abd_age_cat=4 if abd_age1>=18 & abd_age1<=30
	replace abd_age_cat=5 if abd_age1>=30 & abd_age1!=.
	replace abd_age_cat=6 if abd==1 & abd_age1==.
	
	replace abd_age_cat=1 if abd_age2<=4 & abd_age_cat==6
	replace abd_age_cat=2 if abd_age2>=5 & abd_age2<=9 & abd_age_cat==6
	replace abd_age_cat=3 if abd_age2>=10 & abd_age2<=17 & abd_age_cat==6
	replace abd_age_cat=4 if abd_age2>=18 & abd_age2<=30 & abd_age_cat==6
	replace abd_age_cat=5 if abd_age2>=30 & abd_age2!=. & abd_age_cat==6
	replace abd_age_cat=6 if abd==1 & abd_age2==. & abd_age_cat==6
	
	label define age_cat 1 "0-4" 2 "5-9" 3 "10-17" 4 "18-30" 5 "31+" 6 "DK"
	label values abd_age_cat age_cat
	
	gen abd_lgth=hhiih_l
	replace abd_lgth=abd_lgth+hhiii_l if hhiii_l!=.
	
	gen abd_per=1 if abd==1
	replace abd_per=2 if abd_lgth>.07
	replace abd_per=3 if abd_lgth>.5
	replace abd_per=4 if abd_lgth>2
	replace abd_per=5 if abd_lgth>12
	replace abd_per=7 if abd==1 & abd_lgth==.
	replace abd_per=6 if no_return==1
	replace abd_per=0 if abd==0
	
	label define period 0 "Never" 1 "<3D" 2 "3-14D" 3 "2W-2M" 4 "2-12M" 5 ">12M" 6 "DNR" 7 "DK"
	label values abd_per period
	
	tab abd_age1 if abd==1, m
	list age hhiih_l hhiih_yr abd_age2 hhiii_l hhiii_yr if abd==1 & abd_age_cat==6
	
	replace abd_age_cat=1 if abd_age_cat==6 & age<=6
	replace abd_age_cat=2 if abd_age_cat==6 & age>6 & age<=11
	replace abd_age_cat=3 if abd_age_cat==6 & age>=15 & age<=19
	replace abd_age_cat=3 if abd_age_cat==6 & age>=12 & age<=14
	replace abd_age_cat=3 if abd_age_cat==6 & age>=20 & age<=24 & abd_lgth>=9 & abd_lgth!=.
	replace abd_age_cat=3 if abd_age_cat==6 & age>=26 & age<=36 & abd_lgth>=9 & abd_lgth!=.
	replace abd_age_cat=3 if abd_age_cat==6 & abd_lgth>1 & abd_lgth<12 & age>18 & age<22
	replace abd_age_cat=4 if abd_age_cat==6 & age>=23 & age<=32 & (abd_lgth<6 | abd_lgth==.)
	replace abd_age_cat=4 if abd_age_cat==6 & age>=20 & age<=22 & abd_lgth<1
	replace abd_age_cat=4 if abd_age_cat==6 & age>=26 & age<=36 & abd_lgth<36
	replace abd_age_cat=5 if abd_age_cat==6 & age>37 & age!=. & abd_lgth<3
	replace abd_age_cat=4 if abd_age_cat==6 & age>30 & age!=. & abd_lgth>3 & abd_lgth!=.
	replace abd_age_cat=4 if abd_age_cat==5 & age<35 & abd_lgth>3 & abd_lgth!=.
	replace abd_age_cat=3 if abd_age_cat==6 & abd_lgth>=36 & abd_lgth!=.
	replace abd_age_cat=3 if abd_age_cat==6 & age==20
	replace abd_age_cat=3 if abd_age_cat==6 & age==22
	replace abd_age_cat=3 if abd_age_cat==6 & age==.
	replace abd_age_cat=4 if abd_age_cat==6 & age==21
	replace abd_age_cat=5 if abd_age_cat==6 & age>=35 & age!=.
	
	
	gen abd_2w=cond(abd==1 & abd_per>=3 & abd_per!=7 & abd_age_cat>=2 & abd_age_cat<=4,1,0)
	gen abd_under30=cond(abd==1 & abd_age_cat>=2 & abd_age_cat<=4,1,0)
	
	gen abd_times=0
	replace abd_times=1 if hhiig==1
	replace abd_times=2 if hhiig==2
	replace abd_times=3 if hhiig>2 & hhiig!=.
	label define times 0 "Never" 1 "1x" 2 "2x" 3 ">3x"
	label values abd_times times
	
	
	ren abd_lgth abd_lgth_hh
	
	
	aorder
	order id2 hhi*
	save "SWAYI/HHsurvey_sectionII_27May2008.dta", replace
	
	
	
	keep id2 hh_adults05 hh_size05 hh_size96 hh_abd_total hh_abd_youth hh_avg_educ hh_war_death hh_select
	duplicates drop
	sort id2
	
	merge id2 using "SWAYI/HHsurvey_sectionI_27May2008.dta"
	drop _merge
	aorder
	order id2 hhi* hh_*
	sort id2
	
	save "SWAYI/tempdata/SWAY cleaning HH.dta", replace


 ******************** FINISHED CLEANING *************************


** MERGE INDIVIDUAL WITH HH DATA

	insheet using "SWAYI/rawdata/matching.I.complete.csv", comma names clear
	sort id
	save "SWAYI/tempdata/matching.dta", replace

	use "SWAYI/HHsurvey_sectionII_27May2008.dta", clear
	keep if age<31
	keep id2 hhii* abd_lgth_hh
	sort id2 hhiia
	save "SWAYI/tempdata/temp.dta", replace

	use "SWAYI/tempdata/SWAY cleaning - I survey.dta", clear
	ren i1_hh hhi1
	sort id
	merge id using "SWAYI/tempdata/matching"
	tab _merge
	sort id
	list id id2 district camp parish village household person ii2 i5aday i5amonth i5ayear if id2==.
	drop _merge dup

	sort id2
	merge id2 using "SWAYI/tempdata/SWAY cleaning HH.dta"
	drop if _merge==2
	drop _merge

	sort id2 hhiia
	merge id2 hhiia using "SWAYI/tempdata/temp.dta"
	drop if _merge==2

	replace abd_lgth_hh=0 if hhiig==0

	drop _merge


** CLEAN UP

// Drop and adjust ages outside range
	drop if age>30
	replace age=14 if age==13

order i1-i8 ii* iii* iv* v* vi* vii* viii* ix* x* xi* xii*  hh* C_*
order id id2 district camp parish block village household person abd
drop CHECK_AGE MISSING_TRAUMA NO_SCHOOL_DATA abd2 abd3 missing_id yqnno

save "SWAYI/tempdata/sway_temp", replace


** MERGE NON-SURVIVOR AND ABSENTEE DATA

// Bring in absentees
	cap drop _merge

	insheet using "SWAYI/rawdata/absentee.txt", clear
	ren id a_id
	la var a_id "ID including absentees"

	drop dist-block
	sort a_id
	save "SWAYI/tempdata/absentee_data.dta", replace

	insheet using "SWAYI/rawdata/matching.A.complete.csv", clear
	drop if id2==.
	sort id2 hhiia
	save "SWAYI/tempdata/absentee.dta", replace

	use "SWAYI/HHsurvey_sectionII_27May2008.dta", clear
	sort id2 hhiia
	merge id2 hhiia using "SWAYI/tempdata/absentee"
	tab _merge

	gen absentee=cond(a_id!=.,1,0)

	drop _merge d c p v hh lgth_num lgth_unit

// Identify non-survivors from HH survey

	keep if male==1 & age<=30 & camp!=.
	keep if (no_return==1 | died==1 | absentee==1)
	drop if died==1 & no_return==0 & age<8

	replace hh_weight=18 if hh_weight==.

	* Interpolate ages and years
		gen ix2ayear=hhiih_yr
		replace ix2ayear=hhiii_yr if hhiii_yr!=.
		gen randvar=uniform() * 8
		replace randvar=round(randvar)
		replace ix2ayear=1996+randvar if ix2ayear==. & abd==1
		gen age_=age+(age-abd_age1) if no_return==1
		replace age_=age+(age-abd_age2) if abd_age2!=. & no_return==1
		replace age_=age+randvar if age_==. & no_return==1
		replace abd_age1=age if no_return==1 & abd_age1==.
		replace age=age_ if no_return==1
		drop if age<14 & no_return==1
		drop if age>30 | age<14

	* ren vars
		ren hhiie causedeath
		la var causedeath "Cause of death"
		ren abd_times ix1
		sort id2
		gen id=1000+_n
		sort a_id
		replace id=2000+_n if absentee==1
		keep id id2 a_id absentee hhi8 hhi8_log hhi9 hhi10 hhi11 camp C_ach C_akw C_ata C_kma C_oro C_pad C_paj C_pal died causedeath abd abd_age1 abd_age2 ix2ayear age asset_50th asset_75th asset_90th camp_yrs fthr_frm fthr_pro fthr_sol hh_abd_total hh_abd_youth hh_adults05 hh_assets96 hh_avg_educ hh_select hh_size05 hh_size96 hh_wealth96 hh_weight parish village no_return w10 w25 w75 w90 hhii*

	sort id
	save "SWAYI/tempdata/noreturn.dta", replace


// Merge non-survivors

use "SWAYI/tempdata/sway_temp", clear
	sort id
	merge id using "SWAYI/tempdata/noreturn.dta"
	drop _merge
	replace no_return=0 if id<1000 | id>=2000
	replace died=0 if no_return==1
	gen sample=cond(id<1000 | abs==1,1,0)
	gen found=cond(id<1000,1,0)
	la var sample "Equals 1 if in survey sample"
	la var found "Equals 1 if youth found and interviewed"

	replace abd_lgth_hh=0 if hhiig==0
	replace abd_lgth_hh=hhiih_l if hhiig>0
	replace abd_lgth_hh=abd_lgth_hh+hhiii_l if hhiii_l!=.
	replace abd_lgth_hh=abd_lgth if abd_lgth_hh==.


// Create Strata and pweights

	gen stratum=1 if abd_lgth_hh==0 | hhiig==0
	replace stratum=2 if abd_lgth_hh>0 & abd_lgth_hh<.25
	replace stratum=3 if abd_lgth_hh>=.25 & abd_lgth_hh!=.
	replace stratum=4 if no_return==1
	replace stratum=5 if died==1

	gen i_weight=1
	replace i_weight=2 if stratum==2 & camp==2
	replace i_weight=2.5 if stratum==2 & camp==3
	replace i_weight=2.644 if stratum==1 & camp==1
	replace i_weight=3.311 if stratum==1 & camp==2
	replace i_weight=3.333 if stratum==1 & camp==3
	replace i_weight=3.478 if stratum==1 & camp==4
	replace i_weight=2.667 if stratum==1 & camp==5
	replace i_weight=2.881 if stratum==1 & camp==6
	replace i_weight=2.774 if stratum==1 & camp==7
	replace i_weight=2.86 if stratum==1 & camp==8

	la var stratum "Survey Sampling Strata"
	la var i_weight "Sampling Weights: Individual Survey"
	la var hh_weight "Sampling Weights: Household Survey"
	la var hh_select "Household Probability of Sample Selection"

// Merge absentee data
	
	sort a_id
	merge a_id using "SWAYI/tempdata/absentee_data"
	drop if _merge==2
	drop _merge

	replace absentee=0 if absentee!=1
	la var absentee "Equals 1 if absentee survey conducted"

	replace abd_lgth=abd_lgth_hh if abd_lgth==.
	replace ix1=q21 if q21<99 & absentee==1
	replace ix1=hhiig if ix1==.
	replace abd_age_a=abd_age1 if abd_age_a==.
	replace abd_age_b=abd_age2 if abd_age_b==.
	replace abd_lgth_a=hhiih_l if absentee==1
	replace abd_lgth_b=hhiii_l if absentee==1
	replace ix2ayear=hhiih_yr if absentee==1
	replace ix2byear=hhiii_yr if absentee==1


//  Non-survivors
	gen nosurvive=cond(died==1 | no_return==1,1,0)
	replace causedeath=3 if no_return==1
	la var nosurvive "Did not survive"

** CREATE AGE TERMS

	drop age_* abd_age1 abd_age2

    foreach x of numlist 14/35 {
        gen A`x'=cond(age==`x',1,0) if age!=.
        la var A`x' "Age indicator: `x'"
        }

	foreach x of numlist 1/8 {
		gen G`x'_14_17=cond(age>=14 & age<=17 & camp==`x',1,0)
		gen G`x'_18_21=cond(age>=18 & age<=21 & camp==`x',1,0)
		gen G`x'_22_25=cond(age>=22 & age<=25 & camp==`x',1,0)
		gen G`x'_26_30=cond(age>=26 & age<=30 & camp==`x',1,0)
		}

	gen age_cat=.
	replace age_cat=1 if age>=14 & age<=16 
	replace age_cat=2 if age>=17 & age<=19 
	replace age_cat=3 if age>=20 & age<=22 
	replace age_cat=4 if age>=23 & age<=25
	replace age_cat=5 if age>=26 & age<=28
	replace age_cat=6 if age>=29 & age<=30 
	label define ages 1 "14-16" 2 "17-19" 3 "20-22" 4 "23-25" 5 "26-28" 6 "29-30"
	label values age_cat ages
      la var age_cat "Age categories"


** IMPUTE MISSING HH VARIABLES

  foreach v of varlist hh_assets96 hh_wealth96 hh_size96 hhi8_log fthr_frm {
	reg `v' C_ach-C_paj fthr_ed mthr_ed no_fthr96 no_mthr96 catholic aindex educ A* [pweight=i_weight]
		predict Q`v', xb
		replace `v'=Q`v' if `v'==.
		drop Q`v'
		}
	replace fthr_frm=1 if fthr_frm>0 & fthr_frm<1
	replace fthr_sol=0 if fthr_frm==1
	replace fthr_pro=0 if fthr_frm==1

	replace asset_50th=cond(hh_assets96<-.38,1,0) if asset_50th==.
	replace asset_75th=cond(hh_assets96>.08,1,0) if asset_75th==.
	replace asset_90th=cond(hh_assets96>.79,1,0) if asset_90th==.

	replace w10=cond(hh_wealth96<-.62,1,0) if w10==.
	replace w25=cond(hh_wealth96<-.5,1,0) if w25==.
	replace w75=cond(hh_wealth96>.12,1,0) if w75==.
	replace w90=cond(hh_wealth96>.85,1,0) if w90==.

// Rename, create, and label HH vars

	ren hhi8_log log_land 

	gen hh_land=hhi8
	replace hh_land=0 if hh_land==.

	gen hh_cattle=hhi9
	replace hh_cattle=0 if hh_cattle==. | hh_cattle<0
	gen hh_stock=hhi10
	replace hh_stock=0 if hh_stock==.

	replace hhi11=0 if hhi11==99 | hhi11==.
	gen hh_plow=cond(hhi11>0,1,0)	

	ren fthr_frm hh_fthr_frm
	ren fthr_so hh_fthr_sol
	ren fthr_pro hh_fthr_pro
	la var hh_land "HH land (acres)"
	la var hh_cattle "HH cattle"
	la var hh_stock "HH livestock"
	la var hh_plow "HH plows"
	la var hhi8_small "Equals 1 if land holdings <2 katala/acres"

** ABDUCTION VARIABLES

// Replace abduction values with longest abduction

	gen abd_age=abd_age_a
	gen abd_l=abd_lgth_a
	gen abd_yr=ix2ayear
	gen yr_ret=ix14ayear
	gen longest_abd=1
	replace longest_abd=2 if ix1>1 & abd_lgth_b>abd_lgth_a & abd_age_b!=.
	replace longest_abd=3 if ix1>1 & abd_lgth_c>abd_lgth_a & abd_lgth_c>abd_lgth_b & abd_age_c!=.
	replace abd_age=abd_age_b if longest==2
	replace abd_yr=ix2byear if longest==2
	replace yr_ret=ix14byear if longest==2
	replace abd_l=abd_lgth_b if longest==2
	replace abd_age=abd_age_c if longest==3
	replace abd_yr=ix2cyear if longest==3
	replace yr_ret=ix14cyear if longest==3
	replace abd_l=abd_lgth_c if longest==3
	replace abd_l=abd_lgth_hh if abd_l==. & abd==1 

	list id longest_ abd_l ix2ayear ix14ayear ix2byear ix14byear ix2cyear ix14cyear if yr_ret==. & abd==1
	replace yr_ret=2002 if yr_ret==1920
	replace yr_ret=2003 if id==571
	replace yr_ret=2003 if id==734
	replace yr_ret=2004 if id==148
	replace yr_ret=1999 if id==638

la var abd_age "Age at longest abduction"
la var abd_l "Length of longest abduction"
la var abd_yr "Year of longest abduction"
la var yr_ret "Year of return from longest abduction"
la var longest_abd "Longest abduction"

* Times abducted

	gen nonabd=cond(abd==0,1,0)
	gen abd2=cond(ix1>1 & ix1!=., 1, 0) if ix1!=.
	replace abd2=cond(hhiig>1 & hhiig!=., 1, 0) if hhiig!=. & found==.
	gen abd3=cond(ix1>2 & ix1!=., 1, 0) if ix1!=.
	replace abd3=cond(hhiig>2 & hhiig!=., 1, 0) if hhiig!=. & found==.
	la var abd "Ever abducted"
	la var abd2 "Abducted at least twice"
	la var abd3 "Abducted at least three times"
	la var nonabd "Never abducted"

* Abduction length dummies and vars 

	gen abd_l_1d=cond(abd_l>0 & abd_l<.04,1,0)
	gen abd_l_2d=cond(abd_l>.04 & abd_l<.07,1,0)
	gen abd_l_3to6d=cond(abd_l>=.07 & abd_l<.25,1,0)
	gen abd_l_1w=cond(abd_l==.25,1,0)
	gen abd_l_2to4w=cond(abd_l>.25 & abd_l<1,1,0)
	gen abd_l_1=cond(abd_l>=1 & abd_l<2,1,0)
	gen abd_l_2=cond(abd_l>=2 & abd_l<3,1,0)
	gen abd_l_3=cond(abd_l>=3 & abd_l<4,1,0)
	gen abd_l_4=cond(abd_l>=4 & abd_l<5,1,0)
	gen abd_l_5to6=cond(abd_l>=5 & abd_l<=6,1,0)
	gen abd_l_7to11=cond(abd_l>6 & abd_l<12,1,0)
	gen abd_l_12to17=cond(abd_l>=12 & abd_l<18,1,0)
	gen abd_l_18to24=cond(abd_l>=18 & abd_l<=24,1,0)
	gen abd_l_25to30=cond(abd_l>24 & abd_l<=30,1,0)
	gen abd_l_30to36=cond(abd_l>30 & abd_l<=36,1,0)
	gen abd_l_36to50=cond(abd_l>36 & abd_l<=50,1,0)
	gen abd_l_50up=cond(abd_l>50 & abd_l!=.,1,0)

	gen abd_l2_1w=cond(abd_l>0 & abd_l<=.25,1,0)
	gen abd_l2_2m=cond(abd_l>.25 & abd_l<=2,1,0)
	gen abd_l2_12m=cond(abd_l>2 & abd_l<12,1,0)
	gen abd_l2_12up=cond(abd_l>=12 & abd_l!=.,1,0)

	la var abd_l_1d "Abduction length: 1 day"
	la var abd_l_2d "Abduction length: 2 days"
	la var abd_l_3to6d "Abduction length: 3-6 days"
	la var abd_l_1w "Abduction length: 1 week"
	la var abd_l_2to4w "Abduction length: 2-4 weeks"
	la var abd_l_1 "Abduction length: 1 month"
	la var abd_l_2 "Abduction length: 2 months"
	la var abd_l_3 "Abduction length: 3 months"
	la var abd_l_4 "Abduction length: 4 months"
	la var abd_l_5to6 "Abduction length: 5-6 months"
	la var abd_l_7to11 "Abduction length: 7-11 months"
	la var abd_l_12to17 "Abduction length: 12-17 months"
	la var abd_l_18to24 "Abduction length: 18-24 months"
	la var abd_l_25to30 "Abduction length: 25-30 months"
	la var abd_l_30to36 "Abduction length: 30-36 months"
	la var abd_l_36to50 "Abduction length: 36-50 months"
	la var abd_l_50up "Abduction length: 50+ months"
	la var abd_l2_1w "Abduction length: < or = 1 week"
	la var abd_l2_2m "Abduction length: 1 week-2 months"
	la var abd_l2_12m "Abduction length: 2-12 months"
	la var abd_l2_12up "Abduction length: 12+ months"

	gen labd_l=ln(abd_l)
	gen labd_lgth_a=ln(abd_lgth_a)
	gen labd_lgth=ln(abd_lgth)
	la var labd_lgth_a "Log of length of longest abduction"

      gen lt_abd=cond(abd==1 & abd_lgth>=8 & abd_lgth!=.,1,0)
      la var lt_abd "=1 if abducted for >= 8 months"

	gen rescue=cond(ix16a==2,1,0)
      lab var rescue "=1 if respondent was rescued during 1st abduction"

	replace abd_l=0 if abd==0

	gen twoweek=cond(ix32!=. | ix42!=. | ix45!=.,1,0) if abd==1
	la var twoweek "Equals 1 if responded to IX32 to IX91 (i.e. abducted more than two weeks)"

*Abduction Age

	gen abdage=round(abd_age)
	gen abd_child=cond(abd==1 & abd_age<18,1,0)
	gen abd_adult=cond(abd==1 & abd_age>=18,1,0)

	la var abdage "Age of 1st abduction"
	la var abd_child "Equals 1 if abducted before the age of 18"
	la var abd_adult "Equals 1 if abducted at or after the age of 18"

	gen abd5_12=cond(abd_age<=12,1,0)
	gen abd13_17=cond(abd_age>=13 & abd_age<=17,1,0)
	gen abd18_21=cond(abd_age>=18 & abd_age<=21,1,0)
	gen abd22_30=cond(abd_age>=22 & abd_age<=30,1,0)
	
	la var abd5_12 "Abducted between ages 5 and 12"
	la var abd13_17 "Abducted between ages 13 and 17"
	la var abd18_21 "Abducted between ages 18 and 21"
	la var abd22_30 "Abducted between ages 22 and 30"

* Abduction year dummies

	gen yr_abd=abd_yr

	replace abd_yr=2004 if ix2ayear==2005
	replace abd_yr=1994 if ix2ayear<=1994
	foreach n of numlist 1994/2004 {
		quietly gen Y`n'=cond(abd_yr==`n',1,0)
		}
	
* Abuduction unit (year/location) dummies
	foreach y of varlist C_ach-C_pal {
		foreach x of numlist 1994/2004 {	
			quietly gen U`y'`x'=Y`x'*`y'
			}
		}

la var yr_abd "Year abucted 1989-2005"
la var Y1994 "Abduction year indicator: 1994"
la var Y1995 "Abduction year indicator: 1995"
la var Y1996 "Abduction year indicator: 1996"
la var Y1997 "Abduction year indicator: 1997"
la var Y1998 "Abduction year indicator: 1998"
la var Y1999 "Abduction year indicator: 1999"
la var Y2000 "Abduction year indicator: 2000"
la var Y2001 "Abduction year indicator: 2001"
la var Y2002 "Abduction year indicator: 2002"
la var Y2003 "Abduction year indicator: 2003"
la var Y2004 "Abduction year indicator: 2004"

//Orphan/Single parent at time of abduction

    gen lost_fthr_yr=ii3 if no_fthr==1
    gen lost_fthr_age=age-(2006-lost_fthr_yr) if no_fthr==1
    replace lost_fthr_age=0 if lost_fthr_age<0
    
    recode ii7 (2999=2006)
    gen lost_mthr_yr=ii7 if no_mthr==1
    gen lost_mthr_age=age-(2006-lost_mthr_yr) if no_mthr==1
    replace lost_mthr_age=0 if lost_mthr_age<0

	gen abd_no_mthr=0 if abd==1 & no_mthr==0 & abd!=. & no_mthr!=.
	replace abd_no_mthr=0 if abd==1 & no_mthr==1 & abd_age<=lost_mthr_age & abd!=. & abd_age!=. & lost_mthr_age!=.
	replace abd_no_mthr=1 if abd==1 & no_mthr==1 & abd_age>lost_mthr_age & abd!=. & abd_age!=. & lost_mthr_age!=.
	replace abd_no_mthr=1 if abd==1 & no_mthr==1 & abd_no_mthr==. & yr_abd>=2000
	replace abd_no_mthr=1 if abd==1 & no_mthr==1 & abd_no_mthr==. & (ii8==. | ii9==.)

	gen abd_no_fthr=0 if abd==1 & no_fthr==0 & abd!=. & no_fthr!=.
	replace abd_no_fthr=0 if abd==1 & no_fthr==1 & abd_age<=lost_fthr_age & abd!=. & abd_age!=. & lost_fthr_age!=.
	replace abd_no_fthr=1 if abd==1 & no_fthr==1 & abd_age>lost_fthr_age & abd!=. & abd_age!=. & lost_fthr_age!=.
	replace abd_no_fthr=1 if abd==1 & no_fthr==1 & abd_no_fthr==. & yr_abd>=2000
	replace abd_no_fthr=1 if abd==1 & no_fthr==1 & abd_no_fthr==. & (ii4==. | ii5==.)

	gen abd_sing_par=0 if abd==1 & no_fthr==0 & no_mthr==0 & abd!=. & no_mthr!=. & no_fthr!=.
	replace abd_sing_par=0 if abd==1 & abd_no_mthr==0 & abd_no_fthr==0 & abd!=. & abd_no_mthr!=. & abd_no_fthr!=.
	replace abd_sing_par=1 if abd==1 & (abd_no_mthr==1 | abd_no_fthr==1) & abd!=. 

	gen abd_orph=0 if abd==1 & no_fthr==0 & no_mthr==0 & abd!=. & no_mthr!=.& no_fthr!=.
	replace abd_orph=0 if abd==1 & abd_no_mthr==0 & abd_no_fthr==0 & abd!=. & abd_no_mthr!=. & abd_no_fthr!=.
	replace abd_orph=0 if abd==1 & abd_sing_par==1 & abd!=. & abd_sing_par!=.
	replace abd_orph=1 if abd==1 & abd_no_mthr==1 & abd_no_fthr==1 & abd!=. & abd_no_mthr!=. & abd_no_fthr!=.

	replace abd_orph=1 if abd==1 & abd_yr>=2002 & no_fthr==1 & no_mthr==1 & ii3==. & ii7==.

	drop lost_mthr_age lost_mthr_yr lost_fthr_age lost_fthr_yr

	la var abd_no_mthr "Equals 1 if no mother at time of abduction"
	la var abd_no_fthr "Equals 1 if no father at time of abduction"
	la var abd_sing_par "Equals 1 if single parent at time of abduction"
	la var abd_orph "Equals 1 if orphan at time of abduction"
	
// Ceremony performed for abductee?

	gen ceremony=0
	replace ceremony=1 if longest_abd==1 & ix10a==1
	replace ceremony=1 if longest_abd==2 & ix10b==1
	replace ceremony=1 if longest_abd==2 & ix10c==1
	la var ceremony "Given ceremony during longest-lasting abduction?"


** OTHER ABDUCTION EXPERIENCES **

// Guns

	gen gun=cond(ix32==1,1,0) if twoweek==1
	la var gun "Equals 1 if ever given a gun while with LRA"
	
	gen gun_keep=cond(ix35==1,1,0) if twoweek==1
	la var gun_keep "Equals 1 if allowed to keep/sleep with the gun"

	gen gun_mo=ix34
	la var gun_mo "Months before allowed to keep a gun"


// Attitudes & Experiences

	gen depend=cond(ix36==1,1,0) if twoweek==1
	gen rewarded=cond(ix38==1,1,0) if twoweek==1
	gen safer=cond(ix45==1,1,0) if twoweek==1
	gen magic=cond(ix47==1,1,0) if twoweek==1
	gen loyal=cond(ix48==1,1,0) if twoweek==1
	gen loyal_often=cond(ix49==1,1,0) if twoweek==1
	gen loyal_now=cond(ix50==1,1,0) if twoweek==1
	gen stay=cond(ix51==1,1,0) if twoweek==1
	gen important=cond(ix61==1,1,0) if twoweek==1
	gen command=cond(ix53==1,1,0) if twoweek==1

	la var depend "Considered someone leaders could depend on"
	la var rewarded "Ever rewarded for job well done"
	la var safer "Ever felt safer inside LRA than at home"
	la var magic "Ever felt had magical protection from bullets while with LRA"
	la var loyal "Ever felt allegiance to Kony or LRA"
	la var loyal_often "Often felt allegiance to Kony or LRA"
	la var loyal_now "Feel any allegiance to Kony or the LRA now"
	la var stay  "Ever a time when preferred to stay with LRA than escape"
	la var important "Ever felt like an important part of unit" 
	la var command "Ever wanted to be an LRA commander one day"

// Threats and indoctrination
	gen threats=cond(ix541==1 | ix542==1 | ix543==1 | ix544==1 | ix545==1 | ix547=="",1,0) if twoweek==1
	replace threats=0 if id==10 | id==81 | id==158 | id==163 | id==166 | id==202 | id==203 | id==242 | id==268 | id==277 | id==281 | id==351 | id==398 | id==418 | id==611
	replace threats=. if twoweek==0
	la var threats "Received threats"

	gen indoctrinate=cond(ix546==1 | id==17 | id==28 | id==68 | id==69 | id==80 | id==81 | id==94 | id==122 | id==149 | id==154 | id==158 | id==160 | id==163 | id==166 | id==202 | id==203 | id==213 | id==222 | id==242 | id==268 | id==277 | id==281 | id==351 | id==360 | id==398 | id==418 | id==478 | id==491 | id==497 | id==566 | id==611 | id==618 | id==716 | id==749,1,0) if twoweek==1
	la var indoctrinate "Received propaganda"

	gen threats_only=cond(threats==1 & indoc==0,1,0) if twoweek==1
	la var threats_only "Received threats but no indoctrination"

//  Allegiance variable
    foreach x of varlist ix45-ix48 ix51 ix53 {
        replace `x'=0 if `x'==2
        replace `x'=1 if `x'==3
        }

    ren ix45 ix45_safer
    ren ix47 ix47_magical
    ren ix48 ix48_allegiance
    ren ix51 ix51_stay
    ren ix53 ix53_commander

    gen allegiance_total=ix45_safer+ix48_allegiance+ix51_stay if twoweek==1
    gen allegiance_modhi=cond(allegiance_total>0,1,0) if allegiance_total!=.
    la var allegiance_total "Elements of allegiance reported 0-3"
    la var allegiance_modhi "Equals 1 if allegiance_total>0" 

// Leadership 

	gen lead=cond(ix42==1,1,0) if twoweek==1
	gen ranked=cond(ix43>=1 & ix43<=9,1,0)  if twoweek==1
	gen leader=cond(ranked==1 | lead==1,1,0) if twoweek==1 
	
	la var lead "Equals 1 if ever led other soldiers"
	la var ranked "Equals 1 if held rank in LRA"
	la var leader "Equals 1 if held rank in LRA or led other soldiers"

	*Why promoted?
		replace ix44a1=1 if id==149	
		replace ix44a2=1 if id==237
		replace ix44a4=1 if id==102 | id==267 | id==286 | id==300 | id==551
		replace ix44a7=1 if id==531
		replace ix44a99=0 if id==531
		replace ix44a7=0 if id==149 | id==237 | id==102 | id==267 | id==286 | id==300 | id==551
		la var ix44a1 "Obedient"
		la var ix44a2 "Intelligent"
		la var ix44a3 "Strong"
		la var ix44a4 "Good Soldier"
		la var ix44a5 "Listened to"
		la var ix44a6 "Killed officer"
		la var ix44a7 "Other"

// Exit

	replace ix16a=3 if id==168
	replace ix16a=2 if id==183

    gen released=cond((ix16a==1 & longest_abd==1) | (ix16b==1 & longest_abd==2) | (ix16c==1 & longest_abd==3),1,0) if abd==1 & sample==1
    gen released_1w=cond(released==1 & ((abd_lgth_a<=.25 & longest_abd==1) | (abd_lgth_b<=.25 & longest_abd==2) | (abd_lgth_c<=.25 & longest_abd==3)),1,0) if abd==1 & sample==1 
    gen released_2w=cond(released==1 & ((abd_lgth_a<=.5 & longest_abd==1) | (abd_lgth_b<=.5 & longest_abd==2) | (abd_lgth_c<=.5 & longest_abd==3)),1,0) if abd==1 & sample==1 

    gen rescued=cond((ix16a==2 & longest_abd==1) | (ix16b==2 & longest_abd==2) | (ix16c==2 & longest_abd==3),1,0) if abd==1 & sample==1
    gen escaped=cond((ix16a==3 & longest_abd==1) | (ix16b==3 & longest_abd==2) | (ix16c==3 & longest_abd==3),1,0) if abd==1 & sample==1
	
    la var rescued "Equals 1 if rescued from longest abduction by LRA"
    la var escaped "Equals 1 if escaped from longest abduction"
    la var released "Respondent released when abducted"
    la var released_1w "Respondent released & held <=1 week"
    la var released_2w "Respondent released & held <=2 weeks"



** WAR TRAUMA & VIOLENCE

// Violence indices

	foreach k of numlist 2/32 {
		replace x`k'=0 if x`k'==2
		}
	replace x32=0 if x32==.
	foreach k of numlist 21/32 {
		replace x`k'=0 if x`k'==. & (abd==0 | (abd==1 & abd_lgth<.1))
		}

	replace x21=0 if id==597
	replace x7=1 if id==87
	replace x18=1 if id==87
	replace x16=1 if id==148
	replace x10=1 if id==136
	replace x2=1 if id==427
	replace x3=1 if id==427
	replace x5=1 if id==427

	factor x2 x3 x4 x5 x6 x7 x8 x9 x10 x11 x12 x13 x14 x15 x16 x17 x18 x19 x20 x21 x22 x23 x24 x25 x26 x27 x28 x29 x20 x31, means
	rotate, factors(3)

	gen viol_received=x4+x6+x7+x8+x9+x10 if found==1
	gen viol_witnessed=x3+x12+x13+x14+x15+x16 if found==1
	gen viol_experienced=x3+x4+x6+x7+x8+x9+x10+x12+x13+x14+x15+x16 if found==1
	gen viol_loss=x17+x18+x19+x20+x21 if found==1
	gen viol_personal=x3+x4+x6+x7+x8+x9+x10+x12+x13+x14+x15+x16+x17+x18+x19+x20+x21 if found==1
	gen viol_perp=x22+x23+x24+x25+x26+x27+x29+x30 if found==1
	gen viol_total=x3+x4+x6+x7+x8+x9+x10+x12+x13+x14+x15+x16+x17+x18+x19+x20+x21+x22+x23+x24+x25+x26+x27+x29+x30 if found==1

    la var viol_received "Sum of 6 violent acts received"
    la var viol_witnessed "Sum of 6 violent acts witnessed"
    la var viol_experienced "Sum of 12 violent acts received and witnessed"
    la var viol_loss "Sum of 5 violent acts inflicted on family members"
    la var viol_personal "Sum of 17 violent acts received, witnessed, or upon family"
    la var viol_perp "Sum of 8 violent acts perpetrated"
    la var viol_total "Sum of 25 violent acts received, witnessed, upon family, or perpetrated"


// Specific violent acts indicators

	gen tied=cond(x8==1,1,0) if found==1
	gen carry=cond(x10==1,1,0) if found==1
	gen kill_sol=cond(x23==1,1,0) if found==1
	gen kill=cond(x23==1 | x26==1 | x27==1,1,0) if found==1
	gen abusedead=cond(x29==1,1,0) if found==1
	gen harm_fam=cond(x24==1 | x26==1,1,0) if found==1

	la var tied "Tied or imprisoned"
	la var carry "Forced labor"
	la var kill "Ever forced to kill (soldier, civilian, friend, family)"
	la var kill_sol "Ever killed soldier(s)"
	la var harm_fam "Forced to harm family/friend (beat, cut, or kill)"	
	la var abusedead "Forced to abuse dead"


// Unit brutality

	codebook ix63a-ix68b

	foreach n of numlist 63/68 {
	gen ix`n'=0 if twoweek==1
	replace ix`n'=1 if ix`n'a==1 & ix`n'b==3
	replace ix`n'=2 if ix`n'a==1 & (ix`n'b==2 | ix`n'b==.)
	replace ix`n'=3 if ix`n'a==1 & ix`n'b==1
	replace ix`n'=. if ix63a==.
	}

	foreach n of numlist 69/74 {
	gen ix`n'=0 if twoweek==1
	replace ix`n'=1 if (ix`n'a==1 | ix`n'a==3) & (ix`n'b==2 | ix`n'b==3 | ix`n'b==.)
	replace ix`n'=2 if (ix`n'a==1 | ix`n'a==3) & ix`n'b==1
	replace ix`n'=. if ix63a==.
	}


** POST-ABDUCTION EXPERIENCES **

// Time back
    gen back = age - abd_age_a - (abd_lgth_a/12)
    replace back = age - abd_age_b - (abd_lgth_b/12) if abd2==1
    replace back = age - abd_age_c - (abd_lgth_c/12) if abd3==1
    replace back=0 if back<0
    la var back "Years since returning from bush"

// Age back 
    gen age_back = abd_age_a + (abd_lgth_a/12)
    replace age_back = abd_age_b + (abd_lgth_b/12) if abd2==1
    replace age_back = abd_age_c + (abd_lgth_c/12) if abd3==1
    replace age_back=round(age_back)
    gen age_back2 = abd_age + (abd_l/12)
    replace age_back2=round(age_back2)

    gen age_bk_cat=.
    replace age_bk_cat=1 if age_back<=12
    replace age_bk_cat=2 if age_back>=13 & age_back<=17
    replace age_bk_cat=3 if age_back>=18 & age_back<=30
    label define back 1 "<12" 2 "13-17" 3 "18-30"
    label values age_bk_cat back

    la var age_back "Age at return from last abduction"
    la var age_back2 "Age at return from longest-lasting abduction"
    la var age_bk_cat "Recode: Age at return from last abduction"

// Received amnesty

    gen amnesty=cond(ix28a==1 | ix28b==1 | ix28c==1,1,0) if abd==1
    la var amnesty "Equals 1 if received amnesty"

// Returned to school after abduction

    gen ret_skool=cond(ix75==1,1,0) if ix75!=.
    la var ret_skool "Returned to school after longest-lasting abduction"

// Received any services

	gen services=cond(ix27a1==1 | ix27a2==1 | ix27a3==1 | ix27a4==1 | ix27a5==1 | ix27a6==1,1,0)
	replace services=1 if ix781==1 | ix782==1 | ix783==1 | ix785==1 | ix786==1 | ix787==1
	replace services=. if abd==0
	la var services "Equals 1 if received any services"

// Returned home

    gen ix29=ix29a if longest==1 & id!=.
    replace ix29=ix29b if longest==2 & id!=.
    replace ix29=ix29c if longest==3 & id!=.
    recode ix29 (2=0), gen(ret_home)
    la var ret_home "Returned home after longest-lasting abduction"

// Returned home to husband/parent/grandparent after abduction

    *dummy if returned to parent
    gen ret_to_parent=cond(ix771==1 | ix772==1,1,0)
    la var ret_to_parent "Returned to at least one parent after abduction"

    *dummy if returned to wife
    gen ret_wf=0
    replace ret_wf=1 if id==69
    replace ret_wf=1 if id==89
    replace ret_wf=1 if id==145 
    replace ret_wf=1 if id==261 
    replace ret_wf=1 if id==344 
    replace ret_wf=1 if id==418 
    replace ret_wf=1 if id==449 
    replace ret_wf=1 if id==494 
    replace ret_wf=1 if id==497 
    replace ret_wf=1 if id==598 
    replace ret_wf=1 if id==621 
    replace ret_wf=1 if id==627 
    replace ret_wf=1 if id==648  
    replace ret_wf=1 if id==734
    
    *dummy if returned to parents or wife
     gen ret_par_hw=cond(ret_to_parent==1|ret_wf==1,1,0)
     la var ret_par_hw "Returned to parent or husband/wife following abduction"

    *dummy if returned to parent, wife, grandparent
    gen ret_pr_gr=cond(ix771==1 | ix772==1 | ix774==1,1,0)
    gen ret_pr_gr_hw=cond(ret_pr_gr==1 | ret_par_hw==1,1,0)
    la var ret_pr_gr_hw "Returned to parents, grandparents or husband/wife after abduction"

    *dummy if returned to parent, wife, grandparent, sibling(s)
    gen ret_pr_gr_sib=cond(ix771==1 | ix772==1 | ix774==1 | ix775==1,1,0)
    gen ret_pr_gr_hw_sib=cond(ret_pr_gr_sib==1 | ret_par_hw==1,1,0)
    la var ret_pr_gr_hw_sib "Returned to parents, grandparents, husband/wife, or siblings after abduction"

    drop ret_wf ret_pr_gr ret_pr_gr_sib
    replace ret_to_parent=. if ix771==. & ix772==. & ix773==. & ix774==. & ix775==. & ix776
    replace ret_par_hw=. if ix771==. & ix772==. & ix773==. & ix774==. & ix775==. & ix776
    replace ret_pr_gr_hw=. if ix771==. & ix772==. & ix773==. & ix774==. & ix775==. & ix776
    replace ret_pr_gr_hw_sib=. if ix771==. & ix772==. & ix773==. & ix774==. & ix775==. & ix776

//  Family & community acceptance variables

    gen fam_prob_ever=cond(ix84a==1 | ix85a==1 | ix86a==1,1,0) if (ix84a!=. | ix85a!=. | ix86a!=.)
    gen fam_prob_now=cond(ix84b==1 | ix85b==1 | ix86b==1,1,0) if (ix84b!=. | ix85b!=. | ix86b!=.)
    gen comm_prob_ever=cond(ix87a==1 | ix88a==1 | ix89a==1,1,0) if (ix87a!=. | ix88a!=. | ix89a!=.)
    gen comm_prob_now=cond(ix87b==1 | ix88b==1 | ix89b==1,1,0) if (ix87b!=. | ix88b!=. | ix89b!=.)
    gen fam_prob_better=1 if fam_prob_ever==1 & fam_prob_now==0
    replace fam_prob_better=0 if fam_prob_ever==1 & fam_prob_now==1
    replace fam_prob_better=. if fam_prob_ever==0
    gen comm_prob_better=1 if comm_prob_ever==1 & comm_prob_now==0
    replace comm_prob_better=0 if comm_prob_ever==1 & comm_prob_now==1
    replace comm_prob_better=. if comm_prob_ever==0
    la var fam_prob_ever "Equals 1 if reported family problems ever"
    la var fam_prob_now "Equals 1 if report family problems now"
    la var comm_prob_ever "Equals 1 if reported community problems ever"
    la var comm_prob_now "Equals 1 if report community problems now"
    la var fam_prob_better "Equals 1 if reported family problems improved"
    la var comm_prob_better "Equals 1 if reported community problems improved"

// Self-blame variables

    replace x48="" if x48=="DK"
    ren x48 temp
    gen x48=real(temp)
    drop temp

    gen courageous=cond(x35==1 | x35==3,1,0) if x35!=.
    gen parents=cond(x36==1 | x36==3,1,0) if x36!=.
    gen blame=cond(x37==1 | x37==3,1,0) if x37!=.
    gen haunted=cond(x38==1 | x38==3,1,0) if x38!=.
    gen troublefam=cond(x42==1 | x42==3,1,0) if x42!=.
    gen guilt=cond(x43==1 | x43==3,1,0) if x43!=.
    gen troublecomm=cond(x44==1 | x44==3,1,0) if x44!=.
    gen neighborsbad=cond(x45==1 | x45==3,1,0) if x45!=.
    gen konyblame=cond(x46==1 | x46==3,1,0) if x46!=.
    gen godsplan=cond(x47==1 | x47,1,0) if x47!=.
    gen cursed=cond(x48==1 | x48==3,1,0) if x48!=.

    la var courageous "Experiences happened because you were not courageous or strong enough?"
    la var parents "Parents to blame for what happened?"
    la var blame "Blame yourself for what happened?"
    la var haunted "Equals 1 if haunted by spirits"
    la var troublefam "Do you feel you have caused trouble to your family?"
    la var guilt "Do you feel any guilt about what happened?"
    la var troublecomm "Do you feel you have caused trouble for people in your community?"
    la var neighborsbad "Do neighbors see you as a bad person because of anything you did?"
    la var konyblame "Kony and LRA leaders to blame for what happened?"
    la var godsplan "Do you believe this happened to you because it was God's plan for you?"
    la var cursed "Do you believe that bad things happened to you because you were cursed?"

    gen blame_self=cond(blame==1 | guilt==1 | troublefam==1 | troublecom,1,0)
    gen blame_total=troublefam+guilt+troublecomm+neighborsbad
    gen blame_total_modhi=cond(blame_total>0 & blame_total!=.,1,0)
    gen blame_total_hi=cond(blame_total>3 & blame_total!=.,1,0)

    la var blame_self "Equals 1 if any self-blame, guilt, or caused trouble for family or commmunity"
    la var blame_total "Sum of caused trouble for family/community, has self-guilt, or neighbors see as bad person"
    la var blame_total_modhi "Equals 1 if blame_total>0"
    la var blame_total_hi "Equals 1 if blame_total>3"

// Returned to school

	gen back_skool=cond(ix75==1,1,0) if twoweek==1
	la var back_skool "Equals 1 if returned to school after abduction"

** MENTAL AND PHYSICAL HEALTH

// Mental Health

	foreach x of numlist 2/44 {
		codebook viii`x'a
		quietly replace viii`x'a=2 if viii`x'a==. & viii`x'b==.
		quietly replace viii`x'a=1 if viii`x'a==. & viii`x'b!=.
		}

	foreach x of numlist 2/44 {
		quietly gen viii`x'=.
		quietly replace viii`x'=0 if viii`x'a==2
		quietly replace viii`x'=1 if viii`x'b==3
		quietly replace viii`x'=2 if viii`x'b==2 | (viii`x'a==1 & viii`x'b==.)
		quietly replace viii`x'=3 if viii`x'b==1
		}

	replace viii2=(viii2*-1)+4
	replace viii3=(viii3*-1)+4
	replace viii8=(viii8*-1)+4
	replace viii9=(viii9*-1)+4
	replace viii10=(viii10*-1)+4
	replace viii15=(viii15*-1)+4
	replace viii20=(viii20*-1)+4
	replace viii21=(viii21*-1)+4
	replace viii22=(viii22*-1)+4
	replace viii24=(viii24*-1)+4
	replace viii27=(viii27*-1)+4
	replace viii31=(viii31*-1)+4
	replace viii37=(viii37*-1)+4
	replace viii38=(viii38*-1)+4
	replace viii40=(viii40*-1)+4

	replace viii2=0 if viii2==4
	replace viii3=0 if viii3==4
	replace viii8=0 if viii8==4
	replace viii9=0 if viii9==4
	replace viii10=0 if viii10==4
	replace viii15=0 if viii15==4
	replace viii20=0 if viii20==4
	replace viii21=0 if viii21==4
	replace viii22=0 if viii22==4
	replace viii24=0 if viii24==4
	replace viii27=0 if viii27==4
	replace viii31=0 if viii31==4
	replace viii37=0 if viii37==4
	replace viii38=0 if viii38==4
	replace viii40=0 if viii40==4

	factor viii2-viii44, factors(3) means
      rotate, factors(3)

     gen distress=(viii4 + viii5 + viii6 + viii11 + viii12 + viii14 + viii16 + viii26 + viii28 + viii29 + viii32 + viii33 + viii34 + viii35 + viii36 + viii39 + viii41)/3
     gen prosocial=(viii2 + viii8 + viii9 + viii10 + viii20 + viii21 + viii23 + viii24 + viii27 + viii31 + viii40)/3 
     gen hostility = viii13 + viii18 + viii25 + viii30	
     la var distress "Index of emotional distress"
     la var hostility "Index of hostile acts"
     la var prosocial "Index of prosocial behavior"
    
     gen hostile=cond(hostility>0,1,0) if found==1
     la var hostile "Indicator if hostility >0"

** SOCIAL & FAMILY SUPPORT **

// Forms of social support and support index

	foreach x of numlist 50/66 {
		codebook x`x'a x`x'b
		quietly replace x`x'a=2 if x`x'a==. & x`x'b==.
		quietly replace x`x'a=1 if x`x'a==. & x`x'b!=.
		}

	foreach x of numlist 50/66 {
		quietly gen x`x'=.
		quietly replace x`x'=0 if x`x'a==2
		quietly replace x`x'=1 if x`x'b==2 | (x`x'a==1 & x`x'b==.)
		quietly replace x`x'=2 if x`x'b==1
		}

	ren x50 x50_minded
	ren x51 x51_sat
	ren x52 x52_sharedexp
	ren x53 x53_work
	ren x54 x54_compliment
	ren x55 x55_info
	ren x56 x56_takehrt
	ren x57 x57_plans
	ren x58 x58_advice
	ren x59 x59_money
	ren x60 x60_listen
	ren x61 x61_loan	
	ren x62 x62_taught
	ren x63 x63_joked
	ren x64 x64_prayed
	ren x65 x65_healer
	ren x66 x66_chores
	
	la var x50_minded "Recode: X50 Support: Looked after your things? no/sometimes/often"
	la var x51_sat "Recode: X51 Support: Sat with you? no/sometimes/often"
	la var x52_sharedexp "Recode: X52 Support: Shared with you? no/sometimes/often"
	la var x53_work "Recode: X53 Support: Work with you? no/sometimes/often"
	la var x54_compliment "Recode: X54 Support: Appreciated you? no/sometimes/often"
	la var x55_info "Recode: X55 Support: Gave info? no/sometimes/often"
	la var x56_takehrt "Recode: X56 Support: Take heart? no/sometimes/often"
	la var x57_plans "Recode: X57 Support: Help find direction? no/sometimes/often"
	la var x58_advice "Recode: X58 Support: Gave advice? no/sometimes/often"
	la var x59_money "Recode: X59 Support: Lent you things for free? no/sometimes/often"
	la var x60_listen "Recode: X60 Support: Listened? no/sometimes/often"
	la var x61_loan "Recode: X61 Support: Gave you something? no/sometimes/often
	la var x62_taught "Recode: X62 Support: Taught you? no/sometimes/often
	la var x63_joked "Recode: X63 Support: Joked? no/sometimes/often
	la var x64_prayed "Recode: X64 Support: Prayed? no/sometimes/often
	la var x65_healer "Recode: X65 Support: Took to healer? no/sometimes/often
	la var x66_chores "Recode: X66 Support: Helped you? no/sometimes/often

	gen support=(x51_ + x52_ + x53_ + x54_ + x55_ + x56_ + x57_ + x58_ + x59_ + x60_ + x61_ + x62_ + x63_ + x64_)/2
	la var support "Additive index of social support (17 forms, comparable to SWAY II)"

//  Family connectedness

	gen fam_comfort=ii11
		replace fam_comfort=1 if ii11!=1 & ii11!=2 & ii11!=3
		gen fam_quarrel=ii12
		replace fam_quarrel=3 if ii12!=1 & ii12!=2 & ii12!=3
	gen fam_caring=ii13
		replace fam_caring=1 if ii13!=1 & ii13!=2 & ii13!=3
	
	replace fam_com=-fam_com
	replace fam_car=-fam_car
	
    // Change to 0 to 2 scale
    replace fam_comfort=3+fam_comfort
    replace fam_caring=3+fam_caring
    replace fam_quarrel=fam_quarrel-1

    // generate indices
    gen fam_problems=cond(fam_com==0 | fam_q==0 | fam_car==0,1,0) if found!=.
    gen fam_connect=fam_comfort+fam_quarrel+fam_caring

    la var fam_comfort "How comfortable living in household (higher values more comfortable)"
    la var fam_quarrel "Quarrels with family (higher values fewer quarrels)"
    la var fam_caring "Family caring (higher values more caring)"
    la var fam_problems "Equals 1 if poor family relations: Quarrel very often, or not at all comfortable or caring"
    la var fam_connect "Index of family connectedness (0 to 6, low to high)"

// Who do you live with?

	gen live_mthr=cond(ii10a==1 | ii10b==1 | ii10c==1 | ii10d==1,1,0)	
	gen live_fthr=cond(ii10a==2 | ii10b==2 | ii10c==2 | ii10d==2,1,0)
	gen live_par=cond(live_mthr==1 | live_fthr==1,1,0)
	la var live_mthr "Equals 1 if lives with mother"
	la var live_fthr "Equals 1 if lives with father"
	la var live_par "Equals 1 if lives with at least one parent"

// Social and political participation

    gen groups=vii1a+vii1b+vii1c+vii1d+vii1e+vii1f+vii1g+vii1j
    gen group=cond(groups>0 & groups!=.,1,0)
    la var group "Equals 1 if member of at least one group"
    la var groups "Number of groups to which youth belongs"

    gen comm_mobil=cond(vii1i==1,1,0)
    gen volunteer=cond(vii1h==1,1,0)
    la var comm_mobil "Equals 1 if community mobilizer"
    la var volunteer "Equals 1 if volunteer for NGO"

    gen g_peace=cond(vii1b==1,1,0)
    gen g_drama=cond(vii1a==1,1,0)
    gen g_farm=cond(vii1c==1,1,0)
    gen g_water=cond(vii1d==1,1,0)
    gen g_skool=cond(vii1f==1,1,0)
    gen g_sport=cond(vii1g==1,1,0)
    gen g_church=cond(vii1e==1,1,0)
    gen g_oth=cond(vii1j==1,1,0)
    la var g_peace "Equals 1 if member of peace club"
    la var g_drama "Equals 1 if member of drama, music, or dance club"
    la var g_farm "Equals 1 if member of farmers group or cooperative"
    la var g_water "Equals 1 if member of water committee"
    la var g_skool "Equals 1 if member of school committee or school club or a school prefect"
    la var g_sport "Equals 1 if member of sports team"
    la var g_church "Equals 1 if member of church, prayer or bible study group"
    la var g_oth "Equals 1 if member of any other community or church group"

    gen elders_disobey=cond(viii19>=2,1,0)
    la var elders_disobey "Equals 1 if disobey your parents/guardians, teachers or elders"

// Physical Health

	foreach x of numlist 3/10 {
		gen v`x'=.
		replace v`x'=0 if v`x'a==1
		replace v`x'=1 if v`x'b==1
		replace v`x'=2 if v`x'b==2 | (v`x'a==2 & v`x'b==.)
		replace v`x'=3 if v`x'b==3
		replace v`x'=0 if v`x'a==.
		}

	ren v3 v3_walk
	ren v4 v4_20L
	ren v5 v5_dig
	ren v6 v6_run
	ren v7 v7_10L
	ren v8 v8_work
	ren v9 v9_stand
	ren v10 v10_bend
	la var v3_walk "Recode: V3 Able to walk? yes/slight difficulty/great difficulty/not at all"
	la var v4_20L "Recode: V4 Able to carry a 20L jerry can for 20m? yes/slight difficulty/great difficulty/not at all
	la var v5_dig "Recode: V5 Able to dig? yes/slight difficulty/great difficulty/not at all
	la var v6_run "Recode: V6 Able to run? yes/slight difficulty/great difficulty/not at all
	la var v7_10L "Recode: V7 Able to 10L jerry can for 20m? yes/slight difficulty/great difficulty/not at all
	la var v8_work "Recode: V8 Able to work? yes/slight difficulty/great difficulty/not at all
	la var v9_stand "Recode: V9 Able to stand? yes/slight difficulty/great difficulty/not at all
	la var v10_bend "Recode: V10 Able to bend? yes/slight difficulty/great difficulty/not at all

	codebook v1
	list v1 v2 v3_ v4_ v5_ v6_ v7_ v8_ v9_ v10_ if v1==.
	replace v1=v1-1
	replace v1=0 if v1==. & (v3_==0 & v4_==0 & v5_==0 & v6_==0)
	replace v1=2 if v1==. & (v7_!=0 | v8_!=0 | v9_!=0 | v10_!=0)
	replace v1=1 if v1==. 

	list v1 v2 v3_-v10_ if v2==.
	replace v2=0 if v2==.

	codebook v1 v2 v3_-v10_

	factor v3_-v10_, pcf
	rotate
	predict health

	codebook health
	gen hindex=(health+.41203386)/(6.7125135+.41203386)
	replace hindex=0 if hindex<0.001
	codebook hindex

	gen ill=cond(hindex>0,1,0)
	gen v_ill=cond(hindex>.15,1,0)

	la var hindex "Normalized health index"
	la var ill "Equals 1 if hindex>0"
	la var v_ill "Equals 1 if hindex>.15"

	gen injury=cond(v3_>1 | v4_>1 | v5_>1 | v6_>1 | v7_>0 | v8_>0 | v9_>0 | v10_>0,1,0)
	gen lra_inj=cond(injury==1 & v12==1,1,0)
	gen poor_health=cond(v1==2,1,0)
	gen no_dig=cond(v5_==2 | v5_==3,1,0)
	
	la var health "Health score (pcf)"
	la var injury "Equals 1 if has a serious injury"
	la var lra_inj "Equals 1 if injury caused by LRA"
	la var poor_health "Equals 1 if health is somewhat good (vs. good or not good)"
	la var no_dig "Equals 1 if unable to dig in a garden"

// Marriage and sex

	gen married=cond(ii23==1,1,0)
	gen children=cond(ii22>0 & ii22!=.,1,0)
	gen own_condom=cond(ii21==1,1,0)
	gen hiv_aware=cond(ii20==1,1,0)
	tab2 age married
	tab2 age children

	la var married "Equals 1 if ever married"
	la var children "Equals 1 if has had any children"
	la var own_condom "Equals 1 if currently owns condoms"
	la var hiv_aware "Equals 1 if says that Condoms prevent spread of HIV/AIDS"

	codebook ii19 ii20 ii21 ii22 ii23
	gen condom=cond(ii19==1,1,0)
	gen sex_active=cond(ii24==1,1,0) if married==0
	gen use_cond_some=cond(ii25==3 | ii27==3,1,0)
	gen use_cond_often=cond(ii25==1 | ii27==1,1,0)
	gen sleeps_around=cond(married==1 & (ii29==1 | ii29==2 | ii29==3),1,0)
	gen sleeps_around2=cond(married==1 & (ii29==1 | ii29==2 | ii29==3) & ii28!=4,1,0)
	gen use_cond_oth=cond(ii29==1,1,0)

	la var condom "Equals 1 if knows place to obtain condoms"
	la var sex_active "Equals 1 if never married and had sexual partner(s) in last year"
	la var use_cond_some "Equals 1 if uses condom sometimes"
	la var use_cond_often "Equals 1 if uses condom often"
	la var sleeps_around "Equals 1 if married and sleeps with other women"
	la var sleeps_around2 "Equals 1 if married and sleeps with other women and does not report being with only one woman"
	la var use_cond_oth "Equals 1 if married and uses a condom with a woman other than wife"

** ECONOMIC VARIABLES **

// Employment Levels

	foreach x of numlist 1/6 {
		foreach y of numlist 7/9 {
			replace iv`y'_`x'=0 if iv`y'_`x'==.
			}
		}

	gen emp_mo=cond(iv7_1>0,1,0)

	gen emp_wk=cond(iv8_1>0,1,0)

	gen emp_d_mo=iv7_1+iv7_2+iv7_3+iv7_4+iv7_5+iv7_6
		tab emp_d_mo
		replace emp_d_mo=28 if emp_d_mo>28
		tab emp_d_mo

	gen emp_d_wk=iv8_1+iv8_2+iv8_3+iv8_4+iv8_5+iv8_6
		tab emp_d_wk
		replace emp_d_wk=7 if emp_d_wk>7
		tab emp_d_wk

	gen emp_hrs_wk=(iv8_1*iv9_1)+(iv8_2*iv9_2)+(iv8_3*iv9_3)+(iv8_4*iv9_4)+(iv8_5*iv9_5)+(iv8_6*iv9_6)
		tab emp_hrs_wk
		replace emp_hrs_wk=84 if emp_hrs_wk>84
		tab emp_hrs_wk
			
	gen emp_hrs_mo=(iv7_1*iv9_1)+(iv7_2*iv9_2)+(iv7_3*iv9_3)+(iv7_4*iv9_4)+(iv7_5*iv9_5)+(iv7_6*iv9_6)
		tab emp_hrs_mo
		replace emp_hrs_mo=360 if emp_hrs_mo>360
		tab emp_hrs_mo

	gen unocc=cond(emp_mo==0 & inschool==0,1,0)

	reg emp_d_mo age_* C_*, robust
	predict emp_dhat, resid
			
	la var emp_mo "Equals 1 if worked in last 4 weeks"
	la var emp_wk "Equals 1 if worked in last 7 days"
	la var emp_d_mo "Days worked in last 4 weeks"
	la var emp_d_wk "Days worked in last 7 days"
	la var emp_hrs_mo "Hours worked in last 4 weeks"
	la var emp_hrs_wk "Hours worked in last 7 days"
	la var unocc "Equals 1 if neither in school nor working"

	gen land_access=cond(iv3==1,1,0)
	la var land_access "Equals 1 if has access to land"

// Earnings measures

	gen earn_mo=iv1
	gen earn_wk=iv2
	codebook earn_mo earn_wk	

	reg earn_mo A14-A29 C_ach-C_paj if abd==0, robust
	predict earnhat, resid

	gen earn0=cond(earn_mo>0,1,0)

	la var earn0 "Earnings > 0"
	la var earn_mo "Earnings: Last 4 weeks (USH)"
	la var earn_wk "Earnings: Last 7 days (USH)"
	la var earnhat "Age/Location-Adj Earnings Residuals - Censored"

// Wage Measures

	gen wage_mo=earn_mo/emp_d_mo
		replace wage_mo=. if emp_d_mo==0
	gen lwage_mo=ln(wage_mo)
	gen wage_wk=earn_wk/emp_d_wk
		replace wage_wk=. if emp_d_wk==0
	gen lwage_wk=ln(wage_wk)

	la var wage_mo "Daily earnings"
	la var wage_wk "Average wage earned in last 7 days (USH)"
	la var lwage_wk "Log Average wage earned in last 7 days (USH)"

// Occupation

	sort id
	list id iv6other_1 if iv6_1=="S"
	list id iv6other_2 if iv6_2=="S"
	list id iv6other_3 if iv6_3=="S"
	list id iv6other_4 if iv6_4=="S"
	list id iv6other_5 if iv6_5=="S"
	list id iv6other_6 if iv6_6=="S"
	list id iv6other_7 if iv6_7=="S"

	replace iv6_1="A" if iv6_1=="S" & (id==51 | id==201 | id==213 | id==308 | id==616)
	replace iv6_1="C" if iv6_1=="S" & (id==227 | id==600 | id==643 | id==680 | id==108 | id==120 | id==222)
	replace iv6_1="D" if iv6_1=="S" & (id==196)
	replace iv6_1="F" if iv6_1=="S" & (id==3)
	replace iv6_1="G" if iv6_1=="S" & (id==590 | id==646 | id==739 | id==744)
	replace iv6_1="H" if iv6_1=="S" & (id==323)
	replace iv6_1="J" if iv6_1=="S" & (id==153)
	replace iv6_1="L" if iv6_1=="S" & (id==6 | id==7 | id==18 | id==151 | id==200 | id==208 | id==417 | id==515 | id==541 | id==595 | id==604 | id==642 | id==659 | id==666)
	replace iv6_1="O" if iv6_1=="S" & (id==529 | id==729)
	replace iv6_1="R" if iv6_1=="S" & (id==657)
	replace iv6_1="T" if iv6_1=="S" & (id==2 | id==50 | id==140 | id==164 | id==260 | id==272 | id==449 | id==589 | id==591 | id==593 | id==599 | id==629 | id==653 | id==654 | id==673)
	replace iv6_1="V" if iv6_1=="S" & (id==651 | id==691 | id==701)
	replace iv6_1="W" if iv6_1=="S" & (id==37 | id==620 | id==678)
	
	replace iv6_2="D" if iv6_2=="S" & (id==148)
	replace iv6_1="H" if iv6_2=="S" & (id==739)
	replace iv6_2="L" if iv6_2=="S" & (id==260 | id==535 | id==735)
	replace iv6_2="O" if iv6_2=="S" & (id==194)
	replace iv6_2="T" if iv6_2=="S" & (id==100 | id==123 | id==188 | id==308 | id==608 | id==708)
	
	replace iv6_4="T" if iv6_4=="S" & id==68

	replace iv6_6="L" if iv6_6=="S" & id==65
	
	gen mainjob=""
	replace mainjob=iv6_1
	replace mainjob=iv6_2 if iv7_2>iv7_1 & iv7_2!=.
	replace mainjob=iv6_3 if iv7_3>iv7_1 & iv7_3>iv7_2 & iv7_3!=.
	replace mainjob=iv6_4 if iv7_4>iv7_1 & iv7_4>iv7_2 & iv7_4>iv7_3 & iv7_4!=.
	replace mainjob=iv6_5 if iv7_5>iv7_1 & iv7_5>iv7_2 & iv7_5>iv7_3 & iv7_5>iv7_4 & iv7_5!=.
	replace mainjob=iv6_6 if iv7_6>iv7_1 & iv7_6>iv7_2 & iv7_6>iv7_3 & iv7_6>iv7_4 & iv7_6>iv7_5 & iv7_6!=.
	replace mainjob=iv6_7 if iv7_7>iv7_1 & iv7_7>iv7_2 & iv7_7>iv7_3 & iv7_7>iv7_4 & iv7_7>iv7_5 & iv7_7>iv7_6 & iv7_7!=.
	replace mainjob=iv6_8 if iv7_8>iv7_1 & iv7_8>iv7_2 & iv7_8>iv7_3 & iv7_8>iv7_4 & iv7_8>iv7_5 & iv7_8>iv7_6 & iv7_8>iv7_7 & iv7_8!=.
	replace mainjob=iv6_9 if iv7_9>iv7_1 & iv7_9>iv7_2 & iv7_9>iv7_3 & iv7_9>iv7_4 & iv7_9>iv7_5 & iv7_9>iv7_6 & iv7_9>iv7_7 & iv7_9>iv7_8 & iv7_9!=.
	
	tab mainjob, m
	replace mainjob="A" if mainjob=="P"
	replace mainjob="G" if mainjob=="W"
	replace mainjob="H" if mainjob=="I" | mainjob=="R"
	replace mainjob="Z" if mainjob==""	
	
	* Occ choice
	gen occup=0
	replace occup=1 if inschool==1
	replace occup=2 if inschool==1 & emp_mo==1 & earn_mo>0
	replace occup=3 if inschool==0 & emp_mo==1 
	la var occup "Occupation: nothing/school/school+work/work"

	gen occup2=0
	replace occup2=1 if inschool==1
	replace occup2=2 if inschool==0 & emp_mo==1 & earn_mo>0
	la var occup2 "Occupation: nothing/school/work"

	label define activity 0 "Nothing" 1 "School" 2 "Skool/Work" 3 "Work only"
	label values occup activity	
	label define activity2 0 "Nothing" 1 "School" 2 "Work only"
	label values occup2 activity2
	
	encode mainjob, g(job)
	label define occup 15 "Other" 16 "Profession" 17 "Quarrying"
	label values job occup
		
	gen job_skill=.
	replace job_skill=0 if mainjob=="Z"
	replace job_skill=1 if mainjob=="B" | mainjob=="T" | mainjob=="O" | mainjob=="E" | mainjob=="V"
	replace job_skill=2 if mainjob=="N" | mainjob=="A" | mainjob=="L"
	replace job_skill=3 if mainjob=="K"
	replace job_skill=4 if mainjob=="F" | mainjob=="M" | mainjob=="Q" | mainjob=="D"	
	replace job_skill=5 if mainjob=="G" | mainjob=="J" | mainjob=="C" | mainjob=="H"
	replace job_skill=6 if mainjob=="S"
	label define occup3 0 "Nothing" 1 "Unskilled Labor" 2 "SmallCap" 3 "Military" 4 "MedCap" 5 "Biz/Prof" 6 "Other"
	label values job_skill occup3
	
	gen job_skill2=.
	replace job_skill2=0 if job_skill==0
	replace job_skill2=1 if job_skill==1 | job_skill==2
	replace job_skill2=2 if job_skill==3
	replace job_skill2=3 if job_skill==4 | job_skill==5
	replace job_skill2=4 if job_skill==6
	label define occup4 0 "Nothing" 1 "LowSkill" 2 "Military" 3 "HighSkill" 4 "Other"
	label values job_skill2 occup4
	
	replace mainjob="A" if mainjob=="Q"
	replace mainjob="S" if mainjob=="V"

	replace mainjob="Farming" if mainjob=="A"
	replace mainjob="Ag Labor" if mainjob=="B"
	replace mainjob="Vocation" if mainjob=="C"
	replace mainjob="Construction" if mainjob=="D"
	replace mainjob="Quarrying" if mainjob=="E"
	replace mainjob="Boda Boda" if mainjob=="F"
	replace mainjob="Business" if mainjob=="G"
	replace mainjob="Professional" if mainjob=="H"
	replace mainjob="Repairs" if mainjob=="J"
	replace mainjob="Military" if mainjob=="K"
	replace mainjob="Hawking" if mainjob=="L"
	replace mainjob="Brickmaking" if mainjob=="M"
	replace mainjob="Charcoal" if mainjob=="N"
	replace mainjob="Firewood" if mainjob=="O"
	replace mainjob="Other" if mainjob=="S"
	replace mainjob="Casual Labor" if mainjob=="T"
	replace mainjob="None" if mainjob=="Z"

	gen prof=cond(job_skill==5,1,0) if found==1
	la var prof "Occupation: business/professional"

	la var mainjob "Main job"
	la var job "Main job type"
	la var job_skill "Job Skill type"
	la var job_skill2 "Job skill level"
	
    gen pol_job=cond(iv6_1=="R" | iv6_2=="R" | iv6_3=="R" | iv6_4=="R" | iv6_5=="R" | iv6_6=="R" | iv6_7=="R" | iv6_8=="R" | iv6_9=="R",1,0)
    la var pol_job "Equals 1 if has a political job"

// Other occupational measures

	gen risky_job=0
	foreach x in K N O V {
		foreach y of numlist 1	/9 {
			replace risky_job=1 if iv6_`y'=="`x'"
			}
		}
	
	gen risky_job2=0
	foreach x in N O V {
		foreach y of numlist 1/9 {
			replace risky_job2=1 if iv6_`y'=="`x'"
			}
		}
	
	gen skill1=0
	foreach x in C G H I J R {
		foreach y of numlist 1/9 {
			replace skill1=1 if iv6_`y'=="`x'"
			}
		}
	
	gen skill2=0
	foreach x in G H I R J G {
		foreach y of numlist 1/9 {
			replace skill2=1 if iv6_`y'=="`x'"
			}
		}

// Work Experience

	gen exper=age-6-educ
	replace exper=0 if exper<0
	la var exper "Work experience"

** MISC VARIABLES **

// Dry Season

	gen dry=cond(i5amonth==12 | i5amonth==1 | i5amonth==2 | i5amonth==3,1,0)
	la var dry "Equals 1 if Interviewed in Dry season"


// Lives In Town

	gen location=xi1b
	replace location="88" if xi1b=="Kitgum town concil" | xi1b=="KAMPALA" | xi1b=="Kampala" | xi1b=="Town Council" | xi1b=="K.T.C" | xi1b=="TOWN COUNCIL" | xi1b=="Municipality" | xi1b=="N/A" | xi1b=="Kitgum Town Council" | xi1b=="KITGUM TOWN COUNCIL" | xi1b=="KITGUM TOWN" | xi1b=="MUNICIPALITY" | xi1b=="municipality"
	replace location="88" if xi1c=="K.T.C" | xi1c=="KIT.Town Council" | xi1c=="KTC" | xi1c=="Kitgum Town" | xi1c=="Kitgum Town C" | xi1c=="Kitgum Town Council" | xi1c=="Kitgum Town council" | xi1c=="Kitgum Town council." | xi1c=="Kitgum Town concil" | xi1c=="KITGUM TOWN" | xi1c=="kitgum T. Council" | xi1c=="K.Town Council" | xi1c=="TOWN" | xi1c=="TOWN COUNCIL" | xi1c=="Town" | xi1c=="Town Council" | xi1c=="kitgum T. Council"
	gen town=cond(location=="88" | district==3 | district==4 | district==5,1,0)
	drop location
	la var town "Equals 1 if Lives in a town"

// Smoking and Eating

	gen smoke=cond(vi3==1,1,0)
	gen eatmeat=cond(vi2>0 & vi2!=.,1,0)
	gen eattwice=cond(vi1==2 | vi1==3,1,0)

	la var smoke "Equals 1 if smokes"
	la var eatmeat "Equals 1 if has eaten meat in past two weeks"
	la var eattwice "Equals 1 if eats more than once per day"

// No Education

    gen educ0=cond(educ==0,1,0)
    la var educ0 "No education"

// Beatings
	codebook vii9 vii10
	replace vii9=0 if vii9==2
	replace vii10=0 if vii10==2
	replace vii9=1 if vii9==. & vii10!=.
	replace vii9=0 if vii9==. & vii10==.
	replace vii10=vii9 if vii10==.
	
	codebook vii11 vii12
	replace vii11=0 if vii11==2
	replace vii12=0 if vii12==2
	replace vii11=1 if vii11==. & vii12!=.
	replace vii11=0 if vii11==. & vii12==.
	replace vii12=vii11 if vii12==.
	
	codebook vii13 vii14
	replace vii13=0 if vii13==2
	replace vii14=0 if vii14==2
	replace vii13=1 if vii13==. & vii14!=.
	replace vii13=0 if vii13==. & vii14==.
	replace vii14=vii13 if vii14==.
	
	codebook vii15 vii16
	replace vii15=0 if vii15==2
	replace vii16=0 if vii16==2
	replace vii15=1 if vii15==. & vii16!=.
	replace vii15=0 if vii15==. & vii16==.
	replace vii16=vii15 if vii16==.
	
	pca vii9-vii16
	predict beat
	
	gen beat_neighbor=cond(vii9==1,1,0)
	gen beat_wife=cond(vii15==1,1,0)

	gen beat0=cond(beat>0,1,0)
	pctile a=beat, n(95)
	gen beat95=cond(beat>a,1,0)
	drop a
	pctile a=beat, n(90)
	gen beat90=cond(beat>a,1,0)
	drop a

	la var beat "beatings score (pca)"
	la var beat_neighbor "Equals 1 if thinks would be acceptable for a neighbor to seriously beat a man who insulted him"
	la var beat_wife "Equals 1 if thinks it would be acceptable to beat wife if she argues or talks back"
	la var beat0 "Equals 1 if beat pca score >0"
	la var beat95 "Equals 1 if in 95th percentile of beat pca score"
	la var beat90 "Equals 1 if in 90th percentile of beat pca score"

// Ever in army
	gen army=cond(iv11==1 | iv11==2,1,0) if id<1000
	gen noarmy=cond(army==0,1,0) if id<1000
	la var army "Equals 1 if ever in army"
	la var noarmy "Equals 1 if never in army"

// Impute absentee values and generate variables for predicting absenteeism

	replace emp_mo=1 if q15>2 & q15<=10
	replace emp_mo=0 if q15==1 | q15==2
	tab hhiil if absentee==1 & emp_mo==., m
	replace emp_mo=1 if hhiil>2 & hhiil<=10 & absentee==1 & emp_mo==.
	replace emp_mo=0 if (hhiil==1 | hhiil==2) & absentee==1 & emp_mo==.

	replace educ=hhiij if absentee==1
	replace educ=8 if absentee==1 & educ==.

	replace inschool=1 if absentee==1 & q15==2 | (q15==. & hhiil==2)
	replace inschool=0 if absentee==1 & inschool==.

	replace literate=1 if (q14==1 | (q14==. & educ>=7)) & absentee==1
	replace literate=0 if (q14==2 | (q14==. & educ<7)) & absentee==1

	gen military=cond(mainjob=="Military" | (absentee==1 & q15>=3 & q15<=4),1,0)
	la var military "Equals 1 if main job is reported as military"

	replace skill1=1 if q15==6 | q15==7 | q15==9 | id==2104 | id==2036
	replace skill2=1 if q15==6 | q15==7 | q15==9 | id==2104
	replace skill1=0 if absentee==1 & skill1==.
	replace skill2=0 if absentee==1 & skill2==.

	replace injury=1 if q17==1 | q19==1
	replace ill=1 if q17==1 | q19==1
	replace injury=0 if injury==. & absentee==1
	replace ill=0 if ill==. & absentee==1

	gen hh_immed_fam=cond(hhiic>=0 & hhiic<=4,1,0)
	gen hh_wfp=cond(hhiib_05==1,1,0)
	gen hh_no_info=cond(hhiib_05==. | hhiic==.,1,0)
	la var hh_immed_fam "Equals 1 if immediate family"
	la var hh_wfp "Equals 1 if registered with WFP"
	la var hh_no_info "Equals 1 if no info on hh"

	gen incamp=1
	replace incamp=0 if absentee==1 
	replace incamp=1 if id==2012 | id==2001 | id==2014 | id==2054 | id==2058 | id==2112
	replace incamp=0 if hhiik>2 & hhiik<99 & hhiik!=.
	replace incamp=0 if ii17==. & found==1
	replace incamp=0 if xi1b=="88" | id==516 | id==542 | id==462 | id==745 | id==27 | id==179 | id==539 | id==2

	gen out_district=0
	replace out_dist=1 if xi1a==4 | xi1a==5 | q12b==3 | q12b==4 | q12b==5

	la var incamp "Equals 1 if currently in a camp"
	la var out_district "Equals 1 if outside of survey districts"

// Interactions

    foreach var of varlist hh_size96 hh_land hh_cattle hh_stock educ abd_age abd_lgth {
     gen `var'_2 = `var'^2
     gen `var'_3 = `var'^3
     gen `var'_4 = `var'^4
     }


** CREATE WEIGHTS **

// Absentee weights

global D "A14-A29 C_ach-C_paj G*"
global ctrls "hh_fthr_pro hh_fthr_sol hh_wfp hh_immed hh_no_info hh_size96 hh_size96_2 hh_size96_3 hh_size96_4 hh_land hh_land_2 hh_land_3 hh_land_4 hh_cattle hh_cattle_2 hh_cattle_3 hh_cattle_4 hh_stock hh_stock_2 hh_stock_3 hh_stock_4 hh_plow"

	logit sample A14-A29 C_ach-C_pal $ctrls [pweight=i_weight]
	predict samp_hat if sample==1
	gen z=i_weight/samp_hat if sample==1
	egen sumz=sum(z)
      egen sumi=sum(i_weight)
	gen w_samp=z*sumi/sumz if sample==1
	drop z sumz sumi samp_hat

	logit absentee A14-A29 C_ach-C_pal abd abd_lgth abd_lgth_2 abd_lgth_3 abd_lgth_4 $ctrls if sample==1 [pweight=w_samp]
	predict abd_hat if found==1 
	gen z=w_samp/(1-abd_hat) if found==1	
	egen sumz=sum(z)
      egen sumi=sum(i_weight)
	gen w_abs=z*sumi/sumz if found==1
	drop z sumz sumi abd_hat

	logit absentee A14-A29 C_ach-C_paj abd abd_lgth abd_lgth_2 abd_lgth_3 abd_lgth_4 emp_mo skill2 inschool educ educ_2 educ_3 educ_4 literate military incamp out_dist hh_size96 $ctrls if sample==1 [pweight=w_samp]
	predict abd_hat if found==1 
	gen z=w_samp/(1-abd_hat) if found==1	
	egen sumz=sum(z)
      egen sumi=sum(i_weight)
	gen w_abs2=z*sumi/sumz if found==1
	drop z sumz sumi abd_hat

	gen weighthh=i_weight*hh_weight
	replace weighthh=25 if weighth==.

      gen f_weight=round(i_weight)
      gen f_weight_abs=round(w_abs)
      la var f_weight "Rounded i_weight (for use as fweights)"
      la var f_weight_abs "Rounded w_abs (for use as fweights)"

// Generate inverse pscore weights

	logit abd $D $ctrls [pw=w_abs]
		predict pscore if found==1
	gen z=w_abs*((abd/pscore)+((1-abd)/(1-pscore))) if found==1
	egen sumz=sum(z)
      egen sumi=sum(i_weight)
	gen w_sel=z*sumi/sumz if found==1
	drop z sumz sumi 

	logit abd $D $ctrls [pw=w_abs2]
		predict pscore2 if found==1
	gen z=w_abs2*((abd/pscore2)+((1-abd)/(1-pscore2))) if found==1
	egen sumz=sum(z) if found==1
      egen sumi=sum(i_weight)
	gen w_sel_abs=(z*sumi/sumz) if found==1
	drop z sumz sumi

	la var w_samp "sampling weights x inverse attrition prob"
	la var w_abs "sampling weights x inverse attrition prob x inverse absentee prob"
	la var w_abs2 "sampling weights x inverse attrition prob x inverse absentee prob (with detailed absentee data)"
	la var weighthh "hh weight"
	la var w_sel "sampling weights x inverse attrition prob  x inverse absentee prob x inverse selection prob" 
	la var w_sel_abs  "sampling weights x inverse attrition prob  x inverse absentee prob (with detailed absentee data) x inverse selection prob"

//Fill in missing labels

la var child "Equals 1 if currently under the age of 18"
la var adult "Equals one if currently at or after the age of 18" 
la var assets "PCA score of asset ownership"
la var ngo_paid "Equals 1 if NGO paid for higher educ"
la var secskool "Equals 1 if years of educ>7"
la var university "Equals 1 if ever attended university"
la var voc "Equals 1 if has higher education other than a university"
la var voc2 "Equals 1 if has 6+ years of higher education other than a university"
la var died "Equals 1 if youth died"
la var no_return "Equals 1 if youth did not return"
la var absentee "Equals 1 if absentee survey conducted"
la var dup "Duplicate"
la var a_id "ID with absentees"
la var educ_cat "Education categories"
la var no_mthr "Equals 1 if mother died"

//Label HH & absentee survey questions

la var hhi16hr "HHI16 Interview End: Hour"
la var hhi16min "HHI16 Interview End: Min"
la var hhiia "HHIIa ID"
la var hhiic "HHIIc Relation to head"
la var hhiid "HHIId =1 if Male"
la var hhiie "HHIIe =1 if died"
la var hhiif "HHIIf Age category"
la var hhiig "HHIIg Times abducted 0-6"
la var hhiih_lgth "HHIIh Length of first abduction"
la var hhiii_lgth "HHIIi Length of second abduction"
la var hhiih_yr "HHIIh Year of first abduction"
la var hhiii_yr "HHIIi Year of second abduction
la var hhiij "HHIIj Highest school level"
la var hhiik "HHIIk Where they stay at night"
la var hhiil "HHIIl Current work"
la var hh_immed_fam "HH =1 if immediate family"
la var hh_wfp "HH =1 if registered with WFP"
la var hh_no_info "HH =1 if no info on hh"
la var hh_abd_total "HH Abducted in household"
la var hh_abd_youth "HH Youth abducted in household"
la var hh_adults05 "HH Adults in household in 2005"
la var hh_avg_educ "HH Average education in household"
la var hh_size05 "HH Household size in 2005"
la var hh_size96 "HH Household size in 1996"
la var hh_war_death "HH Number of household members died in war"
la var causedeath "HH Cause of death"
rename abd_lgth_hh hh_abd_lgth
la var hh_abd_lgth "HH abduction length"
la var hhi8_small "HH =1 if land holdings <2 katala/acres"
la var hhiiage "HH age"
la var hh_adults05 "HH adults in household in 2005"
la var hhsize "HH household size"
la var hh_size96 "HH household size in 1996"
la var hh_size05 "HH household size in 2005"
la var hh_assets96 "HH 1996 Assets Index"
la var hh_wealth96 "HH 1996 Wealth Index"
la var hh_fthr_frm "HH =1 if Father a farmer"
la var hh_fthr_pro "HH =1 if Father a professional"
la var hh_fthr_sol "HH =1 if Father a soldier"
la var hh_avg_educ "HH average education in household"
la var hh_abd_total "HH abducted in household"
la var hh_abd_youth "HH youth abducted in household"
la var hh_war_death "HH War deaths in household"
la var causedeath "HH cause of death"
la var hh_immed_fam "HH =1 Roster row individual in hh with immediate family"
la var hh_wfp "HH =1 if registered with WFP"
la var hh_no_info "HH =1 if no info on hh"

la var q5 "ABS =1 if willing to be interviwed for absentee survey"
la var q6 "ABS If unwilling, why?"
la var q7 "ABS Reason for absence"
*la var q7other "ABS Reason for absence: other"
la var q8 "ABS =1 if will be coming to camp in future"
*la var q9 "ABS When will individual be coming to this camp?"
la var q10 "ABS For how long will individual be coming to this camp?"
*la var q11 "ABS Will individual come to the camp at other times?"
la var q12a "ABS Where is individual living now? Uganda or other"
la var q12aotherspecify "ABS If other, specify"
la var q12b "ABS District"
la var q12bother "ABS District: other"
la var q12c "ABS Sub-country or town"
la var q14 "ABS Read and write well enough to read a newspaper or write a letter"
la var q15 "ABS Employment or any type of work"
la var q15other "ABS Employment or any type of work: other"
la var q16 "ABS Specific work"
la var q17 "ABS Major physical injuries?"
la var q18 "ABS Major physical injuries: describe"
la var q19 "ABS Chronic illnesses?"
la var q20 "ABS Chronic illnesses: describe"
la var q21 "ABS Number of times gone with LRA"
la var q22 "ABS Still seriously affected by experience in the bush?"
la var q23 "ABS If seriously affected, describe"
la var q24 "ABS =1 if passed through reintegration center"
la var q25a "ABS Received services or assistance since returning: a"
la var q25b "ABS Received services or assistance since returning: b"
la var q25c "ABS Received services or assistance since returning: c"
la var q25d "ABS Received services or assistance since returning: d"
la var q25other "ABS Received services or assistance since returning: other"
la var q26 "ABS After return from bush, accepted, somewhat accepted, or not accepted by your family?"
la var q27 "ABS If not accepted, why did your family not accept him?"
la var q28 "ABS After return from bush, accepted, somewhat accepted, or not accepted by your neighbours and community?"
la var q29 "ABS If not accepted, why did your neighbours and community not accept him?"

//Label Survey Questions

la var id2 "Household ID"

la var iii1 "III1 Read book/newspaper?"
la var iii2 "III2 If no, read poster/notice?"
la var iii3 "III3 Write a letter?"
la var iii5a "III5a Reason not in school: abduction"
la var iii5b "III5b Reason not in school: school closed"
la var iii5c "III5c Reason not in school: displaced"
la var iii5d "III5d Reason not in school: no money for fees"
la var iii5h "III5h Reason not in school: got married"
la var iii5j "III5j Reason not in school: caring for family"
la var iii5k "III5k Reason not in school: other"
la var iii5oth1 "III5other Reason not in school: insecurity"
la var iii5oth2 "III5other Reason not in school: parents died"

la var iv4a "IV4a If land: how much?"
la var iv4b "IV4b If land: how much?"
la var iv6_7 "IV6 Job 7: Activity"
la var iv6_8 "IV6 Job 8: Activity"
la var iv6_9 "IV6 Job 9: Activity"
la var iv6other_7 "IV6 Job 7: Activity (other)"
la var iv6other_8 "IV6 Job 8: Activity (other)"
la var iv6other_9 "IV6 Job 9: Activity (other)"
la var iv7_7 "IV7 Job 7: Days worked in past 4 weeks"
la var iv7_8 "IV7 Job 8: Days worked in past 4 weeks"
la var iv7_9 "IV7 Job 9: Days worked in past 4 weeks"
la var iv8_7 "IV8 Job 7: Days worked in past 7 days"
la var iv8_8 "IV8 Job 8: Days worked in past 7 days"
la var iv8_9 "IV8 Job 9: Days worked in past 7 days"
la var iv9_7 "IV9 Job 7: Hours per day in last 7 years"
la var iv9_8 "IV9 Job 8: Hours per day in last 7 years"
la var iv9_9 "IV9 Job 9: Hours per day in last 7 years"
la var iv10_7 "IV10 Job 7: Paid?"
la var iv10_8 "IV10 Job 8: Paid?"
la var iv10_9 "IV10 Job 9: Paid?"

la var vi1 "VI1 Meals per day"
la var vi2 "VI2 Meat in past 2 wks"
la var vi11 "VI11 Illegal drug use?"

la var vii3 "VII3 People in closest group understand you?"
la var vii4 "VII4 Feel you are important part of closest group?"
la var vii5 "VII5 Had trouble getting along with others in closest group?"
la var vii6 "VII6 Consider others in closest group to be like brothers and sisters?"

la var viii2 "Recode: VIII2 Do you play games, sports, or go dancing with your friends? ever/often/sometimes/rarely"
la var viii3 "Recode: VIII3 Do you think you can do most things like others? ever/often/sometimes/rarely"
la var viii4 "Recode: VIII4 Are you jumpy? ever/often/sometimes/rarely"
la var viii5 "Recode: VIII5 Are you in constant fear of losing your family? ever/often/sometimes/rarely"
la var viii6 "Recode: VIII6 Do you feel lonely? ever/often/sometimes/rarely"
la var viii7 "Recode: VIII7 Do you destroy things that belong to others? ever/often/sometimes/rarely"
la var viii8 "Recode: VIII8 Do you enjoy doing things and talking with peers? ever/often/sometimes/rarely"
la var viii9 "Recode: VIII9 Are you helpful to elders? ever/often/sometimes/rarely"
la var viii10 "Recode: VIII10 Do you feel sympathy for others? ever/often/sometimes/rarely"
la var viii11 "Recode: VIII11 Do you worry about too many things? ever/often/sometimes/rarely"
la var viii12 "Recode: VIII12 Do you feel like you do not treasure your life? ever/often/sometimes/rarely"
la var viii13 "Recode: VIII13 Are you quarrelsome? ever/often/sometimes/rarely"
la var viii14 "Recode: VIII14 Do you feel unloved? ever/often/sometimes/rarely"
la var viii15 "Recode: VIII15 Are you confident doing things on your own? ever/often/sometimes/rarely"
la var viii16 "Recode: VIII16 Do you think that everything you do is wrong? ever/often/sometimes/rarely"
la var viii17 "Recode: VIII17 Do you lie or behave in a dishonest way? ever/often/sometimes/rarely"
la var viii18 "Recode: VIII18 Do you take things from other places without permission? ever/often/sometimes/rarely"
la var viii19 "Recode: VIII19 Do you disobey your parents/guardians, teachers or elders? ever/often/sometimes/rarely"
la var viii20 "Recode: VIII20 Do you enjoy doing things in the community? ever/often/sometimes/rarely"
la var viii21 "Recode: VIII21 Do you have love for your peers? ever/often/sometimes/rarely"
la var viii22 "Recode: VIII22 Are you satisfied with yourself? ever/often/sometimes/rarely"
la var viii23 "Recode: VIII23 Do you prefer being alone rather than being with family and friends? ever/often/sometimes/rarely"
la var viii24 "Recode: VIII24 Do you share with others? ever/often/sometimes/rarely"
la var viii25 "Recode: VIII25 Do you curse or use abusive language? ever/often/sometimes/rarely"
la var viii26 "Recode: VIII26 Do you cry when you remember bad things from the past? ever/often/sometimes/rarely"
la var viii27 "Recode: VIII27 Do you help younger ones? ever/often/sometimes/rarely"
la var viii28 "Recode: VIII28 Do you have difficulty concentrating? ever/often/sometimes/rarely"
la var viii29 "Recode: VIII29 Do you feel helpless? ever/often/sometimes/rarely"
la var viii30 "Recode: VIII30 Do you threaten to hurt others? ever/often/sometimes/rarely"
la var viii31 "Recode: VIII31 Do you share your feelings or ideas with your friends? ever/often/sometimes/rarely"
la var viii32 "Recode: VIII32 Do you keep to yourself when you are worried? ever/often/sometimes/rarely"
la var viii33 "Recode: VIII33 Do you have restless nights? ever/often/sometimes/rarely"
la var viii34 "Recode: VIII34 Do you think about bad things from the past? ever/often/sometimes/rarely"
la var viii35 "Recode: VIII35 Are you easily irritated? ever/often/sometimes/rarely"
la var viii36 "Recode: VIII36 Do you have nightmares? ever/often/sometimes/rarely"
la var viii37 "Recode: VIII37 Do you have confidence about your future? ever/often/sometimes/rarely"
la var viii38 "Recode: VIII38 Do you have confidence to be responsible for others? ever/often/sometimes/rarely"
la var viii39 "Recode: VIII39 Do you feel sad? ever/often/sometimes/rarely"
la var viii40 "Recode: VIII40 Do other youth like associating with you? ever/often/sometimes/rarely"
la var viii41 "Recode: VIII41 Do you find life difficult whether you are home or elsewhere? ever/often/sometimes/rarely"
la var viii42 "Recode: VIII42 Do you get chest pains when you are overthinking? ever/often/sometimes/rarely"
la var viii43 "Recode: VIII43 Do you get headaches from overthinking? ever/often/sometimes/rarely"
la var viii44 "Recode: VIII44 Does your body shake uncontrollably from overthinking? ever/often/sometimes/rarely"

la var ix4a "IX4a If willingly: Why?"
la var ix4b "IX4b If willingly: Why?"
la var ix4c "IX4c If willingly: Why?"
la var ix5a "IX5a Abduction place"
la var ix5b "IX5b Abduction place"
la var ix5c "IX5c Abduction place"
la var ix6a "IX6a Living at time of abduction"
la var ix6b "IX6b Living at time of abduction"
la var ix6c "IX6c Living at time of abduction"
la var ix7a "IX7a How many others abducted?"
la var ix7b "IX7b How many others abducted?"
la var ix7c "IX7c How many others abducted?"
la var ix8a "IX8a How many released right away/within a day?"
la var ix8b "IX8b How many released right away/within a day?"
la var ix8c "IX8c How many released right away/within a day?"

la var ix9a1 "IX9a Who/why released? (Reason 1)"
la var ix9a2 "IX9a Who/why released? (Reason 2)"
la var ix9a3 "IX9a Who/why released? (Reason 3)"
la var ix9aother "IX9a Who/why released? (Other Reason)"
la var ix9b1 "IX9b Who/why released? (Reason 1)"
la var ix9b2 "IX9b Who/why released? (Reason 2)"
la var ix9b3 "IX9b Who/why released? (Reason 3)"
la var ix9bother "IX9b Who/why released? (Other Reason)"
la var ix9c1 "IX9c Who/why released? (Reason 1)"
la var ix9c2 "IX9c Who/why released? (Reason 2)"
la var ix9c3 "IX9c Who/why released? (Reason 3)"
la var ix9cother "IX9c Who/why released? (Other Reason)"

la var ix41a1 "IX41 Reward received 1"
la var ix41a2 "IX41 Reward received 2"
la var ix41a3 "IX41 Reward received 3"
la var ix41a4 "IX41 Reward received 4"
la var ix41a5 "IX41 Reward received 5"
la var ix41a6 "IX41 Reward received 6"
la var ix41a6other "IX41 Reward received (other)"

la var ix44a1 "IX44 Reason for promotion: obedient"
la var ix44a2 "IX44 Reason for promotion: intelligent"
la var ix44a3 "IX44 Reason for promotion: strong"
la var ix44a4 "IX44 Reason for promotion: good fighter"
la var ix44a5 "IX44 Reason for promotion: others listened"
la var ix44a6 "IX44 Reason for promotion: killed officer"
la var ix44a7 "IX44 Reason for promotion: other"
la var ix44a7other "IX44 Reason for promotion: other (specify)"
la var ix44a99 "IX44 Reason for promotion: DK"

la var ix540 "IX54 Escape propaganda: Nothing"
la var ix550 "IX55 Escape propaganda repeated: Nothing"

la var ix63 "Recode: IX63 New abductees beaten? ever/often/sometimes/rarely"
la var ix64 "Recode: IX64 Other abductees forced to beat or kill new abductees? ever/often/sometimes/rarely"
la var ix65 "Recode: IX65 Fighters in your unit mutilate civilians? ever/often/sometimes/rarely"
la var ix66 "Recode: IX66 Abductees in your unit forced to desecrate dead bodies? ever/often/sometimes/rarely"
la var ix67 "Recode: IX67 Abductees in your unit forced to carry body parts? ever/often/sometimes/rarely"
la var ix68 "Recode: IX68 Abductees in your unit punished for no reason? ever/often/sometimes/rarely"
la var ix69 "Recode: IX69 Fighters punished for stealing? ever/often/sometimes/rarely"
la var ix70 "Recode: IX70 Fighters punished for beating/wounding another fighter? ever/often/sometimes/rarely"
la var ix71 "Recode: IX71 Fighters punished for beating/wounding abductee? ever/often/sometimes/rarely"
la var ix72 "Recode: IX72 Fighters punished for killing another fighter? ever/often/sometimes/rarely"
la var ix73 "Recode: IX73 Fighters punished for killing abductee? ever/often/sometimes/rarely"
la var ix74 "Recode: IX74 Fighters punished for killing civilians near base? ever/often/sometimes/rarely"
la var ix780 "IX78 Services after reception center: None"
la var ix781 "IX78 Services after reception center: Medical"
la var ix782 "IX78 Services after reception center: Counseling"
la var ix783 "IX78 Services after reception center: IGA"
la var ix785 "IX78 Services after reception center: School"
la var ix786 "IX78 Services after reception center: Amnesty package"
la var ix787 "IX78 Services after reception center: Other"
la var ix787other "IX78 Services after reception center: Other"

la var x48 "X48 Do you believe that bad things happened to you because you were cursed?"

la var xi1a "XI1a District"
la var xi1b "XI1b Camp"
la var xi1c "XI1c Sub-county or Town"
*la var xi1d "XI1c Parish"

la var xii91 "XII9 Others present: parent/grandparent/guardian"
la var xii92 "XII9 Others present: son/daughter"
la var xii93 "XII9 Others present: wife"
la var xii94 "XII9 Others present: village elder"
la var xii95 "XII9 Others present: guide"
la var xii97 "XII9 Others present: neighbor/co-worker"
la var xii98 "XII9 Others present: teacher/school official"
la var xii99 "XII9 Others present: friend"
la var xii910 "XII9 Others present: other"

// Label dummies

la var G1_14_17 "Age & Location indicator: camp 1, ages 14-17"
la var G1_18_21 "Age & Location indicator: camp 1, ages 18-21"
la var G1_22_25 "Age & Location indicator: camp 1, ages 22-25"
la var G1_26_30 "Age & Location indicator: camp 1, ages 26-30"
la var G2_14_17 "Age & Location indicator: camp 2, ages 14-17"
la var G2_18_21 "Age & Location indicator: camp 2, ages 18-21"
la var G2_22_25 "Age & Location indicator: camp 2, ages 22-25"
la var G2_26_30 "Age & Location indicator: camp 2, ages 26-30"
la var G3_14_17 "Age & Location indicator: camp 3, ages 14-17"
la var G3_18_21 "Age & Location indicator: camp 3, ages 18-21"
la var G3_22_25 "Age & Location indicator: camp 3, ages 22-25"
la var G3_26_30 "Age & Location indicator: camp 3, ages 26-30"
la var G4_14_17 "Age & Location indicator: camp 4, ages 14-17"
la var G4_18_21 "Age & Location indicator: camp 4, ages 18-21"
la var G4_22_25 "Age & Location indicator: camp 4, ages 22-25"
la var G4_26_30 "Age & Location indicator: camp 4, ages 26-30"
la var G5_14_17 "Age & Location indicator: camp 5, ages 14-17"
la var G5_18_21 "Age & Location indicator: camp 5, ages 18-21"
la var G5_22_25 "Age & Location indicator: camp 5, ages 22-25"
la var G5_26_30 "Age & Location indicator: camp 5, ages 26-30"
la var G6_14_17 "Age & Location indicator: camp 6, ages 14-17"
la var G6_18_21 "Age & Location indicator: camp 6, ages 18-21"
la var G6_22_25 "Age & Location indicator: camp 6, ages 22-25"
la var G6_26_30 "Age & Location indicator: camp 6, ages 26-30"
la var G7_14_17 "Age & Location indicator: camp 7, ages 14-17"
la var G7_18_21 "Age & Location indicator: camp 7, ages 18-21"
la var G7_22_25 "Age & Location indicator: camp 7, ages 22-25"
la var G7_26_30 "Age & Location indicator: camp 7, ages 14-17"
la var G8_14_17 "Age & Location indicator: camp 8, ages 14-17"
la var G8_18_21 "Age & Location indicator: camp 8, ages 18-21"
la var G8_22_25 "Age & Location indicator: camp 8, ages 22-25"
la var G8_26_30 "Age & Location indicator: camp 8, ages 14-17"

// Dividers

gen SURVEY_SECTION_I=.
la var SURVEY_SECTION_I "========================================"
gen SURVEY_SECTION_II=.
la var SURVEY_SECTION_II "========================================"
gen SURVEY_SECTION_III=.
la var SURVEY_SECTION_III "========================================"
gen SURVEY_SECTION_IV=.
la var SURVEY_SECTION_IV "========================================"
gen SURVEY_SECTION_V=.
la var SURVEY_SECTION_V "========================================"
gen SURVEY_SECTION_VI=.
la var SURVEY_SECTION_VI "========================================"
gen SURVEY_SECTION_VII=.
la var SURVEY_SECTION_VII "========================================"
gen SURVEY_SECTION_VIII=.
la var SURVEY_SECTION_VIII "========================================"
gen SURVEY_SECTION_IX=.
la var SURVEY_SECTION_IX "========================================"
gen SURVEY_SECTION_X=.
la var SURVEY_SECTION_X "========================================"
gen SURVEY_SECTION_XI=.
la var SURVEY_SECTION_XI "========================================"
gen SURVEY_SECTION_XII=.
la var SURVEY_SECTION_XII "========================================"

gen HH_SURVEY_QUESTIONS=.
la var HH_SURVEY_QUESTIONS "========================================"
gen ABS_SURVEY_QUESTIONS=.
la var ABS_SURVEY_QUESTIONS "========================================"

gen IDENTIFYING_VARIABLES=.
la var IDENTIFYING_VARIABLES "========================================"
gen EMPLOYMENT_VARIABLES=.
la var EMPLOYMENT_VARIABLES "========================================"
gen EDUCATION_VARIABLES=.
la var EDUCATION_VARIABLES "========================================"
gen HEALTH_VARIABLES=.
la var HEALTH_VARIABLES "========================================"
gen ABDUCTION_VARIABLES=.
la var ABDUCTION_VARIABLES "========================================"
gen WAR_EXPERIENCES_VARIABLES=.
la var WAR_EXPERIENCES_VARIABLES "========================================"
gen RETURN_EXPERIENCES_VARIABLES=.
la var RETURN_EXPERIENCES_VARIABLES "========================================"
gen FAMILY_COMMUNITY_VARIABLES=.
la var FAMILY_COMMUNITY_VARIABLES "========================================"
gen SOC_POL_PARTICIPATION_VARIABLES=.
la var SOC_POL_PARTICIPATION_VARIABLES "========================================"
gen HH_BACKGROUND_VARIABLES=.
la var HH_BACKGROUND_VARIABLES "========================================"
gen WEIGHTS=.
la var WEIGHTS "========================================"
gen LOCATION_AGE_ABDYR_INDICATORS=.
la var LOCATION_AGE_ABDYR_INDICATORS"========================================"

//Order variables 

**HH survey questions

order HH_SURVEY_QUESTIONS hhi3aday hhi3amo hhi3ayr hhi3bhr hhi3bmin hhi5 hhi6 hhi7mo hhi7yr hhi8 hhi8_small hhi9 hhi10 hhi11 /*
*/ hhi12 hhi12other hhi16hr hhi16min int_lgth_hhi hhiia hhiic hhiid hhiie hhiif hhiig hhiih_lgth hhiii_lgth hhiih_yr hhiii_yr /*
*/ hhiij hhiik hhiil hhiiage hh_adults05 hhsize hh_size96 hh_size05 hh_assets96 hh_wealth96 hh_fthr_frm hh_fthr_pro hh_fthr_sol/*
*/ hh_land hh_plow hh_cattle hh_stock hh_avg_educ hh_abd_total hh_abd_youth hh_abd_lgth hh_war_death causedeath hh_immed_fam hh_wfp hh_no_info 
 
**Absentee survey questionS
order ABS_SURVEY_QUESTIONS q5 q6 q7 q8 q10 q12a q12aotherspecify q12b q12bother q12c q14 q15 q15other /*
*/ q16 q17 q18 q19 q20 q21 q22 q23 q24 q25a q25b q25c q25d q25other q26 q27 q28 q29 

**Survey questions
order SURVEY_SECTION_XII xii6hr - xii6_length xii7 xii8 xii91 - xii910 xii10 - xii14 
order SURVEY_SECTION_XI xi1a - xi1c 
order SURVEY_SECTION_X x2 x3 x4 x5 x6 x7 x8 x9 x10 x11 x12 x13 x14 x15 x16 x17 x18 x19 x20 x21 x22 x23 x24 x25 x26 x27 x28 x29 x20 x31 x32 x33 x34a x34b x34c x35 x36 x37 x38 x39 x40 x41 x42 x43 x44 x45 x46 x47 x48 x49 x50_minded - x66_chores
order SURVEY_SECTION_IX  ix1 - ix29c  ix31 ix32 ix331 - ix333 ix34 - ix52 ix53_commander ix540 - ix558 ix56 - ix61 ix620 - ix629other /*
*/ ix63 - ix74 ix63a - ix76kother ix771 - ix787other ix79 - ix91
order SURVEY_SECTION_VIII viii2 - viii44  viii451 - viii4512other
order SURVEY_SECTION_VII vii1a - vii22
order SURVEY_SECTION_VI vi1 - vi11
order SURVEY_SECTION_V  v1 v2 v3_walk - v10_bend v11 - v15
order SURVEY_SECTION_IV iv1 - iv15
order SURVEY_SECTION_III iii1 - iii15b
order SURVEY_SECTION_II  ii1 - ii30other
order SURVEY_SECTION_I i1 - i8

**Weights and control dummies
order WEIGHTS w_samp w_abs w_abs2 weighthh w_sel_abs w_sel i_weight f_weight f_weight_abs hh_select hh_weight pscore pscore2 stratum dup
order LOCATION_AGE_ABDYR_INDICATORS C_ach - C_pal A14 - A35 G1_14_17 - G8_26_30 abd_l_1d - abd_l2_12up abd5_12 - abd22_30 Y1994 - Y2004

**HH and background characteristics 
order HH_BACKGROUND_VARIABLES age_cat aindex assets catholic church fthr_ed fthr_ed0 fthr_ed4 fthr_ed7 fthr_edhi mthr_ed mthr_ed0 /*
*/ mthr_ed4 mthr_ed7 mthr_edhi no_fthr no_fthr02 no_fthr96 no_fthr99 no_mthr no_mthr02 no_mthr96 no_mthr99 orphan orphan96 live_mthr /*
*/ live_fthr live_par tooyoung asset_50th asset_75th asset_90th w10 w25 w75 w90 dry town army noarmy

**Social and political participation
order SOC_POL_PARTICIPATION_VARIABLES groups group g_peace g_drama g_farm g_water g_skool g_sport g_church g_oth /*
*/ comm_mobil volunteer pol_job vote01 vote05

**Family/community problems
order FAMILY_COMMUNITY_VARIABLES troublefam troublecomm neighborsbad elders_disobey fam_comfort fam_quarrel fam_caring /*
*/ fam_problems fam_connect fam_prob_ever fam_prob_now fam_prob_better comm_prob_ever comm_prob_now comm_prob_better 

**Return experiences
order RETURN_EXPERIENCES_VARIABLES rescue rescued released released_1w released_2w escaped reception amnesty updf ret_skool ret_home ret_to_parent /*
*/ ret_par_hw ret_pr_gr_hw ret_pr_gr_hw_sib back_skool services allegiance_total allegiance_modhi blame blame_self /*
*/ parents konyblame blame_total blame_total_modhi blame_total_hi punish godsplan cursed guilt saved haunted /*
*/ distress prosocial hostility hostile support 

**War experiences
order threats threats_only indoctrinate stay important command lead ranked leader kill kill_sol courageous 
order viol_perp viol_total tied carry abusedead harm_fam gun gun_keep gun_mo ceremony depend rewarded safer magic loyal loyal_now loyal_often
order WAR_EXPERIENCES_VARIABLES viol_received viol_witnessed viol_experienced viol_loss viol_personal

**Abduction information
order ABDUCTION_VARIABLES abd nonabd yr_ret longest_abd abd_l abd_lgth lt_abd abd_age abd_yr yr_abd twoweek back abd_per abd_per_a /*
*/ abdage abd_child abd_adult abd_age_a abd_age_b abd_age_c abd_age abd_lgth_a abd_lgth_b abd_lgth_c abd2 abd3 abd3 /*
*/ labd_l labd_lgth_a labd_lgth age_back age_back2 age_bk abd_no_mthr abd_no_fthr abd_sing_par abd_orph

**Health 
order HEALTH_VARIABLES health bad_hlth poor_health hindex ill v_ill injury lra_inj head_inj fight no_dig married children /*
*/ sex_active hiv_aware condom own_condom use_cond_some use_cond_often use_cond_oth sleeps_around sleeps_around2 /*
*/ drinks drinks_hvy smoke eatmeat eattwice beat beat_neighbor beat_wife beat0 beat95 beat90 

**Education
order EDUCATION_VARIABLES educ educ_cat educhat illiterate literate inschool ngo_paid p1 p4 p7 educ0 secskool university voc voc2

**Livelihoods
order EMPLOYMENT_VARIABLES emp_mo emp_wk emp_d_mo emp_d_wk emp_hrs_wk emp_hrs_mo emp_dhat earn_mo earn_wk earnhat earn0 /*
*/ wage_mo lwage_mo wage_wk lwage_wk mainjob occup occup2 unocc job job_skill job_skill2 prof military exper land_access log_land 

order IDENTIFYING_VARIABLES id id2 district camp parish block village household person died no_return a_id absentee sample found /*
*/ nosurvive age adult child incamp camp_yrs out_district

// Drop variables

drop viii2a - viii44b x50a - x66b ix63a - ix69b  ix70a - ix74b
drop v3a v3b v4a v4b v5a v5b v6a v6b v7a v7b v8a v8b v9a v9b v10a v10b
drop risky_job risky_job2 skill1 skill2 ix29
drop hh_size96_2 hh_size96_3 hh_size96_4 hh_land_2 hh_land_3 hh_land_4 hh_cattle_2 hh_cattle_3 hh_cattle_4
drop hh_stock_2 hh_stock_3 hh_stock_4 educ_2 educ_3 educ_4 abd_age_2 abd_age_3 abd_age_4 abd_lgth_2 abd_lgth_3 abd_lgth_4 
drop UC_ach1994 - UC_pal2004 
drop labd_lgth lwage_mo lwage_wk labd_l leduc log_land

**For anonymity

*drop q2a q2b q3a q3b q4a q4b 
*drop q12d q12e q12g q12i q12h q13a q13b q13c q13d q13e q13f
drop hhiib  hhiib_05 hhiiii
*drop firstname1 firstname2 firstname3 surname1 surname2 surname3
*drop xi1e xi1g xi1h xi2 xi3 xi3a xi3b xi3c xi3d xi3f xi3g xi3i xi4 xi5 xi6a xi6c xi6d
*drop xi6e xi6f xi6g xi6i xi6j xi6k xi7a xi7c xi7d xi7e xi7f xi7g xi7i xi7j xi7k
drop xi1a xi1b xi1c SURVEY_SECTION_XI
drop xii12

// Save Phase I dataset
sa "SWAYI/SWAY_I_April2013", replace


