* DATA CREATION AND RELABELING FILE FOR SWAY-II DATA

/* This file contains code to create the SWAY-II dataset. It includes:
		 1) The creation code (SWAY_II_create27Jan2009.do)
		 2) The data relabeling code (SWAY_II_relabeling27Jan2009.do)
		*/ 



** USER SETTINGS **
    
    cd "YOUR DIRECTORY\SWAY.Replication.June2013\Data\Raw data and cleaning\"

		
    clear
    set mem 30m
    set more off

// Add current location variables
    insheet using "SWAYII/rawdata/location.csv", clear comma
    foreach i in main_camp sat_camp home town dk {
       ren l_`i' L_`i'
       replace L_`i'=0 if L_`i'==.
       }
    sort qno
    sa "SWAYII/rawdata/location.dta", replace

    insheet using "SWAYII/rawdata/qiv17alternate.csv", clear comma
    sort qno
    sa "SWAYII/rawdata/qiv17.dta", replace

    use "SWAYII/rawdata/sway_ii_final_27Sept2007.dta", clear
    sort qno
    merge qno using "SWAYII/rawdata/location"
    drop _merge 
    sort qno
    merge qno using "SWAYII/rawdata/qiv17"
    drop _merge 

// Add counseling variables
	sort qno
	merge qno using "SWAYII/rawdata/counseling.dta"
	drop _merge

** HIGHLIGHT "DON'T KNOW" **

    cap pr drop dkmiss
    pr de dkmiss
        syntax varname [if] [in]
        cap gen DK_`varlist'=.
        qui replace DK_`varlist'=.d `if' `in'
        cap replace `varlist'=. `if' `in'
        cap replace `varlist'="" `if' `in'
    end

// Search for values of 88 and 99 that signify "don't know"

    foreach j of varlist qi2a-qi2b qi3-qii16 qii18-qiv19 qiv21-qvi4 qvi5b-qxii1  qxiii6-qxiii14 {
     cap dkmiss `j' if `j'=="88"
     cap dkmiss `j' if `j'==88
    }
    foreach j of varlist qi2a-qi2b qi3-qiv19 qiv21-qxiii14 qix2a qix2b qix2c qix9a qix9b qix9c {
     cap dkmiss `j' if `j'=="99"
     cap dkmiss `j' if `j'==99
    }

// Replace "don't know" with .d, so they'll retain informative content
    foreach i of varlist * {
     cap replace `i'=.d if DK_`i'==.d
     cap replace `i'=".d" if DK_`i'==.d
    }

drop DK_*


** DATA CLEANING **

// Strip all leading and trailing blanks in dataset
    for varlist *, noh: cap replace X=trim(X)

// Get rid of any quotes
    foreach i of varlist * {
     cap replace `i'=subinstr(`i',"`","",.)
     cap replace `i'=subinstr(`i',"'","",.)
    }

// Make missing string values consistent
    for varlist *, noh: cap replace X="" if X=="."

// Check survey identifier
    destring qi1, replace
    replace qi1=1270 if qno==78
    drop if qno==489
    replace qi1=1374 if qi1==374

// Incorrectly named location variables
    foreach i in a b c d {
     cap ren q2`i' qi2`i'
    }

// Location description
    *replace qi2=subinstr(qi2, ",", ", ", .)
    *replace qi2=subinstr(qi2, "  ", " ", .)
    *replace qi2=proper(qi2)
    *replace qi2=trim(qi2)

// Correct codes
    replace qi2a=1 if qi2a==14323 & qi1==1507
    replace qi2b=4 if qi2b==10 & qi1==1285
    replace qi2b=4 if qi2b==2 & qi1==1224
    replace qi2b=1 if qi2b==. & qi1==1264
    replace qi2b=1 if qi2b==2 & qi1==1021
    replace qi3="566" if qi3=="" & qi1==1345
    replace qi3="581" if qi1==1366 & qi3=="551"
    replace qi4="4" if qi1==1085 & qi4==""
    replace qi4="8" if qi1==1210 & qi4==""
    replace qi4="5" if qi1==1268 & qi4==""
    replace qi4="2" if qi1==1292 & qi4==""
    replace qi4="17" if qi1==1326 & qi4==""
    replace qi4="1" if qi1==1366 & qi4==""
    replace qi4="9" if qi1==1372 & qi4==""
    replace qi4="4" if qi1==1415 & qi4==""
    replace qi4="1" if qi1==1439 & qi4==""
    replace qi4="5" if qi1==1484 & qi4==""
    replace qi4="13" if qi1==1511 & qi4==""
    replace qi4="18" if qi1==1519 & qi4==""
    replace qi4="2" if qi1==1601 & qi4==""
    replace qi4="3" if qi1==1614 & qi4==""
    replace qi4="2" if qi1==1644 & qi4==""
    replace qi4="2" if qi1==1723 & qi4==""
    replace qi4="2" if qi1==1782 & qi4==""
    replace qi4="9" if qi1==1786 & qi4==""
    replace qi4="15" if qi1==1797 & qi4==""

// Enumerator codes
    replace qi8a="1" if qi8a=="o1"
    replace qi8a="4" if qi8a=="14" & qi1==1554


// Roster row complete
    replace qi9="2" if qi9=="3" & qi1==1750
    replace qi9="2" if qi9=="" & (qi1==1173 | qi1==1723 | qi1==1782 | qi1==1783)

// Permission to interview
    replace qi5="1" if (qi5=="" | qi5=="2")  & qii1!=.
    replace qi5="1" if qi5=="" & qii1!=. & qiii1!=. & qiv1!=. & qv1!=. & qvi1!=. & qvii1!=. & qviii1a!=. & qix1!=. /*
                       */ & qx2a!=. & qxi1!=.

// Age
    replace qii1=26 if qii1==261 & qi1==1433
    // respondent 1202 reports 1989 as year of birth, father died in 1995, so reported age of 10 appears incorrect
    replace qii1=17 if qii1==10 & qi1==1202
    // respondent 1462 reports 1983 as year of birth
    replace qii1=23 if qii1==. & qi1==1462
    // respondent 1126 reports age 12 at interview, 1973 as year of birth, and age 11 when abducted in 2003
    // for now, set age to 15
    replace qii1=15 if qii1==12 & qi1==1126

// Father's year of death
    replace qii5=qii4 if qii5==. & qii4>1960 & qii4<=2007
    replace qii4=. if qii4>1960 & qii4<=2007
    replace qii5=1992 if qii5==19992
    replace qii5=. if (qii5==0 | qii5==1) & qii3==2

// Father alive
    replace qii6=. if qii6==0
    replace qii3=2 if qii3==. & qii5>1960 & qii5<=2007
    replace qii3=2 if qii3==. & qii6>=1 & qii6<=4
    replace qii3=2 if qii3==1 & qii5>1960 & qii5<=2007 & qii4==.
    replace qii3=2 if qii3==1 & qii6>=1 & qii6<=4 & qii4==.
    replace qii3=1 if (qii3==2 | qii3==. | qii3==99) & qii5==. & qii6==. & (qii4==1 | qii4==2)
    replace qii5=. if qii3==1
    replace qii6=. if qii3==1
    replace qii4=. if qii3==2 | qii3==3 | qii3==4

// Mother alive
    replace qii11=. if qii11==0
    replace qii11=1 if qii10==1 & qi1==1075
    replace qii10=. if qii10==1 & qi1==1075
    replace qii8=2 if qii8==. & qii10>1960 & qii10<=2007
    replace qii8=2 if qii8==. & qii11>=1 & qii11<=4
    replace qii8=2 if qii8==1 & qii10>1960 & qii10<=2007 & qii9==.
    replace qii8=2 if qii8==1 & qii11>=1 & qii11<=4 & qii9==.
    replace qii8=1 if (qii8==2 | qii8==.) & qii10==. & qii11==. & (qii9==1 | qii9==2)
    replace qii10=. if qii8==1
    replace qii11=. if qii8==1
    replace qii9=. if qii8==2 | qii8==3 | qii8==4

// People in household
    replace qii13b=2 if qii13b==12 & qi1==1342
    foreach i in a b c d {
     for any a b c d: replace qii13X=. if qii13X==8 & ((qii13`i'>0 & qii13`i'<8) | qii13`i'==9)
    }

// Accommodation
    replace qii15=1 if qii15==12 & qi1==1437
    replace qii16=1 if qii16==10 & qi1==1553
    replace qii17=. if qii17==0 & qi1==1224
    replace qii18=. if qii18==0 & qi1==1224

// Decision to have children
    replace qii22=. if qii22==0 & qi1==1487

// Marital status
    replace qii37=2 if qii37==6 & (qi1==1028 | qi1==1529)

// Last year enrolled in school
    replace qiii9=8888 if qiii9==8885 | qiii9==88888 | qiii9==0

// Job
    for any a b c d e: replace qiv3X=upper(qiv3X)
    replace qiv3b="B" if qiv3b=="2"

    cou if qiv3c=="3" & qiv4c==1 & qiv5c==3 & qiv6c==. & qi1==1073
    if r(N)>0 {
     replace qiv3c="" if qi1==1073
     replace qiv4c=3 if qi1==1073
     replace qiv5c=1 if qi1==1073
     replace qiv6c=3 if qi1==1073
    }

    cou if qiv3d=="1" & qiv4d==2 & qiv5d==4 & qiv6d==. & qi1==3038
    if r(N)>0 {
     replace qiv3d="" if qi1==3038
     replace qiv4d=. if qi1==3038
     replace qiv5d=. if qi1==3038
    }

    cou if qiv3d=="4" & qiv4d==0 & qiv5d==7 & qiv6d==. & qi1==1646
    if r(N)>0 {
     replace qiv3d="" if qi1==1646
     replace qiv4d=4 if qi1==1646
     replace qiv5d=0 if qi1==1646
     replace qiv6d=7 if qi1==1646
    }

    cou if qiv3d=="4" & qiv4d==1 & qiv5d==8 & qiv6d==6000 & qi1==1250
    if r(N)>0 {
     replace qiv3d="" if qi1==1250
     replace qiv4d=4 if qi1==1250
     replace qiv5d=1 if qi1==1250
     replace qiv6d=8 if qi1==1250
     replace qiv7d=6000 if qi1==1250
    }

    replace qiv3e="J" if qiv3e=="Y" & qi1==1689
    replace qiv3a="A" if qiv3a=="" & qiv3b=="B" & (qiv4a!=. | qiv5a!=.)
    replace qiv3b="B" if qiv3b=="" & qiv9b2=="water" & (qiv4b!=. | qiv5b!=.)

// Days worked
    replace qiv4d=. if qiv4d==0 & qiv3d=="" & qi1==1219
    replace qiv4a=12 if qiv4a==0 & qiv5a==3 & qi1==1147
    replace qiv4b=. if qiv4b==1 & qiv5b==2 & qi1==1644

    gen temp=1 if qiv4a==7 & qiv5a==24 & qi1==1333
    replace qiv4a=24 if temp==1
    replace qiv5a=7 if temp==1
    drop temp

    foreach y in a b c d e {
     gen temp=1 if (qiv4`y'==28 & qiv5`y'==78) | (qiv4`y'==2 & qiv5`y'==7) | (qiv4`y'==7 & qiv5`y'==28) | (qiv4`y'==0 & qiv5`y'==7) | (qiv4`y'==. & qiv5`y'==28)
     replace qiv5`y'=7 if temp==1
     replace qiv4`y'=28 if temp==1
     drop temp
    }

    foreach y in a b c d e {
     gen temp=1 if qiv5`y'>qiv4`y' & qiv5`y'!=.
     gen temp2=qiv5`y'
     replace qiv5`y'=qiv4`y' if temp==1
     replace qiv4`y'=temp2 if temp==1
     drop temp temp2
    }

    foreach y in a b c d e {
     gen temp=1 if qiv5`y'>7 & qiv5`y'!=.
     gen temp2=1 if qiv4`y'>28 & qiv4`y'!=.
     replace qiv5`y'=7 if temp==1
     replace qiv4`y'=28 if temp2==1
     drop temp temp2
    }

// Hours worked
    foreach y in a b c d e {
     replace qiv7`y'=qiv6`y' if qiv6`y'>=500 & qiv6`y'!=. & qiv7`y'==.
     replace qiv6`y'=. if qiv6`y'>=500 & qiv6`y'!=.
    }

// Monthly and weekly earnings
    egen temp=rsum(qiv7a qiv7b qiv7c qiv7d qiv7e) if qno!=.
    replace qiv2=temp if (qiv2==. | qiv2/10==temp) & qno!=.
    replace qiv1=4*qiv2 if qiv1==. & qno!=.
    drop temp

// Food and drink
    replace qvi9ii=real(qvi9b) if qvi9ii==. & (qvi9b=="2" | qvi9b=="5")
    replace qvi9b="" if qvi9b=="2" | qvi9b=="5"
    replace qvi10=. if qvi10==8
    replace qvi10=1 if qvi10==. & (qvi11==1 | qvi11==2 | qvi11==3)
    replace qvi10=1 if qvi10==2 & (qvi11==1 | qvi11==2)
    replace qvi12=1 if qvi12==. & qvi13!=""
    replace qvi14=1 if (qvi14==. | qvi14==2) & (qvi15==1 | qvi15==2 | qvi15==3)
    replace qvi15=0 if qvi14==2
    replace qvi16=1 if (qvi16==. | qvi16==2) & (qvi18==1 | qvi18==2 | qvi18==3 | qvi17!="")
    replace qvi18=0 if qvi16==2
    replace qvi19=. if qvi19==8
    replace qvi19=1 if qvi19==. & qvi21!=""
    replace qvi20a=. if qvi19==2
    replace qvi22=1 if qvi23!=""
    replace qvi24=1 if qvi25!=""
    replace qvi26=1 if qvi27!=""
    replace qvi22=1 if qvi23!=""

    la var qvi18 "how often did you receive non-food items"
    la var qvi16 "have you ever received any non-food household items from an ngo or cbo?"

// Psychosocial adjustment
    foreach x of numlist 2/44 {
        replace qvii`x'a=1 if (qvii`x'a==. | qvii`x'a==.d) & (qvii`x'b==1 | qvii`x'b==2 | qvii`x'b==3)
        replace qvii`x'a=1 if qvii`x'a==3 & (qvii`x'b==1 | qvii`x'b==2 | qvii`x'b==3)
        replace qvii`x'a=2 if (qvii`x'a==. | qvii`x'a==.d) & (qvii`x'b==. | qvii`x'b==.d)
        replace qvii`x'a=2 if qvii`x'a==3 & (qvii`x'b==. | qvii`x'b==.d)
    }

// Domestic and community violence
    replace qviii1a=3 if qviii1a==13 & qi1==1728
    replace qviii1a=3 if qviii1a==34 & qi1==1082
    replace qviii9=2 if qviii9==3 & qi1==1643
    replace qviii10="" if qviii10=="2" & qi1==1643
    replace qviii11=2 if qviii11==6 & qi1==1643
    replace qviii14=. if qviii14==0 & qi1==1324
    replace qviii21=2 if qviii21==23 & qi1==1433
    replace qviii21=3 if qviii22==0
    replace qviii22=0 if qviii21==3
    la de qviii22 0 "Never" 1 "1-5" 2 "6-10" 3 ">10"
    la val qviii22 qviii22
    replace qviii26a=. if qviii26a==0 & qi1==1107
    replace qviii26b=. if qviii26b==0 & qi1==1107
    replace qviii26a=10 if qviii26a==11 & qi1==1592
    replace qviii26c=. if qviii26c==0 & qi1==1188

// Times abducted
    replace qix1=0 if qix1==.
    replace qix1=0 if qix3a==. & qix4a==. & qix10a1==. & qix11a==.

// Erroneous entries
    replace qix3c=. if qix3c==3 & qi1==1782

    foreach i in 2 3 4 5 6 7 8 9 11 12 13 14 15 16 17 19 21 22 23 25 26 27 {
     foreach j in a b c {
      cap replace qix`i'`j'=. if qix1==0
      cap replace qix`i'`j'="" if qix1==0
     }
     foreach j in b c {
      cap replace qix`i'`j'=. if qix1==1
      cap replace qix`i'`j'="" if qix1==1
     }
     cap replace qix`i'c=. if qix1==2
     cap replace qix`i'c="" if qix1==2
    }
    foreach i in 2aii 4a2 9aii 10a1 10a2 14a2 16a2 18a1 18a2 20a1 20a2 21aii 22a2 23a2 23a3 24a1 24a2 24a1i {
     cap replace qix`i'=. if qix1==0
     cap replace qix`i'="" if qix1==0
    }
    foreach i in 2bii 4b2 9bii 10b1 10b2 14b2 16b2 18b1 18b2 20b1 20b2 21b2 22b2 23b2 24b1 24b2 24b {
     cap replace qix`i'=. if qix1==0 | qix1==1
     cap replace qix`i'="" if qix1==0 | qix1==1
    }
    foreach i in 2cii 4c2 9cii 10c1 10c2 14c2 16c2 18c1 18c2 20c1 20c2 21c2 22c2 23c2 24c1 24c2 24c {
     cap replace qix`i'=. if qix1==0 | qix1==1 | qix1==2
     cap replace qix`i'="" if qix1==0 | qix1==1 | qix1==2
    }
    foreach i in 28 29a 29b 29c 29d 29 30 31 32 33 33b 34 36 36b 37a 37b 37c 38 39 40a 40b 40c 40d 40 41a 41b /*
     */ 41c 41d 41 42 43 44 45 46 46b 47 48 49 50a 50b 51 52 53 54 55 56 57 58 59 60 61 62a 62b 62c 62 63 64 /*
     */ 65a 65b 65c 65 66 67 68 69a 69b 69c 70 71 72 72b 73 74a 74b 74c 74d 74 75 76a 76b 76c 76d 76 77 /*
     */ 78 79 80 81 82 83 84 85 85ii 85iii 85b 86 87 88 89 90 90ii 90iii 90b 91 92 92ii 93 {
      cap replace qix`i'=. if qix1==0
      cap replace qix`i'="" if qix1==0
    }
    foreach i of numlist 94/111 {
     foreach j in a b {
      cap replace qix`i'`j'=. if qix1==0
      cap replace qix`i'`j'="" if qix1==0
     }
    }

// Place of abduction
    replace qix4a=. if qix4a==0
    replace qix4b=. if qix4b==0
    replace qix4b=real(qix4a2) if qix4b==. & regexm(qix4a2, "^[1-5]$")
    replace qix4a2="" if regexm(qix4a2, "^[1-5]$")
    replace qix4c=real(qix4b2) if qix4c==. & regexm(qix4b2, "^[1-5]$")
    replace qix4b2="" if regexm(qix4b2, "^[1-5]$")
    replace qix4a2="" if qix4a>=1 & qix4a<=4

// Length and dates of first abduction
    replace qix9aii=1989 if qix9aii==1889  
    replace qix9aii=1997 if qno==267  
    replace qix9aii=2003 if qno==5  
    replace qix9aii=2003 if qno==481  
    replace qix9bii=2003 if qno==390  
    
// Length of abduction (missing unit)
    replace qix10a2=4 if qix10a2==. & qix10a1==8 & qi1==1744
    replace qix10a2=4 if qix10a2==7

// Education at abduction
    replace qix5a=0 if qiii8==0 & qix5a==. & qix1>0 & qix1!=.
    replace qix5b=0 if qiii8==0 & qix5b==. & qix1>1 & qix1!=.
    replace qix5c=0 if qiii8==0 & qix5c==. & qix1>2 & qix1!=.

// Rescue, release, escape
    replace qix16a=3 if qix16a==18 & qix16a2=="3"
    replace qix11a=2 if qix11a==3 & qix13a<=4 & qix13a>=1 & qix14a==.
    replace qix11a=3 if qix11a==2 & qix14a<=5 & qix14a>=1 & qix13a==.
    foreach z in a b c {
        replace qix11`z'=1 if qix11`z'==. & qix1>0 & qix1!=. & qno!=. & qix12`z'!=""
        replace qix11`z'=2 if qix11`z'==. & qix1>0 & qix1!=. & qno!=. & qix13`z'!=.
        replace qix11`z'=3 if qix11`z'==. & qix1>0 & qix1!=. & qno!=. & (qix14`z'!=. | qix15`z'!=. | qix16`z'!=.)
        replace qix12`z'="" if qix11`z'==2 | qix11`z'==3
        replace qix13`z'=. if qix11`z'==1 | qix11`z'==3
        replace qix14`z'=. if qix11`z'==1 | qix11`z'==2
        replace qix15`z'=. if qix11`z'==1 | qix11`z'==2
        replace qix16`z'=. if qix11`z'==1 | qix11`z'==2
        }

// UPDF
    foreach y in a b c {
        replace qix18`y'1=qix18`y'2 if qix18`y'1==0 & qix18`y'2>0 & qix18`y'2<5
        foreach z of numlist 0/4 {
            qui replace qix18`y'1=`z' if qix18`y'1==. & qix18`y'2==`z'
            qui replace qix18`y'2=. if qix18`y'1==`z' & qix18`y'2==`z'
            }
        }

// Reception center
    replace qix22a=0 if qix22a==. & (qix24a1!=. | qix24a2!=.)
    replace qix22b=0 if qix22b==. & (qix24b1!=. | qix24b2!=.)
    replace qix22c=0 if qix22c==. & (qix24c1!=. | qix24c2!=.)

    foreach y in a b c {
        foreach z of numlist 1/7 {
            qui replace qix24`y'1=`z' if qix24`y'1==. & qix24`y'2==`z'
            qui replace qix24`y'2=. if qix24`y'1==`z' & qix24`y'2==`z'
            }
        }

// Abducted for longer than two weeks
    // Questionnaire for respondent 1634 skips section following qix35, but then
    // starts again with qix77 (after a set of boxed questions), probably because of enumerator error
    replace qix35=0 if qix35==. & qi1==1634

// Role in LRA
    replace qix36=12 if qix36b=="collecting food" | qix36b=="looking for food from civilians"
    replace qix36=1 if qix36b=="training to be afighter" | qix36b=="Looter"
    replace qix36=10 if qix36b=="Escot to to a commander"
    replace qix36=1 if qix36b=="Commander"
    replace qix37a=. if qix37a==0
    replace qix37a=10 if qix37=="Serving commanders"
    replace qix37c=2 if qix37=="2" & qix37c==. & qix37a!=. & qix37b!=.
    replace qix37="" if qix37=="None"
    replace qix36b="" if qix36>=1 & qix36<=11
    replace qix37="" if qix37a!=12 & qix37b!=12 & qix37c!=12

    for any a b c: replace qix37X=. if qix37X==qix36 & qix36!=.
    for any b c: replace qix37X=. if qix37X==qix37a & qix37a!=.

    replace qix37b=qix37c if qix37b==. & qix37c!=.
    replace qix37c=. if qix37c==qix37b & qix37b!=.
    replace qix37a=qix37b if qix37a==. & qix37b!=.
    replace qix37b=. if qix37b==qix37a & qix37a!=.
    replace qix36=qix37a if qix36==. & qix37a!=.
    replace qix37a=. if qix36==qix37a & qix36!=.

// Forced marriage
    replace qix47=1 if qix48!=. | qix49!=.
    replace qix47=2 if qix47==. & (qix36!=. | qix42!=.) & (qix48==. & qix49==.)
    foreach i in 48 49 50a 50b 51 52 53 54 55 56 57 58 59 60 61 62a 62b 62c 62 63 64 65a 65b /*
     */ 65c 65 66 67 68 69a 69b 69c 70 71 72 72b 73 74a 74b 74c 74d 74 75 76a 76b 76c 76d 76 {
     cap replace qix`i'=. if qix47==2
     cap replace qix`i'="" if qix47==2
    }

// Age at time of forced marriage
    // respondent was 12 when abducted in 1993, reports being forcibly married in 1995 (although after 4 yrs with LRA)
    replace qix49=14 if qix49==1 & qi1==1440

// How long with LRA at time of forcible marriage (unit)
    replace qix50b=4 if qix50b==. & qi1==1571
    replace qix50b=4 if qix50b==. & qi1==1744
    replace qix50a=6 if qix50a==4 & qi1==1744

// Number of wives
    replace qix51=1 if qix51==0 & qix47==1
    la var qix51 "how many wives did this fighter have?"

// Bore child in captivity
    replace qix52=1 if qix52==0 & qi1==3057
    replace qix53=0 if qix52==2

// Who should be responsible for child
    replace qix62c=5 if qix62a==1095 & qix62c==.
    replace qix62b=9 if qix62a==1095 & qix62b==.
    replace qix62a=10 if qix62a==1095

// Experience in unit
    foreach i of numlist 94/111 {
     replace qix`i'b=. if qix`i'a==2
    }

// Group membership
    replace qxi4l=. if qxi4l!=1
    replace qxi4l2="" if qxi4l2=="No"
    replace qxi4l2="FDC" if qxi4l2=="Forum for democratic change" | qxi4l2=="fdc" | qxi4m2=="FDC"
    replace qxi4l2="NRM" if qxi4l2=="NRM party" | qxi4l2=="NRMO"
    replace qxi4l=1 if qxi4l2!=""
    replace qxi4i=1 if qxi4m2=="Cash box introduced recently by EARG" | qxi4m2=="peer educator"
    replace qxi4m=. if qi1==1075 | qi1==1630 | qi1==1798 | qxi4m2=="no"
    replace qxi4m2="" if qi1==1075 | qi1==1630 | qi1==1798 | qxi4m2=="no"

// Religion
   ren qx5 qxi5b
   replace qxi5=2 if qxi5b=="2" | qxi5b=="C R (Christian revival)" | qxi5b=="Christian revival (CR)" /*
                   */ | qxi5b=="christti"
    replace qxi5b="" if qxi5b=="2" | qxi5b=="C R (Christian revival)" | qxi5b=="Christian revival (CR)" /*
                  */ | qxi5b=="christti"
    // assume missing obs are catholic for now but flag to check surveys
    replace qxi5=1 if qxi5==. & qno!=.

// Skipped questions about war trauma
    for varlist qx2a-qx32, noh: qui replace X=. if X==0
    for varlist qx2a-qx32, noh: qui replace X=2 if X==22

// Voting
    replace qxi32=2 if (qxi32==3 | qxi32==4) & qxi33!=""
    replace qxi38=. if qxi38==3
    replace qxi39=. if qxi39==3
    replace qxi37="" if qxi36==2

// Turn any non-empty string variables that contain only numeric characters into numeric variables,
// except qxii4 and qxii5, which explicitly accept numeric entry into a string field
    foreach j of varlist qno-qxii1a qxiii6-qxiii14 {
     cap conf str v `j'
     if !_rc {
      qui count if `j'!=""
      if r(N)>0 cap destring `j', replace
     }
    }

** MERGE HH AND SAMPLING DATA **

// Prepare individual survey data for merge
    ren qi1 id
    so id
    sa "SWAYII/tempdata/temp_i", replace

// Prepare household survey data for merge
    *tempfile hhtemp
    u "SWAYI/HHsurvey_sectionII_27May2008.dta", clear
    ren id2 hh_id
    ren hhiia roster_row
    ren abd hh_abd
    ren abd_age1 hh_abd_age1
    ren abd_age2 hh_abd_age2
    ren age hh_age
    ren farm hh_farm
    ren fthr_frm hh_fthr_frm
    ren fthr_pro hh_fthr_pro
    ren fthr_sol hh_fthr_sol
    ren inschool hh_inschool
    ren job hh_job

    // Sampling strata
    gen stratum=1 if abd_times>0 & abd_times!=.
    replace stratum=2 if abd_times==0
    replace stratum=3 if died==1 | no_return==1
    la define strata 1 "Abducted" 2 "Non-Abd" 3 "Did not survive/return"
    la values stratum strata
    la var stratum "Sampling stratum"

    // Generate Primary Sampling Unit (PSU)
    egen psu=group(camp stratum)
    la var psu "Primary sampling unit: subcounty of birth, abduction status, and over/under 30"

    // Abduction length variable
    gen hh_abd_lgth=0 if hhiig==0
    replace hh_abd_lgth=hhiih_l if hhiig>0
    replace hh_abd_lgth=hh_abd_lgth+hhiii_l if hhiii_l!=.

    // Other variables constructed from household survey only
    gen hh_immed_fam=cond(hhiic>=0 & hhiic<=4,1,0) if hhiic!=.
    gen hh_wfp=cond(hhiib_05==1,1,0) if hhiib_05!=.
    gen hh_land=hhi8
    gen hh_cattle=hhi9
    gen hh_stock=hhi10
    gen hh_plow=hhi11
    // Replace 99s in asset variables and in hhiiage, because they're probably "don't know"
    // Check hhi8-hhi11 to see original values of 99
    foreach i of varlist hh_land hh_cattle hh_stock hh_plow hhiiage {
     replace `i'=. if `i'==99
    }

    // Impute missing HH variables & generate indicators for missing data
    // Rule followed: Impute using other HH variables and, if still missing, replace with median
    gen hh_immed_miss=cond(hh_immed_fam==.,1,0)
    xi: logit hh_immed_fam hhiid i.hhiif hh_abd_total hh_size96
    predict p
    gen E_immed=cond(p>=.5, 1, 0) if p!=.
    replace E_immed=1 if E_immed==.
    replace hh_immed_fam=E_immed if hh_immed_fam==.

    gen hh_land_miss=cond(hh_land==.,1,0)
    reg hh_land hh_war_death hh_avg_educ hh_abd_total hh_abd_youth /*
             */ hh_fthr_pro hh_fthr_sol hh_size96 hh_cattle hh_stock hh_plow
    predict E_land, xb
    replace hh_land=E_land if hh_land==.
    replace hh_land=12 if hh_land==.
    replace hh_land=0 if hh_land<0

    gen hh_cattle_miss=cond(hh_land==.,1,0)
    reg hh_cattle hh_war_death hh_avg_educ hh_abd_total hh_abd_youth /*
               */ hh_fthr_pro hh_fthr_sol hh_size96 hh_land hh_stock hh_plow
    predict E_cattle, xb
    replace hh_cattle=E_cattle if hh_cattle==.
    replace hh_cattle=round(hh_cattle)
    replace hh_cattle=0 if hh_cattle==. | hh_cattle<0

    gen hh_stock_miss=cond(hh_stock==.,1,0)
    reg hh_stock hh_war_death hh_avg_educ hh_abd_total hh_abd_youth /*
              */ hh_fthr_pro hh_fthr_sol hh_size96 hh_cattle hh_land hh_plow
    predict E_stock, xb
    replace hh_stock=E_stock if hh_land==.
    replace hh_stock=round(hh_stock)
    replace hh_stock=4 if hh_stock==.
    replace hh_stock=0 if hh_stock<0

    gen hh_plow_miss=cond(hh_plow==.,1,0)
    reg hh_plow hh_war_death hh_avg_educ hh_abd_total hh_abd_youth /*
             */ hh_fthr_pro hh_fthr_sol hh_size96 hh_cattle hh_stock hh_land
    predict E_plow, xb
    replace hh_plow=E_land if hh_plow==.
    replace hh_plow=round(hh_plow)
    replace hh_plow=0 if hh_plow==. | hh_plow<0
    drop p _I* E_*

    // Logged land assets
    gen log_land=ln(hh_land)
    // replace undefined (i.e. hh_land=0) with minimum
    replace log_land=-7 if hh_land==0

    keep hh_id roster_row /*
    */ stratum psu hhi8 hhi9 hhi10 hhi11 hhiiage hhiib hhiib_05 hhiic hhiid hhiie hhiif hhiig hhiih_lgth hhiih_yr /*
    */ hhiii_lgth hhiii_yr hhiiii hhiij hhiik hhiil hh_age hh_abd hh_abd_age1 hh_abd_age2 hh_abd_lgth /*
    */ asset_50th asset_75th asset_90th camp_yrs /*
    */ hh_farm hh_fthr_frm hh_fthr_pro hh_fthr_sol incamp hh_inschool hh_job w10 w25 w75 /*
    */ w90 hh_abd_total hh_abd_youth hh_adults05 hh_assets96 hh_avg_educ hh_select hh_size05 hh_size96 /*
    */ hh_war_death hh_wealth96 hh_weight hh_immed_fam hh_wfp hh_land log_land hh_cattle hh_stock hh_plow /*
    */ hh_immed_miss hh_land_miss hh_cattle_miss hh_stock_miss hh_plow_miss

    so hh_id roster_row
    save "SWAYII/tempdata/hhtemp", replace

// Prepare sample data
    u "SWAYII/rawdata/sway_ii_sample", clear
    ren age age_sample
    ren abd abd_sample
    so hh_id roster_row

// Merge household data
    mer hh_id roster_row using "SWAYII/tempdata/hhtemp"
    drop if _merge==2
    drop _merge

// Merge individual survey data
    so id
    mer id using "SWAYII/tempdata/temp_i"
    ren _merge i_merge
    la var i_merge "=3 if hh-indiv match, =2 if ind only, =1 if hh only"
    sa "SWAYII/tempdata/temp_i", replace

** ADDL VARS: SECTION I **

// Use data from household survey if individual data missing
    replace qi2b=camp if qi2b==.

// Camp indicators
    gen C_ach=cond(qi2b==1,1,0) if qi2b!=.
    gen C_akw=cond(qi2b==2,1,0) if qi2b!=.
    gen C_kma=cond(qi2b==3,1,0) if qi2b!=.
    gen C_oro=cond(qi2b==4,1,0) if qi2b!=.
    gen C_pad=cond(qi2b==5,1,0) if qi2b!=.
    gen C_paj=cond(qi2b==6,1,0) if qi2b!=.
    gen C_pal=cond(qi2b==7,1,0) if qi2b!=.
    gen C_ata=cond(qi2b==8,1,0) if qi2b!=.

    la var C_ach "Acholibur Indicator"
    la var C_akw "Akwang Indicator"
    la var C_kma "K. Matidi Indicator"
    la var C_oro "Orom Indicator"
    la var C_pad "Pader Indicator"
    la var C_paj "Pajule Indicator"
    la var C_pal "Palabek Indicator"
    la var C_ata "Atanga Indicator"

// Dry Season
    gen dry=cond(month(qi7)==12 | month(qi7)==1 | month(qi7)==2 | month(qi7)==3,1,0) if qi7!=.
    la var dry "Interviewed in dry season"


** ADDL VARS: SECTION II **

// qii1: age
    gen age=qii1
    replace age=2007-qv1 if age==.
    replace age=hh_age if age==.
    // same as above:
    // replace age=round( year(qi7) - qv1) if age==.
    gen age2=age^2
    gen age3=age^3
    gen age4=age^4

    la var age "Respondent Age"
    la var age2 "Respondent Age Squared"
    la var age3 "Respondent Age Cubed"
    la var age4 "Respondent Age Quartic"

// Age indicators
    foreach x of numlist 14/35 {
        gen A`x'=cond(age==`x',1,0) if age!=.
        la var A`x' "Age indicator: `x'"
        }
    replace A35=1 if age>35 & age!=.
    replace A14=1 if age<14 & age!=.
    la var A35 "Age indicator: 35+"
    la var A14 "Age indicator: 14 and younger"
    gen adult=cond(age>=18,1,0) if age!=.
    gen child=cond(age<18,1,0) if age!=.
    la var adult "Respondent at least 18 years old"
    la var child "Respondent less than 18 years old"

    gen age_cat=.
    replace age_cat=1 if age>=14 & age<=16
    replace age_cat=2 if age>=17 & age<=19
    replace age_cat=3 if age>=20 & age<=22
    replace age_cat=4 if age>=23 & age<=25
    replace age_cat=5 if age>=26 & age<=28
    replace age_cat=6 if age>=29 & age<=31
    replace age_cat=7 if age>=32 & age<=34
    replace age_cat=8 if age>=34 & age!=.
    label define ages 1 "14-16" 2 "17-19" 3 "20-22" 4 "23-25" 5 "26-28" 6 "29-31" 7 "32-34" 8 "34+"
    label values age_cat ages
    la var age_cat "Age categories"

// Age/location dummies
    foreach x of numlist 1/8 {
        gen G`x'_14_17=cond(age>=14 & age<=17 & qi2b==`x',1,0) if age!=. & qi2b!=.
        la var G`x'_14_17 "Age/location indicator: camp `x', age 14-17"
        gen G`x'_18_21=cond(age>=18 & age<=21 & qi2b==`x',1,0) if age!=. & qi2b!=.
        la var G`x'_18_21 "Age/location indicator: camp `x', age 18-21"
        gen G`x'_22_25=cond(age>=22 & age<=25 & qi2b==`x',1,0) if age!=. & qi2b!=.
        la var G`x'_22_25 "Age/location indicator: camp `x', age 22-25"
        gen G`x'_26_30=cond(age>=26 & age<=30 & qi2b==`x',1,0) if age!=. & qi2b!=.
        la var G`x'_26_30 "Age/location indicator: camp `x', age 26-30"
        gen G`x'_31_plus=cond(age>=31 & age<=38 & qi2b==`x',1,0) if age!=. & qi2b!=.
        la var G`x'_31_plus "Age/location indicator: camp `x', age 31+"
    }

// Parents alive?
    gen no_fthr=cond(qii3!=1,1,0) if qii3!=.
    gen no_fthr96=cond(qii3!=1 & (qii5<=1996 | qii6==1), 1, 0)
    replace no_fthr96=. if (qii3==. & qii5==. & qii6==.) | (qii3!=1 & qii5==. & qii6==.)
    gen no_fthr01=cond(qii3!=1 & (qii5<=2001 | qii6==2 | qii6==1), 1, 0)
    replace no_fthr01=. if (qii3==. & qii5==. & qii6==.) | (qii3!=1 & qii5==. & qii6==.)
    gen no_fthr04=cond(qii3!=1 & (qii5<=2004 | qii6==3 | qii6==2 | qii6==1), 1, 0)
    replace no_fthr04=. if (qii3==. & qii5==. & qii6==.) | (qii3!=1 & qii5==. & qii6==.)
    gen no_mthr=cond(qii8!=1,1,0) if qii8!=.
    gen no_mthr96=cond(qii8!=1 & (qii10<=1996 | qii11==1), 1, 0)
    replace no_mthr96=. if (qii8==. & qii10==. & qii11==.) | (qii8!=1 & qii10==. & qii11==.)
    gen no_mthr01=cond(qii8!=1 & (qii10<=2001 | qii11==2 | qii11==1), 1, 0)
    replace no_mthr01=. if (qii8==. & qii10==. & qii11==.) | (qii8!=1 & qii10==. & qii11==.)
    gen no_mthr04=cond(qii8!=1 & (qii10<=2004 | qii11==3 | qii11==2 | qii11==1), 1, 0)
    replace no_mthr04=. if (qii8==. & qii10==. & qii11==.) | (qii8!=1 & qii10==. & qii11==.)

    gen orphan=cond(no_fthr==1 & no_mthr==1, 1, 0)
    replace orphan=. if (no_fthr==1 & no_mthr==.) | (no_fthr==. & no_mthr==1) | (no_fthr==. & no_mthr==.)
    gen lost_parent=cond(no_fthr==1 | no_mthr==1,1,0)
    replace lost_parent=. if (no_fthr==0 & no_mthr==.) | (no_fthr==. & no_mthr==0) | (no_fthr==. & no_mthr==.)
    gen orphan96=cond(no_fthr96==1 & no_mthr96==1, 1, 0)
    replace orphan96=. if (no_fthr96==1 & no_mthr96==.) | (no_fthr96==. & no_mthr96==1) | (no_fthr96==. & no_mthr96==.)
    gen lost_parent96=cond(no_fthr96==1 | no_mthr96==1,1,0)
    replace lost_parent96=. if (no_fthr96==0 & no_mthr96==.) | (no_fthr96==. & no_mthr96==0) | (no_fthr96==. & no_mthr96==.)

    la var no_fthr "father died"
    la var no_mthr "mother died"
    la var orphan "father and mother died"
    la var lost_parent "father or mother died"
    la var no_fthr96 "father died before 1997"
    la var no_mthr96 "mother died before 1997"
    la var orphan96 "parents died before 1997"
    la var lost_parent96 "father or mother died before 1997"
    la var no_fthr01 "father died before F2002"
    la var no_mthr01 "mother died before 2002"
    la var no_fthr04 "father died before 2005"
    la var no_mthr04 "mother died before 2005"

    gen lost_fthr_yr=qii5 if no_fthr==1
    replace lost_fthr_yr=2006 if no_fthr==1 & qii5==. & qii6==4
    replace lost_fthr_yr=2004 if no_fthr==1 & qii5==. & qii6==3
    replace lost_fthr_yr=2000 if no_fthr==1 & qii5==. & qii6==2

    gen lost_fthr_age=age-(2007-lost_fthr_yr) if no_fthr==1
    replace lost_fthr_age=0 if lost_fthr_age<0

    gen lost_mthr_yr=qii10 if no_fthr==1
    replace lost_mthr_yr=2006 if no_mthr==1 & qii10==. & qii11==4
    replace lost_mthr_yr=2004 if no_mthr==1 & qii10==. & qii11==3
    replace lost_mthr_yr=2000 if no_mthr==1 & qii10==. & qii11==2

    gen lost_mthr_age=age-(2007-lost_mthr_yr) if no_mthr==1
    replace lost_mthr_age=0 if lost_mthr_age<0

    gen lost_fthr15=0 if no_fthr==0
    replace lost_fthr15=1 if no_fthr==1 & lost_fthr_age<=14
    replace lost_fthr15=0 if no_fthr==1 & lost_fthr_age>14 & lost_fthr_age!=.
    replace lost_fthr15=1 if no_fthr==1 & lost_fthr_age==. & age<=28 & qii6==1

    gen lost_mthr15=0 if no_mthr==0
    replace lost_mthr15=1 if no_mthr==1 & lost_mthr_age<=15
    replace lost_mthr15=0 if no_mthr==1 & lost_mthr_age>15 & lost_mthr_age!=.
    replace lost_mthr15=1 if no_mthr==1 & lost_mthr_age==. & age<=28 & qii11==1

    gen orphan15_both=0
    replace orphan15_both=1 if lost_fthr15==1 & lost_mthr15==1

    gen orphan15_one=0
    replace orphan15_one=1 if lost_fthr15==1 | lost_mthr15==1

// qii7, qii12: Parent's education
    gen fthr_ed=qii7
    gen mthr_ed=qii12
    replace fthr_ed=15 if fthr_ed==5
    replace fthr_ed=13 if fthr_ed==4
    replace fthr_ed=10 if fthr_ed==3
    replace fthr_ed=7 if fthr_ed==2
    replace fthr_ed=4 if fthr_ed==1
    replace mthr_ed=15 if mthr_ed==5
    replace mthr_ed=13 if mthr_ed==4
    replace mthr_ed=10 if mthr_ed==3
    replace mthr_ed=7 if mthr_ed==2
    replace mthr_ed=4 if mthr_ed==1

    // Impute missing values:
        reg fthr_ed age C_*
        predict e
        replace fthr_ed=e if fthr_ed==.
        replace fthr_ed=0 if fthr_ed<3
        replace fthr_ed=4 if fthr_ed>=3 & fthr_ed<6
        replace fthr_ed=7 if fthr_ed>=6 & fthr_ed<=8
        replace fthr_ed=10 if fthr_ed>8 & fthr_ed<=11
        replace fthr_ed=12 if fthr_ed>11 & fthr_ed<=13
        replace fthr_ed=15 if fthr_ed>13 & fthr_ed!=.
        drop e

        reg mthr_ed age fthr_ed C_*
        predict e
        replace mthr_ed=e if mthr_ed==.
        replace mthr_ed=0 if mthr_ed<3
        replace mthr_ed=4 if mthr_ed>=3 & mthr_ed<6
        replace mthr_ed=7 if mthr_ed>=6 & mthr_ed<=8
        replace mthr_ed=10 if mthr_ed>8 & mthr_ed<=11
        replace mthr_ed=12 if mthr_ed>11 & mthr_ed<=13
        replace mthr_ed=15 if mthr_ed>13 & mthr_ed!=.
        drop e

    // Indicator for parent's education
        gen fthr_ed0=cond(fthr_ed==0,1,0) if fthr_ed!=.
        gen fthr_ed4=cond(fthr_ed==4,1,0) if fthr_ed!=.
        gen fthr_ed7=cond(fthr_ed==7,1,0) if fthr_ed!=.
        gen fthr_edhi=cond(fthr_ed>7,1,0) if fthr_ed!=.

        gen mthr_ed0=cond(mthr_ed==0,1,0) if mthr_ed!=.
        gen mthr_ed4=cond(mthr_ed==4,1,0) if mthr_ed!=.
        gen mthr_ed7=cond(mthr_ed==7,1,0) if mthr_ed!=.
        gen mthr_edhi=cond(mthr_ed>7,1,0) if mthr_ed!=.

        la var fthr_ed "Father's education"
        la var mthr_ed "Mother's education"
        la var fthr_ed0 "Father has no schooling"
        la var fthr_ed4 "Father has some primary school"
        la var fthr_ed7 "Father completed P7"
        la var fthr_edhi "Father attended secondary school"
        la var mthr_ed0 "Mother has no schooling"
        la var mthr_ed4 "Mother has some primary school"
        la var mthr_ed7 "Mother completed P7"
        la var mthr_edhi "Mother attended secondary school"

// Lives with parents
    gen live_mthr=cond(qii13a==1 | qii13b==1 | qii13c==1 | qii13d==1,1,0) if qii13a!=. | qii13b!=. | qii13c!=. | qii13d!=.
    gen live_fthr=cond(qii13a==2 | qii13b==2 | qii13c==2 | qii13d==2,1,0) if qii13a!=. | qii13b!=. | qii13c!=. | qii13d!=.
    gen live_par=cond(live_mthr==1 | live_fthr==1,1,0) if qii13a!=. | qii13b!=. | qii13c!=. | qii13d!=.
    gen alone=cond(qii13a==8 | qii13b==8 | qii13c==8 | qii13d==8,1,0) if qii13a!=. | qii13b!=. | qii13c!=. | qii13d!=.
    la var live_mthr "Lives with mother"
    la var live_fthr "Lives with father"
    la var live_par "Lives with at least one parent"
    la var alone "Lives alone"

// qii14, qii17
    gen qii17_dens=qii14/qii17
    rename qii17_dens density
    la var density "people per hut"

// Respondent in camp
    replace incamp=cond(L_main_camp==1 | L_sat_camp==1,1,0) & L_main_camp!=.
    la var incamp "Respondent lives in camp"

// qii32: Assets
    factor qii32a-qii32l, factors(3) pcf
    rotate
    predict assets
    gen aindex=normal(assets)
    la var assets "Index of asset ownership"
    la var aindex "Index of asset ownership: Cumulative normal distribution"

    foreach n in a b c d e f g h i j k l {
     replace qii32`n'=0 if qii32`n'==. & qno!=.
     gen qii32`n'_any=cond(qii32`n'==0,0,1) if qno!=.
     }

// Household assets
    foreach var of varlist hh_land hh_cattle hh_stock hh_plow {
     gen `var'_any=cond(`var'==0,0,1) if `var'!=.
     }

// Children
    gen child_bio_num=qii19
     replace child_bio_num=0 if qii19==. & qno!=.
    gen child_nonbio_num=qii20
     replace child_nonbio_num=0 if qii20==. & qno!=.
    gen child_num=child_bio_num+child_nonbio_num
    gen child_bio=cond(child_bio_num>0,1,0) if qno!=.
    gen child_any=cond(child_num>0,1,0) if qno!=.
    gen many_children=0
    replace many_children=1 if child_num>4

    la var child_bio_num "Number of biological children"
    la var child_nonbio_num "Number of non-biol children"
    la var child_num "Number of all children, biol or non-biol"
    la var child_bio "Has at least one biological child"
    la var child_any "Has at least one biological or non-biol. child"
    la var many_children "equals 1 if number of children greater 4"

// Marital status
    replace qii36=1 if qii36==. & qno!=. & qii37==1
    replace qii36=5 if qii36==. & qno!=. & qii37==5
    replace qii36=6 if qii36==. & qno!=.
    replace qii37=1 if qii37==5
    
    gen married=cond(qii36==1 | qii37==1,1,0) if qno!=.
    gen ever_married=cond(qii36==6,0,1) if qno!=.

    la var married "Married or lives with a partner"
    la var ever_married "Has ever been married or lived with a partner"

// Sexual behavior
    gen own_condom=cond(qii35==1,1,0) if qii35!=.
    gen hiv_aware=cond(qii34==1,1,0) if qii34!=.
    gen get_condom=cond(qii33==1,1,0) if qii33!=.

    gen sex_active=cond(qii38==1,1,0) if married==0 | qii38!=.
    gen use_cond_some=cond(qii40==3 | qii43==3,1,0) if qii40!=. | qii43!=.
    gen use_cond_often=cond(qii40==1 | qii43==1,1,0) if qii40!=. | qii43!=.

    la var own_condom "Says that she owns condom"
    la var hiv_aware "Says that condoms prevent spread of HIV/AIDS"
    la var get_condom "Says she knows place selling/giving out condoms"
    la var sex_active "Unmarried respondent sexually active"
    la var use_cond_some "Respondent uses condom sometimes"
    la var use_cond_often "Respondent uses condom often"


** ADDL VARS: SECTION III **

// education and literacy (change survey responses)
    replace qiii8=0 if qiii9==8888 & qiii8==.
    replace qiii8=hhiij if qiii8==. & qno!=.
    replace qiii6=2 if qiii7!="" & qiii6==.
    replace qiii6=1 if qiii9==2007 & qiii6==.
    replace qiii6=2 if qiii9<2007 & qiii6==.
    replace qiii5=1 if qiii8>6 & qiii1==1 & qiii8!=. & qiii5==. & qno!=.
    replace qiii5=3 if (qiii1==2 | qiii1==3) & qiii5==. & qno!=.
    replace qiii5=2 if qiii1==1 & qiii8<=6 & qiii5==. & qno!=.

// iii1-iii3: Literacy
    gen literate=cond(qiii1==1 & qiii5<=2,1,0) if qiii1!=. & qiii5!=.
    gen illiterate=cond(qiii1==3,1,0) if qiii1!=.
    la var literate "Reads without difficulty and writes with some or no difficulty"
    la var illiterate "Cannot read or write"

// Schooling
    gen inschool=cond(qiii6==1,1,0) if qiii6!=.
    replace inschool=1 if qiii6==. & qiii7!=""
    replace inschool=0 if qiii6==. & qiii7==""
    la var inschool "Currently in school"

// Educational attainment
    gen p7=cond(qiii8>=7 & qiii8!=., 1, 0) if qiii8!=.
    gen p4=cond(qiii8>=4 & qiii8!=., 1, 0) if qiii8!=.
    gen p1=cond(qiii8>=1 & qiii8!=., 1, 0) if qiii8!=.
    la var p1 "Completed P1"
    la var p4 "Completed P4"
    la var p7 "Completed P7"



// Higher education

   list qno qiii11a qiii12a qiii13a1 qiii13a2 if qiii11a!=. | qiii12a!="", noobs clean
   list qno qiii11b qiii12b qiii13b1 qiii13b2 if qiii11b!=. | qiii12b!="", noobs clean

    replace qiii12a="1" if qno==247 | qno==75 | qno==355 | qno==74 | qno==294 | qno==32 | qno==293
    replace qiii12a="2" if qno==589 | qno==593
    replace qiii12a="3" if qno==10 | qno==563 | qno==218 | qno==15 | qno==365 | qno==558 | qno==301 | qno==43 | qno==315 | qno==73 | qno==475 /*
       */ | qno==557 | qno==607 | qno==219 | qno==505 | qno==96 | qno==249 | qno==259 | qno==532 | qno==481 | qno==596 | qno==36 
    replace qiii12a="4" if qno==617 | qno==217
    replace qiii12a="5" if qno==125 | qno==33
    replace qiii12a="6" if qno==445 | qno==71 | qno==300 | qno==578 | qno==37 | qno==339
    replace qiii12a="7" if qno==268 | qno==205 | qno==203 | qno==392 | qno==461
    replace qiii12a="" if qno==498 | qno==499 | qno==437
    replace qiii12b="7" if qno==268
    replace qiii12b="" if qno==593

    destring qiii12a, replace
    destring qiii12b, replace

    label define vocations 1 "Adult Education" 2 "Teacher" 3 "Tailoring" 4 "Brick laying" 5 "Catering" 6 "Other vocation" 7 "Unspecified"
    label values qiii12a vocations
    label values qiii12b vocations

    replace qiii11a=3 if qiii12a==2
    replace qiii11a=2 if qiii12a>1 & qiii12a<8
    replace qiii11a=. if qiii12a==.

    replace qiii11b=3 if qiii12b==2
    replace qiii11b=2 if qiii12b>1 & qiii12b<8
    replace qiii11b=. if qiii12b==.

    gen qiii13a=.
    replace qiii13a=qiii13a1/30 if qiii13a2==1
    replace qiii13a=qiii13a1/4 if qiii13a2==2
    replace qiii13a=qiii13a1 if qiii13a2==3
    replace qiii13a=qiii13a1*12 if qiii13a2==4
    replace qiii13a=24 if qno==294 | qno==247 | qno==32
    replace qiii13a=6 if qno==33
    replace qiii13a=qiii13a1 if qiii13a==. & qiii13a>3 & qiii13a1!=. & qiii13a2==.
    replace qiii13a=qiii13a1*12 if qiii13a==. & qiii13a<=3 & qiii13a1!=. & qiii13a2==.

    gen qiii13b=.
    replace qiii13b=qiii13b1 if qiii13b2==1
    replace qiii13b=qiii13b1/30 if qiii13b2==2
    replace qiii13b=qiii13b1/4 if qiii13b2==3
    replace qiii13b=qiii13b1*12 if qiii13b2==4

    la var qiii13a "For how long was this training? (in months)"
    la var qiii13b "For how long was this training? (in months)"

    replace qiii11a=. if qiii13a<=1
    replace qiii12a=. if qiii13a<=1
    replace qiii13a=. if qiii13a<=1
    replace qiii11b=. if qiii13b<=1
    replace qiii12b=. if qiii13b<=1
    replace qiii13b=. if qiii13b<=1

    replace qiii10=1 if qiii11a!=.
    replace qiii10=0 if qiii11a==. & qno!=.

    gen voc=cond(qiii11a==2 | qiii11b==2,1,0) if qno!=.

    gen ngo_paid=cond(qiii15a==4,1,0) if qno!=.

    gen university=cond(qno==589 | qno==43,1,0) if qno!=.

    la var university "Ever attended university"
    la var voc "Ever received vocational training"
    la var ngo_paid "Training paid for by NGO"

// Years of education
    gen educ=qiii8
    replace educ=educ+(qiii13a/12) if qiii13a!=.
    replace educ=educ+(qiii13b/12) if qiii13b!=.
    replace educ=round(educ)
    replace educ=16 if educ>16 & educ!=.
    la var educ "Years of education attained"

    gen educ0=cond(educ==0,1,0)
    la var educ0 "No education"

    gen secskool=cond(educ>=8,1,0) if educ!=.
    la var secskool "Completed at least some secondary school"

    gen educ_cat=.
    replace educ_cat=1 if educ>=0 & educ<=3
    replace educ_cat=2 if educ>=4 & educ<=7
    replace educ_cat=3 if educ>=8 & educ<=11
    replace educ_cat=4 if educ>=12 & educ<=16
    label define educ_cat 1 "0-3" 2 "4-7" 3 "8-11" 4 "12-16"
    label values educ_cat educ_cat
    la var educ_cat "Years of education, categorical"

// Years of potential work experience
    gen exper=age-6-educ
    replace exper=0 if exper<0
    lab var exper "Expertise: Age minus education minus 6"


** ADDL VARS: SECTION IV **

// qiv1-qiv2: earnings
    // Interbank exchange rate, 05/01/2007 <www.oanda.com/convert/fxhistory>
    gen earn_mo=qiv1
    gen earn_wk=qiv2
    replace earn_mo=earn_wk if earn_wk>earn_mo & earn_wk!=.
    la var earn_mo "Earnings in Uganda shillings: Last 4 weeks"
    la var earn_wk "Earnings in Uganda Shillings: Last 7 days"

    gen earn0=cond(earn_mo==0,1,0) if earn_mo!=.
    la var earn0 "Earnings=0"

// Employment levels
    egen emp_d_mo=rsum(qiv4a qiv4b qiv4c qiv4d qiv4e) if qno!=.
    replace emp_d_mo=28 if emp_d_mo>28
    egen emp_d_wk=rsum(qiv5a qiv5b qiv5c qiv5d qiv5e) if qno!=.
    replace emp_d_wk=7 if emp_d_wk>7
    replace emp_d_mo=emp_d_wk if emp_d_wk>emp_d_mo & emp_d_wk!=.

    for any a b c d e: gen temp_X=qiv4X if qiv7X>0 & qiv7X!=.
    egen emp_cash_d_mo=rsum(temp_*) if qno!=.
    replace emp_cash_d_mo=28 if emp_cash_d_mo>28 & emp_cash_d_mo!=.
    drop temp_*
    for any a b c d e: gen temp_X=qiv5X if qiv7X>0 & qiv7X!=.
    egen emp_cash_d_wk=rsum(temp_*) if qno!=.
    replace emp_cash_d_wk=7 if emp_cash_d_wk>7 & emp_cash_d_wk!=.
    replace emp_cash_d_mo=emp_cash_d_wk if emp_cash_d_wk>emp_cash_d_mo & emp_cash_d_wk!=.
    drop temp_*

    for any a b c d e: gen temp_X=qiv4X if qiv3X=="A" | qiv3X=="B"
    egen emp_hh_d_mo=rsum(temp_*) if qno!=.
    replace emp_hh_d_mo=28 if emp_hh_d_mo>28 & emp_hh_d_mo!=.
    drop temp_*
    for any a b c d e: gen temp_X=qiv5X if qiv3X=="A" | qiv3X=="B"
    egen emp_hh_d_wk=rsum(temp_*) if qno!=.
    replace emp_hh_d_wk=7 if emp_hh_d_wk>7 & emp_hh_d_wk!=.
    replace emp_hh_d_mo=emp_hh_d_wk if emp_hh_d_wk>emp_hh_d_mo & emp_hh_d_wk!=.
    drop temp_*

    for any a b c d e: gen temp_X=qiv4X if qiv3X!="A" & qiv3X!="B"
    egen emp_nohh_d_mo=rsum(temp_*) if qno!=.
    replace emp_nohh_d_mo=28 if emp_nohh_d_mo>28 & emp_nohh_d_mo!=.
    drop temp_*
    for any a b c d e: gen temp_X=qiv5X if qiv3X!="A" & qiv3X!="B"
    egen emp_nohh_d_wk=rsum(temp_*) if qno!=.
    replace emp_nohh_d_wk=7 if emp_nohh_d_wk>7 & emp_nohh_d_wk!=.
    replace emp_nohh_d_mo=emp_nohh_d_wk if emp_nohh_d_wk>emp_nohh_d_mo & emp_nohh_d_wk!=.
    drop temp_*

    la var emp_d_mo "Days worked in last 4 weeks"
    la var emp_d_wk "Days worked in last 7 days"
    la var emp_cash_d_mo "Days worked for cash in last 4 weeks"
    la var emp_cash_d_wk "Days worked for cash in last 7 days"
    la var emp_hh_d_mo "Days worked in household in last 4 weeks"
    la var emp_hh_d_wk "Days worked in household in last 7 days"
    la var emp_nohh_d_mo "Days worked outside household in last 4 weeks"
    la var emp_nohh_d_wk "Days worked outside household in last 7 days"

    gen emp_mo=cond(emp_d_mo>0,1,0) if emp_d_mo!=.
    gen emp_wk=cond(emp_d_wk>0,1,0) if emp_d_wk!=.
    foreach z in cash hh nohh {
        gen emp_`z'_mo=cond(emp_`z'_d_mo>0,1,0) if emp_`z'_d_mo!=.
        gen emp_`z'_wk=cond(emp_`z'_d_wk>0,1,0) if emp_`z'_d_wk!=.
        }

    la var emp_mo "Equals 1 if worked in last 4 weeks"
    la var emp_wk "Equals 1 if worked in last 7 days"
    la var emp_cash_mo "Equals 1 if worked for cash in last 4 weeks"
    la var emp_cash_wk "Equals 1 if worked for cash in last 7 days"
    la var emp_hh_mo "Equals 1 if worked in household in last 4 weeks"
    la var emp_hh_wk "Equals 1 if worked in household in last 7 days"
    la var emp_nohh_mo "Equals 1 if worked outside household in last 4 weeks"
    la var emp_nohh_wk "Equals 1 if worked outside household in last 7 days"

    for any a b c d e: gen hrsX=qiv6X
    for any a b c d e: replace hrsX=8 if qiv6X>18 & qiv6X!=.

    for any a b c d e: gen tempX=qiv5X*hrsX
    egen emp_hrs_wk=rsum(tempa tempb tempc tempd tempe)
    drop tempa tempb tempc tempd tempe
    replace emp_hrs_wk=126 if emp_hrs_wk>126
    for any a b c d e: gen tempX=qiv4X*hrsX
    egen emp_hrs_mo=rsum(tempa tempb tempc tempd tempe)
    drop tempa tempb tempc tempd tempe
    replace emp_hrs_mo=504 if emp_hrs_mo>504
    replace emp_hrs_mo=emp_hrs_wk if emp_hrs_wk>emp_hrs_mo & emp_hrs_wk!=.

    for any a b c d e: gen tempX=qiv5X*hrsX if qiv7X>0 & qiv7X!=.
    egen emp_cash_hrs_wk=rsum(tempa tempb tempc tempd tempe)
    drop tempa tempb tempc tempd tempe
    replace emp_cash_hrs_wk=126 if emp_cash_hrs_wk>126
    for any a b c d e: gen tempX=qiv4X*hrsX if qiv7X>0 & qiv7X!=.
    egen emp_cash_hrs_mo=rsum(tempa tempb tempc tempd tempe)
    drop tempa tempb tempc tempd tempe
    replace emp_cash_hrs_mo=504 if emp_cash_hrs_mo>504
    replace emp_cash_hrs_mo=emp_cash_hrs_wk if emp_cash_hrs_wk>emp_cash_hrs_mo & emp_cash_hrs_wk!=.

    for any a b c d e: gen tempX=qiv5X*hrsX if qiv3X=="A" | qiv3X=="B"
    egen emp_hh_hrs_wk=rsum(tempa tempb tempc tempd tempe)
    drop tempa tempb tempc tempd tempe
    replace emp_hh_hrs_wk=126 if emp_hh_hrs_wk>126
    for any a b c d e: gen tempX=qiv4X*hrsX if qiv3X=="A" | qiv3X=="B"
    egen emp_hh_hrs_mo=rsum(tempa tempb tempc tempd tempe)
    drop tempa tempb tempc tempd tempe
    replace emp_hh_hrs_mo=504 if emp_hh_hrs_mo>504
    replace emp_hh_hrs_mo=emp_hh_hrs_wk if emp_hh_hrs_wk>emp_hh_hrs_mo & emp_hh_hrs_wk!=.

    for any a b c d e: gen tempX=qiv5X*hrsX if qiv3X!="A" & qiv3X!="B"
    egen emp_nohh_hrs_wk=rsum(tempa tempb tempc tempd tempe)
    drop tempa tempb tempc tempd tempe
    replace emp_nohh_hrs_wk=126 if emp_nohh_hrs_wk>126
    for any a b c d e: gen tempX=qiv4X*hrsX if qiv3X!="A" & qiv3X!="B"
    egen emp_nohh_hrs_mo=rsum(tempa tempb tempc tempd tempe)
    drop tempa tempb tempc tempd tempe
    replace emp_nohh_hrs_mo=504 if emp_nohh_hrs_mo>504
    replace emp_nohh_hrs_mo=emp_nohh_hrs_wk if emp_nohh_hrs_wk>emp_nohh_hrs_mo & emp_nohh_hrs_wk!=.

    la var emp_hrs_mo "Hours worked in last 4 weeks"
    la var emp_hrs_wk "Hours worked in last 7 days"
    la var emp_cash_hrs_mo "Hours worked for cash in last 4 weeks"
    la var emp_cash_hrs_wk "Hours worked for cash in last 7 days"
    la var emp_hh_hrs_mo "Hours worked in household in last 4 weeks"
    la var emp_hh_hrs_wk "Hours worked in household in last 7 days"
    la var emp_nohh_hrs_mo "Hours worked outside household in last 4 weeks"
    la var emp_nohh_hrs_wk "Hours worked outside household in last 7 days"

    gen emp_hrs_mo_avg=emp_hrs_mo/28
    gen emp_hrs_wk_avg=emp_hrs_wk/7
    gen emp_cash_hrs_mo_avg=emp_cash_hrs_mo/28
    gen emp_cash_hrs_wk_avg=emp_cash_hrs_wk/7
    gen emp_hh_hrs_mo_avg=emp_hh_hrs_mo/28
    gen emp_hh_hrs_wk_avg=emp_hh_hrs_wk/7
    gen emp_nohh_hrs_mo_avg=emp_nohh_hrs_mo/28
    gen emp_nohh_hrs_wk_avg=emp_nohh_hrs_wk/7

    la var emp_hrs_mo_avg "Average hours/day worked in last 4 weeks"
    la var emp_hrs_wk_avg "Average hours/day worked in last 7 days"
    la var emp_cash_hrs_mo_avg "Average hours/day worked for cash in last 4 weeks"
    la var emp_cash_hrs_wk_avg "Average hours/day worked for cash in last 7 days"
    la var emp_hh_hrs_mo_avg "Average hours/day worked in household in last 4 weeks"
    la var emp_hh_hrs_wk_avg "Average hours/day worked in household in last 7 days"
    la var emp_nohh_hrs_mo_avg "Average hours/day worked outside household in last 4 weeks"
    la var emp_nohh_hrs_wk_avg "Average hours/day worked outside household in last 7 days"

    gen unocc=cond(emp_nohh_mo==0 & inschool==0,1,0)
    replace unocc=. if inschool==. & emp_nohh_mo==0
    la var unocc "Equals 1 if neither in school nor working"

    gen unocc2=cond(emp_mo==0 & inschool==0,1,0)
    replace unocc2=. if inschool==. & emp_nohh_mo==0
    la var unocc2 "Equals 1 if neither in school nor working nor performing chores"

// Wage Measures
    gen wage_mo=earn_mo/emp_nohh_d_mo
    gen lwage_mo=ln(wage_mo)
    gen wage_wk=earn_wk/emp_nohh_d_wk
    gen lwage_wk=ln(wage_wk)

    gen wage_mo2=earn_mo/emp_cash_d_mo
    gen lwage_mo2=ln(wage_mo)

    gen wage_mo_hr=earn_mo/emp_nohh_hrs_mo
    gen lwage_mo_hr=ln(wage_mo)

    la var wage_mo "Average daily wage earned in last 4 weeks"
    la var wage_wk "Average daily wage earned in last 7 days"
    la var lwage_mo "Log daily wage earned in last 4 weeks"
    la var lwage_wk "Log daily wage earned in last 7 days"

    la var wage_mo2 "Average daily wage earned in last 4 weeks (only cash-earning activities)"
    la var lwage_mo2 "Log daily wage earned in last 7 days  (only cash-earning activities)"

    la var wage_mo_hr "Average hourly wage earned in last 4 weeks"
    la var lwage_mo_hr "Log hourly wage earned in last 7 days"
 
// Occupation
    gen mainjob=""
    replace mainjob=qiv3a if qiv3a!="A" & qiv3a!="B" & qiv3a!=""
    replace mainjob=qiv3b if ((qiv4b>qiv4a & qiv4b!=.) | mainjob=="") & qiv3b!="A" & qiv3b!="B" & qiv3b!=""
    replace mainjob=qiv3c if ((qiv4c>qiv4a & qiv4c>qiv4b & qiv4c!=.) | mainjob=="") & qiv3c!="A" & qiv3c!="B" & qiv3c!=""
    replace mainjob=qiv3d if ((qiv4d>qiv4a & qiv4d>qiv4b & qiv4d>qiv4c & qiv4d!=.) | mainjob=="") & qiv3d!="A" & qiv3d!="B" & qiv3d!=""
    replace mainjob=qiv3e if ((qiv4e>qiv4a & qiv4e>qiv4b & qiv4e>qiv4c & qiv4e>qiv4d & qiv4e!=.) | mainjob=="") & qiv3e!="A" & qiv3e!="B" & qiv3e!=""

    replace mainjob="Agriculture (own land/animals)" if mainjob=="D" | mainjob=="F"
    replace mainjob="Agriculture (others' land/animals)" if mainjob=="C" | mainjob=="E"
    replace mainjob="Brewing" if mainjob=="G"
    replace mainjob="Brick/Charcoal making" if mainjob=="I" | mainjob=="H"
    replace mainjob="Firewood collection" if mainjob=="J"
    replace mainjob="Hawking" if mainjob=="K"
    replace mainjob="Vocation" if mainjob=="L"
    replace mainjob="Shop/Hotel" if mainjob=="M"
    replace mainjob="Professional" if mainjob=="N" | mainjob=="O" | mainjob=="P"
    replace mainjob="Quarrying" if mainjob=="Q"
    replace mainjob="Resale" if mainjob=="R"
    replace mainjob="Other" if mainjob=="S"
    replace mainjob="None" if mainjob=="" & emp_nohh_mo==0

    gen mainjob2=mainjob
    replace mainjob2="Skilled work" if mainjob=="Vocation" | mainjob=="Shop/Hotel" | mainjob=="Professional"
    replace mainjob2="Vending/Re-selling" if mainjob=="Hawking" | mainjob=="Resale"
    replace mainjob2="Bricks/Charcoal/Firewood/Quarry" if mainjob=="Firewood collection" | mainjob=="Quarrying" | mainjob=="Brick/Charcoal making"
    replace mainjob2="Agriculture" if mainjob=="Agriculture (own land/animals)" | mainjob=="Agriculture (others' land/animals)"

    encode mainjob, g(mainjob_num)
    la var mainjob "Main job"
    la var mainjob_num "Main job, numeric"
    la var mainjob2 "Main job, compressed"

    gen brewing=cond(qiv3a=="G" | qiv3b=="G" | qiv3c=="G" | qiv3d=="G" | qiv3e=="G",1,0) if qno!=.

    // Occ choice
    gen occup=0
    replace occup=1 if inschool==1
    replace occup=2 if inschool==1 & emp_nohh_mo==1 & earn_mo>0 & earn_mo!=.
    replace occup=3 if inschool==0 & emp_nohh_mo==1

    gen occup2=0
    replace occup2=1 if inschool==1
    replace occup2=2 if inschool==0 & emp_nohh_mo==1 & earn_mo>0 & earn_mo!=.

    label define activity 0 "Nothing" 1 "School" 2 "Skool/Work" 3 "Work only"
    label values occup activity
    label define activity2 0 "Nothing" 1 "School" 2 "Work only"
    label values occup2 activity2
    la var occup "Respondent works/in school/both?"
    la var occup2 "Respondent works only/in school?"

    gen job_skill=.
    replace job_skill=0 if mainjob=="None"
    replace job_skill=1 if mainjob=="Agriculture (own land/animals)" | mainjob=="Agriculture (others' land/animals)" | mainjob=="Firewood collection" | mainjob=="Quarrying"
    replace job_skill=2 if mainjob=="Resale" | mainjob=="Hawking"
    replace job_skill=3 if mainjob=="Brick/Charcoal making" | mainjob=="Brewing"
    replace job_skill=4 if mainjob=="Vocation" | mainjob=="Shop/Hotel" | mainjob=="Professional"
    replace job_skill=5 if mainjob=="Other"
    label define job_skill 0 "Nothing" 1 "Unskilled Labor" 2 "SmallCap" 3 "MedCap" 4 "Biz/Prof" 5 "Other"
    label values job_skill job_skill

    gen job_skill2=.
    replace job_skill2=0 if job_skill==0
    replace job_skill2=1 if job_skill==1 | job_skill==2
    replace job_skill2=2 if job_skill==3 | job_skill==4
    replace job_skill2=3 if job_skill==5
    label define job_skill2 0 "Nothing" 1 "LowSkill" 2 "HighSkill" 3 "Other"
    label values job_skill2 job_skill2

    la var job_skill "Skill level of main job"
    la var job_skill2 "Skill level of main job, compressed"

    gen skill1=0 if qno!=.
    foreach x in L M N O P {
        foreach y in a b c d e {
            replace skill1=1 if qiv3`y'=="`x'"
            }
        }

    // same as skill1, but excluding vocation
    gen skill2=0 if qno!=.
    foreach x in M N O P {
        foreach y in a b c d e {
            replace skill2=1 if qiv3`y'=="`x'"
            }
        }

    la var skill1 "Any high-skill job (codes L-P)?"
    la var skill2 "Any high-skill job (codes M-P)?"

    gen prof=cond(job_skill==4,1,0) if sample==1

// Size of plots
   // Survey respondents reported plot sizes in acres as well as katalas--both the area (A) and length (L) unit of katalas.
   // Note that during data entry a transfromation was made to standardize katala measures to the A form, where 1 katala (A)=5x10 katalas (L).
   // Generally, a katala (L) is 1.8m (although it can vary up to 2.2m). Thus a single standard katala (A) imples 9m x 18m, or 162 sq metres.
   // One international acre equals approximately 4047 sq metres. Thus there are approximately 25 katalas (A) in an acre.
   // This is the thus coversion rate we use: 25 katalas (A)=1 acre

   foreach n in a b c d e f g h {
    cap drop qiv17`n'1 qiv17`n'2
    ren iv17`n'1 qiv17`n'
    replace qiv17`n'=qiv17`n'/25 if iv17`n'2=="katala"
    drop iv17`n'2
    la var qiv17`n' "Plot size in acres, Plot `n'"
    }

// Total land
    egen land=rowtotal(qiv17a qiv17b qiv17c qiv17d qiv17e qiv17f qiv17g qiv17h) if qno!=.
    la var land "Total acres of land reported"

// Access to land
    gen land_access=cond(land>0,1,0) if land!=.
    la var land_access "Equals 1 if access to land"


** ADDL VARS: SECTION V **

// Health
    gen bad_hlth=cond(qv2==3, 1, 0) if qno!=.
    la var bad_hlth "Self-described health is 'not good'"

// Injury
    foreach x of numlist 4/11 {
            gen qv`x'=.
            replace qv`x'=0 if qv`x'a==1 & qno!=.
            replace qv`x'=1 if qv`x'b==1 & qno!=.
            replace qv`x'=2 if qv`x'b==2 | (qv`x'a==2 & qv`x'b==.) & qno!=.
            replace qv`x'=3 if qv`x'b==3 & qno!=.
            replace qv`x'=0 if qv`x'a==. & qv`x'b==. & qno!=.
            }
    ren qv4 qv4_walk
    ren qv5 qv5_20L
    ren qv6 qv6_dig
    ren qv7 qv7_run
    ren qv8 qv8_10L
    ren qv9 qv9_work
    ren qv10 qv10_stand
    ren qv11 qv11_bend
    gen injury=cond((qv4_>1 & qv4_!=.) | (qv5_>1 & qv5_!=.) | (qv6_>1 & qv6_!=.) | (qv7_>1 & qv7_!=.) | /*
    */ (qv8_>0 & qv8_!=.) | (qv9_>0 & qv9_!=.) | (qv10_>0 & qv10_!=.) | (qv11_>0  & qv11_!=.), 1, 0) if qno!=.

    la de injuries 0 "Yes" 1 "Slight difficulty" 2 "Major difficulty" 3 "Not at all"
    for numlist 4/11: la val qvX_ injuries

    * reclassify pregnant or recently pregnant women as uninjured
    replace injury=0 if qno==585 | qno==96 | qno==36

    replace qv14=1 if qno==513 
    replace qv14=2 if qno==228 | qno==456 
    replace qv14=4 if qno==288 
    replace qv14=5 if qno==156 | qno==458 | qno==434 | qno==453 | qno==583 | qno==512 | qno==526 | qno==362 | qno==289 | qno==365

    gen war_inj=cond(qv14==1 | qv14==2,1,0) if sample==1
    gen lra_inj=cond(qv14==2,1,0) if sample==1

    gen head_inj=cond(qv15==1 & (qv17==1 | qv17==.), 1, 0) if (qv15!=. | qv17!=.) & qv17!=.

** ADDL VARS: SECTION VI **

// Nutrition
    gen eatmeat=cond(qvi3>0 & qvi3!=.,1,0) if qvi3!=.
    gen eattwice=cond(qvi1==2 | qvi1==3,1,0) if qvi1!=.
    la var eatmeat "Has eaten meat in past two weeks"
    la var eattwice "Eats more than once per day"

// Alcohol
    gen alcohol=qvi5a
    replace alcohol=alcohol*7 if qvi5b==2
    replace alcohol=alcohol*30 if qvi5b==3
    replace alcohol=alcohol*365 if qvi5b==4

    gen drinks=cond(alcohol<=7, 1, 0) if alcohol!=.
    gen drinks_hvy=cond(alcohol<=7 & qvi6>1, 1, 0) if alcohol!=. | qvi6!=.
    la var alcohol "Days since last took alcohol"
    la var drinks "Took alcohol in past week"
    la var drinks_hvy "Heavy drinker"

// Recode alcohol/resource variables
// Replace missing vars with 2 if follow-up question is skipped
    for any 10 12 14 16 19 22 24 26: recode qviX (0=0), gen(qviXa)
    replace qvi10a=2 if qvi10==. & qvi11==. & qno!=.
    replace qvi12a=2 if qvi12==. & qvi13=="" & qno!=.
    replace qvi14a=2 if qvi14==. & qvi15==. & qno!=.
    replace qvi16a=2 if qvi16==. & qvi18==. & qno!=.
    replace qvi19a=2 if qvi19==. & qvi21=="" & qno!=.
    replace qvi22a=2 if qvi22==. & qvi23=="" & qno!=.
    replace qvi24a=2 if qvi24==. & qvi25=="" & qno!=.
    replace qvi26a=2 if qvi26==. & qvi27=="" & qno!=.

// Husband drinks?
    replace qvi10=. if ever_married==0
    replace qvi11=. if ever_married==0
    replace qvi10=1 if (qvi10==. | qvi10==.d) & ever_married==1 & (qvi11==1 | qvi11==2)
    replace qvi10=2 if (qvi10==. | qvi10==.d) & ever_married==1 & (qvi11==3 | qvi11==.)
    
    gen husb_drinks=cond(qvi10==1,1,0) if ever_married==1
    gen husb_drunk_often=cond(qvi11==1,1,0) if ever_married==1
    gen husb_drunk_some=cond(qvi11==1 | qvi11==2,1,0) if ever_married==1
    
    la var husb_drinks "Indicator for married and husband ever drinks"
    la var husb_drunk_often "Indicator for married and husband is drunk often"
    la var husb_drunk_some "Indicator for married and husband is drunk sometimes"

// Assistance received
    replace qvi14=1 if qvi14==. & qvi15>0 & qvi15<=3
    replace qvi14=2 if qvi14==. & (qvi15==0 | qvi15==.) & qno!=.
    replace qvi15=. if qvi14==2
    replace qvi16=1 if qvi16==. & qvi18>0 & qvi18<=3
    replace qvi16=2 if qvi16==. & (qvi18==0 | qvi18==.) & qno!=.
    replace qvi18=. if qvi16==2
    replace qvi19=1 if qvi19==. & qvi21!=""
    replace qvi19=2 if qvi19==. & qvi21=="" & qno!=.
    replace qvi20a=. if qvi19==2
    replace qvi22=1 if qvi22==. & qvi23!=""
    replace qvi22=2 if qvi22==. & qvi23=="" & qno!=.
    replace qvi24=1 if qvi24==. & qvi25!=""
    replace qvi24=2 if qvi24==. & qvi25=="" & qno!=.
    replace qvi26=1 if qvi26==. & qvi27!=""
    replace qvi26=2 if qvi26==. & qvi27=="" & qno!=.
    replace qvi28=. if qvi28==0
    
    gen aid_food=cond(qvi14==1,1,0) if qno!=.
    gen aid_food_subs=aid_food
      replace aid_food_subs=0 if qvi15==3
    gen aid_hh_items=cond(qvi16==1,1,0) if qno!=.
    gen aid_cash=cond(qvi19==1,1,0) if qno!=.
    gen aid_med=cond(qvi22==1,1,0) if qno!=.

    gen aid_counsel=1 if aid_counsel_cat!=.
    gen aid_school=1 if aid_school_cat!=.

    label define aid_counsel_cat 1 "hiv / aids" 2 "psycho-social" 3 "return specific" 4 "family planning" 5 "child education / motherhood" 6 " marriage / domestic violence" 8 "life skills" 9 "hygiene" 16 "other"
    label define aid_counsel_cat2 1 "hiv / aids" 2 "psycho-social" 3 "return specific" 4 "family planning" 5 "child education / motherhood" 6 " marriage / domestic violence" 8 "life skills" 9 "hygiene" 16 "other"
    label define aid_school_cat 11 "agricultural training" 12 "business skills" 13 "tailoring / weaving" 15 "other vocational training" 18 "human rights" 10 "water and sanitation"

    gen aid_any=cond(aid_food==1 | aid_hh==1 | aid_cash==1 | aid_med==1 | aid_counsel==1 | aid_school==1,1,0) if qno!=.
    gen aid_nonfood=cond(aid_hh==1 | aid_cash==1 | aid_med==1 | aid_counsel==1 | aid_school==1,1,0) if qno!=.
    gen aid_svcs=cond(aid_cash==1 | aid_med==1 | aid_counsel==1 | aid_school==1,1,0) if qno!=.

	gen services=cond(qix23a!=. | qix23a2!=. | qix23a3!=. | qix23b!=. | qix23b2!=. | qix23c!=. ,1,0)
	replace services=1 if aid_svcs==1
	la var services "Equals 1 if received any services"

// Generate different categories of counseling and training

    gen aid_counsel_ret=0
    replace aid_counsel_ret=1 if aid_counsel_cat==2 | aid_counsel_cat==3 | aid_counsel_cat==8 
    replace aid_counsel_ret=1 if aid_counsel_cat2==2 | aid_counsel_cat2==3 | aid_counsel_cat2==8 

    gen aid_counsel_psych=0
    replace aid_counsel_psych=1 if aid_counsel_cat==2 | aid_counsel_cat==3 
    replace aid_counsel_psych=1 if aid_counsel_cat2==2 | aid_counsel_cat2==3 

    gen aid_counsel_med=0
    replace aid_counsel_med=1 if aid_counsel_cat==1 | aid_counsel_cat==9
    replace aid_counsel_med=1 if aid_counsel_cat2==1 | aid_counsel_cat2==9 

    gen aid_counsel_hiv=0
    replace aid_counsel_hiv=1 if aid_counsel_cat==1 
    replace aid_counsel_hiv=1 if aid_counsel_cat2==1 

    gen aid_counsel_fam=0
    replace aid_counsel_fam=1 if aid_counsel_cat==4 | aid_counsel_cat==5 | aid_counsel_cat==6
    replace aid_counsel_fam=1 if aid_counsel_cat2==4 | aid_counsel_cat2==5 | aid_counsel_cat2==6

    gen aid_school_voc=0
    replace aid_school_voc=1 if aid_school_cat==11 | aid_school_cat==12 | aid_school_cat==13 | aid_school_cat==15 

** ADDL VARS: SECTION VII **

// Indices of distress, hostility, and sociality
    foreach x of numlist 2/44 {
    gen qvii`x'=.
    replace qvii`x'=0 if qvii`x'a==2 | (qvii`x'a==. & qvii`x'b==. & qno!=.)
    replace qvii`x'=1 if qvii`x'b==3
    replace qvii`x'=2 if qvii`x'b==2 | (qvii`x'a==1 & qvii`x'b>=4)
    replace qvii`x'=3 if qvii`x'b==1
    }
    
    foreach x in 2 3 8 9 10 15 20 21 22 24 27 31 37 38 40 {
    replace qvii`x'=2 if qvii`x'a==. & qvii`x'b==. & qno!=.
    replace qvii`x'=-qvii`x'
    }

// Transform asocial measures to prosocial -- 0 to 3 scale in survey:
    replace qvii2=(-1)*qvii2
    replace qvii3=(-1)*qvii3
    replace qvii8=(-1)*qvii8
    replace qvii9=(-1)*qvii9
    replace qvii10=(-1)*qvii10
    replace qvii15=(-1)*qvii15
    replace qvii20=(-1)*qvii20
    replace qvii21=(-1)*qvii21
    replace qvii22=(-1)*qvii22
    replace qvii24=(-1)*qvii24
    replace qvii27=(-1)*qvii27
    replace qvii31=(-1)*qvii31
    replace qvii37=(-1)*qvii37
    replace qvii38=(-1)*qvii38
    replace qvii40=(-1)*qvii40

// Generate indices
    gen distress=(qvii4+qvii5+qvii6+qvii11+qvii12+qvii14+qvii16+qvii26+qvii28+qvii29+qvii32+qvii33+qvii34+qvii35+qvii36+qvii39+qvii41)/3
    gen prosocial=(qvii2+qvii8+qvii9+qvii10+qvii20+qvii21+qvii23+qvii24+qvii27+qvii31+qvii40)/3
    egen hostility=rsum(qvii13 qvii18 qvii25 qvii30)

** ADDL VARS: SECTION VIII **

// Ever beaten by family member or partner?
    replace qviii27=0 if qviii27==. & qno!=.
    replace qviii28=0 if qviii28==. & qno!=.
    replace qviii29=0 if qviii29==. & qno!=.
    
    gen dom_abuse_num=qviii28+qviii29
    gen dom_abuse=cond(dom_abuse_num>0 & qno!=.,1,0)

    la var dom_abuse_num "Number of times beatne in past two months"
    la var dom_abuse "Indicator for ever being beaten in past two months"
    

** ADDL VARS: SECTION IX **

// Abducted dummy

    gen abd=cond(qix1>0 & qix1!=., 1, 0) if qix1!=.
    gen abd2=cond(qix1>1 & qix1!=., 1, 0) if qix1!=.
    gen abd3=cond(qix1>2 & qix1!=., 1, 0) if qix1!=.
    replace abd=cond(hhiig>0 & hhiig!=., 1, 0) if hhiig!=. & qno==.
    replace abd2=cond(hhiig>1 & hhiig!=., 1, 0) if hhiig!=. & qno==.
    replace abd3=cond(hhiig>2 & hhiig!=., 1, 0) if hhiig!=. & qno==.
    la var abd "Ever abducted"
    la var abd2 "Abducted at least twice"
    la var abd3 "Abducted at least three times"

    gen nonabd=cond(abd==0,1,0) if abd!=.
    la var nonabd "Never abducted"

// too young
replace abd=0 if qno==507
replace abd2=0 if qno==507
replace qix1=0 if qno==507
replace abd=0 if qno==507
foreach v of varlist qix3a-qix111b qix2a-qix9cii {
    cap replace `v'=. if qno==507
    cap replace `v'="" if qno==507
    }

// generate and standardize length of abduction data
   // use given length
    foreach i in a b c {
        gen abd_lgth_`i'=qix10`i'1 if qix10`i'2!=.
        replace abd_lgth_`i'=abd_lgth_`i'/30 if qix10`i'2==1
        replace abd_lgth_`i'=abd_lgth_`i'/4 if qix10`i'2==2
        replace abd_lgth_`i'=abd_lgth_`i'*12 if qix10`i'2==4
        }

   // calculate length given dates reported
    foreach i in a b c {
	gen abd_lgth_`i'2=((qix9`i'ii-qix2`i'ii)*12)+(qix9`i'-qix2`i') if abd==1 & qno!=.
	}

   // compare two abduction lengths
    foreach i in a b c {
     qui gen diff_`i'=abd_lgth_`i'-abd_lgth_`i'2
     tab diff_`i' if abd_lgth_`i'2!=0
     }
     drop diff_*

   // use given length as default and replace with calculated length where missing or incomplete
   // use 1 day as default for within-month abductions without a clear length
      replace abd_lgth_a=abd_lgth_a2 if abd_lgth_a==. & abd==1 & qno!=.
      replace abd_lgth_a=1/30 if (abd_lgth_a==. | abd_lgth_a<.04) & abd==1 & qno!=.
      replace abd_lgth_b=abd_lgth_b2 if abd_lgth_b==. & abd==1 & qno!=.
      replace abd_lgth_b=1/30 if (abd_lgth_b==. | abd_lgth_b<.04) & abd2==1 & qno!=.
      replace abd_lgth_c=abd_lgth_c2 if abd_lgth_c==. & abd==1 & qno!=.
      replace abd_lgth_c=1/30 if (abd_lgth_c==. | abd_lgth_c<.04) & abd3==1 & qno!=.

// generate abduction age
     foreach i in a b c {
       gen abd_age_`i'=qix3`i'
        replace abd_age_`i'=age-(abd_lgth_`i'/12) if abd_age_`i'==. & abd==1
        replace abd_age_`i'=round(abd_age_`i')
        }

    gen abd_child=cond(abd==1 & abd_age_a<18,1,0) if abd_age_a!=.
    gen abd_adult=cond(abd==1 & abd_age_a>=18,1,0) if abd_age_a!=.
    la var abd_child "Younger than 18 at 1st abduction"
    la var abd_adult "At least 18 yrs old at 1st abduction"

// replace with data from household survey if missing
    replace abd_age_a=hh_abd_age1 if abd_age_a==. & abd==1
    replace abd_age_b=hh_abd_age2 if abd_age_b==. & abd==1

    la var abd_age_a "Age abducted (1st abduction)"
    la var abd_lgth_a "Months abducted (1st abduction)"
    la var abd_age_b "Age abducted (2nd abduction)"
    la var abd_lgth_b "Months abducted (2nd abduction)"
    la var abd_age_c "Age abducted (3rd abduction)"
    la var abd_lgth_c "Months abducted (3rd abduction)"

// Other abduction length measures
    egen abd_lgth=rsum(abd_lgth_a abd_lgth_b abd_lgth_c) if abd_lgth_a!=. | abd_lgth_b!=. | abd_lgth_c!=. | abd==0
    la var abd_lgth "Months abducted (total)"

    replace abd_lgth=hh_abd_lgth if abd==1 & abd_lgth==. & qno==.

    *Correct errors
    replace abd_lgth=33 if id==1181
    replace abd_lgth=14 if id==1158

    gen labd_lgth=ln(abd_lgth)
    la var labd_lgth "Log of total months abducted"

    gen abd_per=1 if abd==1
    replace abd_per=2 if abd==1 & abd_lgth>=.1
    replace abd_per=3 if abd==1 & abd_lgth>.5
    replace abd_per=4 if abd==1 & abd_lgth>2
    replace abd_per=5 if abd==1 & abd_lgth>12
    replace abd_per=7 if abd==1 & abd_lgth==.
    replace abd_per=6 if abd==1 & no_return==1
    replace abd_per=0 if abd==0
    label define abd_per 0 "Never" 1 "Less than 3 days" 2 "3 to 14 days" 3 "2 Weeks to 2 Months" /*
    */ 4 "2 to 12 Months" 5 ">12 Months" 6 "Did Not Return" 7 "No response"
    la values abd_per abd_per
    la var abd_per "Period of abduction--categories"

    gen abd_per_a=.
    replace abd_per_a=0 if abd==0
    replace abd_per_a=1 if abd==1 & abd_lgth_a<.25
    replace abd_per_a=2 if abd_lgth_a>=.25 & abd_lgth_a<1
    replace abd_per_a=3 if abd_lgth_a>=1 & abd_lgth_a<3
    replace abd_per_a=4 if abd_lgth_a>=3 & abd_lgth_a<12
    replace abd_per_a=5 if abd_lgth_a>=12 & abd_lgth_a!=.
    label values abd_per_a period
    la var abd_per_a "Period of 1st abduction--categories"

// Get abduction variables for longest abduction
    gen abd_age=abd_age_a if abd==1 & qno!=.
    gen abd_l=abd_lgth_a if abd==1 & qno!=.
    gen abd_yr=qix2aii if abd==1 & qno!=.
    gen yr_ret=qix9aii if abd==1 & qno!=.
    gen longest_abd=1 if abd_lgth_a!=. & abd==1
    replace longest_abd=2 if abd_lgth_b>abd_lgth_a & abd2==1
    replace longest_abd=3 if abd_lgth_c>abd_lgth_a & abd_lgth_c>abd_lgth_b & abd3==1
    replace abd_age=abd_age_b if longest==2
    replace abd_yr=qix2bii if longest==2
    replace yr_ret=qix9bii if longest==2
    replace abd_l=abd_lgth_b if longest==2
    replace abd_age=abd_age_c if longest==3
    replace abd_yr=qix2cii if longest==3
    replace yr_ret=qix9cii if longest==3
    replace abd_l=abd_lgth_c if longest==3
    replace abd_l=0 if abd==0

    *Correct errors
    replace abd_l=33 if id==1810
    replace abd_l=14 if id==1158
    replace abd_yr=2004 if id==1158

    la var abd_age "Age at longest-lasting abduction"
    la var abd_l "Duration of longest-lasting abduction"
    la var abd_yr "Year of longest-lasting abduction"
    la var yr_ret "Year of return from longest-lasting abduction"
    la var longest_abd "Which abduction lasted the longest"

// Abduction length dummies and vars

    gen abd_l_1d=cond(abd_lgth>0 & abd_lgth<.04,1,0) if abd==1
    gen abd_l_2to6d=cond(abd_lgth>=0.04 & abd_lgth<.25,1,0) if abd==1
    gen abd_l_1w=cond(abd_lgth==.25,1,0) if abd==1
    gen abd_l_1to4w=cond(abd_lgth>.25 & abd_lgth<1.1,1,0) if abd==1
    gen abd_l_1_2=cond(abd_lgth>=1.1 & abd_lgth<2.5,1,0) if abd==1
    gen abd_l_2_4=cond(abd_lgth>=2.5 & abd_lgth<4,1,0) if abd==1
    gen abd_l_4_7=cond(abd_lgth>=4 & abd_lgth<7,1,0) if abd==1
    gen abd_l_7_11=cond(abd_lgth>=7 & abd_lgth<12,1,0) if abd==1
    gen abd_l_12_16=cond(abd_lgth>=12 & abd_lgth<16,1,0) if abd==1
    gen abd_l_16_25=cond(abd_lgth>=16 & abd_lgth<25,1,0) if abd==1
    gen abd_l_25_50=cond(abd_lgth>=25 & abd_lgth<50,1,0) if abd==1
    gen abd_l_50up=cond(abd_lgth>=50 & abd_lgth<200,1,0) if abd==1

    la var abd_l_1d "Total abduction: <1 day"
    la var abd_l_2to6d "Total abduction: 2-6 days"
    la var abd_l_1w "Total abduction: 1 week"
    la var abd_l_1to4w "Total abduction: 2-4 weeks"
    la var abd_l_1_2 "Total abduction: 1-2.5 months"
    la var abd_l_2_4 "Total abduction: 2.5-4 months"
    la var abd_l_4_7 "Total abduction: 4-7 months"
    la var abd_l_7_11 "Total abduction: 7-11 months"
    la var abd_l_12_16 "Total abduction: 12-16 months"
    la var abd_l_16_25 "Total abduction: 16-25 months"
    la var abd_l_25_50 "Total abduction: 25-50 months"
    la var abd_l_50up "Total abduction: >50 months"

    gen lt_abd=cond(abd==1 & abd_lgth>=8 & abd_lgth!=.,1,0)

    gen labd_l=ln(abd_l)
    gen rescue=cond(qix11a==2,1,0) if qix11a!=.

    lab var lt_abd "Long-term abductee (longer than 8 months)"
    lab var labd_l "Log of duration, longest-lasting abduction"
    lab var rescue "=1 if respondent was rescued during 1st abduction"

// Abduction age dummies
    foreach n of numlist 9/25 {
        quietly gen Z`n'=cond(abd_age==`n',1,0) if abd_age!=.
        lab var Z`n' "Age at longest-lasting abduction: `n'"
        }
    replace Z9=1 if abd_age<9
    replace Z25=1 if abd_age>25 & abd_age<=30
    lab var Z9 "Age at longest-lasting abduction: <=9"
    lab var Z25 "Age at longest-lasting abduction: >=25"

    gen abd_age2=abd_age^2
    gen abd_age3=abd_age^3
    gen abd_age4=abd_age^4

    gen abdage_5to12=cond(abd_age_a>=4 & abd_age_a<12,1,0) if abd_age_a!=.
    gen abdage_12to14=cond(abd_age_a>=12 & abd_age_a<14,1,0) if abd_age_a!=.
    gen abdage_14to18=cond(abd_age_a>=14 & abd_age_a<18,1,0) if abd_age_a!=.
    gen abdage_18to35=cond(abd_age_a>=18 & abd_age_a<=35,1,0) if abd_age_a!=.

    lab var abd_age2 "Age at longest-lasting abduction, squared"
    lab var abd_age3 "Age at longest-lasting abduction, cubic"
    lab var abd_age4 "Age at longest-lasting abduction, quartic"
    lab var abdage_5to12 "Age at first abduction: 5 to 12"
    lab var abdage_12to14 "Age at first abduction: 12 to 14"
    lab var abdage_14to18 "Age at first abduction: 14 to 18"
    lab var abdage_18to35 "Age at first abduction: 18 to 35"

// Abduction year dummies
    gen yr_abd=abd_yr
    replace abd_yr=2004 if yr_abd>=2004 & yr_abd!=.
    replace abd_yr=1994 if yr_abd<=1994
    foreach n of numlist 1994/2004 {
        quietly gen Y`n'=cond(yr_abd==`n',1,0) if yr_abd!=.
        lab var Y`n' "Year of longest-lasting abduction: `n'"
        }
    lab var Y1994 "Year of longest-lasting abduction: <=1994"
    lab var Y2004 "Year of longest-lasting abduction: >=2004"

// Time back
    gen back=age - abd_age_a - (abd_lgth_a/12)
    replace back=age - abd_age_b - (abd_lgth_b/12) if abd2==1
    replace back=age - abd_age_c - (abd_lgth_c/12) if abd3==1
    replace back=0 if back<0
    la var back "Years since returning from bush"

    gen age_back=abd_age_a+(abd_lgth_a/12)
    replace age_back=abd_age_b+(abd_lgth_b/12) if abd2==1
    replace age_back=abd_age_c+(abd_lgth_c/12) if abd3==1
    replace age_back=round(age_back)
    gen age_back2=abd_age+(abd_l/12)
    replace age_back2=round(age_back2)

    gen age_bk_cat=.
    replace age_bk_cat=1 if age_back<=12
    replace age_bk_cat=2 if age_back>=13 & age_back<=17
    replace age_bk_cat=3 if age_back>=18 & age_back<=30
    label define back 1 "<12" 2 "13-17" 3 "18-30"
    label values age_bk_cat back

    la var age_back "Age at return from last abduction"
    la var age_back2 "Age at return from longest-lasting abduction"
    la var age_bk_cat "Recode: Age at return from last abduction"

// Education level at the time of abduction
    gen educ_abd=qix5a if qno!=. & abd==1 & longest==1 & qix5a!=0
    replace educ_abd=qix5b if qno!=. & abd==1 & longest==2 & qix5b!=0
    replace educ_abd=qix5c if qno!=. & abd==1 & longest==3 & qix5c!=0
    replace educ_abd=0 if qno!=. & abd==1 & educ==0
    replace educ_abd=educ if educ_abd==. & qiii9<=abd_yr & qno!=. & abd==1
    replace educ_abd=educ-(qiii9-abd_yr-(abd_lgth/12)) if educ_abd==. & qiii9>abd_yr & qno!=. & abd==1
    replace educ_abd=0 if educ_abd<0
    replace educ_abd=round(educ_abd)
    replace educ_abd=educ if educ_abd==. & qiii9==. & qno!=. & abd==1 & abd_age>15
    replace educ_abd=5 if qno==588
    la var educ_abd "Educational attainment at the time of abduction (longest abduction)"

// In school or employed at time of abduction
    gen inschool_abd=cond(qix5a==0,0,1) if abd==1 & longest==1 & qix5a!=.
    replace inschool_abd=cond(qix5b==0,0,1) if abd==1 & longest==2 & qix5b!=.
    replace inschool_abd=cond(qix5c==0,0,1) if abd==1 & longest==3 & qix5c!=.
    replace inschool_abd=0 if qno!=. & qix5a==. & abd==1 & longest==1 & (abd_yr>qiii9 | qiii9==8888)
    replace inschool_abd=0 if qno!=. & qix5b==. & abd==1 & longest==2 & (abd_yr>qiii9 | qiii9==8888)
    replace inschool_abd=0 if qno!=. & qix5c==. & abd==1 & longest==3 & (abd_yr>qiii9 | qiii9==8888)
    la var inschool_abd "In school at the time of abduction (for longest abduction)"

    gen emp_abd=cond(qix6a==3,0,1) if abd==1 & longest==1 & qix6a!=.
    replace emp_abd=cond(qix6b==3,0,1) if abd==1 & longest==2 & qix6b!=.
    replace emp_abd=cond(qix6c==3,0,1) if abd==1 & longest==3 & qix6c!=.
    la var emp_abd "Employed at the time of abduction (for longest abduction)"

    replace emp_abd=0 if inschool_abd==1 & emp_abd==.
    replace inschool_abd=0 if emp_abd==1 & inschool_abd==.
    replace emp_abd=0 if emp_abd==. & inschool_abd!=.
    replace inschool_abd=0 if inschool_abd==. & emp_abd!=.

    gen occ_abd=cond(inschool_abd==1 | emp_abd==1,1,0) if emp_abd!=. & inschool_abd!=.
    la var occ_abd "In school or employed at the time of abduction (for longest abduction)"


//Orphan/single parent at abduction
	gen abd_no_mthr=0 if abd==1 & no_mthr==0 & abd!=. & no_mthr!=.
	replace abd_no_mthr=0 if abd==1 & no_mthr==1 & abd_age<=lost_mthr_age & abd!=. & abd_age!=. & lost_mthr_age!=.
	replace abd_no_mthr=1 if abd==1 & no_mthr==1 & abd_age>lost_mthr_age & abd!=. & abd_age!=. & lost_mthr_age!=.
	replace abd_no_mthr=1 if abd==1 & no_mthr==1 & lost_mthr_yr==. & qii11==1 & yr_abd>=1997
	replace abd_no_mthr=1 if abd==1 & no_mthr==1 & lost_mthr_yr==. & yr_abd>=2000
	replace abd_no_mthr=1 if abd==1 & no_mthr==1 & abd_no_mthr==. & qii12==.

	gen abd_no_fthr=0 if abd==1 & no_fthr==0 & abd!=. & no_fthr!=.
	replace abd_no_fthr=0 if abd==1 & no_fthr==1 & abd_age<=lost_fthr_age & abd!=. & abd_age!=. & lost_fthr_age!=.
	replace abd_no_fthr=1 if abd==1 & no_fthr==1 & abd_age>lost_fthr_age & abd!=. & abd_age!=. & lost_fthr_age!=.
	replace abd_no_fthr=1 if abd==1 & no_fthr==1 & lost_fthr_yr==. & qii6==1 & yr_abd>=1997
	replace abd_no_fthr=1 if abd==1 & no_fthr==1 & lost_fthr_yr==. & yr_abd>=2000
	replace abd_no_fthr=1 if abd==1 & no_fthr==1 & abd_no_fthr==. & qii7==.

	gen abd_sing_par=0 if abd==1 & no_fthr==0 & no_mthr==0 & abd!=. & no_mthr!=. & no_fthr!=.
	replace abd_sing_par=0 if abd==1 & abd_no_mthr==0 & abd_no_fthr==0 & abd!=. & abd_no_mthr!=. & abd_no_fthr!=.
	replace abd_sing_par=1 if abd==1 & (abd_no_mthr==1 | abd_no_fthr==1) & abd!=.

	gen abd_orph=0 if abd==1 & no_fthr==0 & no_mthr==0 & abd!=. & no_mthr!=.& no_fthr!=.
	replace abd_orph=0 if abd==1 & abd_no_mthr==0 & abd_no_fthr==0 & abd!=. & abd_no_mthr!=. & abd_no_fthr!=.
	replace abd_orph=0 if abd==1 & abd_sing_par==1 & abd!=. & abd_sing_par!=.
	replace abd_orph=1 if abd==1 & abd_no_mthr==1 & abd_no_fthr==1 & abd!=. & abd_no_mthr!=. & abd_no_fthr!=.

     	drop lost_mthr_age lost_mthr_yr lost_fthr_age lost_fthr_yr

    	la var abd_no_mthr "=1 if no mother at time of abduction"
    	la var abd_no_fthr "=1 if no father at time of abduction"
    	la var abd_sing_par "=1 if single parent at time of abduction"
    	la var abd_orph "=1 if orphan at time of abduction"

// Rescue, release, escape

    gen released=cond((qix11a==1 & longest_abd==1) | (qix11b==1 & longest_abd==2) | (qix11c==1 & longest_abd==3),1,0) if abd==1 & sample==1
    gen released_1w=cond(released==1 & ((abd_lgth_a<=.25 & longest_abd==1) | (abd_lgth_b<=.25 & longest_abd==2) | (abd_lgth_c<=.25 & longest_abd==3)),1,0) if abd==1 & sample==1 
    gen released_2w=cond(released==1 & ((abd_lgth_a<=.5 & longest_abd==1) | (abd_lgth_b<=.5 & longest_abd==2) | (abd_lgth_c<=.5 & longest_abd==3)),1,0) if abd==1 & sample==1 
    gen rescued=cond((qix11a==2 & longest_abd==1) | (qix11b==2 & longest_abd==2) | (qix11c==2 & longest_abd==3),1,0) if abd==1 & sample==1
    gen escaped=cond((qix11a==3 & longest_abd==1) | (qix11b==3 & longest_abd==2) | (qix11c==3 & longest_abd==3),1,0) if abd==1 & sample==1

    la var released "Respondent released when abducted"
    la var released_1w "Respondent released & held <=1 week"
    la var released_2w "Respondent released & held <=2 weeks"

    gen qix11=qix11a if longest==1 & qno!=.
    replace qix11=qix11b if longest==2 & qno!=.
    replace qix11=qix11c if longest==3 & qno!=.
    la de return 1 "Released" 2 "Rescued" 3 "Escaped"
    la val qix11 return
    rename qix11 manner_depart
    la var manner_depart "Manner of departure - longest abd"

// UPDF
    gen updf=.
    replace updf=0 if qix17a==2 | qix17b==2 | qix17c==2
    replace updf=1 if qix17a==1 | qix17b==1 | qix17c==1
    replace updf=1 if updf==. & abd==1 & qno!=. & /*
                   */ (qix18a1!=. | qix18b1!=. | qix18c1!=. | qix20a1!=. | qix20b1!=. | qix20c1!=.)
    la var updf "Passed thru UPDF"

// Reception center
    gen reception=.
    replace reception=0 if qix22a==0 | qix22b==0 | qix22c==0
    replace reception=1 if (qix22a>0 & qix22a<=8) | (qix22b>0 & qix22b<=8) | (qix22c>0 & qix22c<=8)
    replace reception=1 if reception==. & abd==1 & qno!=. & (qix23a!=. | qix23b!=. | qix23c!=.)
    replace reception=0 if reception==. & abd==1 & qno!=. & (qix24a1!=. | qix24b1!=. | qix24c1!=.)
    la var reception "Reported ever passing through a reception center"

// Returned home
    gen qix25=qix25a if longest==1 & qno!=.
    replace qix25=qix25b if longest==2 & qno!=.
    replace qix25=qix25c if longest==3 & qno!=.
    recode qix25 (2=0), gen(ret_home)
    drop qix25
    la var ret_home "Returned home after longest-lasting abduction"

// Returned home to husband/parent/grandparent after abduction
   
 *dummy if returned to husband
    gen ret_hus=0
    replace ret_hus=1 if id==10080
    replace ret_hus=1 if id==11150
    replace ret_hus=1 if id==11730
    replace ret_hus=1 if id==12850
    replace ret_hus=1 if id==13730
    replace ret_hus=1 if id==14580
    replace ret_hus=1 if id==15100
    replace ret_hus=1 if id==15220
    replace ret_hus=1 if id==15520
    replace ret_hus=1 if id==16430
    replace ret_hus=1 if id==16840
    replace ret_hus=1 if id==17320
    replace ret_hus=1 if id==17450
    replace ret_hus=1 if id==17840
    replace ret_hus=1 if id==17850
    replace ret_hus=1 if id==17920
    replace ret_hus=1 if id==17980
    replace ret_hus=1 if id==30130
    replace ret_hus=1 if id==30320
    replace ret_hus=1 if id==30340
    replace ret_hus=1 if id==30460
    replace ret_hus=1 if id==30470
    replace ret_hus=1 if id==30530
    replace ret_hus=1 if id==30570

    *dummy if returned to parent or husband
    recode qix29a (1=1) (2=1) (3=0) (4=0) (5=0) (7=0), gen(ret_parent1)
    recode qix29b (1=1) (2=1) (3=0) (4=0) (5=0) (7=0), gen(ret_parent2)
    recode qix29c (1=1) (2=1) (3=0) (4=0) (5=0) (7=0), gen(ret_parent3)
    recode qix29d (2=1) (3=0) (4=0) (5=0) (6=0) (7=0), gen(ret_parent4)
    gen ret_to_parent=cond(ret_parent1==1|ret_parent2==1|ret_parent3==1|ret_parent4==1,1,0)
    gen ret_par_hw=cond(ret_to_parent==1|ret_hus==1,1,0)
    drop ret_parent1 ret_parent2 ret_parent3 ret_parent4
    la var ret_par_hw "Returned to parent or husband/wife following abduction"

    *dummy if returned to parent, husband, grandparent
    recode qix29a (1=1) (2=1) (3=0) (4=1) (5=0) (7=0), gen(ret_pr_gr1)
    recode qix29b (1=1) (2=1) (3=0) (4=1) (5=0) (7=0), gen(ret_pr_gr2)
    recode qix29c (1=1) (2=1) (3=0) (4=1) (5=0) (7=0), gen(ret_pr_gr3)
    recode qix29d (2=1) (3=0) (4=1) (5=0) (6=0) (7=0), gen(ret_pr_gr4)
    gen ret_pr_gr_hw=cond(ret_pr_gr1==1|ret_pr_gr2==1|ret_pr_gr3==1|ret_pr_gr4==1|ret_par_hw==1,1,0)
    drop  ret_pr_gr1 ret_pr_gr2 ret_pr_gr3 ret_pr_gr4
    la var ret_pr_gr_hw "Returned to parents, grandparents or husband/wife after abduction"

    *dummy if returned to parent, husband, grandparent, sibling(s)
    recode qix29a (1=1) (2=1) (3=0) (4=1) (5=1) (7=0), gen(ret_pr_gr_sib1)
    recode qix29b (1=1) (2=1) (3=0) (4=1) (5=1) (7=0), gen(ret_pr_gr_sib2)
    recode qix29c (1=1) (2=1) (3=0) (4=1) (5=1) (7=0), gen(ret_pr_gr_sib3)
    recode qix29d (2=1) (3=0) (4=1) (5=1) (6=0) (7=0), gen(ret_pr_gr_sib4)
    gen ret_pr_gr_hw_sib=cond(ret_pr_gr_sib1==1|ret_pr_gr_sib2==1|ret_pr_gr_sib3==1|ret_pr_gr_sib4==1|ret_pr_gr_hw==1,1,0)
    drop ret_pr_gr_sib1 ret_pr_gr_sib2 ret_pr_gr_sib3 ret_pr_gr_sib4
    la var ret_pr_gr_hw_sib "Returned to parents, grandparents, husband/wife, or siblings after abduction"

    la var ret_hus "Returned to husband after abduction"
    la var ret_to_parent "Returned to parent after abduction"
    replace ret_to_parent=. if qix29a==. & qix29b==. & qix29c==. & qix29d==. & qix29==""
    replace ret_par_hw=. if qix29a==. & qix29b==. & qix29c==. & qix29d==. & qix29==""
    replace ret_pr_gr_hw=. if qix29a==. & qix29b==. & qix29c==. & qix29d==. & qix29==""
    replace ret_pr_gr_hw_sib=. if qix29a==. & qix29b==. & qix29c==. & qix29d==. & qix29==""

// Returned to school
    gen qix26=qix26a if longest==1 & qno!=.
    replace qix26=qix26b if longest==2 & qno!=.
    replace qix26=qix26c if longest==3 & qno!=.
    replace qix26=2 if qix26==. & qix27a!="" & longest==1
    replace qix26=2 if qix26==. & qix27b!="" & longest==2
    replace qix26=2 if qix26==. & qix27c!="" & longest==3
    replace qix26=2 if yr_ret>qiii9 & abd==1 & qix26==. & qno!=.
    replace qix26=1 if yr_ret<=qiii9 & abd==1 & qix26==. & qno!=.
    recode qix26 (2=0), gen(ret_skool)
    drop qix26
    la var ret_skool "Returned to school after longest-lasting abduction"

// Amnesty certificate
    replace qix34=. if qix32==2
    replace qix32=1 if qix34==1 | qix34==2
    replace qix32=2 if abd_lgth<=3.1 & qix32==. & abd==1
    gen amnesty=cond(qix32==1,1,0) if abd==1 & qix32!=.

// Abducted for more than 2 weeks?
    gen twoweek=cond(qix42!=. | qix44!=. | qix45!=.,1,0) if abd==1
    la var twoweek "Equals 1 if responded to IX36 to IX111 (i.e. abducted more than two weeks)"

// Make sure skip patterns are consistent (change survey responses)
    foreach i in 36 36b 37a 37b 37c 38 39 40a 40b 40c 40d 40 41a 41b 41c 41d 41 42 43 44 /*
    */ 45 46 46b 47 48 49 50a 50b 51 52 53 54 55 56 57 58 59 60 61 62a 62b 62c 62 63 64 65a 65b /*
    */ 65c 65 66 67 68 69a 69b 69c 70 71 72 72b 73 74a 74b 74c 74d 74 75 76a 76b 76c 76d 76 77 /*
    */ 78 79 80 81 82 83 84 85 85ii 85iii 85b 86 87 88 89 90 90ii 90iii 90b 91 92 93 {
     cap replace qix`i'=. if twoweek==0
     cap replace qix`i'="" if twoweek==0
    }
    foreach i of numlist 94/111 {
     foreach j in a b {
      cap replace qix`i'`j'=. if twoweek==0
      cap replace qix`i'`j'="" if twoweek==0
     }
    }

// Forced wife, mother
    gen forcewife=cond(qix47==1,1,0) if twoweek==1 & qno!=.
    gen force_wife=cond(forcewife==1,1,0) if abd==1 & qno!=.
    gen bushmom=cond(qix52==1,1,0) if twoweek==1 & qno!=.
    gen bush_mom=cond(bushmom==1,1,0) if abd==1 & qno!=.
    la var forcewife "Respondent was forced wife (for abd >2 wks)"
    la var force_wife "Was forced wife (for all abductees; assumes 0 for abd <=2 wks)"
    la var bushmom "Had child from forced marriage (for abd >2 wks; 0 if no forced marriage)"
    la var bush_mom "Had child from forced marriage (for all abductees; assumes 0 for abd <=2 wks)"

// Categorization of abductees
    gen abd_cat=0 if nonabd==1 & qno!=.
    replace abd_cat=1 if force_wife==0
    replace abd_cat=2 if force_wife==1
    replace abd_cat=3 if bush_mom==1
    la de abd_cat 0 "Not abducted" 1 "Abducted only" 2 "Abducted & forced wife" 3 "Abducted & captive mother"
    la val abd_cat abd_cat
    la var abd_cat "Categories by abduction, forced marriage and motherhood"

// Number of (biological) children born out of captivity
    gen children_nonbush=qii19-qix53
    replace children_nonbush=0 if children_nonbush<0 | qii19==0
    replace children_nonbush=qii19 if forcewife==0 | twoweek==0
    la var children_nonbush "Number of (biological) children born out of captivity"

// Children born in and out of captivity
    gen children_both=1 if bushmom==1 & children_nonbush>0 & children_nonbush!=.
    replace children_both=0 if children_both!=1 & qno!=.
    gen children_bush_only=1 if bushmom==1 & children_nonbush==0
    replace children_bush_only=0 if children_bush_only!=1 & qno!=.
    gen children_nonbush_only=1 if children_both==0 & children_nonbush>0 & children_nonbush!=.
    replace children_nonbush_only=0 if children_nonbush_only!=1 & qno!=.
    la var children_nonbush_only "Respondent had children only out of captivity (>2 wks)"
    la var children_bush_only "Respondent had children only in captivity (>2 wks)"
    la var children_both "Respondent had children both in and out of captivity (>2 wks)"

// Girl motherhood
    // Age of first-born child
    gen age_child1=.
    // Manually code age of first-born child on the basis of qii27
    #delimit ;
    replace age_child1=12 if id==1002; replace age_child1=1 if id==1003; replace age_child1=13 if id==1004; replace age_child1=1 if id==1005; replace age_child1=3 if id==1008; replace age_child1=12 if id==1013; replace age_child1=3 if id==1016; replace age_child1=3 if id==1017; replace age_child1=14 if id==1018; replace age_child1=10 if id==1021; replace age_child1=10 if id==1026; replace age_child1=5 if id==1027; replace age_child1=2 if id==1029; replace age_child1=3 if id==1030; replace age_child1=9 if id==1031; replace age_child1=1.5 if id==1033; replace age_child1=7 if id==1041; replace age_child1=7 if id==1045; replace age_child1=0.25 if id==1051; replace age_child1=10 if id==1052; replace age_child1=3 if id==1054; replace age_child1=4 if id==1058; replace age_child1=9 if id==1062; replace age_child1=1 if id==1063; replace age_child1=2 if id==1064; replace age_child1=2 if id==1065; replace age_child1=7 if id==1073; replace age_child1=10 if id==1075; replace age_child1=2 if id==1079; replace age_child1=6 if id==1083; replace age_child1=6 if id==1087; replace age_child1=10 if id==1090; replace age_child1=0.5 if id==1091; replace age_child1=15 if id==1093; replace age_child1=1.5833333 if id==1094; replace age_child1=12 if id==1096; replace age_child1=10 if id==1098; replace age_child1=5 if id==1101; replace age_child1=2.3333333 if id==1104; replace age_child1=8 if id==1105; replace age_child1=4 if id==1106; replace age_child1=6 if id==1107; replace age_child1=14 if id==1113; replace age_child1=5 if id==1115; replace age_child1=17 if id==1118; replace age_child1=11 if id==1119; replace age_child1=15 if id==1120; replace age_child1=2.3333333 if id==1121; replace age_child1=0.66666666 if id==1123; replace age_child1=10 if id==1124; replace age_child1=4 if id==1128; replace age_child1=4 if id==1129; replace age_child1=9 if id==1130; replace age_child1=13 if id==1131; replace age_child1=4 if id==1132; replace age_child1=1 if id==1133; replace age_child1=0.5 if id==1135; replace age_child1=12 if id==1137; replace age_child1=19 if id==1139; replace age_child1=12 if id==1140; replace age_child1=11 if id==1141; replace age_child1=9 if id==1142; replace age_child1=15 if id==1143; replace age_child1=8 if id==1144; replace age_child1=4 if id==1145; replace age_child1=9 if id==1151; replace age_child1=12 if id==1153; replace age_child1=4 if id==1154; replace age_child1=9 if id==1155; replace age_child1=4 if id==1156; replace age_child1=6 if id==1157; replace age_child1=1.4166667 if id==1158; replace age_child1=4 if id==1160; replace age_child1=6 if id==1161; replace age_child1=16 if id==1162; replace age_child1=12 if id==1163; replace age_child1=3 if id==1164; replace age_child1=8 if id==1165; replace age_child1=12 if id==1172; replace age_child1=2 if id==1173; replace age_child1=16 if id==1175; replace age_child1=0.08333333 if id==1178; replace age_child1=12 if id==1180; replace age_child1=1.5 if id==1181; replace age_child1=7 if id==1182; replace age_child1=6 if id==1185; replace age_child1=1.25 if id==1186; replace age_child1=0.08333333 if id==1188; replace age_child1=4 if id==1189; replace age_child1=13 if id==1192; replace age_child1=8 if id==1193; replace age_child1=4 if id==1195; replace age_child1=13 if id==1196; replace age_child1=6 if id==1198; replace age_child1=8 if id==1200; replace age_child1=7 if id==1201; replace age_child1=0.75 if id==1202; replace age_child1=12 if id==1205; replace age_child1=15 if id==1206; replace age_child1=0.83333333 if id==1207; replace age_child1=0.5 if id==1210; replace age_child1=12 if id==1211; replace age_child1=8 if id==1212; replace age_child1=14 if id==1213; replace age_child1=13 if id==1216; replace age_child1=2 if id==1220; replace age_child1=2 if id==1221; replace age_child1=5 if id==1226; replace age_child1=8 if id==1228; replace age_child1=12 if id==1230; replace age_child1=13 if id==1231; replace age_child1=12 if id==1232; replace age_child1=20 if id==1243; replace age_child1=7 if id==1244; replace age_child1=12 if id==1247; replace age_child1=15 if id==1248; replace age_child1=1 if id==1250; replace age_child1=15 if id==1257; replace age_child1=3 if id==1259; replace age_child1=5 if id==1260; replace age_child1=8 if id==1261; replace age_child1=1 if id==1263; replace age_child1=13 if id==1264; replace age_child1=11 if id==1267; replace age_child1=13 if id==1269; replace age_child1=14 if id==1270; replace age_child1=13 if id==1271; replace age_child1=13 if id==1275; replace age_child1=12 if id==1278; replace age_child1=3 if id==1279; replace age_child1=18 if id==1281; replace age_child1=13 if id==1285; replace age_child1=12 if id==1288; replace age_child1=7 if id==1290; replace age_child1=10 if id==1292; replace age_child1=6 if id==1293; replace age_child1=5 if id==1296; replace age_child1=5 if id==1299; replace age_child1=5 if id==1300; replace age_child1=2 if id==1301; replace age_child1=2 if id==1302; replace age_child1=3 if id==1303; replace age_child1=7 if id==1307; replace age_child1=9 if id==1309; replace age_child1=7 if id==1310; replace age_child1=4 if id==1313; replace age_child1=8 if id==1315; replace age_child1=1.25 if id==1316; replace age_child1=4 if id==1319; replace age_child1=10 if id==1322; replace age_child1=15 if id==1323; replace age_child1=3 if id==1325; replace age_child1=11 if id==1332; replace age_child1=5 if id==1333; replace age_child1=13 if id==1334; replace age_child1=2.5 if id==1335; replace age_child1=0.83333333 if id==1336; replace age_child1=7 if id==1338; replace age_child1=4 if id==1341; replace age_child1=7 if id==1342; replace age_child1=12 if id==1343; replace age_child1=2 if id==1344; replace age_child1=15 if id==1345; replace age_child1=9 if id==1346; replace age_child1=7 if id==1349; replace age_child1=2 if id==1350; replace age_child1=3 if id==1351; replace age_child1=12 if id==1352; replace age_child1=10 if id==1356; replace age_child1=6 if id==1358; replace age_child1=6 if id==1360; replace age_child1=9 if id==1362; replace age_child1=15 if id==1364; replace age_child1=1 if id==1366; replace age_child1=5 if id==1369; replace age_child1=1.6666667 if id==1371; replace age_child1=4 if id==1372; replace age_child1=7 if id==1373; replace age_child1=3 if id==1374; replace age_child1=5 if id==1375; replace age_child1=5 if id==1381; replace age_child1=0.25 if id==1383; replace age_child1=8 if id==1384; replace age_child1=4 if id==1385; replace age_child1=2 if id==1386; replace age_child1=8 if id==1390; replace age_child1=1.25 if id==1391; replace age_child1=0.16666 if id==1392; replace age_child1=2 if id==1393; replace age_child1=2 if id==1394; replace age_child1=0.166666 if id==1395; replace age_child1=18 if id==1399; replace age_child1=9 if id==1401; replace age_child1=18 if id==1402; replace age_child1=19 if id==1403; replace age_child1=4 if id==1404; replace age_child1=8 if id==1406; replace age_child1=6 if id==1407; replace age_child1=14 if id==1409; replace age_child1=4 if id==1411; replace age_child1=3 if id==1412; replace age_child1=6 if id==1413; replace age_child1=1.5 if id==1415; replace age_child1=2.5 if id==1420; replace age_child1=5 if id==1421; replace age_child1=0.66666667 if id==1424; replace age_child1=10 if id==1425; replace age_child1=13 if id==1426; replace age_child1=8 if id==1428; replace age_child1=3 if id==1432; replace age_child1=11 if id==1433; replace age_child1=6 if id==1434; replace age_child1=5 if id==1435; replace age_child1=6 if id==1436; replace age_child1=10 if id==1437; replace age_child1=16 if id==1439; replace age_child1=7 if id==1440; replace age_child1=2.6666667 if id==1442; replace age_child1=7 if id==1443; replace age_child1=5 if id==1448; replace age_child1=10 if id==1450; replace age_child1=0.33333333 if id==1453; replace age_child1=8 if id==1454; replace age_child1=1.0833333 if id==1455; replace age_child1=1.5 if id==1456; replace age_child1=7 if id==1457; replace age_child1=14 if id==1458; replace age_child1=8 if id==1460; replace age_child1=7 if id==1461; replace age_child1=0.5 if id==1462; replace age_child1=14 if id==1463; replace age_child1=10 if id==1465; replace age_child1=7 if id==1466; replace age_child1=7 if id==1467; replace age_child1=7 if id==1470; replace age_child1=3 if id==1471; replace age_child1=10 if id==1472; replace age_child1=14 if id==1473; replace age_child1=18 if id==1477; replace age_child1=10 if id==1478; replace age_child1=3 if id==1479; replace age_child1=1.3333333 if id==1480; replace age_child1=5 if id==1482; replace age_child1=10 if id==1483; replace age_child1=16 if id==1484; replace age_child1=9 if id==1485; replace age_child1=8 if id==1487; replace age_child1=6 if id==1488; replace age_child1=4 if id==1491; replace age_child1=12 if id==1492; replace age_child1=7 if id==1498; replace age_child1=2 if id==1501; replace age_child1=16 if id==1502; replace age_child1=0.33333333 if id==1507; replace age_child1=7 if id==1508; replace age_child1=9 if id==1509; replace age_child1=12 if id==1510; replace age_child1=3 if id==1513; replace age_child1=15 if id==1515; replace age_child1=0.1666666 if id==1516; replace age_child1=5 if id==1518; replace age_child1=4 if id==1519; replace age_child1=2 if id==1520; replace age_child1=6 if id==1521; replace age_child1=6 if id==1522; replace age_child1=14 if id==1524; replace age_child1=0.3333333 if id==1526; replace age_child1=0.66666667 if id==1527; replace age_child1=2 if id==1528; replace age_child1=1 if id==1530; replace age_child1=10 if id==1532; replace age_child1=3 if id==1535; replace age_child1=6 if id==1541; replace age_child1=3 if id==1542; replace age_child1=7 if id==1545; replace age_child1=18 if id==1546; replace age_child1=20 if id==1547; replace age_child1=6 if id==1548; replace age_child1=3 if id==1549; replace age_child1=0.5 if id==1550; replace age_child1=10 if id==1551; replace age_child1=10 if id==1552; replace age_child1=2 if id==1553; replace age_child1=14 if id==1554; replace age_child1=3 if id==1555; replace age_child1=5 if id==1556; replace age_child1=8 if id==1557; replace age_child1=14 if id==1559; replace age_child1=9 if id==1560; replace age_child1=11 if id==1562; replace age_child1=4 if id==1566; replace age_child1=12 if id==1567; replace age_child1=10 if id==1569; replace age_child1=3 if id==1570; replace age_child1=6 if id==1571; replace age_child1=0.75 if id==1574; replace age_child1=12 if id==1576; replace age_child1=0.33333333 if id==1578; replace age_child1=10 if id==1579; replace age_child1=11 if id==1580; replace age_child1=2 if id==1586; replace age_child1=12 if id==1587; replace age_child1=11 if id==1588; replace age_child1=0.75 if id==1589; replace age_child1=1.4166666 if id==1590; replace age_child1=1.4166666 if id==1591; replace age_child1=9 if id==1593; replace age_child1=15 if id==1595; replace age_child1=2 if id==1598; replace age_child1=4 if id==1602; replace age_child1=7 if id==1605; replace age_child1=1 if id==1607; replace age_child1=0.16666667 if id==1609; replace age_child1=1.5833333 if id==1610; replace age_child1=1.5833333 if id==1611; replace age_child1=8 if id==1612; replace age_child1=1.5833333 if id==1613; replace age_child1=2.5 if id==1617; replace age_child1=3 if id==1618; replace age_child1=11 if id==1621; replace age_child1=9 if id==1622; replace age_child1=8 if id==1627; replace age_child1=7 if id==1632; replace age_child1=19 if id==1633; replace age_child1=0.91666667 if id==1634; replace age_child1=10 if id==1635; replace age_child1=10 if id==1638; replace age_child1=11 if id==1639; replace age_child1=16 if id==1643; replace age_child1=7 if id==1644; replace age_child1=9 if id==1648; replace age_child1=14 if id==1650; replace age_child1=7 if id==1652; replace age_child1=10 if id==1655; replace age_child1=15 if id==1656; replace age_child1=0.16666667 if id==1657; replace age_child1=12 if id==1659; replace age_child1=12 if id==1660; replace age_child1=15 if id==1661; replace age_child1=6 if id==1662; replace age_child1=5 if id==1664; replace age_child1=4 if id==1665; replace age_child1=15 if id==1666; replace age_child1=4 if id==1667; replace age_child1=18 if id==1672; replace age_child1=5 if id==1677; replace age_child1=13 if id==1678; replace age_child1=18 if id==1679; replace age_child1=4 if id==1681; replace age_child1=4 if id==1683; replace age_child1=5 if id==1684; replace age_child1=13 if id==1689; replace age_child1=0.166666 if id==1690; replace age_child1=5 if id==1691; replace age_child1=5 if id==1692; replace age_child1=10 if id==1693; replace age_child1=6 if id==1694; replace age_child1=0.08333333 if id==1697; replace age_child1=12 if id==1698; replace age_child1=18 if id==1699; replace age_child1=2 if id==1701; replace age_child1=12 if id==1703; replace age_child1=1.3333333 if id==1705; replace age_child1=2 if id==1708; replace age_child1=13 if id==1709; replace age_child1=10 if id==1710; replace age_child1=2 if id==1716; replace age_child1=0.5833333 if id==1717; replace age_child1=6 if id==1718; replace age_child1=6 if id==1719; replace age_child1=15 if id==1720; replace age_child1=15 if id==1721; replace age_child1=8 if id==1723; replace age_child1=4 if id==1726; replace age_child1=1.4166666 if id==1727; replace age_child1=12 if id==1728; replace age_child1=5 if id==1729; replace age_child1=14 if id==1732; replace age_child1=4 if id==1734; replace age_child1=4 if id==1738; replace age_child1=5 if id==1739; replace age_child1=12 if id==1740; replace age_child1=13 if id==1741; replace age_child1=8 if id==1742; replace age_child1=4 if id==1743; replace age_child1=3 if id==1744; replace age_child1=3 if id==1745; replace age_child1=14 if id==1747; replace age_child1=0.41666667 if id==1750; replace age_child1=18 if id==1751; replace age_child1=1.333333 if id==1752; replace age_child1=3 if id==1753; replace age_child1=14 if id==1755; replace age_child1=13 if id==1756; replace age_child1=3 if id==1757; replace age_child1=4 if id==1759; replace age_child1=4 if id==1761; replace age_child1=11 if id==1764; replace age_child1=7 if id==1765; replace age_child1=8 if id==1767; replace age_child1=13 if id==1768; replace age_child1=0.16666 if id==1769; replace age_child1=2.3333333 if id==1770; replace age_child1=0.33333333 if id==1771; replace age_child1=0.04166667 if id==1772; replace age_child1=10 if id==1774; replace age_child1=7 if id==1775; replace age_child1=3 if id==1777; replace age_child1=8 if id==1778; replace age_child1=10 if id==1780; replace age_child1=8 if id==1781; replace age_child1=1 if id==1783; replace age_child1=5 if id==1784; replace age_child1=18 if id==1785; replace age_child1=10 if id==1786; replace age_child1=7 if id==1788; replace age_child1=6 if id==1789; replace age_child1=5 if id==1792; replace age_child1=3.25 if id==1796; replace age_child1=12 if id==1797; replace age_child1=7 if id==1798; replace age_child1=0.58333333 if id==1799; replace age_child1=7 if id==1800; replace age_child1=14 if id==3003; replace age_child1=11 if id==3004; replace age_child1=13 if id==3006; replace age_child1=18 if id==3008; replace age_child1=14 if id==3009; replace age_child1=14 if id==3010; replace age_child1=16 if id==3012; replace age_child1=12 if id==3013; replace age_child1=15 if id==3018; replace age_child1=8 if id==3019; replace age_child1=2 if id==3020; replace age_child1=20 if id==3021; replace age_child1=13 if id==3024; replace age_child1=3 if id==3025; replace age_child1=10 if id==3029; replace age_child1=14 if id==3030; replace age_child1=12 if id==3031; replace age_child1=18 if id==3032; replace age_child1=7 if id==3033; replace age_child1=11 if id==3034; replace age_child1=10 if id==3038; replace age_child1=3 if id==3039; replace age_child1=14 if id==3041; replace age_child1=18 if id==3045; replace age_child1=14 if id==3046; replace age_child1=17 if id==3047; replace age_child1=16 if id==3052; replace age_child1=13 if id==3053; replace age_child1=9 if id==3057;
    #delimit cr

    // Age at first birth
    gen age_firstbirth=age-age_child1
    // Indicator for girl motherhood
    gen girl_mother=1 if age_firstbirth<18
    replace girl_mother=0 if girl_mother!=1 & qno!=. & ((age!=. & age_child1!=.) | child_bio==0)
    // Indicator for girl mother in and out of captivity
    gen girl_mom_bush=0 if girl_mother==0
    gen girl_mom_nonbush=0 if girl_mother==0
    replace girl_mom_bush=1 if girl_mother==1 & children_bush_only==1
    replace girl_mom_nonbush=0 if girl_mother==1 & children_bush_only==1
    replace girl_mom_bush=0 if girl_mother==1 & children_nonbush_only==1
    replace girl_mom_nonbush=1 if girl_mother==1 & children_nonbush_only==1
    // Twelve remaining cases of girl mothers with children both in and out of captivity were manually coded
    // on the basis of respondent's age at forced marriage (qix49) and abduction and return dates (abd_age_a,
    // abd_lgth_a, abd_age_b, abd_lgth_b, abd_age_c, abd_lgth_c), taking into consideration the ages of
    // second- and third-born children (from qii27) because respondent may have had multiple children while
    // under the age of 18, possibly both in and out of captivity (in the end, there were no such cases)
    #delimit ;
    replace girl_mom_bush=1 if id==1145; replace girl_mom_nonbush=0 if id==1145; replace girl_mom_bush=1 if id==1279; replace girl_mom_nonbush=0 if id==1279; replace girl_mom_bush=1 if id==1465; replace girl_mom_nonbush=0 if id==1465; replace girl_mom_bush=1 if id==1487; replace girl_mom_nonbush=0 if id==1487; replace girl_mom_bush=0 if id==1516; replace girl_mom_nonbush=1 if id==1516; replace girl_mom_bush=0 if id==1519; replace girl_mom_nonbush=1 if id==1519; replace girl_mom_bush=1 if id==1598; replace girl_mom_nonbush=0 if id==1598; replace girl_mom_bush=0 if id==1611; replace girl_mom_nonbush=1 if id==1611; replace girl_mom_bush=1 if id==1635; replace girl_mom_nonbush=0 if id==1635; replace girl_mom_bush=1 if id==1716; replace girl_mom_nonbush=0 if id==1716; replace girl_mom_bush=0 if id==1726; replace girl_mom_nonbush=1 if id==1726; replace girl_mom_bush=1 if id==1743; replace girl_mom_nonbush=0 if id==1743;
    #delimit cr

    la var age_child1 "Age of first-born biological child"
    la var age_firstbirth "Age at first birth"
    la var girl_mother "=1 if respondent bore child before the age of 18"
    la var girl_mom_bush "=1 if respondent bore child conceived in captivity before the age of 18"
    la var girl_mom_nonbush "=1 if respondent bore child conceived out of captivity before the age of 18"

// Convert time-to-forced-marriage to a single variable
    gen qix50=qix50a/30 if qix50b==1
    replace qix50=qix50a/4 if qix50b==2
    replace qix50=qix50a if qix50b==3
    replace qix50=qix50a*12 if qix50b==4
    replace qix50=24 if id==1182
    la var qix50 "Months before forced marriage"

// Indicator for wanting bush husband to support your child
    gen sup_bush_hus=cond(qix61==1 & ((qix62a==1 | qix62a==2 | qix62a==3) | /*
      */ (qix62b==1 | qix62b==2 | qix62b==3) | (qix62c==1 | qix62c==2 | qix62c==3)),1,0) if bushmom==1
    la var sup_bush_hus "Says that bush husband/family/clan should help support child"

// If more than 2 weeks abducted
    gen rewarded=cond(qix38==1,1,0) if twoweek==1
    gen lead=cond(qix42==1,1,0) if twoweek==1.
    gen orders=cond(qix43==1 | qix42==1,1,0) if twoweek==1.
    gen safer=cond(qix44==1,1,0) if twoweek==1
    gen loyal=cond(qix79==1,1,0) if twoweek==1
    gen loyal_often=cond(qix80==1,1,0) if twoweek==1
    gen stay=cond(qix77==1,1,0) if twoweek==1

    foreach n of numlist 94/97 {
    gen qix`n'=0 if twoweek==1
    replace qix`n'=1 if qix`n'a==1 & qix`n'b==3
    replace qix`n'=2 if qix`n'a==1 & (qix`n'b==2 | qix`n'b==.)
    replace qix`n'=3 if qix`n'a==1 & qix`n'b==1
    replace qix`n'=. if qix`n'a==.
    }

    foreach n of numlist 98/101 {
    gen qix`n'=0 if twoweek==1
    replace qix`n'=1 if (qix`n'a==1 | qix`n'a==3) & (qix`n'b==2 | qix`n'b==3 | qix`n'b==.)
    replace qix`n'=2 if (qix`n'a==1 | qix`n'a==3) & qix`n'b==1
    replace qix`n'=. if qix`n'a==.
    }

    label de new_abductees 0 "never" 1 "rarely" 2 "sometimes" 3 "often"
    for any 4 5 6 7: label val qix9X new_abductees

    label de unit_discipline 0 "no" 1 "beaten" 2 "killed"
    for any 98 99 100 101: label val qixX unit_discipline

// Fighter

    gen fighter=cond(qix36==1 | qix37a==1 | qix37b==1 | qix37c==1,1,0) if twoweek==1
    la var fighter "=1 if served as fighter during abduction"

// Allegiance variables

    foreach x of varlist qix44 qix45 qix77 qix79 {
        replace `x'=0 if `x'==2
        replace `x'=1 if `x'==3
        }

    rename qix44 qix44_safer
    rename qix79 qix79_allegiance
    rename qix77 qix77_stay

    gen allegiance_total=qix44_safer+qix77_stay+qix79_allegiance if twoweek==1
    gen allegiance_modhi=cond(allegiance_total>0,1,0) if allegiance_total!=.
    
// Community acceptance (change survey responses)
    foreach n of numlist 102/107 {
     replace qix`n'b=2 if qix`n'a==2 & twoweek==1
    }
    foreach n of numlist 108/111 {
     replace qix`n'b=2 if qix`n'a==2 & bushmom==1
    }

// Community and family acceptance
    replace qix107a=2 if qix107a==. & qix102a!=.
    replace qix107b=2 if qix107b==. & qix107a==2

    recode qix102a (1=1) (2=0)
    recode qix103a (1=1) (2=0)
    recode qix104a (1=1) (2=0)
    recode qix102b (1=1) (2=0)
    recode qix103b (1=1) (2=0)
    recode qix104b (1=1) (2=0)

    recode qix105a (1=1) (2=0)
    recode qix106a (1=1) (2=0)
    recode qix107a (1=1) (2=0)
    recode qix105b (1=1) (2=0)
    recode qix106b (1=1) (2=0)
    recode qix107b (1=1) (2=0)

    gen fam_prob_ever=cond(qix102a==1 | qix103a==1 | qix104a==1,1,0) & (qix102a!=. | qix103a!=. | qix104a!=.)
    gen fam_prob_now=cond(qix102b==1 | qix103b==1 | qix104b==1,1,0) & (qix102b!=. | qix103b!=. | qix104b!=.)
    gen comm_prob_ever=cond(qix105a==1 | qix107a==1 | qix106a==1,1,0) & (qix105a!=. | qix107a!=. | qix106a!=.)
    gen comm_prob_now=cond(qix105b==1 | qix107b==1 | qix106b==1,1,0) & (qix105b!=. | qix107b!=. | qix106b!=.)
    gen fam_prob_better=1 if fam_prob_ever==1 & fam_prob_now==0
    replace fam_prob_better=0 if fam_prob_ever==1 & fam_prob_now==1
    replace fam_prob_better=. if fam_prob_ever==0
    gen comm_prob_better=1 if comm_prob_ever==1 & comm_prob_now==0
    replace comm_prob_better=0 if comm_prob_ever==1 & comm_prob_now==1
    replace comm_prob_better=. if comm_prob_ever==0


** ADDL VARS: SECTION X **

// Fix erroneously missing violence variables
    foreach n of numlist 13/31 {
        ren qx`n' qx`n'a
        }
    foreach n of numlist 2/31 {
        recode qx`n'a (0=0), gen(qx`n')
        replace qx`n'=0 if qno!=. & qx2a==. & qx3a==. & qx4a==. & qx5a==. & qx6a==. & qx7a==. & /*
                        */ qx8a==. & qx9a==. & qx10a==. & qx11a==. & qx12a==. & qx13a==. & /*
                        */ qx14a==. & qx15a==. & qx16a==. & qx17a==. & qx18a==. & qx19a==. & /*
                        */ qx20a==. & qx21a==. & qx22a==. & qx23a==. & qx24a==. & qx25a==. & /*
                        */ qx26a==. & qx27a==. & qx28a==. & qx29a==. & qx30a==. & qx31a==.
        }
    // Replace erroneously missing values
    foreach n of numlist 2/12 {
        replace qx`n'=1 if qno!=. & qx`n'a==. & qx`n'b!=.
        replace qx`n'=2 if qno!=. & qx`n'a==. & qx`n'b==.
        }
    foreach n of numlist 13/31 {
        replace qx`n'=2 if qx`n'==. & qno!=.
        }
    // Turn into indicator
    foreach n of numlist 2/31 {
        replace qx`n'=. if qx`n'==0
        replace qx`n'=0 if qx`n'==2
        }

   replace qx13=1 if qx13==4

// Violence
   gen viol_received=qx2+qx3+qx4+qx5+qx6+qx14
   gen viol_witnessed=qx7+qx8+qx9+qx13+qx18+qx19
   gen viol_experienced=qx2+qx3+qx4+qx5+qx6+qx7+qx8+qx9+qx13+qx14+qx18+qx19
   gen viol_loss=qx11+qx12+qx20+qx21+qx22
   gen viol_personal=qx2+qx3+qx4+qx5+qx6+qx7+qx8+qx9+qx11+qx12+qx13+qx14+qx18+qx19+qx20+qx21+qx22
   gen viol_perp=qx23+qx24+qx25+qx26+qx27+qx28+qx30+qx31 
   gen viol_total=qx2+qx3+qx4+qx5+qx6+qx7+qx8+qx9+qx11+qx12+qx13+qx14+qx18+qx19+qx20+qx21+qx22+qx23+qx24+qx25+qx26+qx27+qx28+qx30+qx31
   gen viol_sex=qx29+qx10

// Specific violent acts indicators

    gen tied=cond(qx5==1,1,0) if qno!=.
    gen carry=cond(qx6==1,1,0) if qno!=.
    gen kill_sol=cond(qx24==1,1,0) if qno!=.
    gen abusedead=cond(qx30==1,1,0) if qno!=.
    gen kill=cond(qx24==1 | qx27==1 | qx28==1,1,0) if qno!=.
    gen harm_fam=cond(qx25==1 | qx27==1,1,0) if qno!=.


// Self blame

    gen courageous=cond(qx38==1 | qx38==3,1,0) if qx38!=.
    gen parents=cond(qx39==1 | qx39==3,1,0) if qx39!=.
    gen blame=cond(qx42==1 | qx42==3,1,0) if qx42!=.
    gen haunted=cond(qx43==1 | qx43==3,1,0) if qx43!=.
    gen troublefam=cond(qx47==1 | qx47==3,1,0) if qx47!=.
    gen guilt=cond(qx48==1 | qx48==3,1,0) if qx48!=.
    gen troublecomm=cond(qx49==1 | qx49==3,1,0) if qx49!=.
    gen neighborsbad=cond(qx50==1 | qx50==3,1,0) if qx50!=.
    gen konyblame=cond(qx51==1 | qx51==3,1,0) if qx51!=.
    gen godsplan=cond(qx52==1 | qx52,1,0) if qx52!=.
    gen cursed=cond(qx53==1 | qx53==3,1,0) if qx53!=.

    gen blame_self=cond(blame==1 | guilt==1 | troublefam==1 | troublecom,1,0)

    gen blame_total=troublefam+guilt+troublecomm+neighborsbad
    gen blame_total_modhi=cond(blame_total>0 & blame_total!=.,1,0)
    gen blame_total_hi=cond(blame_total>3 & blame_total!=.,1,0)

// Support
    foreach x of numlist 55/72 {
        qui replace qx`x'a=2 if qx`x'a==. & qx`x'b==. & qno!=.
        qui replace qx`x'a=2 if qx`x'a==1 & qx`x'b==3
        qui replace qx`x'a=2 if qx`x'a==0 & qx`x'b==.
        qui replace qx`x'a=1 if qx`x'a==. & (qx`x'b!=. | qx`x'b!=3)
        qui gen qx`x'=.
        qui replace qx`x'=0 if qx`x'a==2 | (qx`x'a==. & qx`x'b==.)
        qui replace qx`x'=1 if qx`x'b==2 | (qx`x'a==1 & qx`x'b!=1 & qx`x'b!=3)
        qui replace qx`x'=2 if qx`x'b==1
        }

    label de support 0 "no" 1 "sometimes" 2 "often"
    for num 55/72, noh: label val qxX support

    la var qx55 "Recode: Someone look after family/possessions?"
    la var qx56 "Recode: Someone sit with you?"
    la var qx57 "Recode: Someone who shared experiences?"
    la var qx58 "Recode: Someone take you to work/activities?"
    la var qx59 "Recode: Anyone in community tasked with helping?"
    la var qx60 "Recode: Someone appreciate you?"
    la var qx61 "Recode: Someone give information about program?"
    la var qx62 "Recode: Someone tell you to take heart?"
    la var qx63 "Recode: Someone assist in making plans?"
    la var qx64 "Recode: Someone give advice?"
    la var qx65 "Recode: Someone lend or give you money?"
    la var qx66 "Recode: Someone listen to you?"
    la var qx67 "Recode: Someone loan you something other than money?"
    la var qx68 "Recode: Someone teach you something?"
    la var qx69 "Recode: Someone joke with you?"
    la var qx70 "Recode: Someone pray with you?"
    la var qx71 "Recode: Someone take you to traditional leader?"
    la var qx72 "Recode: Someone pitch in to help you?"

    ren qx55 qx55_minded
    ren qx56 qx56_sat
    ren qx57 qx57_sharedexp
    ren qx58 qx58_work
    ren qx59 qx59_community
    ren qx60 qx60_compliment
    ren qx61 qx61_info
    ren qx62 qx62_takehrt
    ren qx63 qx63_plans
    ren qx64 qx64_advice
    ren qx65 qx65_money
    ren qx66 qx66_listen
    ren qx67 qx67_loan
    ren qx68 qx68_taught
    ren qx69 qx69_joked
    ren qx70 qx70_prayed
    ren qx71 qx71_healer
    ren qx72 qx72_chores

    // Support (excluding QX59)
    egen s=rsum(qx56_sat-qx58_work qx60_compliment-qx70_prayed)
    gen support=s/2
    drop s

    // Social support measure, including all factors with a loading greater than 0.3
    factor qx55_-qx72_ if qno!=., factors(3) means
    rotate, factors(1)
    egen s=rsum(qx55_-qx66_ qx68_-qx70_ qx72_) if qno!=.
    gen support_F=s/2
    drop s

    la var support "Additive index of social support (17 forms, comparable to SWAY I)"
    la var support_F "Additive index of social support (16 forms), based on factor analysis of females only"


** ADDITIONAL VARS: SECTION XI **

// Family support
    gen fam_comfort=qxi1
    gen fam_quarrel=qxi2
    gen fam_caring=qxi3
    replace fam_com=-fam_com
    replace fam_car=-fam_car

    replace fam_com=-2 if fam_com==. & qno!=.
    replace fam_qua=2 if fam_qua==. & qno!=.
    replace fam_car=-2 if fam_car==. & qno!=.

    // Change to 0 to 2 scale
    replace fam_comfort=3+fam_comfort
    replace fam_caring=3+fam_caring
    replace fam_quarrel=fam_quarrel-1

    // generate indices
    gen fam_problems=cond(fam_com==0 | fam_q==0 | fam_car==0,1,0) if qno!=.
    gen fam_connect=fam_comfort+fam_quarrel+fam_caring

    la var fam_comfort "How comfortable living in household (higher values more comfortable)"
    la var fam_quarrel "Quarrels with family (higher values fewer quarrels)"
    la var fam_caring "Family caring (higher values more caring)"
    la var fam_problems "Equals 1 if poor family relations: Quarrel very often, or not at all comfortable or caring"
    la var fam_connect "Index of family connectedness (0 to 6, low to high)"

// Community groups
    foreach x in a b c d e f g h i j k m {
        replace qxi4`x'=. if qxi4`x'>1 | qxi4`x'==0
        }

    egen groups=rsum(qxi4a-qxi4h qxi4k qxi4l qxi4m)
    la var groups "Number of social groups in which respondent is member"

    gen group=cond(groups>0,1,0) if groups!=.
    la var group "Equals 1 if a member of at least one social group"

    gen comm_mobil=cond(qxi4j==1,1,0)
    gen volunteer=cond(qxi4i==1,1,0)

    gen g_peace=cond(qxi4b==1,1,0)
    gen g_drama=cond(qxi4a==1,1,0)
    gen g_farm=cond(qxi4c==1,1,0)
    gen g_water=cond(qxi4e==1,1,0)
    gen g_skool=cond(qxi4g==1,1,0)
    gen g_sport=cond(qxi4h==1,1,0)
    gen g_church=cond(qxi4f==1,1,0)
    gen g_oth=cond(qxi4m==1,1,0)

    gen g2_women=cond(qxi4k==1,1,0)
    gen g2_credit=cond(qxi4d==1,1,0)
    gen g2_political=cond(qxi4l==1,1,0)

// Religion
    gen catholic=cond(qxi5==1, 1, 0) if qxi5!=.
    gen saved=cond(qxi5==2, 1, 0) if qxi5!=.
    gen church=cond(qxi6==1, 1, 0) if qxi6!=.
    la var catholic "Catholic"
    la var saved "Saved"
    la var church "Attends church regularly"

// Community Connectedness
    gen comm_understand=qxi7 
    gen comm_important=qxi8  
    gen comm_quar=qxi9
    gen comm_sisters=qxi10
    
    // Change to 0 to 2 scale
    replace comm_quar=comm_quar-1
    replace comm_understand=-comm_understand
    replace comm_understand=3+comm_understand
    replace comm_sisters=-comm_sisters
    replace comm_sisters=3+comm_sisters

    replace comm_qua=0 if qno==353
    replace fam_qua=0 if qno==353
    replace fam_car=0 if qno==353
    replace comm_qua=1 if comm_qua==. & qno!=.
    replace fam_qua=1 if fam_qua==. & qno!=.
    replace fam_car=1 if fam_car==. & qno!=.

    // Generate indices
    gen comm_connect=comm_understand+comm_quar+comm_sisters

// Voting
    gen vote05=cond(qxi39==1, 1, 0) if qxi39!=.
    replace vote05=. if qii1<17
    gen vote06=cond(qxi34==1, 1, 0) if qxi34!=.
    replace vote06=. if qii1<17

** ADDL VARS: SECTION XII **

// Correlates of Movement
    gen movement=1
    replace movement=0 if  L_main_camp==1
    replace movement=. if  L_town==1
    replace movement=. if  L_dk==1
    la variable movement "Dummy for movement (0 if in main camp or town, 1 if in sat camp or home)"

    // Fix interview start and end times
    foreach i in qxiii6 qi7b {
        egen `i'hr_=ends(`i'), head punct(":") trim
        egen `i'min_=ends(`i'), tail punct(":") trim
        gen `i'hr=real(`i'hr_)
        gen `i'min=real(`i'min_)
        drop `i'hr_ `i'min_
        replace `i'hr=`i'hr+12 if `i'hr>=1 & `i'hr<=5
        }

        // Length of interview
        gen qxiii6_length=((qxiii6hr-qi7bhr)*60)+(qxiii6min-qi7bmin)
        replace qxiii6_length=. if qxiii6_length<0
        la var qxiii6hr "Interview end hour"
        la var qxiii6min "Interview end min"
        la var qi7bhr "Interview start hour"
        la var qi7bmin "Interview start min"
        la var qxiii6_length "Interview length (mins)"

        // RA evaluations
        *table qi8b
       * table qi8b if qxiii6_length!=., c(mean qxiii6_length)  f(%9.0f)


** ABSENTEES **

// Longest abduction for absentees

    gen temp_abd_age_abs=abd_age_a if abd==1 & qno==.
    gen temp_abd_l_abs=hhiih_lgth if abd==1 & qno==.
    gen temp_abd_yr_abs=hhiih_yr if abd==1 & qno==.
    gen temp_longest_abd_abs=1 if hhiih_lgth!=. & abd==1 & qno==.
    replace temp_longest_abd_abs=2 if hhiii_lgth>hhiih_lgth & hhiig>1 & hhiii_lgth!=. & qno==.
    replace temp_abd_age_abs=abd_age_b if temp_longest_abd_abs==2 & qno==.
    replace temp_abd_yr_abs=hhiii_yr if temp_longest_abd_abs==2 & qno==.
    replace temp_abd_l_abs=hhiii_lgth if temp_longest_abd_abs==2 & qno==.
    replace temp_abd_l_abs=0 if abd==0 & qno==.
 
    replace abd_age=temp_abd_age_abs if qno==.
    replace abd_l=temp_abd_l_abs if qno==.
    replace abd_yr=temp_abd_yr_abs if qno==.
    replace longest_abd=temp_longest_abd_abs if qno==.

    drop temp*

** ADDITIONAL IMPUTATION **

// Impute missing values for control variables no_fthr96 and no_mthr96 (temporarily)
    probit no_fthr96 age age2 age3 age4 C_ach-C_pal catholic fthr_ed0 fthr_ed7 fthr_edhi mthr_ed0 /*
                  */ mthr_ed7 mthr_edhi hh_size96 log_land hh_fthr_frm hh_cattle hh_stock hh_plow
    predict yhat
    replace no_fthr96=round(yhat) if no_fthr96==. & qno!=.
    drop yhat
    probit no_mthr96 age age2 age3 age4 C_ach-C_pal catholic fthr_ed0 fthr_ed7 fthr_edhi mthr_ed0 /*
                  */ mthr_ed7 mthr_edhi hh_size96 log_land hh_fthr_frm hh_cattle hh_stock hh_plow
    predict yhat
    replace no_mthr96=round(yhat) if no_mthr96==. & qno!=.
    drop yhat

** Interactions **

foreach var of varlist hh_size96 hh_land hh_cattle hh_stock abd_lgth {
 gen `var'_2=`var'^2
 gen `var'_3=`var'^3
 gen `var'_4=`var'^4
 }

** CONSTRUCT WEIGHTS **

global D "A14-A29 C_ach-C_paj G*"
global ctrls "hh_fthr_pro hh_fthr_sol hh_wfp hh_immed_fam hh_size96 hh_no_info hh_size96_2 hh_size96_3 hh_size96_4 hh_land hh_land_2 hh_land_3 hh_land_4 hh_cattle hh_cattle_2 hh_cattle_3 hh_cattle_4 hh_stock hh_stock_2 hh_stock_3 hh_stock_4 hh_plow"

gen hh_no_info=cond(hhiib_05==. | hhiic==.,1,0)

// Construct weights for observable attrition due to non-survival/non-return
    gen A33_34=cond(A33==1 | A34==1,1,0) if A34!=. | A33!=.
    logit sample A14-A32 A33_34 C_ach-C_pal $ctrls [pweight=i_weight]
       predict samp_hat if sample==1
    gen z=i_weight/samp_hat
    egen sumz=sum(z)
    egen sumi=sum(i_weight)
    gen w_samp=z*sumi/sumz
    drop A33_34 samp_hat z sumz sumi

// Generate dummies for found and unfound youth (absentees)
    gen absentee=cond(sample==1 & qno!=.,0,1) if sample==1
    la var absentee "Indicator for unfound but surviving youth in sample"

    gen found=cond(absentee==0,1,0) if sample==1
    la var found "Indicator for found and surviving youth in sample"

    gen died=no_surv==1 & no_ret==0
    la var died "Indicator for died since 1996"

// Construct weights for observable attrition of unfound youth in the sample
    
    logit absentee A14-A29 C_ach-C_pal abd abd_lgth abd_lgth_2 abd_lgth_3 abd_lgth_4 $ctrls hh_immed_miss hh_land_miss hh_cattle_miss hh_stock_miss hh_plow_miss if sample==1 [pweight=w_samp]
       predict abs_hat if absentee==0
    gen z=w_samp/(1-abs_hat)
    egen sumz=sum(z)
    egen sumi=sum(i_weight)
    gen w_abs=z*sumi/sumz
    drop abs_hat z sumz sumi

// Generate inverse pscore weights

	logit abd $D $ctrls [pw=w_abs]
		predict pscore if found==1
	gen z=w_abs*((abd/pscore)+((1-abd)/(1-pscore))) if found==1
	egen sumz=sum(z)
      egen sumi=sum(i_weight)
	gen w_sel=z*sumi/sumz if found==1
	drop z sumz sumi 

// Convert i_weights to f_weights for use in histograms
    gen f_weight=round(i_weight)
    gen f_weight_abs=round(w_abs)

//Hostile dummy

    gen hostile=cond(hostility>0,1,0) if found==1

** HIGHLIGHT ERRORS **

// Functions to mark values as missing due to error;
    cap pr drop errmiss
    pr de errmiss
        syntax varname [if] [in]
        cap gen ERR_`varlist'=.
        qui replace ERR_`varlist'=.e `if' `in'
        cap replace `varlist'=. `if' `in'
        cap replace `varlist'="" `if' `in'
    end

// Errmiss for experience in unit and for return experience with children
    foreach i of numlist 94/111 {
     errmiss qix`i'b if qix`i'a==1 & qix`i'b==.
    }
    foreach i of numlist 108/111 {
     errmiss qix`i'a if qix52==1 & qix108a==. & qix109a==. & qix110a==. & qix111a==.
    }

// Replace erroneously missing values with error code .e, and "don't know" with .d,
    foreach i of varlist * {
     cap replace `i'=.e if ERR_`i'==.e
     cap replace `i'=".e" if ERR_`i'==.e
    }
    drop ERR_*

//Add missing variable labels, reorder variables, drop variables
   * Code taken from "SWAYII\SWAY_II_relabeling27Jan2009.do"
   	   
	//Fill in missing labels
	
	**Survey questions
	
	la var qi7 "Date of interview"
	
	la var qii32a_any "=1 if owns at least one jerry can"
	la var qii32b_any "=1 if owns at least one wash basin"
	la var qii32c_any "=1 if owns at least one bicycle"
	la var qii32d_any "=1 if owns at least one mattresse"
	la var qii32e_any "=1 if owns at least one radio"
	la var qii32f_any "=1 if owns at least one plate for eating"
	la var qii32g_any "=1 if owns at least one cattle/oxen"
	la var qii32h_any "=1 if owns at least one pig, goat or sheep"
	la var qii32i_any "=1 if owns at least one bird (chickens, turkeys, pigeons, ducks)"
	la var qii32j_any "=1 if owns at least one chair"
	la var qii32k_any "=1 if owns at least one mobile phone"
	la var qii32l_any "=1 if owns at least one ox plough"
	
	la var qvii2 "Recode: How often do you play games, sports or go dancing with your friends?"
	la var qvii3 "Recode: How often do you think you can do most things like others?
	la var qvii4 "Recode: How often are you jumpy?"
	la var qvii5 "Recode: How often are you in constant fear of losing your family?"
	la var qvii6 "Recode: How often do you feel lonely?"
	la var qvii7 "Recode: How often do you destroy things that belong to others? 
	la var qvii8 "Recode: How often do you enjoy talking and spending time with peers?
	la var qvii9 "Recode: How often are you helpful to elders? 
	la var qvii10 "Recode: How often do you feel sympathy for others?
	la var qvii11 "Recode: How often do you worry about too many things? 
	la var qvii12 "Recode: How often do you feel like you do not value your life? 
	la var qvii13 "Recode: How often are you quarrelsome? 
	la var qvii14 "Recode: How often do you feel unloved?
	la var qvii15 "Recode: How often are you confident doing things on your own?
	la var qvii16 "Recode: How often do you think that everything you do is wrong?
	la var qvii17 "Recode: How often do you lie or behave in a dishonest way?
	la var qvii18 "Recode: How often do you take things from other places without permission?
	la var qvii19 "Recode: How often do you disobey your parents/guardians, teachers or elders?
	la var qvii20 "Recode: How often do you enjoy doing things in the community?
	la var qvii21 "Recode: How often do you have love for your peers?
	la var qvii22 "Recode: How often are you satisfied with yourself?
	la var qvii23 "Recode: How often do you prefer being alone rather than being with family and friends?
	la var qvii24 "Recode: How often do you share with others?
	la var qvii25 "Recode: How often do you curse or use abusive language?
	la var qvii26 "Recode: How often do you cry when you remember bad things from the past?
	la var qvii27 "Recode: How often do you help younger ones?
	la var qvii28 "Recode: How often do you have difficulty concentrating?
	la var qvii29 "Recode: How often do you feel helpless?
	la var qvii30 "Recode: How often do you threaten to hurt others? 
	la var qvii31 "Recode: How often do you share your feelings or ideas with your friends?
	la var qvii32 "Recode: How often do you keep to yourself when you are worried?
	la var qvii33 "Recode: How often do you have restless nights?
	la var qvii34 "Recode: How often do you think about bad things from the past?
	la var qvii35 "Recode: How often are you easily irritated? 
	la var qvii36 "Recode: How often do you have nightmares?
	la var qvii37 "Recode: How often do you have confidence about your future?
	la var qvii38 "Recode: How often do you have confidence to be responsible for others?
	la var qvii39 "Recode: How often do you feel sad?
	la var qvii40 "Recode: How often Do other youth like associating with you?
	la var qvii41 "Recode: How often Do you find life difficult whether you are home or elsewhere?
	la var qvii42 "Recode: How often Do you get chest pains when you are over thinking?
	la var qvii43 "Recode: How often Do you get headaches from over thinking?
	la var qvii44 "Recode: How often Does your body shake uncontrollably from over thinking?
	
	la var qix1 "How many times have you been with the LRA?"
	la var qix2a "When did you go with the LRA (1st time)?"
	la var qix2aii "When did you go with the LRA (1st time): Year?"
	la var qix2b "When did you go with the LRA (2nd time)?"
	la var qix2bii "When did you go with the LRA (2nd time): Year?"
	la var qix2c "When did you go with the LRA (3rd time)?"
	la var qix2cii "When did you go with the LRA (3rd time): Year?"
	 
	la var qix9a "In what month and year did you leave the LRA (1st time)?"
	la var qix9aii "In what month and year did you leave the LRA (1st time): Year?"
	la var qix9b "In what month and year did you leave the LRA (2nd time)?"
	la var qix9bii "In what month and year did you leave the LRA (2nd time): Year?"
	la var qix9c "In what month and year did you leave the LRA (3rd time)?"
	la var qix9cii "In what month and year did you leave the LRA (3rd time): Year?"
	
	la var qxi1 "How do you feel living in your household?"
	
	**HH survey questions
	
	la var hhiiage "HH age"
	la var hh_land_any "HH =1 if HH has land"
	la var hh_cattle_any "HH =1 if HH has cattle"
	la var hh_stock_any "HH =1 if HH has livestock"
	la var hh_plow_any "HH =1 if HH has one or more plows"
	la var hh_no_info "HH =1 if no info on hh"
	la var hh_abd "HH =1 if individual in roster row ever abducted"
	
	la var hh_size96 "HH household size in 1996"
	la var hh_size05 "HH household size in 2005"
	la var hh_assets96 "HH 1996 Assets Index"
	la var hh_wealth96 "HH 1996 Wealth Index"
	la var hh_fthr_frm "HH =1 if Father a farmer"
	la var hh_fthr_pro "HH =1 if Father a professional"
	la var hh_fthr_sol "HH =1 if Father a soldier"
	la var hh_avg_educ "HH average education in household"
	la var hh_abd_total "HH abducted in household"
	la var hh_abd_youth "HH youth abducted in household"
	la var hh_war_death "HH Number of household members died in war"
	
	la var hh_abd_age1 "HH age of individual in roster row at first abduction"
	la var hh_abd_age2 "HH age of individual in roster row at 2nd abduction"
	la var hh_age "HH age of individual in roster row"
	la var hh_adults05 "HH total number of adults in household in 2005"
	
	la var hh_farm "HH =1 if individual in roster row a farmer"
	la var hh_land "HH 1996 Land (acres) in household"
	la var hh_cattle "HH 1996 Cattle in household"
	la var hh_stock "HH 1996 Goats/sheep/pigs in household"
	la var hh_plow "HH 1996 Ploughs in household"
	la var hh_inschool "HH =1 if individual in roster row is in school"
	la var hh_job "HH = 1 if individual in roster row has job"
	la var hh_abd_lgth "HH total abduction length for roster row individual"
	
	la var hh_immed_fam "HH =1 Roster row individual in hh with immediate family"
	la var hh_wfp "HH =1 if registered with WFP"
	la var hh_no_info "HH =1 if no info on hh"
	la var hh_immed_miss "HH Imputation flag: Roster row individual in hh with immediate family"
	la var hh_land_miss "HH Imputation flag: 1996 Land (Acres) in household"
	la var hh_cattle_miss "HH Imputation flag: 1996 Cattle in household"
	la var hh_stock_miss "HH Imputation flag: 1996 Goats/sheep/pigs in household"
	la var hh_plow_miss "HH Imputation flag: 1996 Ploughs in household"
	
	la var hh_weight "HH Household weights"
	la var incamp "HH =1 if individual in roster row stays in camp"
	la var hh_weight "HH Sampling Weights: Household Survey"
	la var hh_select "HH Household Probability of Sample Selection"
	
	    lab var hhiib "HHIIb Name"
	    lab var hhiib_05 "HHIIb World Food Program"
	    lab var hhiic "HHIIc Relation to Head"
	    lab var hhiid "HHIId Sex"
	    lab var hhiie "HHIIe Cause of Death"
	    lab var hhiif "HHIIf Age category"
	    lab var hhiig "HHIIg How many times abducted"
	    lab var hhiih_lgth "HHIIh Length of first abduction"
	    lab var hhiih_yr "HHIIh Year of first abduction"
	    lab var hhiii_lgth "HHIIi Length of second abduction"
	    lab var hhiii_yr "HHIIi Year of second abduction"
	    lab var hhiij "HHIIj Highest school level"
	    lab var hhiik "HHIIk Where stay at night"
	    lab var hhiil "HHIIl Current work"
	
	**Constructed variables
	
	    la var L_main_camp "Indicator for living in a main camp"
	    la var L_sat_camp "Indicator for living in a satellite camp"
	    la var L_home "Indicator for living in home village/on own land"
	    la var L_town "Indicator for living in a town (typically Kitgum or Gulu)"
	    la var L_dk "Indicator for do not know current location"
	
	la var brewing "=1 if main job is brewing"
	la var prof "Occupation: business/professional"
	la var war_inj "=1 if has war-related injury"
	la var lra_inj "=1 if has LRA-inflicted injury"
	
	    la var injury "Has lost ability or major difficulty doing basic task"
	    la var qv4_walk "Can walk for 3 miles?"
	    la var qv5_20L "Can carry 20L jerry can for 20 meters?"
	    la var qv6_dig "Can dig in garden?"
	    la var qv7_run "Can run short distance?"
	    la var qv8_10L "Can carry 10L jerry can for 20 meters?"
	    la var qv9_work "Can work half-day?"
	    la var qv10_stand "Can stand up?"
	    la var qv11_bend "Can bow, squat, kneel?"
	    la var head_inj "Unconscious from head injury?"
	
	    la var lost_fthr15 "Father died before respondent turned 15"
	    la var lost_mthr15 "Mother died before respondent turned 15"
	    la var orphan15_both "Lost both parents before respondent turned 15"
	    la var orphan15_one "Lost one parent before respondent turned 15"
	
	la var distress "Index of distress"
	la var hostility "Index of hostile acts"
	la var prosocial "Index of prosocial behavior"
	la var hostile "Indicator if hostility >0"
	
	la var yr_abd "Year abducted 1987-2006"
	la var rescued "=1 if rescued from longest abduction by LRA"
	la var escaped "=1 if escaped from longest abduction"
	
	la var amnesty "=1 if received amnesty certificate"
	
	la var fam_prob_ever "=1 if reported family problems ever"
	la var fam_prob_now "=1 if report family problems now"
	la var comm_prob_ever "=1 if reported community problems ever"
	la var comm_prob_now "=1 if report community problems now"
	la var fam_prob_better "=1 if reported family problems improved"
	la var comm_prob_better "=1 if reported community problems improved"
	
	la var allegiance_total "Elements of allegiance reported 0-3"
	la var allegiance_modhi "=1 if allegiance_total>0" 
	la var courageous "Experiences happened because you were not courageous or strong enough?"
	la var parents "Parents to blame for what happened?"
	la var blame "Blame yourself for what happened?"
	la var troublefam "Do you feel you have caused trouble to your family?"
	la var guilt "Do you feel any guilt about what happened?"
	la var troublecomm "Do you feel you have caused trouble for people in your community?"
	la var konyblame "Kony and LRA leaders to blame for what happened?"
	la var godsplan "Do you believe this happened to you because it was God's plan for you?"
	la var cursed "Do you believe that bad things happened to you because you were cursed?"
	la var neighborsbad "Do neighbors see you as a bad person because of anything you did?"
	
	la var group "=1 if member of at least one group"
	la var comm_mobil "=1 if community mobilizer"
	la var volunteer "=1 if volunteer for NGO"
	la var g_peace "=1 if member of peace club"
	la var g_drama "=1 if member of drama, music, or dance club"
	la var g_farm "=1 if member of farmers group or cooperative"
	la var g_water "=1 if member of water committee"
	la var g_skool "=1 if member of school committee or school club or a school prefect"
	la var g_sport "=1 if member of sports team"
	la var g_church "=1 if member of church, prayer or bible study group"
	la var g_oth "=1 if member of any other community or church group"
	la var g2_women "=1 if member of a women's group"
	la var g2_credit "=1 if member of a loan or credit group"
	la var g2_political "=1 if member of a member of a political party or group"
	la var comm_important "Do you feel like you are an important part of your community?"
	    la var vote05 "Voted in 2005 referendum"
	    la var vote06 "Voted in 2006 Presidential elections"
	
	    la var comm_understand "How much do your neighbors understand you?"
	    la var comm_quar "Quarrels with neighbors"
	    la var comm_sisters "Community closeness"
	    la var comm_connect "Index of community connectedness (0 to 6, low to high)"
	
	la var aid_counsel_cat2 "Category of counseling aid"
	la var aid_school_cat "Category of school/educ aid"
	la var aid_food "Equals 1 if ever received food from an aid agency"
	la var aid_hh_ "Equals 1 if ever received household items from an aid agency"
	la var aid_cas "Equals 1 if ever received cash from an aid agency"
	    la var aid_med "Equals 1 if ever received medicines from an aid agency"
	    la var aid_any "Equals 1 if ever received any aid (food, items, cash, drugs, counseling or schooling) from an aid agency"
	    la var aid_non "Equals 1 if ever received any non-food aid from an aid agency"
	    la var aid_svc "Equals 1 if ever received any non food, non hh item aid from an aid agency"
	    la var aid_food_subs "1 if received food aid regularly or a few times (qvi15==1 or qvi15==2)"
	    la var aid_school "Equals 1 if ever received schooling from an aid agency"
	    la var aid_counsel "Equals 1 if ever received counseling from an aid agency"
	    la var aid_counsel_ret "1 if ever received psycho-social, war trauma, reintegration or life skills counseling"
	    la var aid_counsel_psych "1 if ever received psycho-social, war trauma, or reintegration counseling"
	    la var aid_counsel_med "1 if ever received counseling on HIV/Aids or hygiene"
	    la var aid_counsel_hiv "1 if ever received counseling on HIV/Aids"
	    la var aid_counsel_fam "1 if ever received counseling on family planning, child education, marriage or domestic violence"
	    la var aid_school_voc "1 if ever received vocational / business skill training"
	
	    label var haunted "Equals 1 if haunted by spirits"
	    label var blame_self "Equals 1 if any self-blame, guilt, or caused trouble for family or commmunity"
	    label var blame_total "Sum of caused trouble for family/community, has self-guilt, or neighbors see as bad person"
	    label var blame_total_modhi "Equals 1 if blame_total>0"
	    label var blame_total_hi "Equals 1 if blame_total>3"
	
	la var w_sel "sampling weights x inverse attrition prob  x inverse absentee prob x inverse selection prob"
	la var i_weight "Individual sample weight (for data on all sampled, incl. non-survivors)"
	la var w_samp "Survivor sample weight (for data on all survivors, incl. absentees)"
	la var w_abs "Respondent sample wght (for data on respondents, excl. absentees or non-surviv.)"
	la var f_weight "Rounded i_weight (for use as fweights)"
	la var f_weight_abs "Rounded w_abs (for use as fweights)"
	
	    la var rewarded "Ever rewarded for a job well done"
	    la var lead "Ever give orders to fighters"
	    la var orders "Ever give orders to abductees or fighters"
	    la var safer "Ever feel safer inside rebel group"
	    la var loyal "Ever feel allegiance to Kony and LRA"
	    la var loyal_often "Often felt allegiance to Kony or LRA"
	    la var stay "Ever preferred to stay with rebels"
	
	    la var qix94 "Recode: abductees forced to beat or kill new abductees?"
	    la var qix95 "Recode: fighters mutilate civilians?"
	    la var qix96 "Recode: abductees forced to desecrate dead bodies?"
	    la var qix97 "Recode: abductees punished for no reason?"
	    la var qix98 "Recode: fighter punished for beating/wounding abductee?"
	    la var qix99 "Recode: fighter punished for killing other fighter?"
	    la var qix100 "Recode: fighter punished for killing abductee?"
	    la var qix101 "Recode: fighter punished for killing civilian?"
	
	   la var viol_received "Sum of 6 violent acts received"
	   la var viol_witnessed "Sum of 6 violent acts witnessed"
	   la var viol_experienced "Sum of 12 violent acts received and witnessed"
	   la var viol_loss "Sum of 5 violent acts inflicted on family members"
	   la var viol_personal "Sum of 17 violent acts received, witnessed, or upon family"
	   la var viol_perp "Sum of 8 violent acts perpetrated"
	   la var viol_total "Sum of 25 violent acts received, witnessed, upon family, or perpetrated"
	   la var viol_sex "Sum of 2 acts of sexual violence"
	
	    la var tied "Tied or imprisoned"
	    la var carry "Forced labor"
	    la var kill "Ever forced to kill (soldier, civilian, friend, family)"
	    la var kill_sol "Ever killed soldier(s)"
	    la var harm_fam "Forced to harm family/friend (beat, cut, or kill)"	
	    la var abusedead "Forced to abuse dead"
	
	
	// Dividers
	
	gen SURVEY_SECTION_I=.
	label var SURVEY_SECTION_I "========================================"
	gen SURVEY_SECTION_II=.
	label var SURVEY_SECTION_II "========================================"
	gen SURVEY_SECTION_III=.
	label var SURVEY_SECTION_III "========================================"
	gen SURVEY_SECTION_IV=.
	label var SURVEY_SECTION_IV "========================================"
	gen SURVEY_SECTION_V=.
	label var SURVEY_SECTION_V "========================================"
	gen SURVEY_SECTION_VI=.
	label var SURVEY_SECTION_VI "========================================"
	gen SURVEY_SECTION_VII=.
	label var SURVEY_SECTION_VII "========================================"
	gen SURVEY_SECTION_VIII=.
	label var SURVEY_SECTION_VIII "========================================"
	gen SURVEY_SECTION_IX=.
	label var SURVEY_SECTION_IX "========================================"
	gen SURVEY_SECTION_X=.
	label var SURVEY_SECTION_X "========================================"
	gen SURVEY_SECTION_XI=.
	label var SURVEY_SECTION_XI "========================================"
	gen SURVEY_SECTION_XIII=.
	label var SURVEY_SECTION_XIII "========================================"
	
	gen HH_SURVEY_QUESTIONS=.
	label var HH_SURVEY_QUESTIONS "========================================"
	
	gen IDENTIFYING_VARIABLES=.
	la var IDENTIFYING_VARIABLES "========================================"
	gen EMPLOYMENT_VARIABLES=.
	la var EMPLOYMENT_VARIABLES "========================================"
	gen EDUCATION_VARIABLES=.
	la var EDUCATION_VARIABLES "========================================"
	gen HEALTH_VARIABLES=.
	la var HEALTH_VARIABLES "========================================"
	gen ABDUCTION_VARIABLES=.
	la var ABDUCTION_VARIABLES "========================================"
	gen WAR_EXPERIENCES_VARIABLES=.
	la var WAR_EXPERIENCES_VARIABLES "========================================"
	gen RETURN_EXPERIENCES_VARIABLES=.
	la var RETURN_EXPERIENCES_VARIABLES "========================================"
	gen FAMILY_COMMUNITY_VARIABLES=.
	la var FAMILY_COMMUNITY_VARIABLES "========================================"
	gen SOC_POL_PARTICIPATION_VARIABLES=.
	la var SOC_POL_PARTICIPATION_VARIABLES "========================================"
	gen ASSISTANCE_VARIABLES=.
	la var ASSISTANCE_VARIABLES "========================================" 
	gen HH_BACKGROUND_VARIABLES=.
	la var HH_BACKGROUND_VARIABLES "========================================"
	gen WEIGHTS=.
	la var WEIGHTS "========================================"
	gen LOCATION_AGE_ABDYR_INDICATORS=.
	la var LOCATION_AGE_ABDYR_INDICATORS"========================================"
	
	//Re-order
	
	order HH_SURVEY_QUESTIONS hhi8 hhi9 hhi10 hhi11 hhiib_05 hhiic hhiid hhiie hhiif hhiig hhiih_lgth hhiih_yr hhiii_lgth /*
	*/ hhiii_yr hhiij hhiik hhiil hhiiage hh_abd hh_abd_age1 hh_abd_age2 hh_age hh_size05 hh_size96 hh_adults05 hh_assets96 /*
	*/ hh_wealth96 hh_farm hh_fthr_frm hh_fthr_pro hh_fthr_sol hh_land hh_cattle hh_stock hh_plow hh_land_any hh_cattle_any /*
	*/ hh_stock_any hh_plow_any hh_avg_educ hh_inschool hh_job hh_abd_total hh_abd_youth hh_abd_lgth hh_war_death hh_immed_fam /*
	*/ hh_wfp hh_immed_miss hh_land_miss hh_cattle_miss hh_stock_miss hh_plow_miss hh_no_info
	
	**Survey questions
	
	order SURVEY_SECTION_XIII qxiii6 qxiii6hr qxiii6min qxiii6_length qxiii7 - qxiii14 
	order SURVEY_SECTION_XI qxi1 - qxi41
	order SURVEY_SECTION_X qx1 qx2 - qx31 qx32 - qx54b qx55_minded - qx72_chores qx73 qx74 qx75
	order SURVEY_SECTION_IX qix1 - qix2cii qix3a - qix8c qix9a - qix9cii qix10a1 - qix50b qix50 qix51 - qix93 qix94 - qix101 qix102a - qix111b
	order SURVEY_SECTION_VIII qviii1 qviii1a - qviii26o qviii26 qviii27 - qviii32o qviii32 qviii33 qviii34
	order SURVEY_SECTION_VII qvii1 qvii2 - qvii44 qvii45 qvii45a - qvii45o
	order SURVEY_SECTION_VI qvi1 - qvi10 qvi10a qvi11 qvi12 qvi12a qvi13 qvi14 qvi14a qvi15 /*
	*/ qvi16 qvi16a qvi17 qvi18 qvi19 qvi19a qvi20a qvi20b qvi20c qvi21 qvi22 qvi22a qvi23 qvi24 /*
	*/ qvi24a qvi25 qvi26 qvi26a qvi27 qvi28
	order SURVEY_SECTION_V qv1a -  qv3 qv4_walk - qv11_bend  qv12 - qv17
	order SURVEY_SECTION_IV  qiv1 qiv2 qiv3a qiv3b qiv3c qiv3d qiv3e qiv4a qiv4b qiv4c qiv4d qiv4e qiv5a qiv5b qiv5c qiv5d qiv5e qiv6a /*
	*/ qiv6b qiv6c qiv6d qiv6e qiv7a qiv7b qiv7c qiv7d qiv7e qiv8a1 qiv8b1 qib8c1 qiv8d1 qiv8e1 qiv8a2 qiv8b2 qiv8c2 qiv8d2 qiv8e2 /*
	*/ qiv9a1 qiv9b1 qiv9c1 qiv9d1 qiv9e1 qiv9a2 qiv9b2 qiv9c2 qiv9d2 qiv9e2 qiv10 - qiv17g3 qiv17a - qiv17h qiv18a - qiv24
	order SURVEY_SECTION_III qiii1 - qiii10 qiii11a qiii11a2 qiii11b qiii11b2 qiii12a qiii12b /*
	*/ qiii13a qiii13a1 qiii13a2 qiii13b qiii13b1 qiii13b2 qiii14a qiii14b qiii15a qiii15b
	order SURVEY_SECTION_II qii1 - qii32l qii32a_any - qii32l_any qii33 - qii45
	order SURVEY_SECTION_I qno - qi7b qi7bhr qi7bmin qi8a - qi12   
	
	**Weights and control dummies
	order WEIGHTS w_samp w_abs w_sel f_weight f_weight_abs i_weight hh_select hh_weight sample_prob pscore stratum psu
	order LOCATION_AGE_ABDYR_INDICATORS C_ach - C_ata A14 - A35 G1_14_17 - G8_31_plus abd_l_1d - abd_l_50up Z9 - Z25 abdage_5to12 - abdage_18to35 Y1994 - Y2004
	
	**HH and background characteristics
	order HH_BACKGROUND_VARIABLES age_cat no_fthr no_fthr96 no_fthr01 no_fthr04 no_mthr no_mthr96 no_mthr01 no_mthr04 orphan lost_parent /*
	*/ orphan96 lost_parent96 lost_fthr15 lost_mthr15 orphan15_both orphan15_one fthr_ed mthr_ed fthr_ed0 /*
	*/ fthr_ed4 fthr_ed7 fthr_edhi mthr_ed0 mthr_ed4 mthr_ed7 mthr_edhi live_mthr live_fthr live_par alone /* 
	*/ married ever_married child_bio_num child_nonbio_num child_num child_bio child_any many_children /*
	*/ movement density assets aindex asset_50th asset_75th asset_90th w10 w25 w75 w90 dry catholic church saved /*
	*/ L_main_camp - L_dk
	
	**Services
	order ASSISTANCE_VARIABLES aid_food - aid_school_voc aid_school_cat aid_counsel_cat aid_counsel_cat2  
	
	**Social and political participation
	order SOC_POL_PARTICIPATION_VARIABLES group groups comm_mobil volunteer g_peace g_drama g_farm g_water g_skool g_sport /*
	*/ g_church g_oth g2_women g2_credit g2_political vote05 vote06
	
	**Family/community problems
	order FAMILY_COMMUNITY_VARIABLES comm_understand comm_important comm_quar comm_sisters comm_connect /*
	*/ fam_comfort fam_quarrel fam_caring fam_problems fam_connect troublefam troublecomm neighborsbad /*
	*/ comm_prob_ever comm_prob_now comm_prob_better fam_prob_ever fam_prob_now fam_prob_better
	
	**Return experiences
	order RETURN_EXPERIENCES_VARIABLES rescue rescued released escaped manner_depart reception amnesty updf ret_skool ret_home /*
	*/ ret_par_hw ret_pr_gr_hw ret_pr_gr_hw_sib ret_hus ret_to_parent distress prosocial hostility hostile haunted guilt /*
	*/ godsplan cursed support support_F blame parents konyblame blame_self - blame_total_hi allegiance_total allegiance_modhi
	
	**War experiences
	order WAR_EXPERIENCES_VARIABLES forcewife force_wife bushmom bush_mom abd_cat fighter viol_received - viol_sex /*
	*/ tied carry kill kill_sol abusedead harm_fam rewarded lead safer courageous loyal loyal_often stay
	
	**Abduction information
	order ABDUCTION_VARIABLES  abd abd2 abd3 nonabd abd_lgth_a abd_lgth_b abd_lgth_c abd_age_a abd_age_b /*
	*/ abd_age_c abd_child abd_adult  abd_lgth abd_per abd_per_a abd_age abd_l abd_yr yr_ret longest_abd /*
	*/ abd_age2 abd_age3 abd_age4 lt_abd labd_l yr_abd back age_back age_back2 age_bk_cat educ_abd /*
	*/ inschool_abd emp_abd occ_abd abd_no_mthr abd_no_fthr abd_sing_par abd_orph twoweek
	
	**Health
	order HEALTH_VARIABLES bad_hlth children_nonbush children_both children_bush_only children_nonbush_only age_child1 /*
	*/ age_firstbirth girl_mother girl_mom_bush girl_mom_nonbush sup_bush_hus injury war_inj lra_inj head_inj /*
	*/ sex_active hiv_aware own_condom get_condom use_cond_some use_cond_often eatmeat eattwice alcohol drinks drinks_hvy /*
	*/ husb_drinks husb_drunk_often husb_drunk_some dom_abuse_num dom_abuse
	
	**Education
	order EDUCATION_VARIABLES educ educ_cat inschool illiterate literate ngo_paid p1 p4 p7 educ0 secskool university voc
	
	**Livelihoods
	order EMPLOYMENT_VARIABLES earn_mo - emp_nohh_wk  emp_hrs_wk - prof exper land land_access 
	
	order age adult child incamp camp_yrs
	order age_sample - sample absentee found i_merge died
	order IDENTIFYING_VARIABLES id - block
	
	move orders lead
	move released_1w escaped
	move released_2w escaped
	
	//Drop variables
	
	drop hh_size96_2 hh_size96_3 hh_size96_4 hh_land_2 hh_land_3 hh_land_4 hh_cattle_2 hh_cattle_3 hh_cattle_4 /*
	*/ hh_stock_2 hh_stock_3 hh_stock_4 abd_lgth_2 abd_lgth_3 abd_lgth_4 age2 age3 age4 abd_lgth_a2 abd_lgth_b2 abd_lgth_c2 /*
	*/ hrsa hrsb hrsc hrsd hrse qv4a - qv11b  qvii2a - qvii44b qix94a - qix101b  qx2a - qx31a  qx55a - qx72b log_land hhiiii
	drop lwage_mo lwage_wk lwage_mo2 lwage_mo_hr skill1 skill2 labd_lgth abd_age2 abd_age3 abd_age4
	drop qvi10 qvi12 qvi14 qvi16 qvi19 qvi22 qvi24 qvi26
	
	**drop variables for anonymity
	*drop qxii1c qxii1d qxii1e qxii3c qxii3d qxii3f qxii3g qxii3i qxii1g qxii1h qxii3 qxii6a qxii6c qxii6d qxii6e
	*qxii6f qxii6g qxii6i qxii6j qxii6k qxii7a qxii7c qxii7d qxii7e qxii7f qxii7g qxii7i qxii7j qxii7k qviii1
	drop qx1 qvii1 qxii1a qxiii9a qxiii9b qxiii9c qxiii9d hhiib
	*qxii1 qxii1b qxii2 qxii3a qxii3a2 qxii3b qxii3b2 qxii4 qxii5 qxii7k 
	
	*drop hh_head1_name hh_head1_surname hh_head2_name hh_head2_surname name qi8b

   
   ***************************

// Save Phase II dataset
sa "SWAYII/SWAY_II_April2013.dta", replace
